@@ -6,22 +6,7 @@
 //
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2010 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -29,7 +14,7 @@
 // 
 // Description: 
 //
-// Status: QS-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -43,13 +28,13 @@
 
 #define DEFTMPSIZE 100
 #define DEFSYSSIZE 100
+#define DEFAPPSIZE 3000
 #define DEFLOGNUM 3
 #define DEFLOGSIZE 1000000
 #define DEFSORTAREASIZE 10000000
-#define DEFTSROOT "./"
-#define DEFPRIMARY "localhost"
-#define DEFSECONDARY "localhost"
-#define DEFTSTICKET "tsticket.xml"
+#define DEFTSROOT ""
+#define DEFPRIMARY Chain()
+#define DEFSECONDARY Chain()
 #define DEFTIMEOUT 10
 
 #define MAXHOSTLEN 25
@@ -60,7 +45,7 @@ CegoAdmAction::CegoAdmAction(CegoAdminHandler *pAH, CegoModule *pModule, CegoAdm
     _pC = 0;
     _pModule = pModule;
     _pAH = pAH;
-    _pit = 0;
+    _pit = Chain();
     _rawMode = rawMode;
     _resultMode = resultMode;
     _keepTicket = false;
@@ -110,7 +95,7 @@ void CegoAdmAction::setCommandChain(char *pC)
     _buMsg = Chain("NONE");
     _syncCmd = Chain("");
     _timeout = DEFTIMEOUT;
- }
+}
 
 char CegoAdmAction::nextChar() 
 {
@@ -152,7 +137,6 @@ void CegoAdmAction::backChar()
 {
     _i--;
 }
-
     
 void CegoAdmAction::printTokenList() 
 {
@@ -166,7 +150,6 @@ void CegoAdmAction::printTokenList()
 	cout << *pS << endl;
 	pS = tlist.Next();
     }
-    
 }
 
 void CegoAdmAction::cleanUp() 
@@ -174,31 +157,248 @@ void CegoAdmAction::cleanUp()
     
 }
 
+void CegoAdmAction::listQueryCacheAction()
+{
+
+    Chain tableSet;
+
+    Chain* pS = getTokenList().First();
+    if ( pS )
+    {
+	tableSet = *pS;
+    }
+
+    CegoAdminHandler::ResultType res;
+
+    res = _pAH->reqListQueryCache(tableSet);
+    handleMedResult(res);
+    
+    CegoTableObject oe;
+    ListT<ListT<CegoFieldValue> > info;
+    
+    Chain format;
+    if ( _pAH->getCacheList(oe, info, format) )
+    {
+	
+	CegoOutput output(oe.getSchema(), format);
+	output.setRawMode(_rawMode);
+	output.tabOut(info);
+    }
+    Chain msg;
+    _pAH->getMsg(msg);
+    
+    if ( _rawMode == false )
+	cout << msg << endl;
+}
+
+void CegoAdmAction::cleanQueryCacheAction()
+{
+    
+    Chain tableSet;
+
+    Chain* pS = getTokenList().First();
+    if ( pS )
+    {
+	tableSet = *pS;
+    }
+
+    CegoAdminHandler::ResultType res;
+
+    res = _pAH->reqCleanQueryCache(tableSet);
+    handleMedResult(res);
+
+    Chain msg;
+    _pAH->getMsg(msg);
+    if ( _rawMode == false )
+	cout << msg << endl;
+}
+
+void CegoAdmAction::setQueryCacheEntryAction()
+{
+    Chain tableSet;
+    int numQueryEntry;
+
+    Chain* pS = getTokenList().First();
+    if ( pS )
+    {
+	tableSet = *pS;
+    }
+    pS = getTokenList().Next();
+    pS = getTokenList().Next();
+    if ( pS )
+    {
+	numQueryEntry = pS->asInteger();
+    }
+
+    CegoAdminHandler::ResultType res;
+
+    res = _pAH->reqSetQueryCacheEntry(tableSet, numQueryEntry);
+    handleMedResult(res);
+
+    Chain msg;
+    _pAH->getMsg(msg);
+    if ( _rawMode == false )
+	cout << msg << endl;    
+}
+
+void CegoAdmAction::setQueryCacheSizeAction()
+{
+
+    Chain tableSet;
+    int queryCacheSize = 0;
+
+    Chain* pS = getTokenList().First();
+    if ( pS )
+    {
+	tableSet = *pS;
+    }
+    pS = getTokenList().Next();
+    pS = getTokenList().Next();
+    if ( pS )
+    {
+	queryCacheSize = pS->asInteger();
+    }
+
+    CegoAdminHandler::ResultType res;
+
+    res = _pAH->reqSetQueryCacheSize(tableSet, queryCacheSize);
+    handleMedResult(res);
+
+    Chain msg;
+    _pAH->getMsg(msg);
+    if ( _rawMode == false )
+	cout << msg << endl;
+}
+
+void CegoAdmAction::listTableCacheAction()
+{
+
+    Chain tableSet;
+    Chain* pS = getTokenList().First();
+    if ( pS )
+    {
+	tableSet = *pS;
+    }
+
+    CegoAdminHandler::ResultType res;
+    res = _pAH->reqListTableCache(tableSet);
+    handleMedResult(res);
+    
+    CegoTableObject oe;
+    ListT<ListT<CegoFieldValue> > info;
+    
+    Chain format;
+    if ( _pAH->getCacheList(oe, info, format) )
+    {
+	
+	CegoOutput output(oe.getSchema(), format);
+	output.setRawMode(_rawMode);
+	output.tabOut(info);
+    }
+    Chain msg;
+    _pAH->getMsg(msg);
+    
+    if ( _rawMode == false )
+	cout << msg << endl;
+}
+
+void CegoAdmAction::cleanTableCacheAction()
+{
+
+    Chain tableSet;
+    
+    Chain* pS = getTokenList().First();
+    if ( pS )
+    {
+	tableSet = *pS;
+    }
+
+    CegoAdminHandler::ResultType res;
+
+    res = _pAH->reqCleanTableCache(tableSet);
+    handleMedResult(res);
+
+    Chain msg;
+    _pAH->getMsg(msg);
+    if ( _rawMode == false )
+	cout << msg << endl;
+}
+
+void CegoAdmAction::setTableCacheEntryAction()
+{
+    Chain tableSet;
+    int numEntry;
+    
+    Chain* pS = getTokenList().First();
+    if ( pS )
+    {
+	tableSet = *pS;
+    }
+    pS = getTokenList().Next();
+    pS = getTokenList().Next();
+    if ( pS )
+    {
+	numEntry = pS->asInteger();
+    }
+
+    CegoAdminHandler::ResultType res;
+
+    res = _pAH->reqSetTableCacheEntry(tableSet, numEntry);
+    handleMedResult(res);
+
+    Chain msg;
+    _pAH->getMsg(msg);
+    if ( _rawMode == false )
+	cout << msg << endl;
+}
+
+void CegoAdmAction::setTableCacheSizeAction()
+{
+    Chain tableSet;
+    int cacheSize = 0;
+    
+    Chain* pS = getTokenList().First();
+    if ( pS )
+    {
+	tableSet = *pS;
+    }
+    pS = getTokenList().Next();
+    pS = getTokenList().Next();
+    if ( pS )
+    {
+	cacheSize = pS->asInteger();
+    }
+
+    CegoAdminHandler::ResultType res;
+
+    res = _pAH->reqSetTableCacheSize(tableSet, cacheSize);
+    handleMedResult(res);
+
+    Chain msg;
+    _pAH->getMsg(msg);
+    if ( _rawMode == false )
+	cout << msg << endl;
+}
+
 void CegoAdmAction::setDefault()
 {
     _primary = DEFPRIMARY;
     _secondary = DEFSECONDARY;
     _tsRoot = DEFTSROOT;
-    _tsTicket = DEFTSTICKET;
     _logSize = DEFLOGSIZE;
     _logNum = DEFLOGNUM;
     _sortAreaSize = DEFSORTAREASIZE;
     _sysSize = DEFSYSSIZE;
+    _appSize = DEFAPPSIZE;
     _tmpSize = DEFTMPSIZE;
 }
 
-void CegoAdmAction::setPitFormat(const Chain& pitFormat)
-{
-    _pitFormat = pitFormat;
-}
-
 /////////////////////////////////
 // cego semantic action methods //
 /////////////////////////////////
 
 void CegoAdmAction::lockInfoAction() 
 {
-
     Chain* pS = getTokenList().First();
     pS = getTokenList().Next();
     if ( pS )
@@ -273,15 +473,12 @@ void CegoAdmAction::lockInfoAction()
 	output.tabOut(info);
 	if ( _rawMode == false )
 	    cout << msg << endl;
-
     }
 }
 
-
 void CegoAdmAction::lockStatAction() 
 {
 
-    
     CegoAdminHandler::ResultType res;
     
     res = _pAH->reqLockStat();    
@@ -300,7 +497,6 @@ void CegoAdmAction::lockStatAction()
     output.tabOut(info);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::poolInfoAction() 
@@ -342,31 +538,40 @@ void CegoAdmAction::poolInfoAction()
 	cout << msg << endl;
 }
 
-
 void CegoAdmAction::poolEntryListAction() 
 {
         
     CegoAdminHandler::ResultType res;
     
     res = _pAH->reqPoolEntryList();
-    handleMedResult(res);    
 
-    Chain msg;
-    _pAH->getMsg(msg);
+    while ( res == CegoAdminHandler::ADM_INFO )
+    {
+
+	Chain msg;
+	_pAH->getMsg(msg);
+	
+	CegoTableObject oe;
+	ListT<ListT<CegoFieldValue> > info;
+	
+	_pAH->getPoolEntryList(oe, info);
+
+	CegoOutput output(oe.getSchema());
+	output.setRawMode(_rawMode);
+	output.tabOut(info);
+		
+	res = _pAH->nextInfo();
+	
+    }
     
-    CegoTableObject oe;
-    ListT<ListT<CegoFieldValue> > info;
-    
-    _pAH->getPoolEntryList(oe, info);
-    
-    CegoOutput output(oe.getSchema());
-    output.setRawMode(_rawMode);
-    output.tabOut(info);
     if ( _rawMode == false )
+    {
+	Chain msg;
+	_pAH->getMsg(msg);	
 	cout << msg << endl;
+    }
 }
 
-
 void CegoAdmAction::dbSessionInfoAction() 
 {
         
@@ -389,10 +594,8 @@ void CegoAdmAction::dbSessionInfoAction()
     
     if ( _rawMode == false )
 	cout << msg << endl;
-	
 }
 
-
 void CegoAdmAction::listTableSetAction() 
 {
         
@@ -414,7 +617,6 @@ void CegoAdmAction::listTableSetAction()
     output.tabOut(info);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::threadInfoAction() 
@@ -439,7 +641,6 @@ void CegoAdmAction::threadInfoAction()
 
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::dbThreadInfoAction() 
@@ -464,10 +665,8 @@ void CegoAdmAction::dbThreadInfoAction()
     output.tabOut(info);
     if ( _rawMode == false )
 	cout << msg << endl;
-    
 }
 
-
 void CegoAdmAction::dbThreadLastQueryAction() 
 {
 
@@ -478,7 +677,6 @@ void CegoAdmAction::dbThreadLastQueryAction()
     {
 	threadId = pS->asInteger();
     }
-
         
     CegoAdminHandler::ResultType res;
     
@@ -499,7 +697,6 @@ void CegoAdmAction::dbThreadLastQueryAction()
     output.tabOut(info);
     if ( _rawMode == false )
 	cout << msg << endl;
-    
 }
 
 void CegoAdmAction::admThreadInfoAction() 
@@ -524,7 +721,6 @@ void CegoAdmAction::admThreadInfoAction()
     output.tabOut(info);
     if ( _rawMode == false )
 	cout << msg << endl;
-    
 }
 
 void CegoAdmAction::logThreadInfoAction() 
@@ -549,7 +745,6 @@ void CegoAdmAction::logThreadInfoAction()
     output.tabOut(info);
     if ( _rawMode == false )
 	cout << msg << endl;
-    
 }
 
 void CegoAdmAction::copyInfoAction() 
@@ -573,8 +768,7 @@ void CegoAdmAction::copyInfoAction()
     output.setRawMode(_rawMode);
     output.tabOut(info);
     if ( _rawMode == false )
-	cout << msg << endl;
-	
+	cout << msg << endl;	
 }
 
 void CegoAdmAction::buFileInfoAction()
@@ -610,7 +804,6 @@ void CegoAdmAction::buFileInfoAction()
 	cout << msg << endl;
 }
 
-
 void CegoAdmAction::buStatInfoAction()
 {
 
@@ -683,7 +876,6 @@ void CegoAdmAction::abortDbThreadAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::retrieveTableSetAction()
@@ -710,11 +902,11 @@ void CegoAdmAction::defineTableSetAction()
     CegoAdminHandler::ResultType res;
     res = _pAH->medDefineTableSet(tableSet,
 				  _tsRoot,
-				  _tsTicket,
 				  _primary,
 				  _secondary,
 				  _sysSize,
 				  _tmpSize,
+				  _appSize,
 				  _logSize,
 				  _logNum, 
 				  _sortAreaSize);
@@ -724,10 +916,8 @@ void CegoAdmAction::defineTableSetAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
-
+    
     setDefault();
-
 }
 
 void CegoAdmAction::startTableSetAction() 
@@ -768,7 +958,6 @@ void CegoAdmAction::stopTableSetAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::copyTableSetAction() 
@@ -790,7 +979,6 @@ void CegoAdmAction::copyTableSetAction()
 
 }
 
-
 void CegoAdmAction::switchTableSetAction() 
 {
     CegoAdminHandler::ResultType res;
@@ -807,10 +995,8 @@ void CegoAdmAction::switchTableSetAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
-
 void CegoAdmAction::dropTableSetAction() 
 {
     CegoAdminHandler::ResultType res;   
@@ -827,7 +1013,6 @@ void CegoAdmAction::dropTableSetAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::removeTableSetAction() 
@@ -846,7 +1031,6 @@ void CegoAdmAction::removeTableSetAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::resetTableSetAction() 
@@ -865,7 +1049,6 @@ void CegoAdmAction::resetTableSetAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::createTableSetAction() 
@@ -884,7 +1067,6 @@ void CegoAdmAction::createTableSetAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::syncTableSetAction() 
@@ -905,7 +1087,6 @@ void CegoAdmAction::syncTableSetAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::beginBackupAction()
@@ -924,7 +1105,6 @@ void CegoAdmAction::beginBackupAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::endBackupAction()
@@ -944,7 +1124,6 @@ void CegoAdmAction::endBackupAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::keepTicketAction()
@@ -952,7 +1131,6 @@ void CegoAdmAction::keepTicketAction()
     _keepTicket = true;
 }
 
-
 void CegoAdmAction::syncCmdAction()
 {
     _syncCmd = Chain(_stringBuf);
@@ -1014,11 +1192,13 @@ void CegoAdmAction::recoverTableSetAction()
     res = _pAH->medRecover(tableSet, _pit);
     handleMedResult(res);
 
+    // reset pit to 0
+    _pit = Chain();
+
     Chain msg;
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::setLogMngAction()
@@ -1035,7 +1215,6 @@ void CegoAdmAction::setLogMngAction()
 	cout << msg << endl;
 
     _timeout = DEFTIMEOUT;
-
 }
 
 
@@ -1072,10 +1251,8 @@ void CegoAdmAction::showLogMngAction()
     }
 }
 
-
 void CegoAdmAction::setTSInitFileAction()
 {
-
     Chain initFileName(_stringBuf);
 
     Chain tableSet;
@@ -1092,13 +1269,10 @@ void CegoAdmAction::setTSInitFileAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
-
 void CegoAdmAction::setTSRootPathAction()
 {
-
     Chain tsRoot(_stringBuf);
 
     Chain tableSet;
@@ -1111,29 +1285,6 @@ void CegoAdmAction::setTSRootPathAction()
     CegoAdminHandler::ResultType res;
     res = _pAH->reqSetTSRootPath(tableSet, tsRoot);
 
-    Chain msg;
-    _pAH->getMsg(msg);
-    if ( _rawMode == false )
-	cout << msg << endl;
-
-}
-
-
-void CegoAdmAction::setTSTicketAction()
-{
-
-    Chain tsTicket(_stringBuf);
-
-    Chain tableSet;
-    Chain* pS = getTokenList().First();
-    if ( pS )
-    {
-	tableSet = *pS;
-    }
-
-    CegoAdminHandler::ResultType res;
-    res = _pAH->reqSetTSTicket(tableSet, tsTicket);
-
     Chain msg;
     _pAH->getMsg(msg);
     if ( _rawMode == false )
@@ -1142,9 +1293,7 @@ void CegoAdmAction::setTSTicketAction()
 
 void CegoAdmAction::setTSLogNumAction()
 {
-
     int logNum;
-
     Chain tableSet;
     Chain* pS = getTokenList().First();
     if ( pS )
@@ -1170,16 +1319,13 @@ void CegoAdmAction::setTSLogNumAction()
 
 void CegoAdmAction::setTSLogSizeAction()
 {
-
     int logSize;
-
     Chain tableSet;
     Chain* pS = getTokenList().First();
     if ( pS )
     {
 	tableSet = *pS;
     }
-
     pS = getTokenList().Next();
     pS = getTokenList().Next();
     if ( pS )
@@ -1198,21 +1344,18 @@ void CegoAdmAction::setTSLogSizeAction()
 
 void CegoAdmAction::setTSSortAreaSizeAction()
 {
-
-    int sortAreaSize;
-
+    unsigned long long sortAreaSize;
     Chain tableSet;
     Chain* pS = getTokenList().First();
     if ( pS )
     {
 	tableSet = *pS;
     }
-
     pS = getTokenList().Next();
     pS = getTokenList().Next();
     if ( pS )
     {
-	sortAreaSize = pS->asInteger();
+	sortAreaSize = pS->asUnsignedLongLong();
     }
 
     CegoAdminHandler::ResultType res;
@@ -1226,7 +1369,6 @@ void CegoAdmAction::setTSSortAreaSizeAction()
 
 void CegoAdmAction::setTSSysSizeAction()
 {
-
     Chain tableSet;
     int sysSize;
     Chain* pS = getTokenList().First();
@@ -1248,12 +1390,10 @@ void CegoAdmAction::setTSSysSizeAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::setTSTmpSizeAction()
 {
-
     Chain tableSet;
     int tmpSize;
     Chain* pS = getTokenList().First();
@@ -1275,12 +1415,35 @@ void CegoAdmAction::setTSTmpSizeAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
+}
 
+void CegoAdmAction::setTSAppSizeAction()
+{
+    Chain tableSet;
+    int appSize;
+    Chain* pS = getTokenList().First();
+    if ( pS )
+    {
+	tableSet = *pS;
+    }
+    pS = getTokenList().Next();
+    pS = getTokenList().Next();
+    if ( pS )
+    {
+	appSize = pS->asInteger();
+    }
+
+    CegoAdminHandler::ResultType res;
+    res = _pAH->reqSetTSAppSize(tableSet, appSize);
+
+    Chain msg;
+    _pAH->getMsg(msg);
+    if ( _rawMode == false )
+	cout << msg << endl;
 }
 
 void CegoAdmAction::setCheckpointAction()
 {
-
     Chain tableSet;
     int interval;
     Chain* pS = getTokenList().First();
@@ -1302,7 +1465,6 @@ void CegoAdmAction::setCheckpointAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::switchSecondaryAction()
@@ -1313,15 +1475,16 @@ void CegoAdmAction::switchSecondaryAction()
     {
 	tableSet = *pS;
     }
+    
     CegoAdminHandler::ResultType res;
     res = _pAH->medSecSwitch(tableSet);
+
     handleMedResult(res);
 
     Chain msg;
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::switchMediatorAction()
@@ -1332,18 +1495,18 @@ void CegoAdmAction::switchMediatorAction()
     {
 	tableSet = *pS;
     }
+    
     CegoAdminHandler::ResultType res;
     res = _pAH->secMedSwitch(tableSet);
+
     handleMedResult(res);
 
     Chain msg;
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
-
 void CegoAdmAction::relocateSecondaryAction()
 {
     Chain secondary(_stringBuf);
@@ -1355,15 +1518,16 @@ void CegoAdmAction::relocateSecondaryAction()
     {
 	tableSet = *pS;
     }
+
     CegoAdminHandler::ResultType res;
     res = _pAH->medSecRelocate(tableSet, secondary);
+
     handleMedResult(res);
 
     Chain msg;
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::relocateMediatorAction()
@@ -1377,15 +1541,16 @@ void CegoAdmAction::relocateMediatorAction()
     {
 	tableSet = *pS;
     }
+    
     CegoAdminHandler::ResultType res;
     res = _pAH->secMedRelocate(tableSet, mediator);
+
     handleMedResult(res);
 
     Chain msg;
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::checkTableSetAction()
@@ -1396,9 +1561,12 @@ void CegoAdmAction::checkTableSetAction()
     {
 	tableSet = *pS;
     }
+    
     CegoAdminHandler::ResultType res;
     res = _pAH->medCheckTableSet(tableSet);
+
     handleMedResult(res);
+
     CegoTableObject oe;
     ListT<ListT<CegoFieldValue> > info;
     _pAH->getTableSetCheck(oe, info);
@@ -1415,6 +1583,7 @@ void CegoAdmAction::verifyTableSetAction()
     {
 	tableSet = *pS;
     }
+
     CegoAdminHandler::ResultType res;
     res = _pAH->medVerifyTableSet(tableSet);
     if ( res == CegoAdminHandler::ADM_ERROR )
@@ -1462,7 +1631,6 @@ void CegoAdmAction::verifyTableSetAction()
     }
 }
 
-
 void CegoAdmAction::correctTableSetAction()
 {
     Chain tableSet;
@@ -1471,6 +1639,7 @@ void CegoAdmAction::correctTableSetAction()
     {
 	tableSet = *pS;
     }
+
     CegoAdminHandler::ResultType res;
     res = _pAH->medCorrectTableSet(tableSet);
     if ( res == CegoAdminHandler::ADM_ERROR )
@@ -1480,7 +1649,6 @@ void CegoAdmAction::correctTableSetAction()
 	throw Exception(EXLOC, msg);
     }
 
-
     CegoTableObject oe;
     ListT<ListT<CegoFieldValue> > info;
     _pAH->getTableSetCorrectionInfo(oe, info);
@@ -1517,10 +1685,8 @@ void CegoAdmAction::correctTableSetAction()
     {
 	throw Exception(EXLOC, msg);
     }
-
 }
 
-
 void CegoAdmAction::getDbSpecAction() 
 {
     CegoAdminHandler::ResultType res;   
@@ -1541,7 +1707,6 @@ void CegoAdmAction::getDbSpecAction()
     }
 }
 
-
 void CegoAdmAction::listNodeAction() 
 {
     CegoAdminHandler::ResultType res;   
@@ -1576,9 +1741,7 @@ void CegoAdmAction::listNodeAction()
 
 void CegoAdmAction::getDbInfoAction() 
 {
-
-    CegoAdminHandler::ResultType res;
-    
+    CegoAdminHandler::ResultType res;   
     res = _pAH->reqGetDbInfo();
 
     if ( res == CegoAdminHandler::ADM_OK )
@@ -1609,6 +1772,7 @@ void CegoAdmAction::tableSetInfoAction()
 
     CegoAdminHandler::ResultType res;
     res = _pAH->medGetDetailedTableSetInfo(tableSet);
+    
     handleMedResult(res);    
 
     Chain msg;
@@ -1624,7 +1788,6 @@ void CegoAdmAction::tableSetInfoAction()
     if ( _rawMode == false )
 	cout << msg << endl;
 }
- 
 
 void CegoAdmAction::dataFileInfoAction() 
 {
@@ -1637,6 +1800,7 @@ void CegoAdmAction::dataFileInfoAction()
 
     CegoAdminHandler::ResultType res;   
     res = _pAH->medGetDetailedTableSetInfo(tableSet);
+
     handleMedResult(res);
     
     CegoTableObject oe;
@@ -1652,7 +1816,6 @@ void CegoAdmAction::dataFileInfoAction()
     output.tabOut(info);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::archLogInfoAction() 
@@ -1666,6 +1829,7 @@ void CegoAdmAction::archLogInfoAction()
 
     CegoAdminHandler::ResultType res;
     res = _pAH->medGetDetailedTableSetInfo(tableSet);
+    
     handleMedResult(res);
 
     Chain msg;
@@ -1694,6 +1858,7 @@ void CegoAdmAction::logInfoAction()
 
     CegoAdminHandler::ResultType res;
     res = _pAH->medGetDetailedTableSetInfo(tableSet);
+
     handleMedResult(res);
 
     Chain msg;
@@ -1711,7 +1876,6 @@ void CegoAdmAction::logInfoAction()
 	cout << msg << endl;
 }
 
-
 void CegoAdmAction::objInfoAction() 
 {
     Chain tableSet;
@@ -1740,7 +1904,6 @@ void CegoAdmAction::objInfoAction()
 	cout << msg << endl;
 }
 
-
 void CegoAdmAction::transactionInfoAction() 
 {
     Chain tableSet;
@@ -1769,7 +1932,6 @@ void CegoAdmAction::transactionInfoAction()
 	cout << msg << endl;
 }
 
-
 void CegoAdmAction::setTableSetNodeAction() 
 {
     Chain tableSet;
@@ -1970,8 +2132,6 @@ void CegoAdmAction::importTableAction()
 
     if ( _rawMode == false )
 	cout << msg << endl;
-
-
 }
 
 void CegoAdmAction::importTableSetAction()
@@ -2000,7 +2160,6 @@ void CegoAdmAction::importTableSetAction()
 
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::importTableSetStructureAction()
@@ -2030,7 +2189,6 @@ void CegoAdmAction::importTableSetStructureAction()
 
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::addDataFileAction()
@@ -2076,10 +2234,8 @@ void CegoAdmAction::addDataFileAction()
 	    throw Exception(EXLOC, Chain("Invalid file type"));
 	}
     }
-   
-        
-    CegoAdminHandler::ResultType res;
-    
+
+    CegoAdminHandler::ResultType res;    
     res = _pAH->medAddDataFile(tableSet, fileType, dataFile, numPages);
 
     handleMedResult(res);
@@ -2088,15 +2244,13 @@ void CegoAdmAction::addDataFileAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::listUserAction()
 {
-        
     CegoAdminHandler::ResultType res;
-    
     res = _pAH->reqShowUser();
+
     handleMedResult(res);
 
     CegoTableObject oe;
@@ -2106,22 +2260,11 @@ void CegoAdmAction::listUserAction()
     
     CegoOutput output(oe.getSchema());
     output.setRawMode(_rawMode);
-    output.tabOut(info);
-    
-    /*
-    if ( _rawMode == false )
-    {
-	Chain msg;
-	_pAH->getMsg(msg);
-	cout << msg << endl;
-    }
-    
-    */
+    output.tabOut(info);    
 }
 
 void CegoAdmAction::addUserAction()
 {
-
     Chain user;
     Chain password(_stringBuf);
 
@@ -2143,12 +2286,10 @@ void CegoAdmAction::addUserAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::removeUserAction()
 {
-
     Chain tableSet;
     Chain user;
 
@@ -2167,12 +2308,10 @@ void CegoAdmAction::removeUserAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::changePwdAction()
 {
-
     Chain user;
     Chain password(_stringBuf);
 
@@ -2185,21 +2324,19 @@ void CegoAdmAction::changePwdAction()
 	user = *pS;
     }
         
-    CegoAdminHandler::ResultType res;
-    
+    CegoAdminHandler::ResultType res;   
     res = _pAH->reqChangePwd(user, password);
+    
     handleMedResult(res);
 
     Chain msg;
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::assignRoleAction()
 {
-
     Chain user;
     Chain role;
 
@@ -2215,22 +2352,19 @@ void CegoAdmAction::assignRoleAction()
 	role = *pS;
     }
         
-    CegoAdminHandler::ResultType res;
-    
+    CegoAdminHandler::ResultType res;   
     res = _pAH->reqAssignRole(user, role);
+
     handleMedResult(res);
 
     Chain msg;
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
-
 }
 
 void CegoAdmAction::removeRoleAction()
 {
-
     Chain user;
     Chain role;
 
@@ -2246,24 +2380,22 @@ void CegoAdmAction::removeRoleAction()
 	role = *pS;
     }
         
-    CegoAdminHandler::ResultType res;
-    
+    CegoAdminHandler::ResultType res;   
     res = _pAH->reqRemoveRole(user, role);
+
     handleMedResult(res);
 
     Chain msg;
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::listRoleAction()
 {
-
-    CegoAdminHandler::ResultType res;
-    
+    CegoAdminHandler::ResultType res;   
     res = _pAH->reqListRole();
+    
     handleMedResult(res);
 
     CegoTableObject oe;
@@ -2274,21 +2406,10 @@ void CegoAdmAction::listRoleAction()
     CegoOutput output(oe.getSchema());
     output.setRawMode(_rawMode);
     output.tabOut(info);
-
-    /*
-    if ( _rawMode == false )
-    {
-	Chain msg;
-	_pAH->getMsg(msg);
-	cout << msg << endl;
-    }
-    */
-
 }
 
 void CegoAdmAction::showRoleAction()
 {
-
     Chain role;
 
     Chain* pS = getTokenList().First();
@@ -2297,9 +2418,9 @@ void CegoAdmAction::showRoleAction()
 	role = *pS;
     }
 
-    CegoAdminHandler::ResultType res;
-    
+    CegoAdminHandler::ResultType res;   
     res = _pAH->reqShowRole(role);
+    
     handleMedResult(res);
 
     CegoTableObject oe;
@@ -2312,10 +2433,8 @@ void CegoAdmAction::showRoleAction()
     output.tabOut(info);
 }
 
-
 void CegoAdmAction::createRoleAction()
 {
-
     Chain role;
 
     Chain* pS = getTokenList().First();
@@ -2324,21 +2443,19 @@ void CegoAdmAction::createRoleAction()
 	role = *pS;
     }
         
-    CegoAdminHandler::ResultType res;
-    
+    CegoAdminHandler::ResultType res;   
     res = _pAH->reqCreateRole(role);
+    
     handleMedResult(res);
 
     Chain msg;
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::dropRoleAction()
 {
-
     Chain role;
 
     Chain* pS = getTokenList().First();
@@ -2348,15 +2465,14 @@ void CegoAdmAction::dropRoleAction()
     }
         
     CegoAdminHandler::ResultType res;
-    
     res = _pAH->reqDropRole(role);
+
     handleMedResult(res);
 
     Chain msg;
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::setPermAction()
@@ -2373,22 +2489,20 @@ void CegoAdmAction::setPermAction()
     pS = getTokenList().Next();
     pS = getTokenList().Next();
     pS = getTokenList().Next();
-    pS = getTokenList().Next();
     if ( pS )
     {
 	permid = *pS;
     }
 
     CegoAdminHandler::ResultType res;
-    
     res = _pAH->reqSetPermission(role, permid, _tsPerm, _filterPerm, _rightPerm);
+    
     handleMedResult(res);
 
     Chain msg;
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::setTableSetPerm()
@@ -2427,28 +2541,24 @@ void CegoAdmAction::removePermAction()
     }
     pS = getTokenList().Next();
     pS = getTokenList().Next();
-    pS = getTokenList().Next();
     if ( pS )
     {
 	permid = *pS;
     }
 
     CegoAdminHandler::ResultType res;
-    
     res = _pAH->reqRemovePermission(role, permid);
+    
     handleMedResult(res);
 
     Chain msg;
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
-
 void CegoAdmAction::traceOnAction()
 {
-
     Chain user;
 
     Chain* pS = getTokenList().First();
@@ -2457,21 +2567,19 @@ void CegoAdmAction::traceOnAction()
 	user = *pS;
     }
         
-    CegoAdminHandler::ResultType res;
-    
+    CegoAdminHandler::ResultType res;   
     res = _pAH->reqUserTrace(user, true);
+    
     handleMedResult(res);
 
     Chain msg;
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::traceOffAction()
 {
-
     Chain user;
 
     Chain* pS = getTokenList().First();
@@ -2480,22 +2588,19 @@ void CegoAdmAction::traceOffAction()
 	user = *pS;
     }
 
-    CegoAdminHandler::ResultType res;
-    
+    CegoAdminHandler::ResultType res;   
     res = _pAH->reqUserTrace(user, false);
+    
     handleMedResult(res);
 
     Chain msg;
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
-
 }
 
 void CegoAdmAction::addArchLogAction()
 {
-
     Chain tableSet;
     Chain archId;
     Chain archPath(_stringBuf);
@@ -2513,24 +2618,20 @@ void CegoAdmAction::addArchLogAction()
     {
 	archId = *pS;
     }
-
-        
-    CegoAdminHandler::ResultType res;
-    
+       
+    CegoAdminHandler::ResultType res;    
     res = _pAH->medAddArchLog(tableSet, archId, archPath);
+    
     handleMedResult(res);
 
     Chain msg;
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
-
 void CegoAdmAction::removeArchLogAction()
 {
-
     Chain tableSet;
     Chain archId;
 
@@ -2546,21 +2647,19 @@ void CegoAdmAction::removeArchLogAction()
 	archId = *pS;
     }
         
-    CegoAdminHandler::ResultType res;
+    CegoAdminHandler::ResultType res;   
+    res = _pAH->medRemoveArchLog(tableSet, archId);
     
-    res = _pAH->medRemoveArchLog(tableSet, archId);    
     handleMedResult(res);
 
     Chain msg;
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::enableArchLogAction()
 {
-
     Chain tableSet;
 
     Chain* pS = getTokenList().First();
@@ -2569,21 +2668,19 @@ void CegoAdmAction::enableArchLogAction()
 	tableSet = *pS;
     }
         
-    CegoAdminHandler::ResultType res;
+    CegoAdminHandler::ResultType res;   
+    res = _pAH->medEnableArchLog(tableSet);
     
-    res = _pAH->medEnableArchLog(tableSet);    
     handleMedResult(res);
 
     Chain msg;
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
-
 }
 
 void CegoAdmAction::disableArchLogAction()
 {
-
     Chain tableSet;
 
     Chain* pS = getTokenList().First();
@@ -2601,7 +2698,132 @@ void CegoAdmAction::disableArchLogAction()
     _pAH->getMsg(msg);
     if ( _rawMode == false )
 	cout << msg << endl;
+}
 
+void CegoAdmAction::enableAutoCorrectAction()
+{
+    Chain tableSet;
+
+    Chain* pS = getTokenList().First();
+    if ( pS )
+    {
+	tableSet = *pS;
+    }
+        
+    CegoAdminHandler::ResultType res;   
+    res = _pAH->medEnableAutoCorrect(tableSet);
+    
+    handleMedResult(res);
+
+    Chain msg;
+    _pAH->getMsg(msg);
+    if ( _rawMode == false )
+	cout << msg << endl;
+}
+
+void CegoAdmAction::disableAutoCorrectAction()
+{
+    Chain tableSet;
+
+    Chain* pS = getTokenList().First();
+    if ( pS )
+    {
+	tableSet = *pS;
+    }
+        
+    CegoAdminHandler::ResultType res;   
+    res = _pAH->medDisableAutoCorrect(tableSet);
+    
+    handleMedResult(res);
+
+    Chain msg;
+    _pAH->getMsg(msg);
+    if ( _rawMode == false )
+	cout << msg << endl;
+}
+
+void CegoAdmAction::enableTableCacheAction()
+{
+    Chain tableSet;
+
+    Chain* pS = getTokenList().First();
+    if ( pS )
+    {
+	tableSet = *pS;
+    }
+        
+    CegoAdminHandler::ResultType res;   
+    res = _pAH->reqEnableTableCache(tableSet);
+    
+    handleMedResult(res);
+
+    Chain msg;
+    _pAH->getMsg(msg);
+    if ( _rawMode == false )
+	cout << msg << endl;
+}
+
+void CegoAdmAction::disableTableCacheAction()
+{
+    Chain tableSet;
+
+    Chain* pS = getTokenList().First();
+    if ( pS )
+    {
+	tableSet = *pS;
+    }
+        
+    CegoAdminHandler::ResultType res;   
+    res = _pAH->reqDisableTableCache(tableSet);
+    
+    handleMedResult(res);
+
+    Chain msg;
+    _pAH->getMsg(msg);
+    if ( _rawMode == false )
+	cout << msg << endl;
+}
+
+void CegoAdmAction::enableQueryCacheAction()
+{
+    Chain tableSet;
+
+    Chain* pS = getTokenList().First();
+    if ( pS )
+    {
+	tableSet = *pS;
+    }
+        
+    CegoAdminHandler::ResultType res;   
+    res = _pAH->reqEnableQueryCache(tableSet);
+    
+    handleMedResult(res);
+
+    Chain msg;
+    _pAH->getMsg(msg);
+    if ( _rawMode == false )
+	cout << msg << endl;
+}
+
+void CegoAdmAction::disableQueryCacheAction()
+{
+    Chain tableSet;
+
+    Chain* pS = getTokenList().First();
+    if ( pS )
+    {
+	tableSet = *pS;
+    }
+        
+    CegoAdminHandler::ResultType res;   
+    res = _pAH->reqDisableQueryCache(tableSet);
+    
+    handleMedResult(res);
+
+    Chain msg;
+    _pAH->getMsg(msg);
+    if ( _rawMode == false )
+	cout << msg << endl;
 }
 
 void CegoAdmAction::storeSysSizeAttr()
@@ -2622,6 +2844,15 @@ void CegoAdmAction::storeTmpSizeAttr()
     }
 }
 
+void CegoAdmAction::storeAppSizeAttr()
+{
+    Chain* pS = getTokenList().First();
+    if ( pS )
+    {
+	_appSize = pS->asInteger();
+    }
+}
+
 void CegoAdmAction::storeLogSizeAttr()
 {
     Chain* pS = getTokenList().First();
@@ -2640,13 +2871,12 @@ void CegoAdmAction::storeLogNumAttr()
     }
 }
 
-
 void CegoAdmAction::storeSortAreaAttr()
 {
     Chain* pS = getTokenList().First();
     if ( pS )
     {
-	_sortAreaSize = pS->asLong();
+	_sortAreaSize = pS->asUnsignedLongLong();
     }
 }
 
@@ -2665,22 +2895,14 @@ void CegoAdmAction::storeTSRootAttr()
     _tsRoot = Chain(_stringBuf);
 }
 
-void CegoAdmAction::storeTSTicketAttr()
-{
-    _tsTicket = Chain(_stringBuf);
-}
-
 void CegoAdmAction::storePit()
 {
     Chain* pS = getTokenList().First();
-    Chain dateVal = Chain(_stringBuf);
-    Datetime dt(dateVal, _pitFormat);   
-    _pit = dt.asInt(); 
+    _pit = Chain(_stringBuf);
 }
 
 void CegoAdmAction::dumpTable()
 {
-
     Chain tableSet;
     Chain table;
 
@@ -2696,13 +2918,11 @@ void CegoAdmAction::dumpTable()
 	table = *pS;
     }
         
-    CegoAdminHandler::ResultType res;
-    
+    CegoAdminHandler::ResultType res;   
     res = _pAH->reqDumpObject(tableSet, CegoObject::TABLE, table);
 	
     if ( res == CegoAdminHandler::ADM_INFO )
     {
-	
 	Chain tableDump;
 
 	_pAH->nextInfo();
@@ -2721,7 +2941,6 @@ void CegoAdmAction::dumpTable()
 
 void CegoAdmAction::dumpUIndex()
 {
-
     Chain tableSet;
     Chain table;
 
@@ -2737,9 +2956,8 @@ void CegoAdmAction::dumpUIndex()
 	table = *pS;
     }
         
-    CegoAdminHandler::ResultType res;
-    
-    res = _pAH->reqDumpObject(tableSet, CegoObject::UINDEX, table);
+    CegoAdminHandler::ResultType res;   
+    res = _pAH->reqDumpObject(tableSet, CegoObject::UAVLTREE, table);
 	
     if ( res == CegoAdminHandler::ADM_INFO )
     {
@@ -2760,10 +2978,8 @@ void CegoAdmAction::dumpUIndex()
     }    
 }
 
-
 void CegoAdmAction::handleMedResult(CegoAdminHandler::ResultType res)
 {
-
     Chain msg;
     _pAH->getMsg(msg);
     
@@ -2779,8 +2995,7 @@ void CegoAdmAction::handleMedResult(CegoAdminHandler::ResultType res)
 	output.headOut();
 
     while ( res == CegoAdminHandler::ADM_INFO )
-    {
-	
+    {	
 	if ( _resultMode == VERBOSE )
 	{
 	    Chain hostRole;
@@ -2808,7 +3023,8 @@ void CegoAdmAction::handleMedResult(CegoAdminHandler::ResultType res)
 	{
 	    Chain msg;
 	    _pAH->getMsg(msg);
-	    cout << msg << endl;
+	    cout << msg;
+	    cout.flush();
 	}
 
 	res = _pAH->nextInfo();
@@ -2819,14 +3035,9 @@ void CegoAdmAction::handleMedResult(CegoAdminHandler::ResultType res)
     
     _pAH->getMsg(msg);
     
-    /* if ( res == CegoAdminHandler::ADM_OK )
-    {
-	if ( _rawMode == false )
-	    cout << msg << endl;
-    }
-    else */
     if ( res == CegoAdminHandler::ADM_ERROR )
     {
 	throw Exception(EXLOC, msg);
     }
 }
+
