@@ -6,38 +6,23 @@
 //         
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2013 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoBTreeCursor
 // 
-// Description: Index cursors are used for reading tables based on an available index 
-//              for the given attribute condition
+// Description: Cursor class for traversing btree objects
 //
-// Status: QG-2.13
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
+// POSIX INCLUDES
 #include <string.h>
 #include <stdlib.h>
 
-// cego includes
+// CEGO INCLUDES
 #include "CegoBTreeCursor.h"
 #include "CegoTupleState.h"
 #include "CegoBTreeNode.h"
@@ -55,13 +40,13 @@ CegoBTreeCursor::CegoBTreeCursor(CegoTableManager *pTabMng, int tabSetId, const
     _type = type;
     _pAttrCond = pAttrCond;
 
-    _dataLock = 0;
-
     _tabSetId = tabSetId;
     _ignoreTouched = ignoreTouched;
+
     _readUncommitted = readUncommitted;
     _cursorCached = false;
     _eoc=true;
+    _lockId = 0;
 }
 
 CegoBTreeCursor::~CegoBTreeCursor()
@@ -71,7 +56,6 @@ CegoBTreeCursor::~CegoBTreeCursor()
 
 bool CegoBTreeCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 {
-
     _eoc=false;
 
     if ( fl.isEmpty() )
@@ -82,19 +66,31 @@ bool CegoBTreeCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
     if ( _cursorCached == false )
     {
 	CegoBTreeObject btoe;
-	
+
 	_pTabMng->getObject(_tabSetId, _btreeName, _type, btoe);
 
 	_btreeSchema = btoe.getSchema();
 
+	_keyLen = CegoBTreeValue::getKeyLen(_btreeSchema);
+
+	/*
 	CegoField *pF = _btreeSchema.First();
 	_keyLen = 0;
 	while ( pF )
 	{
+	    _keyLen += 1; // null value indicator
 	    _keyLen += pF->getLength();
+	    
+	    if ( pF->getType() == VARCHAR_TYPE
+		 || pF->getType() == BIGINT_TYPE
+		 || pF->getType() == DECIMAL_TYPE
+		 || pF->getType() == FIXED_TYPE )
+		_keyLen += 1; // termination character
+	    
 	    pF = _btreeSchema.Next();
 	}
-
+	*/
+	
 	_rootFileId = btoe.getDataFileId();
 	_rootPageId = btoe.getDataPageId();
 	
@@ -106,11 +102,13 @@ bool CegoBTreeCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 	// btm.dumpBTree(btoe);
     }
 
+    // cout << "Read Locking " << _rootFileId << " " << _rootPageId << endl;
+    _lockId = _pTabMng->getLockHandler()->lockData(CegoObject::BTREE, _rootFileId, _rootPageId, CegoLockHandler::READ);
+    
     bool found = false;
 
     if (_pAttrCond)
     { 
-
 	if ( _pAttrCond->getPrimaryCompMode() == CegoAttrComp::BTWN )
 	    _pAttrCond->setPrimaryComparison( MORE_EQUAL_THAN );
 
@@ -130,14 +128,11 @@ bool CegoBTreeCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 
 	    while ( leafFound == false )
 	    {
-
 		CegoBufferPage bp;
-		
 		_pDBMng->bufferFix(bp, _tabSetId, _curFileId, _curPageId, CegoBufferPool::SYNC, _pTabMng->getLockHandler());
 				
 		if ( bp.getType() == CegoBufferPage::BTREE_NODE )
-		{
-		    
+		{		    
 		    CegoBTreeNode traceNode;
 		    traceNode.setType(CegoBTreeNode::NODE);
 		    traceNode.setPtr(bp.getChunkEntry(), bp.getChunkLen());	    
@@ -147,12 +142,11 @@ bool CegoBTreeCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 		    
 		    traceNode.reset();
 		    traceNode.nextChildPointer(_curFileId, _curPageId);
-
+		    
 		    _parentPageStack.Push(bp);
 		}
 		else if ( bp.getType() == CegoBufferPage::BTREE_LEAF )
 		{
-
 		    _curPage = bp;
 		    _curLeaf.setType(CegoBTreeNode::LEAF);
 		    _curLeaf.setPtr(_curPage.getChunkEntry(), _curPage.getChunkLen());
@@ -163,29 +157,64 @@ bool CegoBTreeCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 		    leafFound=true;
 
 		    _curLeaf.reset();
-		    bool hasValue = _curLeaf.nextValue(iv, dp);
-		}
+
+		    bool goOn = true;
+		    while ( goOn )
+		    {
+			CegoBTreeValue iv;
+			bool hasValue = _curLeaf.nextValue(iv, dp);
+					
+			if ( hasValue )
+			{	
+			    CegoComparison comp = compValue(iv);
+			    
+			    if ( comp == LESS_THAN
+				 || ( comp == EQUAL && _pAttrCond->getPrimaryComparison() == LESS_EQUAL_THAN )
+				 || ( comp != EQUAL && _pAttrCond->getPrimaryComparison() == NOT_EQUAL ) )
+			    {	       
+				found = true;
+				goOn = false;
+			    }
+			    else if ( _pAttrCond->getPrimaryComparison() == NOT_EQUAL )
+			    {
+				return getNext(fl, dp);
+			    }
+			}
+			else
+			{   
+			    while ( hasValue == false && goOn )
+			    {				
+				int fileId = _curPage.getNextFileId();
+				int pageId = _curPage.getNextPageId();
+								
+				_pDBMng->bufferUnfix(_curPage, false, _pTabMng->getLockHandler());
+				
+				if ( fileId || pageId )
+				{
+				    _pDBMng->bufferFix(_curPage, _tabSetId, fileId, pageId, CegoBufferPool::SYNC, _pTabMng->getLockHandler());
+				    
+				    _curLeaf.setPtr(_curPage.getChunkEntry(), _curPage.getChunkLen());
+				    _curLeaf.setFileId(_curPage.getFileId());
+				    _curLeaf.setPageId(_curPage.getPageId());
+				    _curLeaf.setSchema(&_btreeSchema, _keyLen);
+				
+				    _curLeaf.reset();
+				    hasValue = _curLeaf.numEntries() > 0;				    
+				}
+				else
+				{
+				    goOn = false;
+				}
+			    }
+			}
+		    }	    
+		}   
 	    }
-	    	    
-	    CegoComparison comp = compValue(iv);
-	    
-	    if ( comp == LESS_THAN
-		 || ( comp == EQUAL && _pAttrCond->getPrimaryComparison() == LESS_EQUAL_THAN )
-		 || ( comp != EQUAL && _pAttrCond->getPrimaryComparison() == NOT_EQUAL ) )
-	    {	       
-		found = true;
-	    }
-	    else if ( _pAttrCond->getPrimaryComparison() == NOT_EQUAL )
-	    {
-		return getNext(fl, dp);
-	    }
-	    	    
 	    break;
 	}
 	case EQUAL:
 	case MORE_EQUAL_THAN:
-	{
-
+	{	    
 	    bool leafFound = false;
 
 	    _curFileId = _rootFileId;
@@ -194,11 +223,11 @@ bool CegoBTreeCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 	    while ( leafFound == false )
 	    {
 		CegoBufferPage bp;
+
 		_pDBMng->bufferFix(bp, _tabSetId, _curFileId, _curPageId, CegoBufferPool::SYNC, _pTabMng->getLockHandler());
-				
+
 		if ( bp.getType() == CegoBufferPage::BTREE_NODE )
-		{
-		    
+		{	    
 		    CegoBTreeNode traceNode;
 		    traceNode.setType(CegoBTreeNode::NODE);
 		    traceNode.setPtr(bp.getChunkEntry(), bp.getChunkLen());	    
@@ -213,7 +242,7 @@ bool CegoBTreeCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 		    {
 			CegoBTreeValue iv;
 			if ( traceNode.rightChild(iv) )
-			{
+			{			    
 			    CegoComparison comp = compValue(iv);
 			    
 			    if ( comp == EQUAL
@@ -234,8 +263,7 @@ bool CegoBTreeCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 		    _parentPageStack.Push(bp); 
 		}
 		else if ( bp.getType() == CegoBufferPage::BTREE_LEAF )
-		{
-		
+		{		
 		    _curPage = bp;
 		    _curLeaf.setType(CegoBTreeNode::LEAF);
 		    _curLeaf.setPtr(_curPage.getChunkEntry(), _curPage.getChunkLen());
@@ -244,25 +272,55 @@ bool CegoBTreeCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 		    _curLeaf.setSchema(&_btreeSchema, _keyLen);
 
 		    leafFound=true;
-
-		    CegoBTreeValue iv;
 		    
 		    _curLeaf.reset();
-		    bool goOn = _curLeaf.nextValue(iv, dp);
+
+		    bool goOn = true;
 		    while ( goOn )
 		    {
-			CegoComparison comp = compValue(iv);			
-			if ( comp == EQUAL || comp == MORE_THAN )
-			{	       
-			    goOn = false;			    
-			    if ( comp == MORE_THAN &&  _pAttrCond->getPrimaryComparison() == EQUAL )
-				found = false;
-			    else
-				found = true;			    
+			CegoBTreeValue iv;
+			bool hasValue = _curLeaf.nextValue(iv, dp);
+	       	
+			if ( hasValue )
+			{
+			    CegoComparison comp = compValue(iv);			
+			    if ( comp == EQUAL || comp == MORE_THAN )
+			    {	       
+				goOn = false;			    
+				if ( comp == MORE_THAN && _pAttrCond->getPrimaryComparison() == EQUAL )
+				    found = false;
+				else
+				    found = true;
+			    }			    
 			}
 			else
-			{
-			    goOn = _curLeaf.nextValue(iv, dp);
+			{			    
+			    while ( hasValue == false && goOn )
+			    {								
+				int fileId = _curPage.getNextFileId();
+				int pageId = _curPage.getNextPageId();
+				
+				// cout << "Skipping leave, going to next " << pageId << endl;
+			
+				_pDBMng->bufferUnfix(_curPage, false, _pTabMng->getLockHandler());
+			
+				if ( fileId || pageId )
+				{
+				    _pDBMng->bufferFix(_curPage, _tabSetId, fileId, pageId, CegoBufferPool::SYNC, _pTabMng->getLockHandler());
+				    
+				    _curLeaf.setPtr(_curPage.getChunkEntry(), _curPage.getChunkLen());
+				    _curLeaf.setFileId(_curPage.getFileId());
+				    _curLeaf.setPageId(_curPage.getPageId());
+				    _curLeaf.setSchema(&_btreeSchema, _keyLen);
+				    
+				    _curLeaf.reset();
+				    hasValue = _curLeaf.numEntries() > 0;
+				}
+				else
+				{
+				    goOn = false;			      
+				}
+			    }
 			}
 		    }
 		}
@@ -271,67 +329,117 @@ bool CegoBTreeCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 	}
 	case MORE_THAN:
 	{
-
-	    // go to the most right
-
 	    bool leafFound = false;
+
 	    _curFileId = _rootFileId;
 	    _curPageId = _rootPageId;
 
-
-	    while ( ! found )
+	    while ( leafFound == false )
 	    {
+		CegoBufferPage bp;
+		_pDBMng->bufferFix(bp, _tabSetId, _curFileId, _curPageId, CegoBufferPool::SYNC, _pTabMng->getLockHandler());
+				
+		if ( bp.getType() == CegoBufferPage::BTREE_NODE )
+		{	    
+		    CegoBTreeNode traceNode;
+		    traceNode.setType(CegoBTreeNode::NODE);
+		    traceNode.setPtr(bp.getChunkEntry(), bp.getChunkLen());	    
+		    traceNode.setSchema(&_btreeSchema, _keyLen);
+		    traceNode.setFileId(bp.getFileId());
+		    traceNode.setPageId(bp.getPageId());
 
-		CegoBTreeValue startVal;
-
-		while ( leafFound == false )
-		{
-		    
-		    CegoBufferPage bp;
-		    
-		    _pDBMng->bufferFix(bp, _tabSetId, _curFileId, _curPageId, CegoBufferPool::SYNC, _pTabMng->getLockHandler());
-		    
-		    
-		    if ( bp.getType() == CegoBufferPage::BTREE_NODE )
+		    traceNode.reset();
+		   
+		    bool goOn = traceNode.nextChildPointer(_curFileId, _curPageId);
+		    while ( goOn )
 		    {
-		    
-			CegoBTreeNode traceNode;
-			traceNode.setType(CegoBTreeNode::NODE);
-			traceNode.setPtr(bp.getChunkEntry(), bp.getChunkLen());	    
-			traceNode.setSchema(&_btreeSchema, _keyLen);
-			traceNode.setFileId(bp.getFileId());
-			traceNode.setPageId(bp.getPageId());
-			
-			
-			traceNode.getChildPage(startVal, _curFileId, _curPageId);
-		    
+			CegoBTreeValue iv;
+			if ( traceNode.rightChild(iv) )
+			{
+			    CegoComparison comp = compValue(iv);
+			    
+			    if ( comp == MORE_THAN )
+			    {	       
+				goOn = false;
+			    }
+			    else
+			    {
+				goOn = traceNode.nextChildPointer(_curFileId, _curPageId);
+			    }
+			}
+			else
+			{
+			    goOn = false;
+			}			
+		    }				
+		    _parentPageStack.Push(bp); 
+		}
+		else if ( bp.getType() == CegoBufferPage::BTREE_LEAF )
+		{		
+		    _curPage = bp;
+		    _curLeaf.setType(CegoBTreeNode::LEAF);
+		    _curLeaf.setPtr(_curPage.getChunkEntry(), _curPage.getChunkLen());
+		    _curLeaf.setFileId(_curPage.getFileId());
+		    _curLeaf.setPageId(_curPage.getPageId());
+		    _curLeaf.setSchema(&_btreeSchema, _keyLen);
 
-		    }
-		    else if ( bp.getType() == CegoBufferPage::BTREE_LEAF )
+		    leafFound=true;
+		    
+		    _curLeaf.reset();
+
+		    bool goOn = true;
+		    while ( goOn )
 		    {
-			
-			leafFound=true;
-			
-			CegoBTreeNode minLeaf;
-			minLeaf.setType(CegoBTreeNode::LEAF);
-			minLeaf.setPtr(bp.getChunkEntry(), bp.getChunkLen());
-			minLeaf.setFileId(bp.getFileId());
-			minLeaf.setPageId(bp.getPageId());
-			minLeaf.setSchema(&_btreeSchema, _keyLen);
-			
-			// dp = minLeaf.getNextValue(startVal);
-			
+			CegoBTreeValue iv;
+			bool hasValue = _curLeaf.nextValue(iv, dp);
+			if  ( hasValue )
+			{
+			    CegoComparison comp = compValue(iv);			
+			    if ( comp == MORE_THAN )
+			    {	       
+				goOn = false;			    
+				if ( comp == MORE_THAN &&  _pAttrCond->getPrimaryComparison() == EQUAL )
+				    found = false;
+				else
+				    found = true;			    
+			    }			    
+			}
+			else
+			{			    
+			    while ( hasValue == false && goOn )
+			    {								
+				int fileId = _curPage.getNextFileId();
+				int pageId = _curPage.getNextPageId();
+												
+				_pDBMng->bufferUnfix(_curPage, false, _pTabMng->getLockHandler());
+				
+				if ( fileId || pageId )
+				{
+				    _pDBMng->bufferFix(_curPage, _tabSetId, fileId, pageId, CegoBufferPool::SYNC, _pTabMng->getLockHandler());
+				    
+				    _curLeaf.setPtr(_curPage.getChunkEntry(), _curPage.getChunkLen());
+				    _curLeaf.setFileId(_curPage.getFileId());
+				    _curLeaf.setPageId(_curPage.getPageId());
+				    _curLeaf.setSchema(&_btreeSchema, _keyLen);
+				    
+				    _curLeaf.reset();
+				    hasValue = _curLeaf.numEntries() > 0;				    
+				}
+				else
+				{
+				    goOn = false;;
+				}
+			    }
+			}
 		    }
 		}
-		
-	    }
+	    }	    	    
 	    break;	    
-	}       	
+	}
 	}
     }
     else
     {
-	
 	// go to the beginning
 	
 	bool leafFound = false;
@@ -341,16 +449,13 @@ bool CegoBTreeCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 	CegoBTreeValue minVal;
 	
 	while ( leafFound == false )
-	{
-	    
+	{	    
 	    CegoBufferPage bp;
 	    
 	    _pDBMng->bufferFix(bp, _tabSetId, _curFileId, _curPageId, CegoBufferPool::SYNC, _pTabMng->getLockHandler());
-
 	    
 	    if ( bp.getType() == CegoBufferPage::BTREE_NODE )
 	    {
-		
 		CegoBTreeNode traceNode;
 		traceNode.setType(CegoBTreeNode::NODE);
 		traceNode.setPtr(bp.getChunkEntry(), bp.getChunkLen());	    
@@ -365,8 +470,7 @@ bool CegoBTreeCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 
 	    }
 	    else if ( bp.getType() == CegoBufferPage::BTREE_LEAF )
-	    {
-		
+	    {	
 		_curPage = bp;
 		_curLeaf.setType(CegoBTreeNode::LEAF);
 		_curLeaf.setPtr(bp.getChunkEntry(), bp.getChunkLen());
@@ -374,60 +478,44 @@ bool CegoBTreeCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 		_curLeaf.setPageId(bp.getPageId());
 		_curLeaf.setSchema(&_btreeSchema, _keyLen);
 
-
 		CegoBTreeValue iv;
 		
 		_curLeaf.reset();
-		bool goOn = _curLeaf.nextValue(iv, dp);
-		
+		found = _curLeaf.nextValue(iv, dp);
 		
 		leafFound=true;
-	    }
-	   	    
-	    found = true;
+	    }	   	    
 	}
     }
 
-    // cout << "After search ..." << dp  << endl;
-
-    if (found )
+    if ( found )
     {
-
 	char* p;
 	int len;
 	
-	if (_dataLock)
-	{
-	    _pTabMng->releaseDataPtr(_dataLock, false);
-	    _dataLock = 0;
-	}
-       
-	_dataLock = _pTabMng->claimDataPtr(_tabSetId, CegoLockHandler::READ, CegoBufferPool::SYNC, dp, p, len);
+	_pTabMng->releaseDataPtrUnlocked(_dataPtr, false);
+	_dataPtr = _pTabMng->claimDataPtrUnlocked(_tabSetId, CegoBufferPool::SYNC, dp, p, len);
 	
 	// skipping tid
 	
-	int tid;
-	int tastep;
+	unsigned long long tid;
+	unsigned long long tastep;
 	CegoTupleState ts;
 
-	memcpy(&tid, p, sizeof(int));
+	int toff = _qh.decodeTupleHeader(tid, tastep, ts, p);
 
-	char* ptastep = p + sizeof(int);
-	memcpy(&tastep, ptastep, sizeof(int));
-
-	char* pts = p + sizeof(int) + sizeof(int);
-	memcpy(&ts, pts, sizeof(CegoTupleState));
+	char* tp = p + toff;
+	int tlen = len - toff;
 	
 	if (tid != 0)
-	{
+	{	    
 	    if ( _ignoreTouched )
 	    {			
 		if ( ts == INSERTED 
 		     && tid == _pTabMng->getTID(_tabSetId) 
 		     && tastep < _pTabMng->getTAStep(_tabSetId) )
 		{
-		    CegoTupleState ts;
-		    _qh.decodeFVL(fl, p, len, tid, tastep, ts);
+		    _qh.decodeFVL(fl, tp, tlen);
 		    return true;   
 		}
 		else
@@ -437,14 +525,13 @@ bool CegoBTreeCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 	    }
 	    else
 	    {
-		
 		if ( ( _readUncommitted == true 
 		       && ts == INSERTED )
 		     || ( _readUncommitted == false 
 			  && (( ts == INSERTED && tid == _pTabMng->getTID(_tabSetId)) 
 			      || ( ts == DELETED && tid != _pTabMng->getTID(_tabSetId)))))
 		{
-		    _qh.decodeFVL(fl, p, len, tid, tastep, ts);
+		    _qh.decodeFVL(fl, tp, tlen);
 		    return true;   
 		}
 		else
@@ -455,21 +542,19 @@ bool CegoBTreeCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 	}
 	else
 	{
-	    _qh.decodeFVL(fl, p, len, tid, tastep, ts);
+	    _qh.decodeFVL(fl, tp, tlen);
 	    return true;
 	}
     }
     else
     {
 	_eoc=true;
-    }
-    
+    }   
     return false;
 }
 
 bool CegoBTreeCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
 {
-
     if ( _eoc )
 	return false;
 
@@ -482,15 +567,14 @@ bool CegoBTreeCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
     
     bool moreValue = _curLeaf.nextValue(iv, dp);
 
-    while ( moreValue == false && _eoc == false )
+    while ( moreValue == false )
     {
-
 	int fileId = _curPage.getNextFileId();
 	int pageId = _curPage.getNextPageId();
 
-	_pDBMng->bufferUnfix(_curPage, true, _pTabMng->getLockHandler());	
+	_pDBMng->bufferUnfix(_curPage, false, _pTabMng->getLockHandler());
 
-	if ( fileId && pageId )
+	if ( fileId || pageId )
 	{
 	    _pDBMng->bufferFix(_curPage, _tabSetId, fileId, pageId, CegoBufferPool::SYNC, _pTabMng->getLockHandler());
 	    
@@ -498,30 +582,27 @@ bool CegoBTreeCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
 	    _curLeaf.setFileId(_curPage.getFileId());
 	    _curLeaf.setPageId(_curPage.getPageId());
 	    _curLeaf.setSchema(&_btreeSchema, _keyLen);
-	    
-	    
+	        
 	    _curLeaf.reset();
 	    moreValue = _curLeaf.nextValue(iv, dp);
 	}
 	else
 	{
 	    _eoc = true;
+	    return false;	    
 	}
     }
 
-
     do {
 
-	
 	bool attrMatch = false;
 
 	// get next available entry from leaf page
-	
-	CegoComparison comp = compValue(iv);
         
 	if (_pAttrCond)
 	{
-
+	    CegoComparison comp = compValue(iv);
+	    
 	    if ( _pAttrCond->getPrimaryCompMode() == CegoAttrComp::BTWN )
 	    {
 		_pAttrCond->setPrimaryComparison(LESS_EQUAL_THAN);
@@ -560,8 +641,7 @@ bool CegoBTreeCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
 		break;
 	    }
 	    case LESS_THAN:
-	    {
-		
+	    {		
 		switch (comp)
 		{
 		case LESS_THAN:
@@ -598,8 +678,7 @@ bool CegoBTreeCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
 		    break;
 		}
 		break;
-	    }
-	    
+	    }   
 	    case MORE_EQUAL_THAN:
 	    {
 		switch (comp)
@@ -630,67 +709,56 @@ bool CegoBTreeCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
 	    	    
 	    // dp = _ie.getData();
 	    
-	    if (_dataLock)
-	    {
-		_pTabMng->releaseDataPtr(_dataLock, false);
-		_dataLock = 0;
-	    }
-	    _dataLock = _pTabMng->claimDataPtr(_tabSetId, CegoLockHandler::READ, CegoBufferPool::SYNC, dp, p, len);
+	    _pTabMng->releaseDataPtrUnlocked(_dataPtr, false);
+	    _dataPtr = _pTabMng->claimDataPtrUnlocked(_tabSetId, CegoBufferPool::SYNC, dp, p, len);
 	    
 	    // skipping tid
 	    
-	    int tid;
-	    int tastep;
+	    unsigned long long tid;
+	    unsigned long long tastep;
 	    CegoTupleState ts;
-	    memcpy(&tid, p, sizeof(int));
 
-	    char* ptastep = p + sizeof(int);
-	    memcpy(&tastep, ptastep, sizeof(int));
+	    int toff = _qh.decodeTupleHeader(tid, tastep, ts, p);
+
+	    char* tp = p + toff;
+	    int tlen = len - toff;
 
-	    char* pts = p + sizeof(int) + sizeof(int);
-	    memcpy(&ts, pts, sizeof(CegoTupleState));
-	    
 	    if (tid != 0)
-	    {
-		if ( _ignoreTouched == true )
+	    {		    
+		if ( _ignoreTouched )
 		{
 		    if ( ts == INSERTED 
 			 && tid == _pTabMng->getTID(_tabSetId) 
 			 && tastep < _pTabMng->getTAStep(_tabSetId) )
 		    {
-			CegoTupleState ts;
-			_qh.decodeFVL(fl, p, len, tid, tastep, ts);
+			_qh.decodeFVL(fl, tp, tlen);
 			return true;   
 		    }
 		    else
-		    {		
-			// ignore entry
+		    {
+			return getNext(fl, dp);		
 		    }
 		}
 		else
 		{
-
 		    if ( ( _readUncommitted == true 
 			   && ts == INSERTED )
 			 || ( _readUncommitted == false 
 			      && (( ts == INSERTED && tid == _pTabMng->getTID(_tabSetId)) 
 				  || ( ts == DELETED && tid != _pTabMng->getTID(_tabSetId)))))
 		    {
-			CegoTupleState ts;
-			_qh.decodeFVL(fl, p, len, tid, tastep, ts);
+			_qh.decodeFVL(fl, tp, tlen);
 			return true;
 		    }
 		    else
 		    {
-			// ignore entry
+			return getNext(fl, dp);		
 		    }
 		}
 	    }
 	    else
-	    {
-		
-		CegoTupleState ts;
-		_qh.decodeFVL(fl, p, len, tid, tastep, ts);
+	    {	       
+		_qh.decodeFVL(fl, tp, tlen);
 		return true;
 	    }
 	}
@@ -705,19 +773,25 @@ bool CegoBTreeCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
 
 void CegoBTreeCursor::abort()
 {
+    _pTabMng->releaseDataPtrUnlocked(_dataPtr, false);
+    _dataPtr = CegoBufferPage();
+	
+    if ( _curPage.isFixed() )
+	_pDBMng->bufferUnfix(_curPage, false, _pTabMng->getLockHandler());	
+
     // unfix node pages on stack
     CegoBufferPage bp;
     while ( _parentPageStack.Pop(bp) )
     {
 	// cout << "Unfixing parent page " << bp.getFileId() << " " << bp.getPageId() << endl;
-	_pDBMng->bufferUnfix(bp, true, _pTabMng->getLockHandler());
+	_pDBMng->bufferUnfix(bp, false, _pTabMng->getLockHandler());
     }
 
-    while ( _fixedPageStack.Pop(bp) )
+    if ( _lockId )
     {
-	// cout << "Unfixing new page " << bp.getFileId() << " " << bp.getPageId() << endl;
-	_pDBMng->bufferUnfix(bp, true, _pTabMng->getLockHandler());
-    }       
+	_pTabMng->getLockHandler()->unlockData(CegoObject::BTREE, _lockId);
+	_lockId = 0;
+    }
 }
 
 void CegoBTreeCursor::reset()
@@ -728,8 +802,6 @@ void CegoBTreeCursor::reset()
 CegoComparison CegoBTreeCursor::compValue(const CegoBTreeValue& val)
 {
 
-    // cout << "Comparing value " << val.toChain(_btreeSchema) << endl;
-
     ListT<CegoFieldValue> fvl = val.valueToFVL(_btreeSchema);
 
     CegoField* pF = _btreeSchema.First();
@@ -737,21 +809,17 @@ CegoComparison CegoBTreeCursor::compValue(const CegoBTreeValue& val)
 
     while ( pF ) 
     {
-
 	CegoAttrComp* pAC = _pAttrCond->getAttrCompSet().First();
 
 	while ( pAC )
 	{
-	    // cout << "Comp is " << pAC->toChain() << endl;
-
 	    if  ( (Chain)pAC->getAttrName() == (Chain)pF->getAttrName()  )
 	    {	       
 		if ( *pFV < pAC->getFieldValue() )
 		{
 		    return LESS_THAN;
-		}
-		
-		if ( pAC->getCompMode() == CegoAttrComp::VAL
+		}		
+		else if ( pAC->getCompMode() == CegoAttrComp::VAL
 		     || pAC->getCompMode() == CegoAttrComp::ATTR )
 		{
 		    if ( *pFV > pAC->getFieldValue() )
@@ -768,15 +836,16 @@ CegoComparison CegoBTreeCursor::compValue(const CegoBTreeValue& val)
 		}		
 	    }
 	    pAC = _pAttrCond->getAttrCompSet().Next();
-	}		
-
 
+	    // check for partial modus here
+	    if ( pAC )
+	    {
+		if ( pAC->getComparison() != EQUAL )
+		    return EQUAL;
+	    }
+	}
 	pF = _btreeSchema.Next();
 	pFV = fvl.Next();
-
     }
-
     return EQUAL;
-
 }
-
