@@ -6,22 +6,7 @@
 //     
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2013 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -30,23 +15,29 @@
 // Description: This class handles operations on btree elements.
 //              These elements either can be nodes or leafs and depending on the type, the corresponding methods are available.
 //
-// Status: 2.16
+//
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
-// base includes
+// LFC INCLUDES
 #include <lfcbase/Exception.h>
 
-// cego includes
+// CEGO INCLUDES
 #include "CegoBTreeNode.h"
 #include "CegoDataPointer.h"
+#include "CegoTupleState.h"
+#include "CegoQueryHelper.h"
 
+// POSIX INCLUDES
 #include <string.h>
 #include <stdlib.h>
 
-// must match the value of method CegoDataPointer.getEncodingLenth 
+// must match the value of method CegoDataPointer.getEncodingLength 
 #define DPENCODINGLENGTH 3*sizeof(int)
 
+char __allowDuplicateNull = 0;
+
 CegoBTreeNode::CegoBTreeNode()
 {
     _pI = 0;
@@ -68,7 +59,7 @@ void CegoBTreeNode::setType(NodeType nt)
     _nt = nt;
 }
 
-const CegoBTreeNode::NodeType CegoBTreeNode::getType()
+const CegoBTreeNode::NodeType CegoBTreeNode::getType() const
 {
     return _nt;
 }
@@ -137,10 +128,10 @@ void CegoBTreeNode::initNode()
 	memcpy(_pI, &numEntry, sizeof(int));
 	return;
     }
-    throw Exception(EXLOC, Chain("BTree node not set up"));
+    throw Exception(EXLOC, Chain("Btree node not set up"));
 }
 
-int CegoBTreeNode::numEntries()
+int CegoBTreeNode::numEntries() const
 {
     if ( _pI )
     {
@@ -148,7 +139,7 @@ int CegoBTreeNode::numEntries()
 	memcpy(&n, _pI, sizeof(int));
 	return n;
     }
-    throw Exception(EXLOC, Chain("BTree node not set up"));
+    throw Exception(EXLOC, Chain("Btree node not set up"));
 }
 
 void CegoBTreeNode::incEntries()
@@ -161,7 +152,7 @@ void CegoBTreeNode::incEntries()
 	memcpy(_pI, &n, sizeof(int));
 	return;
     }
-    throw Exception(EXLOC, Chain("BTree node not set up"));
+    throw Exception(EXLOC, Chain("Btree node not set up"));
 }
 
 void CegoBTreeNode::decEntries()
@@ -174,10 +165,10 @@ void CegoBTreeNode::decEntries()
 	memcpy(_pI, &n, sizeof(int));
 	return;
     }
-    throw Exception(EXLOC, Chain("BTree node not set up"));
+    throw Exception(EXLOC, Chain("Btree node not set up"));
 }
 
-int CegoBTreeNode::maxEntries()
+int CegoBTreeNode::maxEntries() const
 {
     if ( _nt == LEAF )
     {
@@ -185,7 +176,7 @@ int CegoBTreeNode::maxEntries()
     }
     else if ( _nt == NODE )
     {	
-	return ( _len - sizeof(int) - 2 * sizeof(int) )  / ( _keyLen + 2 * sizeof(int) );
+	return ( _len - sizeof(int) - 2 * sizeof(int) ) / ( _keyLen + 2 * sizeof(int) );
     }
     else
     {
@@ -224,10 +215,12 @@ void CegoBTreeNode::getChildPage(const CegoBTreeValue& iv, int& fileId, int& pag
     while ( i < ne && posFound == false)
     {
 
-	char* pKey = (char*)((long)_pI + sizeof(int)) + 2 * sizeof(int) + i * getEntrySize();
+	char* pKey = (char*)((long long)_pI + sizeof(int)) + 2 * sizeof(int) + i * getEntrySize();
 	CegoBTreeValue nv;
 	nv.setPtr(pKey);
 
+	// cout << "getChildPage : Comparing " << iv.toChain(_pSchema) << " and " << nv.toChain(_pSchema) << "i=" << i << "ne=" << ne << endl;
+
 	if ( iv.isHigher(nv, _pSchema) )
 	{
 	    i++;
@@ -244,125 +237,65 @@ void CegoBTreeNode::getChildPage(const CegoBTreeValue& iv, int& fileId, int& pag
     cout << "NumEntries are " << ne << endl;
     */
 
-    char* p = (char*)((long)_pI + sizeof(int)) + i * getEntrySize();
+    char* p = (char*)((long long)_pI + sizeof(int)) + i * getEntrySize();
     
     memcpy(&fileId, p, sizeof(int));
     p+=sizeof(int);
     memcpy(&pageId, p, sizeof(int));
-    return;
-    
-}
 
+    // cout << "Found child page " << pageId << endl;
+
+    return;
+}
 
 void CegoBTreeNode::setSchema(ListT<CegoField>* pSchema, int keyLen)
 {
     _pSchema = pSchema;
-
     _keyLen = keyLen;
-
-    /*
-    CegoField *pF = _schema.First();
-    while ( pF )
-    {
-	_keyLen += pF->getLength();
-	pF = _schema.Next();
-    }
-    */
 }
 
-/* addValue - old version */
-
-/*
-bool CegoBTreeNode::addValue(const CegoBTreeValue& iv, const CegoDataPointer& dp, const Chain& btreeName, bool isUnique)
+bool CegoBTreeNode::addValue(const CegoBTreeValue& iv, const CegoDataPointer& dp, const Chain& btreeName, bool isUnique, CegoObjectManager *pOM, int tabSetId, unsigned long long tid, bool& duplicateAdded)
 {
+    duplicateAdded = false;
 
-    if ( _nt == NODE )
-	throw Exception(EXLOC, Chain("Invalid method for node type"));
-
-    int ne = numEntries();
-
-    if ( ne == maxEntries() )
-	return false;
-
-    char* pKey = (char*)((long)_pI + sizeof(int));
-
-    CegoBTreeValue nv;
-
-    bool isInserted = false;
+    if ( isUnique && __allowDuplicateNull && iv.hasNull(*_pSchema))
+	isUnique=false;
     
-    int i=0;
-    while ( i < ne )
+    if ( _nt == NODE )
+	     throw Exception(EXLOC, Chain("Invalid method for node type"));
+    
+    int ne = numEntries();
+    
+    if ( ne == maxEntries() )
     {
-	nv.setPtr(pKey);
-	
-	if ( isUnique )
+	CegoDataPointer dp1;
+	if ( valueExists(iv, dp1) )
 	{
-	    if ( nv.isEqual(iv, _pSchema))
-	    {
-		Chain msg = Chain("Duplicate key on unique btree ") + btreeName;
-		throw Exception(EXLOC, msg);
+	    // if we have detected duplicate on unique btree, we can throw exception
+	    if ( isUnique && ! ( tid && checkForDeleted(pOM, dp1, tabSetId, tid) ) )
+	    {		
+		Chain msg = Chain("Duplicate key ") + iv.toChain(_pSchema) + Chain(" on unique btree ") + btreeName;
+		throw Exception(EXLOC, msg);		    
 	    }
-	}
-	if ( nv.isHigher(iv, _pSchema) )
-	{
-	    shiftEntries(i, 1);
 	    
-            // copy iv to free slot
-	    memcpy(pKey, iv.getPtr(), _keyLen);
-	    char* pDP = (char*)((long)pKey + _keyLen);
-	    dp.encode(pDP);
-
-	    incEntries();
-
-	    return true; 	    
+	    duplicateAdded=true;
 	}
-	pKey += getEntrySize();
-	i++;
+	return false;
     }
 
-    // adding value at the end
-    memcpy(pKey, iv.getPtr(), _keyLen);
-    char* pDP = (char*)((long)pKey + _keyLen);
-    dp.encode(pDP);
-
-    incEntries();
-
-    return true;
-}
-*/
-
-
-/* add value - new version */
-
-bool CegoBTreeNode::addValue(const CegoBTreeValue& iv, const CegoDataPointer& dp, const Chain& btreeName, bool isUnique)
-{
-
-    // cout << "Adding value " << iv.toChain(_pSchema) << endl;
-
-    if ( _nt == NODE )
-	throw Exception(EXLOC, Chain("Invalid method for node type"));
-    
-    int ne = numEntries();
-    
-    if ( ne == maxEntries() )
-	return false;
-   
     if ( ne == 0 )
     {
-
-	char* pKey = (char*)((long)_pI + sizeof(int));
+	char* pKey = (char*)((long long)_pI + sizeof(int));
 	memcpy(pKey, iv.getPtr(), _keyLen);
-	char* pDP = (char*)((long)pKey + _keyLen);
+	char* pDP = (char*)((long long)pKey + _keyLen);
 	dp.encode(pDP);
 	
 	incEntries();
-	return true;
-	
+	return true;	
     }
 
     bool isInserted = false;
-    
-    
+        
     int lb=0;
     int rb=ne;
 
@@ -372,7 +305,6 @@ bool CegoBTreeNode::addValue(const CegoBTreeValue& iv, const CegoDataPointer& dp
 
     while ( posFound == false )
     {
-	
 	// cout << "lb=" << lb << " rb=" << rb << endl;
 
 	i = (lb+rb) / 2;
@@ -381,9 +313,13 @@ bool CegoBTreeNode::addValue(const CegoBTreeValue& iv, const CegoDataPointer& dp
 
 	CegoBTreeValue nv1;
 	CegoBTreeValue nv2;
-	
 
-	nv1.setPtr((char*)((long)_pI + sizeof(int)) + i * getEntrySize());
+	CegoDataPointer dp1;
+	CegoDataPointer dp2;
+
+	nv1.setPtr((char*)((long long)_pI + sizeof(int)) + i * getEntrySize());
+	dp1.decode((char*)((long long)_pI + sizeof(int)) + i * getEntrySize() + _keyLen);
+
 	CegoBTreeValue::Comparison valueComp1 = iv.comp(nv1, _pSchema);
 	CegoBTreeValue::Comparison valueComp2;
 
@@ -394,20 +330,43 @@ bool CegoBTreeNode::addValue(const CegoBTreeValue& iv, const CegoDataPointer& dp
 	}
 	else
 	{
-	    nv2.setPtr((char*)((long)_pI + sizeof(int)) + (i+1)*getEntrySize());
+	    nv2.setPtr((char*)((long long)_pI + sizeof(int)) + (i+1)*getEntrySize());
+	    dp2.decode((char*)((long long)_pI + sizeof(int)) + (i+1)*getEntrySize() + _keyLen);
+
 	    valueComp2 = iv.comp(nv2, _pSchema);
 	}
-	
-	
-	
+		
 	// cout << "VC1=" << valueComp1 << endl;
 	// cout << "VC2=" << valueComp2 << endl;
-	
 
-	if ( isUnique && ( valueComp1 == CegoBTreeValue::EQUAL || valueComp2 == CegoBTreeValue::EQUAL ))
-	{
-	    Chain msg = Chain("Duplicate key on unique btree ") + btreeName;
-	    throw Exception(EXLOC, msg);
+	// cout << "Comparing " << iv.toChain(_pSchema) << " with " << nv1.toChain(_pSchema) << " and " << nv2.toChain(_pSchema) << endl;
+
+	if ( valueComp1 == CegoBTreeValue::EQUAL || valueComp2 == CegoBTreeValue::EQUAL )
+	{       	    
+	    if ( valueComp1 == CegoBTreeValue::EQUAL )
+	    {
+		if ( isUnique && ! ( tid && checkForDeleted(pOM, dp1, tabSetId, tid) ) )
+		{
+		    Chain msg = Chain("Duplicate key ") + iv.toChain(_pSchema) + Chain(" on unique btree ") + btreeName;
+		    throw Exception(EXLOC, msg);		    
+		}
+		else
+		{
+		    duplicateAdded = true;
+		}
+	    }
+	    if (  valueComp2 == CegoBTreeValue::EQUAL )
+	    {
+		if ( isUnique && ! ( tid && checkForDeleted(pOM, dp2, tabSetId, tid) ) )
+		{
+		    Chain msg = Chain("Duplicate key ") + iv.toChain(_pSchema) + Chain(" on unique btree ") + btreeName;
+		    throw Exception(EXLOC, msg);		    
+		}
+		else
+		{
+		    duplicateAdded = true;
+		}
+	    }
 	}
 
 	if ( ( valueComp1 == CegoBTreeValue::MORE || valueComp1 == CegoBTreeValue::EQUAL )
@@ -443,19 +402,17 @@ bool CegoBTreeNode::addValue(const CegoBTreeValue& iv, const CegoDataPointer& dp
 	shiftEntries(i, 1);
     }
 
-    char* pKey = (char*)((long)_pI + sizeof(int) + i* getEntrySize());    
+    char* pKey = (char*)((long long)_pI + sizeof(int) + i* getEntrySize());    
     memcpy(pKey, iv.getPtr(), _keyLen);
-    char* pDP = (char*)((long)pKey + _keyLen);
+    char* pDP = (char*)((long long)pKey + _keyLen);
     dp.encode(pDP);
-	
+
     incEntries();
     return true;
 }
 
-
 bool CegoBTreeNode::deleteValue(const CegoBTreeValue& iv, const CegoDataPointer& dp)
 {
-
     if ( _nt == NODE )
 	throw Exception(EXLOC, Chain("Invalid method for node type"));
 
@@ -464,7 +421,7 @@ bool CegoBTreeNode::deleteValue(const CegoBTreeValue& iv, const CegoDataPointer&
     if ( ne == 0 )
 	return false;
 
-    char* pKey = (char*)((long)_pI + sizeof(int));
+    char* pKey = (char*)((long long)_pI + sizeof(int));
 
     CegoBTreeValue lv;
 
@@ -491,6 +448,42 @@ bool CegoBTreeNode::deleteValue(const CegoBTreeValue& iv, const CegoDataPointer&
     return false;
 }
 
+bool CegoBTreeNode::valueExists(const CegoBTreeValue& iv)
+{
+    CegoDataPointer dp;
+    return valueExists(iv, dp);
+}
+
+bool CegoBTreeNode::valueExists(const CegoBTreeValue& iv, CegoDataPointer& dp)
+{
+    if ( _nt == NODE )
+	throw Exception(EXLOC, Chain("Invalid method for node type"));
+
+    int ne = numEntries();
+
+    if ( ne == 0 )
+	return false;
+
+    char* pKey = (char*)((long long)_pI + sizeof(int));
+
+    CegoBTreeValue lv;
+    
+    int i=0;
+    while ( i < ne )
+    {
+	lv.setPtr(pKey);
+
+	if ( lv.isEqual(iv, _pSchema))
+	{
+	    dp.decode(pKey + _keyLen);
+	    return true; 	    
+	}
+	pKey += getEntrySize();
+	i++;
+    }
+
+    return false;
+}
 
 bool CegoBTreeNode::addNode(const CegoBTreeValue& iv, const CegoBTreeNode& node)
 {
@@ -507,20 +500,38 @@ bool CegoBTreeNode::addNode(const CegoBTreeValue& iv, const CegoBTreeNode& node)
     
     while ( i < ne && posFound == false)
     {
-	p = (char*)((long)_pI + sizeof(int) + 2 * sizeof(int) + i * getEntrySize());
+	
+	p = (char*)((long long)_pI + sizeof(int) + 2 * sizeof(int) + i * getEntrySize());
 	CegoBTreeValue nv(p);
 	if ( nv.isHigher(iv, _pSchema) )
 	{
 	    shiftEntries(i, 1);
 	    posFound=true;
 	}
+	else if ( iv.isEqual(nv, _pSchema) )
+	{
+	    // if equal, we also have to check the maximum value
+	    if ( node.getMax().isHigher(nv, _pSchema) )
+	    {
+		i++;
+	    }
+	    else
+	    {
+		shiftEntries(i, 1);
+		posFound=true;		    
+	    }
+	}
 	else
 	{	    
 	    i++;
 	}
     }
-    
-    p = (char*)((long)_pI + sizeof(int) + 2 * sizeof(int) + i * getEntrySize());
+ 
+    ////////////////////////////////////
+    // Setting up new node information //
+    ////////////////////////////////////
+
+    p = (char*)((long long)_pI + sizeof(int) + 2 * sizeof(int) + i * getEntrySize());
     
     // put index value
     memcpy(p, iv.getPtr(), _keyLen);
@@ -537,7 +548,6 @@ bool CegoBTreeNode::addNode(const CegoBTreeValue& iv, const CegoBTreeNode& node)
     incEntries();
     
     return true;
-
 }
 
 void CegoBTreeNode::split(CegoBTreeNode& n)
@@ -547,8 +557,8 @@ void CegoBTreeNode::split(CegoBTreeNode& n)
     if ( _nt == LEAF )
     {
 	pos = numEntries() / 2;
-	char* srcPtr = (char*)((long)_pI + sizeof(int) + pos * getEntrySize());
-	char* destPtr = (char*)((long)n._pI + sizeof(int));
+	char* srcPtr = (char*)((long long)_pI + sizeof(int) + pos * getEntrySize());
+	char* destPtr = (char*)((long long)n._pI + sizeof(int));
 	int s =  ( numEntries() - pos ) * getEntrySize();
 	memcpy(destPtr, srcPtr, s);
 	
@@ -556,8 +566,8 @@ void CegoBTreeNode::split(CegoBTreeNode& n)
     else if ( _nt == NODE )
     {
 	pos = numEntries() / 2;
-	char* srcPtr = (char*)((long)_pI + sizeof(int) + pos * getEntrySize());
-	char* destPtr = (char*)((long)n._pI + sizeof(int));
+	char* srcPtr = (char*)((long long)_pI + sizeof(int) + pos * getEntrySize());
+	char* destPtr = (char*)((long long)n._pI + sizeof(int));
 	int s =  ( numEntries() - pos ) * getEntrySize() + 2 * sizeof(int);
 	memcpy(destPtr, srcPtr, s);	
     }
@@ -565,45 +575,41 @@ void CegoBTreeNode::split(CegoBTreeNode& n)
     int s1 = pos;
     int s2 = numEntries() - pos;
     memcpy(_pI, &s1, sizeof(int));
-    memcpy(n._pI, &s2, sizeof(int));    
-
+    memcpy(n._pI, &s2, sizeof(int));
 }
 
 void CegoBTreeNode::shiftEntries(int pos, int offset)
 {
-
     if ( _nt == LEAF )
     {
-	char* srcPtr = (char*)((long)_pI + sizeof(int) + pos * getEntrySize());
-	char* destPtr = (char*)((long)_pI + sizeof(int) + ( pos + offset) * getEntrySize());
+	char* srcPtr = (char*)((long long)_pI + sizeof(int) + pos * getEntrySize());
+	char* destPtr = (char*)((long long)_pI + sizeof(int) + ( pos + offset) * getEntrySize());
 	int shiftSize = ( numEntries() - pos ) * getEntrySize();
-	memcpy(destPtr, srcPtr, shiftSize);
+	memmove(destPtr, srcPtr, shiftSize);
     }
     else if ( _nt == NODE )
     {
-	char* srcPtr = (char*)((long)_pI + sizeof(int) + 2 * sizeof(int) + pos * getEntrySize());
-	char* destPtr = (char*)((long)_pI + sizeof(int) + 2 * sizeof(int) + ( pos + offset ) * getEntrySize());
+	char* srcPtr = (char*)((long long)_pI + sizeof(int) + 2 * sizeof(int) + pos * getEntrySize());
+	char* destPtr = (char*)((long long)_pI + sizeof(int) + 2 * sizeof(int) + ( pos + offset ) * getEntrySize());
 	int shiftSize = ( numEntries() - pos ) * getEntrySize();
-	memcpy(destPtr, srcPtr, shiftSize);
+	memmove(destPtr, srcPtr, shiftSize);
     }
 }
 
-
-CegoBTreeValue CegoBTreeNode::getMin()
+CegoBTreeValue CegoBTreeNode::getMin() const
 {
-
     if ( numEntries() == 0 )
-	throw Exception(EXLOC, Chain("No index values available in node"));
+	throw Exception(EXLOC, Chain("No values values available in node"));
           
     if ( _nt == LEAF )    
     {
-	char* p = (char*)((long)_pI + sizeof(int));
+	char* p = (char*)((long long)_pI + sizeof(int));
 	CegoBTreeValue iv(p);
 	return iv;
     }
     else if ( _nt == NODE )
     {
-	char* p = (char*)((long)_pI + sizeof(int) + 2*sizeof(int));
+	char* p = (char*)((long long)_pI + sizeof(int) + 2*sizeof(int));
 	CegoBTreeValue iv(p);
 	return iv;
     }
@@ -613,7 +619,7 @@ CegoBTreeValue CegoBTreeNode::getMin()
     }
 }
 
-CegoBTreeValue CegoBTreeNode::getMax()
+CegoBTreeValue CegoBTreeNode::getMax() const
 {
     int pos = numEntries() - 1;
 
@@ -622,11 +628,11 @@ CegoBTreeValue CegoBTreeNode::getMax()
 	char* p;
 	if ( _nt == LEAF )
 	{
-	    p = (char*)((long)_pI + sizeof(int) + pos * getEntrySize());
+	    p = (char*)((long long)_pI + sizeof(int) + pos * getEntrySize());
 	}
 	else
 	{
-	    p = (char*)((long)_pI + sizeof(int) + 2 * sizeof(int) + pos * getEntrySize());
+	    p = (char*)((long long)_pI + sizeof(int) + 2 * sizeof(int) + pos * getEntrySize());
 	}
 	CegoBTreeValue iv(p);
 	return iv;
@@ -635,13 +641,8 @@ CegoBTreeValue CegoBTreeNode::getMax()
     throw Exception(EXLOC, "No index values available in node");
 }
 
-
-
-/* propagate - old version */
-
 bool CegoBTreeNode::propagate(const CegoBTreeValue& iv, const CegoBTreeNode& leftNode, const CegoBTreeNode& rightNode)
 {
-
     if ( _nt == LEAF )
 	throw Exception(EXLOC, Chain("Invalid method for node type"));
  
@@ -653,8 +654,8 @@ bool CegoBTreeNode::propagate(const CegoBTreeValue& iv, const CegoBTreeNode& lef
     {
 	// first entry
 	
-	char* p = (char*)((long)_pI + sizeof(int));
-	int fileId, pageId;
+	char* p = (char*)((long long)_pI + sizeof(int));
+	int fileId, pageId, nextFileId, nextPageId;
 	
 	// put left child pointer
 	fileId = leftNode.getFileId();
@@ -664,17 +665,17 @@ bool CegoBTreeNode::propagate(const CegoBTreeValue& iv, const CegoBTreeNode& lef
 	pageId = leftNode.getPageId();
 	memcpy(p, &pageId, sizeof(int));
 	p+=sizeof(int);
-	
+
 	// put index value
 	memcpy(p, iv.getPtr(), _keyLen);
 	p+=_keyLen;
 	
 	// put right child pointer
-	fileId = rightNode.getFileId();
-	memcpy(p, &fileId, sizeof(int));
+	nextFileId = rightNode.getFileId();
+	memcpy(p, &nextFileId, sizeof(int));
 	p+=sizeof(int);
-	pageId = rightNode.getPageId();
-	memcpy(p, &pageId, sizeof(int));
+	nextPageId = rightNode.getPageId();
+	memcpy(p, &nextPageId, sizeof(int));
 	p+=sizeof(int);
 	
 	incEntries();
@@ -684,19 +685,34 @@ bool CegoBTreeNode::propagate(const CegoBTreeValue& iv, const CegoBTreeNode& lef
     else
     {
 
-
 	bool posFound=false;
 	char* p = 0;
 	int i=0;	    
-	
+
 	while ( i < ne && posFound == false)
 	{
-	    p = (char*)((long)_pI + sizeof(int) + 2 * sizeof(int) + i * getEntrySize());
+	    p = (char*)((long long)_pI + sizeof(int) + 2 * sizeof(int) + i * getEntrySize());
 	    CegoBTreeValue nv(p);
+
+	    // cout << "Comparing " << iv.toChain(_pSchema) << " and " << nv.toChain(_pSchema) << "i=" << i << "ne=" << ne << endl;
+	    
 	    if ( iv.isHigher(nv, _pSchema) )
-	    {
+	    {		
 		i++;
 	    }
+	    else if ( iv.isEqual(nv, _pSchema) )
+	    {
+		// if equal, we also have to check the maximum value
+		if ( rightNode.getMax().isHigher(nv, _pSchema) )
+		{
+		    i++;
+		}
+		else
+		{
+		    shiftEntries(i, 1);
+		    posFound=true;		    
+		}
+	    }
 	    else
 	    {
 		shiftEntries(i, 1);
@@ -704,138 +720,11 @@ bool CegoBTreeNode::propagate(const CegoBTreeValue& iv, const CegoBTreeNode& lef
 	    }
 	}
 	
-	p = (char*)((long)_pI + sizeof(int) + 2 * sizeof(int) + i * getEntrySize());
-	
-	// put index value
-	memcpy(p, iv.getPtr(), _keyLen);
-	p+=_keyLen;
-	
-	// put child pointer
-	int fileId = rightNode.getFileId();
-	memcpy(p, &fileId, sizeof(int));
-	p+=sizeof(int);
-	int pageId = rightNode.getPageId();
-	memcpy(p, &pageId, sizeof(int));
-	p+=sizeof(int);
-	
-	incEntries();
-	
-	return true;
+	////////////////////////////////////
+	// Setting up new node information //
+	////////////////////////////////////
 
-
-    }
-}
-
-
-/* propagate - new version - still contains a bug and dows not improve performance */
-
-/*
-bool CegoBTreeNode::propagate(const CegoBTreeValue& iv, const CegoBTreeNode& leftNode, const CegoBTreeNode& rightNode)
-{
-
-    if ( _nt == LEAF )
-	throw Exception(EXLOC, Chain("Invalid method for node type"));
- 
-    int ne = numEntries();       	
-    if ( ne == maxEntries() )
-	return false;
-    
-    if ( ne == 0 )
-    {
-	// first entry
-	
-	char* p = (char*)((long)_pI + sizeof(int));
-	int fileId, pageId;
-	
-	// put left child pointer
-	fileId = leftNode.getFileId();
-	memcpy(p, &fileId, sizeof(int));
-	p+=sizeof(int);
-	
-	pageId = leftNode.getPageId();
-	memcpy(p, &pageId, sizeof(int));
-	p+=sizeof(int);
-	
-	// put index value
-	memcpy(p, iv.getPtr(), _keyLen);
-	p+=_keyLen;
-	
-	// put right child pointer
-	fileId = rightNode.getFileId();
-	memcpy(p, &fileId, sizeof(int));
-	p+=sizeof(int);
-	pageId = rightNode.getPageId();
-	memcpy(p, &pageId, sizeof(int));
-	p+=sizeof(int);
-	
-	incEntries();
-	
-	return true;
-    }
-    else
-    {
-
-	int lb=0;
-	int rb=ne;
-	
-	int i = (lb+rb) / 2;
-	
-	bool posFound=false;
-	
-	while ( posFound == false )
-	{
-	    i = (lb+rb) / 2;
-	    CegoBTreeValue nv1;
-	    CegoBTreeValue nv2;
-
-	    nv1.setPtr((char*)((long)_pI + sizeof(int)) + 2*sizeof(int) + i * getEntrySize());
-	    CegoBTreeValue::Comparison valueComp1 = iv.comp(nv1, _pSchema);
-	    CegoBTreeValue::Comparison valueComp2;
-	    
-	    if ( i+1 == ne )
-	    {
-		valueComp2 = CegoBTreeValue::LESS;
-	    }
-	    else
-	    {
-		nv2.setPtr((char*)((long)_pI + sizeof(int)) + 2*sizeof(int) + (i+1)*getEntrySize());
-		valueComp2 = iv.comp(nv2, _pSchema);
-	    }
-	    
-	    if ( ( valueComp1 == CegoBTreeValue::MORE || valueComp1 == CegoBTreeValue::EQUAL )
-		 && ( valueComp2 == CegoBTreeValue::LESS || valueComp2 == CegoBTreeValue::EQUAL ) )
-	    {
-		// cout << "Propagation case : A" << endl;
-		i++;
-		posFound=true;
-	    }
-	    else if ( valueComp2 == CegoBTreeValue::MORE )
-	    {
-
-		// cout << "Propagation case : B" << endl;
-		if ( i == ne )
-		    posFound = true;
-		else
-		    lb=i;	    	    	    
-	    }
-	    else
-	    {
-		// cout << "Propagation case : C" << endl;
-		if ( i == 0 )
-		    posFound = true;
-		else
-		    rb=i;
-	    }
-	    // sleep(1);
-	}
-
-
-	if ( i < ne )
-	{
-	    shiftEntries(i, 1);
-	}
-
-	char* p = (char*)((long)_pI + sizeof(int) + 2 * sizeof(int) + i * getEntrySize());
+	p = (char*)((long long)_pI + sizeof(int) + 2 * sizeof(int) + i * getEntrySize());
 	
 	// put index value
 	memcpy(p, iv.getPtr(), _keyLen);
@@ -855,9 +744,6 @@ bool CegoBTreeNode::propagate(const CegoBTreeValue& iv, const CegoBTreeNode& lef
 
     }
 }
-
-*/
-
  
 CegoBTreeNode& CegoBTreeNode::operator = (const CegoBTreeNode& n)
 {
@@ -873,23 +759,100 @@ CegoBTreeNode& CegoBTreeNode::operator = (const CegoBTreeNode& n)
     return (*this);
 }
 
+bool CegoBTreeNode::verify()
+{
+    int n = numEntries();
+
+    if ( _nt == LEAF )
+    {
+	int i = 0;
+	while ( i < n )
+	{	    
+	    char* srcPtr = (char*)((long long)_pI + sizeof(int) + i * getEntrySize());
+	    CegoBTreeValue iv;
+	    iv.setPtr(srcPtr);
+	    srcPtr += _keyLen;
+	    CegoDataPointer dp;
+	    dp.decode(srcPtr);
+
+	    // we check fileId and offset, must not be zero in any case
+	    if ( dp.getFileId() == 0 || dp.getOffset() == 0 )
+	    {
+		return false;
+	    }
+	    // cout << iv.toChain(_pSchema) << " " << dp << endl;	  
+	    i++;
+	}
+    }
+    else if ( _nt == NODE )
+    {
+	int fileId, pageId;
+
+	char* p = (char*)((long long)_pI + sizeof(int));
+	
+	memcpy(&fileId, p, sizeof(int));
+	p+=sizeof(int);
+	memcpy(&pageId, p, sizeof(int));
+			  
+	// cout << levelIndent(level) << "Child : " << fileId << "," << pageId << endl;
+
+	// we check fileId, must not be zero in any case
+	if ( fileId == 0  )
+	{
+	    return false;
+	}
+	
+	int i = 0;
+	while ( i < n )
+	{
+	    // cout << levelIndent(level) << "Key " << i << " : ";
+	    
+	    p = (char*)((long long)_pI + sizeof(int) + 2*sizeof(int) + i * getEntrySize());
+	    CegoBTreeValue iv;
+	    iv.setPtr(p);
+	    // cout << iv.toChain(_pSchema) << endl;
+	    p += _keyLen;
+	    
+	    memcpy(&fileId, p, sizeof(int));
+	    p+=sizeof(int);
+	    memcpy(&pageId, p, sizeof(int));
+
+	    // we check fileId, must not be zero in any case
+	    if ( fileId == 0 )
+	    {
+		return false;
+	    }
+
+	    // cout << levelIndent(level) << "Child : " << fileId << "," << pageId << endl;
+
+	    i++;
+	}
+	// cout << levelIndent(level) << "################## " << endl;
+    }
+    else
+    {
+	return false;
+    }
+
+    return true;
+}
+
 void CegoBTreeNode::printNode(int level)
 {
-
     int n = numEntries();
 
     if ( _nt == LEAF )
     {
 
 	cout << levelIndent(level) << "------- LEAF -------" << endl;
-	cout << levelIndent(level) << "FileId=" << _fileId << " PageId=" << _pageId << endl;
+	cout << levelIndent(level) << "NumEntries=" << n << " FileId=" << _fileId << " PageId=" << _pageId << endl;
 
 	int i = 0;
 	while ( i < n )
 	{
 	    cout << levelIndent(level) << "Entry " << i << " : ";
 	    
-	    char* srcPtr = (char*)((long)_pI + sizeof(int) + i * getEntrySize());
+	    char* srcPtr = (char*)((long long)_pI + sizeof(int) + i * getEntrySize());
 	    CegoBTreeValue iv;
 	    iv.setPtr(srcPtr);
 	    srcPtr += _keyLen;
@@ -899,17 +862,16 @@ void CegoBTreeNode::printNode(int level)
 	    i++;
 	}
 	cout << levelIndent(level) << "Next Page : " << _nextFileId << " " << _nextPageId << endl;
-
 	cout << levelIndent(level) << "--------------------" << endl;
     }
     else if ( _nt == NODE )
     {
 	cout << levelIndent(level) << "####### NODE ####### " << endl;
-	cout << levelIndent(level) << "FileId=" << _fileId << " PageId=" << _pageId << endl;
+	cout << levelIndent(level) << "NumEntries=" << n << " FileId=" << _fileId << " PageId=" << _pageId << endl;
 
 	int fileId, pageId;
 
-	char* p = (char*)((long)_pI + sizeof(int));
+	char* p = (char*)((long long)_pI + sizeof(int));
 	
 	memcpy(&fileId, p, sizeof(int));
 	p+=sizeof(int);
@@ -922,7 +884,7 @@ void CegoBTreeNode::printNode(int level)
 	{
 	    cout << levelIndent(level) << "Key " << i << " : ";
 	    
-	    p = (char*)((long)_pI + sizeof(int) + 2*sizeof(int) + i * getEntrySize());
+	    p = (char*)((long long)_pI + sizeof(int) + 2*sizeof(int) + i * getEntrySize());
 	    CegoBTreeValue iv;
 	    iv.setPtr(p);
 	    cout << iv.toChain(_pSchema) << endl;
@@ -947,16 +909,15 @@ void CegoBTreeNode::printNode(int level)
 
 void CegoBTreeNode::reset()
 {
-    _pNextChild = (char*)((long)_pI + sizeof(int));
+    _pNextChild = (char*)((long long)_pI + sizeof(int));
 }
 
 bool CegoBTreeNode::nextChildPointer(int &fileId, int &pageId)
 {
-
     if ( _nt == LEAF )
 	throw Exception(EXLOC, Chain("Invalid method for node type"));
 
-    if ( (long)_pNextChild > (long)_pI + sizeof(int) + numEntries() * getEntrySize() )
+    if ( (long long)_pNextChild > (long long)_pI + sizeof(int) + numEntries() * getEntrySize() )
 	return false;
     
     memcpy(&fileId, _pNextChild, sizeof(int));
@@ -967,17 +928,46 @@ bool CegoBTreeNode::nextChildPointer(int &fileId, int &pageId)
     _pNextChild+=_keyLen;
     
     return true;
+}
 
+void CegoBTreeNode::getLastChildPointer(int &fileId, int &pageId) const
+{
+    if ( _nt == LEAF )
+	throw Exception(EXLOC, Chain("Invalid method for node type"));
+
+    if ( numEntries() == 0 )
+    {
+	fileId = 0;
+	pageId = 0;
+    }
+    else
+    {
+	char *p = (char*)((long long)_pI + sizeof(int) + numEntries() * getEntrySize());
+    
+	memcpy(&fileId, p, sizeof(int));
+	p+=sizeof(int);
+	memcpy(&pageId, p, sizeof(int));
+    }
+}
+
+void CegoBTreeNode::setFirstChildPointer(int fileId, int pageId)
+{
+    if ( _nt == LEAF )
+	throw Exception(EXLOC, Chain("Invalid method for node type"));
+
+    char *p = (char*)((long long)_pI + sizeof(int));
+    
+    memcpy(p, &fileId, sizeof(int));
+    p+=sizeof(int);
+    memcpy(p, &pageId, sizeof(int));
 }
 
 bool CegoBTreeNode::rightChild(CegoBTreeValue& val)
 {
-
     if ( _nt == LEAF )
 	throw Exception(EXLOC, Chain("Invalid method for node type"));
-
-    
-    if ( (long)_pNextChild > (long)_pI + sizeof(int) + numEntries() * getEntrySize() )
+   
+    if ( (long long)_pNextChild > (long long)_pI + sizeof(int) + numEntries() * getEntrySize() )
 	return false;
     
     val.setPtr(_pNextChild - _keyLen);
@@ -989,7 +979,7 @@ bool CegoBTreeNode::nextValue(CegoBTreeValue& val, CegoDataPointer& dp)
     if ( _nt == NODE )
 	throw Exception(EXLOC, Chain("Invalid method for node type"));
 
-    if ( (long)_pNextChild >= (long)_pI + sizeof(int) + numEntries() * getEntrySize() )
+    if ( (long long)_pNextChild >= (long long)_pI + sizeof(int) + numEntries() * getEntrySize() )
 	return false;
     
     val.setPtr(_pNextChild);
@@ -997,8 +987,7 @@ bool CegoBTreeNode::nextValue(CegoBTreeValue& val, CegoDataPointer& dp)
     dp.decode(_pNextChild);
     _pNextChild += DPENCODINGLENGTH;
     
-    return true;
-    
+    return true;    
 }
 
 Chain CegoBTreeNode::levelIndent(int level)
@@ -1011,3 +1000,36 @@ Chain CegoBTreeNode::levelIndent(int level)
     }
     return s;
 }
+
+bool CegoBTreeNode::checkForDeleted(CegoObjectManager* pOM, const CegoDataPointer &dp, int tabSetId, unsigned long long tid)
+{    
+    char* p;
+    int len;
+
+    CegoBufferPage bp = pOM->claimDataPtrUnlocked(tabSetId, CegoBufferPool::SYNC, dp, p, len);
+    
+    // skipping tid
+    unsigned long long dataTid;
+    unsigned long long tastep;
+    CegoTupleState ts;
+
+    CegoQueryHelper::decodeTupleHeader(dataTid, tastep, ts, p);
+    
+    pOM->releaseDataPtrUnlocked(bp, false);
+
+    /*
+    cout << "Datapointer = " << dp << endl;
+    cout << "tid = " << tid << endl;
+    cout << "dataTid = " << dataTid << endl;
+    cout << "ts = " << ts << endl;
+    */
+
+    if (tid == dataTid && ( ts == DELETED || ts == OBSOLETE ))
+    {
+	return true;
+    }
+    else
+    {
+	return false;
+    }
+}
