@@ -2,40 +2,33 @@
 //                                                         
 // CegoBlowTab.cc
 // --------------                           
-// Blowing data to tables
-// 
+// Simulation program to perform random insert, update, delete and stored procedure queries.
+//
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2010 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
-// Class: Main
+// Class: main
 // 
-// Description: 
+// Description: This program can be used to execute random queries for any kind of testing reasons.
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
+// POSIX INCLUDES
 #include <iostream> 
 #include <stdlib.h> 
 
+// improved bsd random functions still not supported for mingw
+#ifdef HAVE_MINGW
+#define srandom srand
+#define random rand
+#endif
+
+// LFC INCLUDES
 #include <lfcbase/Exception.h>
 #include <lfcbase/GetLongOpt.h>
 #include <lfcbase/Chain.h>
@@ -43,47 +36,47 @@
 #include <lfcbase/Timer.h>
 #include <lfcbase/Net.h>
 #include <lfcbase/NetHandler.h>
+#include <lfcbase/Sleeper.h>
 
+// CEGO INCLUDES
 #include "CegoDefs.h"
-#include "CegoDbHandler.h"
+#include "CegoNet.h"
 #include "CegoBlob.h"
-#include "CegoModule.h"
+#include "CegoOutput.h"
 
 #define DEFAULTSERVER "localhost"
 #define DEFAULTPORT 2200
 #define DEFAULTPROTOCOL "xml"
+#define DEFAULTPROGID "cgblow"
 #define DEFAULTINTERVAL 1
 
-enum Operation { INSERT_OP, DELETE_OP, UPDATE_OP, ARBITRARY_OP, PROC_OP, BLOB_OP }; 
+enum Operation { INSERT_OP, DELETE_OP, UPDATE_OP, ARB1_OP, ARB2_OP, ARB3_OP, PROC_OP, BLOB_OP }; 
 
 #define INTCOL "i" 
 #define STRCOL "s" 
 #define KEYCOL "i" 
 
-// #define USAGE "Usage: cgblow [ -i | -d | -k | -a | -l | -b <op>:<arg> ]  -s <host> -p <port> -u <user>/<password> -f <format> -e <format> -c <count> -t <tablename> -j <tableset> -n <report-interval> -g -z -v -h"
-
-#define USAGE "Usage: cgblow --mode=< insert | delete | update | blob | arbitrary | proc>\n\
+#define USAGE "Usage: cgblow --id=<id> --mode=< insert | delete | update | blob | proc |arb1 | arb2 | arb3 >\n\
           --tableset=<tableset>\n\
           --user=<user>/<password>\n\
           [ --server=<host>]\n\
           [ --port=<port> ]\n\
           [ --protocol={serial|xml} ]\n\
           [ --iset=<insert format> ] [ --pset=<proc format> ] [ --dcond=<delete condition> ] [ --uset=<update format> ] [ --ucond=<update condition>] [ --blobop=<format> ] \n\
-          [ --count=<count> ]\n\
-          [ --proc=<tablename> ]\n\
-          [ --table=<procname> ]\n\
-          [ --interval<report-interval> ]\n\
+          [ --count=<number of operations> ]\n\
+          [ --proc=<procname> ]\n\
+          [ --table=<tablename> ]\n\
+          [ --interval=<report interval> ]\n\
           [ --simulate ]\n\
           [ --append ] [ --dotransaction ] [ --version  ] [ --help ]"
 
-bool dispatchCmd(CegoDbHandler* pNetHandle, Chain cmd);
+bool dispatchCmd(CegoNet* pCegoNet, Chain cmd);
 
 extern char __lfcVersionString[];
 extern char __lfcxmlVersionString[];
 
 int main(int argc, char **argv) 
 { 
-
     try { 
 	
 	GetLongOpt longOpt(argc, argv);
@@ -95,6 +88,8 @@ int main(int argc, char **argv)
 	longOpt.addOpt("tableset");
 	longOpt.addOpt("user");
 
+	longOpt.addOpt("id", Chain(DEFAULTPROGID));
+
 	longOpt.addOpt("iset");
 	longOpt.addOpt("pset");
 	longOpt.addOpt("dcond");
@@ -128,8 +123,8 @@ int main(int argc, char **argv)
 	}
 
 	// for random generation in CegoFunction
-	int seed = 5 * (time(NULL) % 100000); 	
-	srand( seed  ); 
+	unsigned seed = 5 * (time(NULL) % 100000); 	
+	srandom( seed  ); 
 	
 	bool isDebug=false;
 	if ( longOpt.isSet("debug") )
@@ -143,13 +138,14 @@ int main(int argc, char **argv)
 
 	if ( longOpt.isSet( Chain("version") ) )
 	{
-	    cout << CEGO_PRODUCT << " (" << sizeof(long) * 8 << " bit), Version " << CEGO_VERSION 
-		 << " [ lfc : " << __lfcVersionString  
-		 << ", lfcxml : " <<  __lfcxmlVersionString << " ]" << endl;
+	    cout << CEGO_PRODUCT << " Simulation Client, (" << sizeof(void*) * 8 << " bit), Version " << CEGO_VERSION 
+		 << " [ lfc: " << __lfcVersionString  
+		 << ", lfcxml: " <<  __lfcxmlVersionString << " ]" << endl;
 	    cout << CEGO_COPYRIGHT << endl;
 	    exit(0);
 	}
 
+	Chain progid = longOpt.getOptValue("id");
 	Chain mode = longOpt.getOptValue("mode");
 	Chain logFile = longOpt.getOptValue("logfile");
 	Chain serverName = longOpt.getOptValue("server");
@@ -189,7 +185,7 @@ int main(int argc, char **argv)
 	Chain procName = longOpt.getOptValue("proc");
 	Chain tableSet = longOpt.getOptValue("tableset");
 
-	int count = longOpt.getOptValue("count").asInteger();
+	unsigned long long count = longOpt.getOptValue("count").asUnsignedLongLong();
 	int interval = longOpt.getOptValue("interval").asInteger();
 
 	Operation op;    
@@ -205,13 +201,25 @@ int main(int argc, char **argv)
 	{
 	    op=UPDATE_OP;
 	}
+	else if ( mode == Chain("proc") )
+	{
+	    op=PROC_OP;
+	}
 	else if ( mode == Chain("blob") )
 	{
 	    op=BLOB_OP;
 	}
-	else if ( mode == Chain("arbitrary") )
+	else if ( mode == Chain("arb1") )
 	{
-	    op=ARBITRARY_OP;	    
+	    op=ARB1_OP;	    
+	}
+	else if ( mode == Chain("arb2") )
+	{
+	    op=ARB2_OP;	    
+	}
+	else if ( mode == Chain("arb3") )
+	{
+	    op=ARB3_OP;	    
 	}
 	else
 	{
@@ -219,7 +227,6 @@ int main(int argc, char **argv)
 	    exit(1);	    
 	}
 
-
 	CegoDbHandler::ProtocolType protType;
 
 	Chain prot = longOpt.getOptValue("protocol");
@@ -238,49 +245,26 @@ int main(int argc, char **argv)
 	    exit (1);
 	}
 
-       
-	// op=BLOB_OP;		
-	// blobArg=Chain(myOpt.getArg()); 
-
-
-	CegoDbHandler* pSH = 0;
-	CegoModule *pModule = 0;
-	NetHandler* pN = 0; 
+	CegoNet* pCegoNet = 0;
 
 	if ( doSimulate == false )
 	{
-	    Net n (  NETMNG_MSG_BUFLEN, NETMNG_SIZEBUFLEN );
-
-	    if ( serverName.length() > 0 && dataPort > 0 ) 
-		pN = n.connect(serverName, dataPort);
-	    
-	    pModule = new CegoModule(Chain("cgblow.log"));
-	    
-	    if ( pN )
-		pSH = new CegoDbHandler(pN, protType, pModule);
-	    
-	    if ( pSH )
-	    {
-		CegoDbHandler::ResultType res = pSH->requestSession(tableSet, user, password);
-		if ( res != CegoDbHandler::DB_OK )
-		{
-		    Chain msg = pSH->getMsg();
-		    throw Exception(EXLOC, msg);
-		}
-	    }
+	    Chain logMode("notice");
+	    Chain logFile; // no logfile
+	    pCegoNet = new CegoNet( protType, logFile, Chain(""), logMode );	    
+	    pCegoNet->connect(serverName, dataPort, tableSet, user, password);
 	}
-
 		
 	Chain cmd;
 
-	if ( op == INSERT_OP || op == ARBITRARY_OP || op == UPDATE_OP )
+	if ( op == INSERT_OP || op == ARB1_OP || op == ARB2_OP || op == ARB3_OP || op == UPDATE_OP )
 	{
 	    if ( appendMode )
 		cmd = "set append on;";
 	    else
 		cmd = "set append off;";
 	 
-	    dispatchCmd(pSH, cmd);
+	    dispatchCmd(pCegoNet, cmd);
 	    
 	}
 	
@@ -290,16 +274,19 @@ int main(int argc, char **argv)
 	tim1.start();
 	tim2.start();
 
-
 	if ( transactionMode )
 	{
 	    cmd = "start transaction;";	     
-	    dispatchCmd(pSH, cmd);
+	    dispatchCmd(pCegoNet, cmd);
 	}
 
-	for ( int i=1; i<=count; i++ ) 
+	unsigned long long numInsert=0;
+	unsigned long long numUpdate=0;
+	unsigned long long numDelete=0;
+	unsigned long long  numCall=0;
+	
+	for ( unsigned long long i=1; i<=count; i++ ) 
 	{ 
-
 	    
 	    Operation actOp;
 	    if ( op == INSERT_OP )
@@ -312,16 +299,34 @@ int main(int argc, char **argv)
 		actOp = PROC_OP;
 	    else if ( op == BLOB_OP )
 		actOp = BLOB_OP;
-	    else if ( op == ARBITRARY_OP )
+	    else if ( op == ARB1_OP )
 	    {
-		int m = rand() % 4;
-		if ( m == 1 )
+		int m = random() % 2;
+		if ( m == 0 )
 		    actOp = INSERT_OP;
-		else if ( m == 2 )
+		else if ( m == 1 )
 		    actOp = DELETE_OP;
-		else if ( m == 3 )
+	    }
+	    else if ( op == ARB2_OP )
+	    {
+		int m = random() % 3;		
+		if ( m == 0 )
+		    actOp = INSERT_OP;
+		else if ( m == 1 )
+		    actOp = DELETE_OP;
+		else if ( m == 2 )
 		    actOp = UPDATE_OP;
-		else
+	    }
+	    else if ( op == ARB3_OP )
+	    {
+		int m = random() % 4;
+		if ( m == 0 )
+		    actOp = INSERT_OP;
+		else if ( m == 1 )
+		    actOp = DELETE_OP;
+		else if ( m == 2 )
+		    actOp = UPDATE_OP;
+		else if ( m == 3 )
 		    actOp = PROC_OP;
 	    }
 
@@ -329,6 +334,7 @@ int main(int argc, char **argv)
 	    {
 	    case INSERT_OP:
 	    {
+		numInsert++;
 		cmd = "insert into " + tableName + " values ("; 
 		Tokenizer tok(insertSet, Chain(",")); 
 		Chain col; 
@@ -344,7 +350,7 @@ int main(int argc, char **argv)
 			tok.nextToken(s); 
 			tok.nextToken(len); 
 			int l = len.asInteger();
-			cmd += Chain(rand() % l); 
+			cmd += Chain(random() % l); 
 		    } 
 		    else if ( col[0] == 's' )
 		    { 
@@ -353,14 +359,14 @@ int main(int argc, char **argv)
 			Chain s, len; 
 			tok.nextToken(s); 
 			tok.nextToken(len); 
-			int l = rand() % len.asInteger();
+			int l = random() % len.asInteger();
 			if ( l == 0 )
 			    l = len.asInteger();
 			
 			cmd += Chain("'"); 
 			for ( int j=0; j<l; j++ ) 
 			{ 
-			    cmd += Chain((char)(65 + (rand()%26))); 
+			    cmd += Chain((char)(65 + (random()%26))); 
 			} 
 			cmd += Chain("'"); 
 		    }
@@ -378,7 +384,7 @@ int main(int argc, char **argv)
 
 			if ( doSimulate == false )
 			{
-			    pSH->putBlob(tableSet, blob);
+			    pCegoNet->putBlob(blob);
 			
 			    cout << "Blob stored to " << blob.getFileId() << " " << blob.getPageId() << endl;
 			}
@@ -388,6 +394,29 @@ int main(int argc, char **argv)
 			}
 			cmd += Chain("[") + Chain(blob.getFileId()) + Chain(",") + Chain(blob.getPageId()) + Chain("]"); 
 		    }
+		    else if ( col[0] == 'c' )
+		    {
+			Tokenizer tok(col, Chain(":")); 
+
+			Chain s;
+			Chain clobString;
+			tok.nextToken(s);
+			tok.nextToken(clobString);
+			
+			CegoClob clob((char*)s, s.length());
+
+			if ( doSimulate == false )
+			{
+			    pCegoNet->putClob(clob);
+			
+			    cout << "Clob stored to " << clob.getFileId() << " " << clob.getPageId() << endl;
+			}
+			else
+			{
+			    cout << "put clob .." << endl;
+			}
+			cmd += clob.toChain();
+		    }
 		    else
 		    {
 			throw Exception(EXLOC, "Invalid insert format");
@@ -402,7 +431,7 @@ int main(int argc, char **argv)
 	    }
 	    case DELETE_OP:
 	    {
-		
+		numDelete++;
 		cmd = "delete from " + tableName;
 		Tokenizer tok1(deleteCond, Chain(",")); 
 		Chain col; 
@@ -422,18 +451,18 @@ int main(int argc, char **argv)
 		    if ( type[0] == 'i' )
 		    { 
 			int l = arg.asInteger();
-			cmd += Chain(rand() % l);
+			cmd += Chain(random() % l);
 		    }
 		    else if ( type[0] == 's' )
 		    {
-			int l = rand() % arg.asInteger();
+			int l = random() % arg.asInteger();
 			if ( l == 0 )
 			    l = arg.asInteger();
 			
 			cmd += Chain("'"); 
 			for ( int j=0; j<l; j++ ) 
 			{ 
-			    cmd += Chain((char)(65 + (rand()%26))); 
+			    cmd += Chain((char)(65 + (random()%26))); 
 			} 
 			cmd += Chain("'");
 		    }
@@ -452,6 +481,7 @@ int main(int argc, char **argv)
 	    }
 	    case PROC_OP:
 	    {
+		numCall++;
 		cmd = "call " + procName + "("; 
 		Tokenizer tok(procSet, Chain(",")); 
 		Chain col; 
@@ -460,36 +490,33 @@ int main(int argc, char **argv)
 		{ 
 		    // cout << "col is " << col << endl; 
 		    if ( col[0] == 'i' ) 
-		    { 
-			
+		    { 	
 			Tokenizer tok(col, Chain(":")); 
 			Chain s, len; 
 			tok.nextToken(s); 
 			tok.nextToken(len); 
 			int l = len.asInteger();
-			cmd += Chain(rand() % l); 
+			cmd += Chain(random() % l); 
 		    } 
 		    else if ( col[0] == 's' ) 
-		    { 
-			
+		    { 			
 			Tokenizer tok(col, Chain(":")); 
 			Chain s, len; 
 			tok.nextToken(s); 
 			tok.nextToken(len); 
-			int l = rand() % len.asInteger();
+			int l = random() % len.asInteger();
 			if ( l == 0 )
 			    l = len.asInteger();
 			
 			cmd += Chain("'"); 
 			for ( int j=0; j<l; j++ ) 
 			{ 
-			    cmd += Chain((char)(65 + (rand()%26))); 
+			    cmd += Chain((char)(65 + (random()%26))); 
 			} 
 			cmd += Chain("'"); 
 		    } 
 		    else if ( col[0] == 'c' )
 		    {
-
 			Tokenizer tok(col, Chain(":")); 
 			Chain s, arg; 
 			tok.nextToken(s); 
@@ -510,6 +537,7 @@ int main(int argc, char **argv)
 	    }
 	    case UPDATE_OP:
 	    {
+		numUpdate++;
 		
 		cmd = "update " + tableName + " set ";
 		Tokenizer tokA(updateSet, Chain(",")); 
@@ -518,7 +546,6 @@ int main(int argc, char **argv)
 
 		while ( moreToken ) 
 		{
-
 		    Tokenizer updTok(col, Chain(":"));
 		    Chain utype, uattr, ulen; 
 		    
@@ -531,18 +558,18 @@ int main(int argc, char **argv)
 		    if ( utype[0] == 'i' )
 		    { 
 			int l = ulen.asInteger();
-			cmd += Chain(rand() % l);
+			cmd += Chain(random() % l);
 		    }
 		    else if ( utype[0] == 's' )
 		    {
-			int l = rand() % ulen.asInteger();
+			int l = random() % ulen.asInteger();
 			if ( l == 0 )
 			    l = ulen.asInteger();
 			
 			cmd += Chain("'"); 
 			for ( int j=0; j<l; j++ ) 
 			{ 
-			    cmd += Chain((char)(65 + (rand()%26))); 
+			    cmd += Chain((char)(65 + (random()%26))); 
 			} 
 			cmd += Chain("'");
 		    }
@@ -564,7 +591,6 @@ int main(int argc, char **argv)
 		
 		while ( moreToken ) 
 		{
-		
 		    Tokenizer condTok(col, Chain(":"));
 		    Chain wtype, wattr, warg;
 		    
@@ -577,18 +603,18 @@ int main(int argc, char **argv)
 		    if ( wtype[0] == 'i' )
 		    { 
 			int l = warg.asInteger();
-			cmd += Chain(rand() % l);
+			cmd += Chain(random() % l);
 		    }
 		    else if ( wtype[0] == 's' )
 		    {
-			int l = rand() % warg.asInteger();
+			int l = random() % warg.asInteger();
 			if ( l == 0 )
 			    l = warg.asInteger();
 			
 			cmd += Chain("'"); 
 			for ( int j=0; j<l; j++ ) 
 			{ 
-			    cmd += Chain((char)(65 + (rand()%26))); 
+			    cmd += Chain((char)(65 + (random()%26))); 
 			} 
 			cmd += Chain("'");
 		    }
@@ -609,8 +635,7 @@ int main(int argc, char **argv)
 	    }
 	    case BLOB_OP:
 	    {
-
-		cout << "Blob mode .." << endl;
+		// cout << "Blob mode .." << endl;
 
 		Chain blobArg = longOpt.getOptValue("blobop");
 
@@ -629,7 +654,7 @@ int main(int argc, char **argv)
 		    
 		    if ( doSimulate == false )
 		    {
-			pSH->putBlob(tableSet, blob);
+			pCegoNet->putBlob(blob);
 			cout << "Blob stored to " << blob.getFileId() << " " << blob.getPageId() << endl;
 		    }
 		    else
@@ -641,7 +666,7 @@ int main(int argc, char **argv)
 		    Chain blobFile;
 		    tok.nextToken(blobRef);
 		    tok.nextToken(blobFile);
-		    Tokenizer btok(blobRef, Chain(BLOBSEP));
+		    Tokenizer btok(blobRef, Chain(LOBSEP));
 
 		    Chain fstr;
 		    btok.nextToken(fstr);
@@ -651,7 +676,7 @@ int main(int argc, char **argv)
 		    CegoBlob blob(fstr.asInteger(), pstr.asInteger());
 		    
 		    cout << "Getting blob " << fstr << " " << pstr << " to " << blobFile << endl;
-		    pSH->getBlob(tableSet, blob);
+		    pCegoNet->getBlob(blob);
 		    blob.writeBlob(blobFile);
 		    
 		}
@@ -659,7 +684,7 @@ int main(int argc, char **argv)
 		{
 		    Chain blobRef;
 		    tok.nextToken(blobRef);
-		    Tokenizer btok(blobRef, Chain(BLOBSEP));
+		    Tokenizer btok(blobRef, Chain(LOBSEP));
 
 		    Chain fstr;
 		    btok.nextToken(fstr);
@@ -668,19 +693,21 @@ int main(int argc, char **argv)
 		    		    
 		    CegoBlob blob(fstr.asInteger(), pstr.asInteger());
 		    
-		    pSH->delBlob(tableSet, blob);
+		    pCegoNet->deleteBlob(blob);
 		    
 		}
 		break;
 	    }
-	    case ARBITRARY_OP:
+	    case ARB1_OP:
+	    case ARB2_OP:
+	    case ARB3_OP:
 		break;
 	    }
 
 	    if ( actOp != BLOB_OP )
-		dispatchCmd(pSH, cmd);
+		dispatchCmd(pCegoNet, cmd);
 	
-	    if ( pSH )
+	    if ( pCegoNet )
 	    {
 		if ( i % interval == 0)
 		{
@@ -688,20 +715,29 @@ int main(int argc, char **argv)
 		    if ( transactionMode )
 		    {
 			cmd = "commit;";	     
-			dispatchCmd(pSH, cmd);
+			dispatchCmd(pCegoNet, cmd);
 		    }
 		    
 		    tim1.stop();
-		    cout << interval << " operations : " << tim1 << endl;
+		    Chain msg = progid + Chain(" : ") + Chain(numInsert) + Chain(" insert / ")
+			+ Chain(numDelete) + Chain(" delete / ")
+			+ Chain(numUpdate) + Chain(" update / ")
+			+ Chain(numCall) + Chain(" call in ") + tim1.getUsed() + Chain(" sec");
+		    cout << msg << endl;
+		    cout.flush();
+
+		    numInsert=0;
+		    numDelete=0;
+		    numUpdate=0;
+		    numCall=0;
+		    
 		    tim1.start();
 
 		    if ( transactionMode )
 		    {
 			cmd = "start transaction;";	     
-			dispatchCmd(pSH, cmd);
+			dispatchCmd(pCegoNet, cmd);
 		    }
-
-
 		}
 	    }
 	}
@@ -709,29 +745,37 @@ int main(int argc, char **argv)
 	if ( transactionMode )
 	{
 	    cmd = "commit;";	     
-	    dispatchCmd(pSH, cmd);
+	    dispatchCmd(pCegoNet, cmd);
 	}
 		
-	if ( pSH )
+	if ( pCegoNet )
 	{
 	    
 	    tim1.stop();
-	    cout << count << " operations : " << tim1 << endl;
-	    tim2.stop();
-	    cout << "Overall time : " << tim2 << endl;
-	    
-	    pSH->closeSession();	 
-	    delete pSH;
-	}
 
-	if ( pModule )
-	    delete pModule;
-	if ( pN )
-	    delete pN;
-	
+	    if ( numInsert || numDelete || numUpdate || numCall )
+	    {
+		Chain msg = progid + Chain(" : ") + Chain(numInsert) + Chain(" insert / ")
+		    + Chain(numDelete) + Chain(" delete / ")
+		    + Chain(numUpdate) + Chain(" update / ")
+		    + Chain(numCall) + Chain(" call in ") + tim1.getUsed() + Chain(" sec");
+		
+		cout << msg << endl;
+		cout.flush();
+	    }
+	    
+	    tim2.stop();
+	    Chain overall = progid + Chain(" : ") + Chain("Overall time = ") + tim2.getUsed() + Chain(" sec");
+	    cout << overall << endl;
+	    cout.flush();
+	    
+	    pCegoNet->disconnect();	 
+	    delete pCegoNet;
+	}	
     } 
     catch (Exception e) 
-    { 
+    {
+	
 	Chain msg;
 	e.pop(msg);
 	cerr << "CegoBlow Error : " << msg << endl;
@@ -739,65 +783,56 @@ int main(int argc, char **argv)
     }   
 } 
 
-bool dispatchCmd(CegoDbHandler *pSH, Chain cmd)
+bool dispatchCmd(CegoNet *pCegoNet, Chain cmd)
 {
-
-    if ( pSH == 0 )
+    if ( pCegoNet == 0 )
     {
 	cout << cmd << endl;
 	return true;
     }
     else
     {
-	Chain msg;
-	CegoDbHandler::ResultType res;
-	
-	res = pSH->reqQueryOp(cmd);
-	
-	if ( res == CegoDbHandler::DB_OK )
+
+
+	try
 	{
-	    Chain msg =  pSH->getMsg();
-	    
-	    // if ( msg == Chain("Goodbye") )
-	    //    return true;
-	    
-	    // cout << msg << endl;
-	    return true;
+	    unsigned long long affCount = pCegoNet->doQuery(cmd);
 	}
-	else if ( res == CegoDbHandler::DB_DATA )
-	{
-	    ListT<CegoField> schema = pSH->getSchema();
+	catch ( Exception e )
+	{	    
+	    Chain msg;
+	    e.pop(msg);
+	    return false;	
+	}
+	
+    
+	if ( pCegoNet->isFetchable() )
+	{		
+	    ListT<CegoField> schema;
+	    pCegoNet->getSchema(schema);
 	    
-	    CegoField *pF = schema.First();
-	    while ( pF ) 
-	    {
-		cout << pF->getAttrName() << "\t";
-		pF = schema.Next();
-	    }
-	    // cout << endl;
+	    Chain format;
+	    pCegoNet->getFormat(format);
 	    
-	    // print out schema
+	    CegoOutput output(schema, format);
+	    
+	    ListT< ListT<CegoFieldValue> > outData;
+	    
+	    output.headOut();
 	    
 	    ListT<CegoFieldValue> fvl;
-	    while ( pSH->receiveTableData(schema, fvl) )
+	
+	    unsigned long long count=0;
+	    
+	    while ( pCegoNet->fetchData(schema, fvl)  )
 	    {
-		CegoFieldValue *pFV = fvl.First();
-		while ( pFV ) 
-		{
-		    cout << *pFV << "\t";
-		    pFV = fvl.Next();
-		}
-		cout << endl;
+		output.rowOut(fvl);
 		fvl.Empty();
 	    }
-	}
-	else if ( res == CegoDbHandler::DB_ERROR )
-	{
-	    Chain msg = pSH->getMsg();
-	    cout << msg << endl;
-	    cout << "Query failed" << endl;
+	    output.tailOut();
 	    
 	}
-	return false;
+	
+	return true;
     }
 }
