@@ -6,49 +6,37 @@
 //
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2013 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoBufferPool
 // 
-// Description: 
+// Description: The buffer pool management class
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
+// LFC INCLUDES
 #include <lfcbase/Exception.h>
 #include <lfcbase/CommandExecuter.h>
 #include <lfcbase/Datetime.h>
 #include <lfcbase/Sleeper.h>
 
+// CEGO INCLUDES
 #include "CegoBufferPool.h"
 #include "CegoDefs.h"
+#include "CegoXMLdef.h"
 
+// POSIX INCLUDES
 #include <string.h>
 #include <stdlib.h>
 
 #define BUFFERPOOLHEAD (((sizeof(BufferPoolHead)-1)/BUPMNG_ALIGNMENT)+1)*BUPMNG_ALIGNMENT
 #define BUFFERHEAD (((sizeof(BufferHead)-1)/BUPMNG_ALIGNMENT)+1)*BUPMNG_ALIGNMENT
 
-
-CegoBufferPool::CegoBufferPool(const Chain& xmlDef, const Chain& logFile) : CegoLogManager(xmlDef, logFile)
+CegoBufferPool::CegoBufferPool(const Chain& xmlDef, const Chain& logFile, const Chain& progName) : CegoLogManager(xmlDef, logFile, progName)
 {    
     _pBufPool = 0;
     _numDiskRead=0;
@@ -58,7 +46,8 @@ CegoBufferPool::CegoBufferPool(const Chain& xmlDef, const Chain& logFile) : Cego
     _avgWriteDelay=0;
 
     Datetime ts;
-    _statStart = ts.asInt(); 
+    _statStart = ts.asInt();
+    _poolStart = ts.asInt();
     _modId = getModId("CegoBufferPool");
 }
 
@@ -66,77 +55,96 @@ CegoBufferPool::~CegoBufferPool()
 {
 }
 
-void CegoBufferPool::initPool(long numPages)
+void CegoBufferPool::initPool(unsigned long long numSegment, unsigned long long numPages)
 {
 
 #ifdef DEBUG
     log(_modId, Logger::DEBUG, Chain("Reading xml def ..."));
 #endif
 
-    xml2Doc();
-
-    initDoc();
-    
     Chain dbName = getDbName();
     int pageSize = getPageSize();
     _maxFixTries = getMaxFixTries();
-
+    // _hashkey = getBufferPoolHashKey();
+    
     _dbName = dbName;
+    _numSegment = numSegment;
     _numPages = numPages;
     _pageSize = pageSize;
 
     if (_pBufPool == 0)
     {
 	
-	log(_modId, Logger::NOTICE, Chain("Allocating ") + Chain(_numPages) + Chain(" buffer pool pages ..."));
-
-	_pBufPool = malloc(_numPages * (BUFFERHEAD + _pageSize) + BUFFERPOOLHEAD);
+	log(_modId, Logger::NOTICE, Chain("Allocating ") + Chain(_numSegment) + Chain(" buffer pool segments ( each ") +  Chain(_numPages) + Chain(" pages ) ..."));
 
+	
+	_pBufPool = (void**)malloc(_numSegment * sizeof(void*));
 	if ( _pBufPool == NULL )
 	{
 	    throw Exception(EXLOC, "Cannot initialize pool");
 	}
 
-	log(_modId, Logger::NOTICE, Chain("Initializing buffer pool pages ..."));
 	
-	void *base = (void*)_pBufPool;
-	void *ptr = (void*)_pBufPool;
-
-	BufferPoolHead bph;
-	bph.numPages=_numPages;
-	
-	memcpy(base, &bph, sizeof(BufferPoolHead));
-	
-	ptr = (void*)(BUFFERPOOLHEAD + (long)base);	
-
-	BufferHead ibh;
-	ibh.isOccupied = NOT_OCCUPIED;
-	ibh.isDirty = 0;
-	ibh.numFixes = 0;
-	
-	for (int i = 0; i<_numPages; i++)
+	for ( int i=0; i<_numSegment; i++)
 	{
-	    memcpy(ptr, &ibh,  sizeof(BufferHead) );
-	    ptr = (void*)((long)ptr + BUFFERHEAD + _pageSize);
-	}
+	    _pBufPool[i] = malloc(_numPages * (BUFFERHEAD + _pageSize) + BUFFERPOOLHEAD);
+
+	    if ( _pBufPool[i] == NULL )
+	    {
+		throw Exception(EXLOC, "Cannot initialize pool");
+	    }
+	    
+	    log(_modId, Logger::NOTICE, Chain("Initializing buffer pool pages for segment ") + Chain(i) + Chain(" ..."));
 	
-	log(_modId, Logger::NOTICE, Chain("Pool started"));
+	    void *base = (void*)_pBufPool[i];
+	    void *ptr = (void*)_pBufPool[i];
+	    
+	    BufferPoolHead bph;
+	    bph.numPages=_numPages;
+	    
+	    memcpy(base, &bph, sizeof(BufferPoolHead));
+	    
+	    ptr = (void*)(BUFFERPOOLHEAD + (long long)base);	
+	    
+	    BufferHead ibh;
+	    ibh.isOccupied = NOT_OCCUPIED;
+	    ibh.isDirty = 0;
+	    ibh.numFixes = 0;
+	    ibh.tabSetId = 0;
+	    ibh.fileId = 0;
+	    ibh.pageId = 0;
+	    ibh.fixStat = 0;
+	    ibh.numUsage = 0;
+	    
+	    for (int i = 0; i<_numPages; i++)
+	    {
+		memcpy(ptr, &ibh,  sizeof(BufferHead) );
+		ptr = (void*)((long long)ptr + BUFFERHEAD + _pageSize);
+	    }
+	    
+	    log(_modId, Logger::NOTICE, Chain("Buffer pool initialized"));
+	}
     }
     else
     {
 	throw Exception(EXLOC, "Buffer pool already created");
-    }
-    
+    }    
 }
 
 void CegoBufferPool::removePool()
 {
     if (_pBufPool)
     {
-	log(_modId, Logger::NOTICE, Chain("Stopping pool ..."));
+	log(_modId, Logger::NOTICE, Chain("Removing pool ..."));
+	
+	for ( int i=0; i<_numSegment; i++)
+	{
+	    free ( _pBufPool[i] );	   
+	}
 	free ( _pBufPool );
+	
 	_pBufPool = 0;
-	log(_modId, Logger::NOTICE, Chain("Pool stopped"));
+	log(_modId, Logger::NOTICE, Chain("Pool removed"));
     }
 }
 
@@ -145,9 +153,34 @@ Chain CegoBufferPool::getDBName()
     return _dbName;
 }
 
+int CegoBufferPool::calcHash(int fileId, int pageId)
+{
+    // we have to cast to long, since for large fileid values, we might exceed integer range
+    // unsigned long long s = (unsigned long long)(fileId+1) * (unsigned long long)_hashkey + (unsigned long long)(pageId+1);    
+
+    unsigned long long s = getRegPageOffset(fileId) + (unsigned long long)(pageId+1);    
+    unsigned long long d = (unsigned long long)calcSegment(fileId, pageId) * (unsigned long long)_numPages;
+    
+    if ( s > d )
+	s = s - d;
+    
+    return (int)( s % (unsigned long long)_numPages);
+}
+
+int CegoBufferPool::calcSegment(int fileId, int pageId)
+{
+    // we have to cast to long, since for large fileid values, we might exceed integer range
+    // unsigned long long s = (unsigned long long)(fileId+1) * (unsigned long long)_hashkey + (unsigned long long)(pageId+1);
+
+    unsigned long long s = getRegPageOffset(fileId) + (unsigned long long)(pageId+1);    
+    // for a better usage of the whole buffer pool, we use segments in a linear way
+    s = s / _numPages;
+
+    return (int)( s % (unsigned long long)_numSegment);
+}
+
 void CegoBufferPool::bufferFix(CegoBufferPage &bp, int tabSetId, int fileId, int pageId, FixMode m, CegoLockHandler *pLockHandle, int numTry)
 {
-
     _fixCount++;
 
     if ( _fixCount % BUPMNG_STATSPERIOD == 0 )
@@ -160,11 +193,13 @@ void CegoBufferPool::bufferFix(CegoBufferPage &bp, int tabSetId, int fileId, int
 	throw Exception(EXLOC, "No valid bufferpool" );
     }
 
-    void *base = (void*)_pBufPool;
+    int segid = calcSegment(fileId, pageId);
 
-    int hashId = ( (fileId+1) * (pageId+1) ) % _numPages;
+    void *base = (void*)_pBufPool[segid];
 
-    void* bufPtr = (void*)((long)base + BUFFERPOOLHEAD + hashId * ( BUFFERHEAD + _pageSize ));
+    int hashId = calcHash(fileId, pageId);
+
+    void* bufPtr = (void*)((long long)base + (long long)BUFFERPOOLHEAD + (long long )hashId * (long long)( BUFFERHEAD + _pageSize ));
     
     bool isFixed = false;
     
@@ -172,19 +207,17 @@ void CegoBufferPool::bufferFix(CegoBufferPage &bp, int tabSetId, int fileId, int
     
     // Step I : searching for possible slot
 
-    
-    while ( ! isFixed && numTries < _maxFixTries )
+    while ( isFixed == false && numTries < _maxFixTries )
     {
-
 	pLockHandle->lockBufferPool(hashId, CegoLockHandler::WRITE);
 
 	BufferHead bh;
 	memcpy(&bh, bufPtr, sizeof(BufferHead));
-
+	
 	int minFixStat = -1;
 
 	if ( bh.isOccupied != NOT_OCCUPIED && bh.pageId == pageId && bh.fileId == fileId)
-	{
+	{	    
 	    isFixed = true;
 
 	    if ( m == CegoBufferPool::PERSISTENT )
@@ -198,6 +231,7 @@ void CegoBufferPool::bufferFix(CegoBufferPage &bp, int tabSetId, int fileId, int
 	    
 	    bh.numFixes = bh.numFixes + 1;
 	    bh.fixStat++;
+	    bh.numUsage++;
 
 	    memcpy(bufPtr, &bh, sizeof(BufferHead));
 	    
@@ -206,16 +240,15 @@ void CegoBufferPool::bufferFix(CegoBufferPage &bp, int tabSetId, int fileId, int
 	}
 	else 
 	{
-
 	    pLockHandle->unlockBufferPool(hashId);
 
-	    hashId =  ( hashId + 1 ) % ( _numPages - 1 );
-	    bufPtr = (void*)( hashId * ( BUFFERHEAD + _pageSize ) + (long)base + BUFFERPOOLHEAD );
+	    hashId =  ( hashId + 1 ) % _numPages;
+	    bufPtr = (void*)( (long long)hashId * (long long)( BUFFERHEAD + _pageSize ) + (long long)base + (long long)BUFFERPOOLHEAD );
 	    numTries++;
 	}
 	
     }
-
+    
     // Step II : page is not in buffercache yet, searching free and not occupied slot 
 
     void* minFixStatBufPtr = 0;
@@ -223,16 +256,13 @@ void CegoBufferPool::bufferFix(CegoBufferPage &bp, int tabSetId, int fileId, int
 
     if ( ! isFixed )
     {
-	numTries = 0;    
-	
-	hashId = ( (fileId+1 ) * (pageId+1) ) % _numPages;
-	bufPtr = (void*)((long)base + BUFFERPOOLHEAD + hashId * ( BUFFERHEAD + _pageSize ));
-
+	numTries = 0;
+	hashId = calcHash(fileId, pageId);
+	bufPtr = (void*)((long long)base + (long long)BUFFERPOOLHEAD + (long long)hashId * (long long)( BUFFERHEAD + _pageSize ));
     }
     
     while ( ! isFixed && numTries < _maxFixTries )
     {
-
 	try
 	{
 	    pLockHandle->lockBufferPool(hashId, CegoLockHandler::WRITE);
@@ -241,7 +271,7 @@ void CegoBufferPool::bufferFix(CegoBufferPage &bp, int tabSetId, int fileId, int
 	{
 	    if ( minHashId != -1 )
 		pLockHandle->unlockBufferPool(minHashId);		
-	    throw Exception(EXLOC, "Cannot fix buffer", e);
+	    throw e;
 	}
 	
 	// cout << "Trying to fix with try " << numTries << endl;
@@ -266,25 +296,26 @@ void CegoBufferPool::bufferFix(CegoBufferPage &bp, int tabSetId, int fileId, int
 	    
 	    try
 	    {		
-		
 		_numDiskRead++;
 		_diskReadTimer.start();
-		readPage(fileId, pageId, bh.tabSetId, bh.fixStat, (char*)((long)bufPtr+BUFFERHEAD), pLockHandle);
+		readPage(fileId, pageId, bh.tabSetId, bh.fixStat, (char*)((long long)bufPtr+BUFFERHEAD), pLockHandle);
 		_diskReadTimer.stop();
-		_avgReadDelay = ( _diskReadTimer.getSum() /  (_numDiskRead+(long)1)  ) / 1000;
-
+		_avgReadDelay = ( _diskReadTimer.getSum() /  (_numDiskRead+(unsigned long long)1)  ) / 1000;
 	    }
 	    catch ( Exception e )
 	    {
 		pLockHandle->unlockBufferPool(hashId);
 		if ( minHashId != -1 )
 		    pLockHandle->unlockBufferPool(minHashId);		
-		throw Exception(EXLOC, "Cannot fix buffer", e);
+		throw e;
 	    }
 
 	    bh.fixStat++;
-	    memcpy(bufPtr, &bh, sizeof(BufferHead));	
+
+	    bh.numUsage++;
 	    
+	    memcpy(bufPtr, &bh, sizeof(BufferHead));	
+
 	    isFixed = true;
 
 	    pLockHandle->unlockBufferPool(hashId);
@@ -320,17 +351,17 @@ void CegoBufferPool::bufferFix(CegoBufferPage &bp, int tabSetId, int fileId, int
 		catch ( Exception e )
 		{
 		    pLockHandle->unlockBufferPool(hashId);
-		    throw Exception(EXLOC, "Cannot fix buffer", e);
+		    throw e;
 		}
 	    }
 	    
 	    pLockHandle->unlockBufferPool(hashId);
 	    
-	    hashId =  ( hashId + 1 ) % ( _numPages - 1 );
-	    bufPtr = (void*)( hashId * ( BUFFERHEAD + _pageSize ) + (long)base + BUFFERPOOLHEAD );
+	    hashId =  ( hashId + 1 ) % _numPages;
+
+	    bufPtr = (void*)( (long long)hashId * (long long)( BUFFERHEAD + _pageSize ) + (long long)base + (long long)BUFFERPOOLHEAD );
 	    numTries++;
-	}
-	
+	}	
     }
 
     // Step III : page is not in buffercache yet and all slots are occupied, using the occupied slot
@@ -347,7 +378,7 @@ void CegoBufferPool::bufferFix(CegoBufferPage &bp, int tabSetId, int fileId, int
 	BufferHead bh;
 	memcpy(&bh, bufPtr, sizeof(BufferHead));
 	
-	if ( bh.isOccupied == WRITE_ON_DIRTY &&  bh.isDirty == 1 )
+	if ( bh.isOccupied == WRITE_ON_DIRTY && bh.isDirty == 1 )
 	{
 	    _numDiskWrite++;
 	    
@@ -358,20 +389,20 @@ void CegoBufferPool::bufferFix(CegoBufferPage &bp, int tabSetId, int fileId, int
 	    // cout << "---- >>>> Async write of page [" << bh.fileId << "," << bh.pageId << "]" << endl;
 
 	    _diskWriteTimer.start();
-	    writePage(bh.fileId, bh.pageId, bh.fixStat, (char*)((long)bufPtr + BUFFERHEAD), pLockHandle);
+	    writePage(bh.fileId, bh.pageId, bh.fixStat, (char*)((long long)bufPtr + BUFFERHEAD), pLockHandle);
 	    _diskWriteTimer.stop();
 
-	    _avgWriteDelay = ( _diskWriteTimer.getSum() /  (_numDiskWrite+(long)1)  ) / 1000;
+	    _avgWriteDelay = ( _diskWriteTimer.getSum() /  (_numDiskWrite+(unsigned long long)1)  ) / 1000;
 
 	}
 	
-	// cout << "Using occupied page with fileId " << bh.fileId << " pageId " << bh.pageId << " numFixes " << bh.numFixes << endl;
+	// cout << "Using occupied page ( type " << bh.isOccupied << " ) with fileId " << bh.fileId << " pageId " << bh.pageId << " numFixes " << bh.numFixes << endl;
 
 	if ( m == CegoBufferPool::PERSISTENT )
 	    bh.isOccupied = PERSISTENT_OCCUPIED;
-	else if (m == CegoBufferPool::SYNC )
+	else if ( m == CegoBufferPool::SYNC )
 	    bh.isOccupied = WRITE_ON_SYNC;
-	else if (m == CegoBufferPool::NOSYNC )
+	else if ( m == CegoBufferPool::NOSYNC )
 	    bh.isOccupied = WRITE_ON_DIRTY;
 
 	bh.pageId = pageId;
@@ -383,24 +414,26 @@ void CegoBufferPool::bufferFix(CegoBufferPage &bp, int tabSetId, int fileId, int
 	{
 	    _numDiskRead++;
 	    _diskReadTimer.start();
-	    readPage(fileId, pageId, bh.tabSetId, bh.fixStat, (char*)((long)bufPtr+BUFFERHEAD), pLockHandle);
+	    readPage(fileId, pageId, bh.tabSetId, bh.fixStat, (char*)((long long)bufPtr+BUFFERHEAD), pLockHandle);
 	    _diskReadTimer.stop();
 
-	    _avgReadDelay = ( _diskReadTimer.getSum() /  (_numDiskRead+(long)1) ) / 1000;
+	    _avgReadDelay = ( _diskReadTimer.getSum() /  (_numDiskRead+(unsigned long long)1) ) / 1000;
 
 	}
 	catch ( Exception e )
 	{
 	    pLockHandle->unlockBufferPool(minHashId);
-	    throw Exception(EXLOC, "Cannot fix buffer", e);
+	    throw e;
 	}
 	bh.fixStat++;
+	bh.numUsage++;
+	
 	memcpy(bufPtr, &bh, sizeof(BufferHead));
 	
 	pLockHandle->unlockBufferPool(minHashId);
 
 	isFixed = true;
-	
+
     }
 
     if ( ! isFixed )
@@ -410,10 +443,8 @@ void CegoBufferPool::bufferFix(CegoBufferPage &bp, int tabSetId, int fileId, int
 
 	writeCheckPoint(tabSetId, true, Chain(""), 0,  pLockHandle);
 
-	// try again
-	
-	// if ( numTry > 5 )
-	//     throw Exception(EXLOC, "Cannot fix buffer");
+	if ( numTry > BUPMNG_FIXLOOP  )
+	    throw Exception(EXLOC, "No more buffers available");
 
 	numTry++;
 	return bufferFix(bp, tabSetId, fileId, pageId, m, pLockHandle, numTry);
@@ -421,21 +452,15 @@ void CegoBufferPool::bufferFix(CegoBufferPage &bp, int tabSetId, int fileId, int
     }
 
     bp.setPageSize(_pageSize);
-    bp.setPagePtr((char*)((long)bufPtr+BUFFERHEAD));
-    bp.setPageHead((CegoBufferPage::PageHead*)((long)bufPtr+BUFFERHEAD));
-    
-    // CegoBufferPage* pBP = new CegoBufferPage((void*)((long)bufPtr+BUFFERHEAD), _pageSize);
+    bp.setPagePtr((char*)((long long)bufPtr+BUFFERHEAD));
+    bp.setPageHead((CegoBufferPage::PageHead*)((long long)bufPtr+BUFFERHEAD));
     bp.setFileId(fileId);
     bp.setPageId(pageId);
     bp.setFixed(true);
-
 }
 
-
 void CegoBufferPool::emptyFix(CegoBufferPage &bp, int tabSetId, FixMode m, CegoFileHandler::FileType ft, CegoLockHandler *pLockHandle, bool doAppend)
 {
-
-
     if (_pBufPool == 0)
     {
 	throw Exception(EXLOC, "No valid bufferpool" );
@@ -445,15 +470,17 @@ void CegoBufferPool::emptyFix(CegoBufferPage &bp, int tabSetId, FixMode m, CegoF
     
     unsigned* fbm;
     int fbmSize=0;
-    
+
     allocatePage(tabSetId, ft, fileId, pageId, pLockHandle, fbm, fbmSize, doAppend);
     
+    // cout << "Allocated page " << fileId << "/" << pageId << endl;
+
     if ( fbmSize > 0 )
     {
 	logBM(tabSetId, fileId, fbm, fbmSize);
 	delete fbm;
     }
-    
+
     bufferFix(bp, tabSetId, fileId, pageId, m, pLockHandle);
     bp.initPage(CegoBufferPage::TABLE);
     bp.setFixed(true);
@@ -461,18 +488,19 @@ void CegoBufferPool::emptyFix(CegoBufferPage &bp, int tabSetId, FixMode m, CegoF
 
 void CegoBufferPool::bufferUnfix(CegoBufferPage &bp, bool isDirty, CegoLockHandler *pLockHandle)
 {
-
+ 
     // cout << "BufferUnfix of page " << bp.getFileId() << "," << bp.getPageId() << endl;
     
-
     if (_pBufPool == 0)
     {
 	throw Exception(EXLOC, "No valid bufferpool" );
     }
+
+    int segid = calcSegment(bp.getFileId(), bp.getPageId());
     
-    void *base = (void*)_pBufPool;
+    void *base = (void*)_pBufPool[segid];
     char* bufPtr = (char*)bp.getPagePtr();
-    int hashId = ( (long)bufPtr - (long)base - BUFFERPOOLHEAD ) / (  BUFFERHEAD + _pageSize );
+    int hashId = ( (long long)bufPtr - (long long)base - BUFFERPOOLHEAD ) / (  BUFFERHEAD + _pageSize );
     
     bufPtr = bufPtr - BUFFERHEAD;
     
@@ -484,9 +512,9 @@ void CegoBufferPool::bufferUnfix(CegoBufferPage &bp, bool isDirty, CegoLockHandl
     
     // cout << "NumFixes = " << bh.numFixes << endl;
     
-    if (bh.numFixes > 0 )
+    if ( bh.numFixes > 0 )
     {
-	if (isDirty)
+	if ( isDirty )
 	{
 	    bh.isDirty = 1;
 	}
@@ -504,7 +532,6 @@ void CegoBufferPool::bufferUnfix(CegoBufferPage &bp, bool isDirty, CegoLockHandl
     pLockHandle->unlockBufferPool(hashId);
 }    
 
-
 void CegoBufferPool::bufferRelease(CegoBufferPage &bp, CegoLockHandler *pLockHandle)
 {
 
@@ -518,7 +545,7 @@ void CegoBufferPool::bufferRelease(CegoBufferPage &bp, CegoLockHandler *pLockHan
     
     BufferHead bh;
     
-    int hashId = ( (bp.getFileId()+1) * (bp.getPageId()+1) ) % _numPages;
+    int hashId = calcHash(bp.getFileId(), bp.getPageId());
     
     pLockHandle->lockBufferPool(hashId, CegoLockHandler::WRITE);
     
@@ -528,12 +555,18 @@ void CegoBufferPool::bufferRelease(CegoBufferPage &bp, CegoLockHandler *pLockHan
 	
 	bh.isOccupied = NOT_OCCUPIED;
 	bh.numFixes=0;
-	
+	bh.isDirty = 0;
+	bh.tabSetId = 0;
+	bh.fileId = 0;
+	bh.pageId = 0;
+	bh.fixStat = 0;
+	bh.numUsage = 0;
+	       
 	memcpy(bufPtr, &bh, sizeof(BufferHead));
 	
 	int fbmSize=0;
 	unsigned* fbm;
-	
+
 	releasePage(bp.getFileId(), bp.getPageId(), pLockHandle, fbm, fbmSize);
 	
 	if ( fbmSize > 0 )
@@ -558,7 +591,7 @@ int CegoBufferPool::writeCheckPoint(int tabSetId, bool switchLog, const Chain& e
     
     pLockHandle->lockBufferPool();
     
-    int lsn;
+    unsigned long long lsn;
     try
     {
 	
@@ -567,19 +600,23 @@ int CegoBufferPool::writeCheckPoint(int tabSetId, bool switchLog, const Chain& e
 	    throw Exception(EXLOC, "No valid bufferpool" );
 	}
 	
-	
 	if ( escCmd != Chain("") )
 	{
 	    log(_modId, Logger::NOTICE, Chain("Executing escape command <") + escCmd + Chain(">"));
 	    
-
 	    char *pShell = getenv(CGEXESHELLVARNAME);
+
+	    Chain shellCmd;
 	    if ( pShell == NULL )
 	    {
-		throw Exception(EXLOC, "Execution shell not set");  		    
+		shellCmd = Chain(CGSTDEXESHELL);
+	    }
+	    else
+	    {
+		shellCmd = Chain(pShell);
 	    }
 
-	    CommandExecuter cmdExe(pShell);
+	    CommandExecuter cmdExe(shellCmd);
 	    
 	    try 
 	    {
@@ -614,91 +651,99 @@ int CegoBufferPool::writeCheckPoint(int tabSetId, bool switchLog, const Chain& e
 	    lsn--;
 
 	setCommittedLSN(tabSetId, lsn);
-	
+
+	Chain currentState = getTableSetRunState(tabSetId);
+	setTableSetRunState(tabSetId, XML_CHECKPOINT_VALUE);
+
+	// sync control information to xml database file
 	doc2Xml();
-
-
-	void *bufPtr = (void*)( (long)_pBufPool + BUFFERPOOLHEAD );
 	
-	int pageCount = 0;
-	
-	while ( pageCount < _numPages )
+	for ( int i=0; i<_numSegment; i++)
 	{
-	    BufferHead bh;
-	    memcpy(&bh, bufPtr, sizeof(BufferHead));
+	    void *bufPtr = (void*)( (long long)_pBufPool[i] + BUFFERPOOLHEAD );
 	    
-	    if (bh.isDirty != 0 && bh.tabSetId == tabSetId )
+	    int pageCount = 0;
+	    
+	    while ( pageCount < _numPages )
 	    {
-		
-		if ( isBackup(bh.fileId) )
-		{
-		    if (  isMarked(bh.fileId, bh.pageId) == false )
+		BufferHead bh;
+		memcpy(&bh, bufPtr, sizeof(BufferHead));
+		       
+		if ( bh.isOccupied != NOT_OCCUPIED && bh.isDirty != 0 && bh.tabSetId == tabSetId )
+		{		    
+		    if ( isBackup(bh.fileId) )
 		    {
-			
+			if (  isMarked(bh.fileId, bh.pageId) == false )
+			{	    
 #ifdef DEBUG
-			log(_modId, Logger::DEBUG, Chain("Reading page (") + Chain(bh.fileId) + Chain(",") + Chain(bh.pageId) + Chain(") to log ..."));
+			    log(_modId, Logger::DEBUG, Chain("Reading page (") + Chain(bh.fileId) + Chain(",") + Chain(bh.pageId) + Chain(") to log ..."));
 #endif	
-			int ts;
-			int fixStat;
-			char *pageData = new char[_pageSize];
-			readPage(bh.fileId, bh.pageId, ts, fixStat, pageData, pLockHandle);
-			
-			CegoLogRecord lr;
-			lr.setAction(CegoLogRecord::LOGREC_BUPAGE);
-			lr.setData(pageData);
-			lr.setDataLen(_pageSize);
-			lr.setFileId(bh.fileId);
-			lr.setPageId(bh.pageId);
-			
-			if (  logAction(tabSetId, lr) == CegoLogManager::LOG_FULL )
-			{
+			    int ts;
+			    int fixStat;
+			    char *pageData = new char[_pageSize];
+			    readPage(bh.fileId, bh.pageId, ts, fixStat, pageData, pLockHandle);
 			    
-			    int gid;
-			    Chain logFile;
-			    int logSize = 0 ;
+			    CegoLogRecord lr;
+			    lr.setAction(CegoLogRecord::LOGREC_BUPAGE);
+			    lr.setData(pageData);
+			    lr.setDataLen(_pageSize);
+			    lr.setFileId(bh.fileId);
+			    lr.setPageId(bh.pageId);
 			    
+			    if (  logAction(tabSetId, lr) == CegoLogManager::LOG_FULL )
+			    {
+				
+				int gid;
+				Chain logFile;
+				int logSize = 0 ;
+				
 #ifdef DEBUG
-			    log(_modId, Logger::DEBUG, Chain("Switching logFiles ..."));
+				log(_modId, Logger::DEBUG, Chain("Switching logFiles ..."));
 #endif	    
-			    
-			    while ( switchLogFile(tabSetId) == false )
-			    {
-				log(_modId, Logger::NOTICE, Chain("Logfile for tabSetId ") + Chain(tabSetId) + Chain(" still active, switch failed"));
 				
-				Sleeper s;
-				s.secSleep(LOGMNG_LOGSWITCH_WAIT_DELAY);
+				while ( switchLogFile(tabSetId) == false )
+				{
+				    log(_modId, Logger::NOTICE, Chain("Logfile for tabSetId ") + Chain(tabSetId) + Chain(" still active, switch failed"));
+				    
+				    Sleeper s;
+				    s.secSleep(LOGMNG_LOGSWITCH_WAIT_DELAY);
+				    
+				}
 				
+				if ( logAction(tabSetId, lr) != CegoLogManager::LOG_SUCCESS )
+				{
+				    throw Exception(EXLOC, "Cannot write to log");
+				}	    		
 			    }
-			    
-			    if ( logAction(tabSetId, lr) != CegoLogManager::LOG_SUCCESS )
-			    {
-				throw Exception(EXLOC, "Cannot write to log");
-			    }	    		
 			}
 		    }
+		    
+		    _numDiskWrite++;
+		    _diskWriteTimer.start();
+		    writePage(bh.fileId, bh.pageId, bh.fixStat, (char*)((long long)bufPtr + BUFFERHEAD), pLockHandle);
+		    _diskWriteTimer.stop();
+		    
+		    _avgWriteDelay = ( _diskWriteTimer.getSum() /  (_numDiskWrite+(unsigned long long)1) ) / 1000;
+		        
+		    bh.isDirty = 0;
+		    memcpy(bufPtr, &bh, sizeof(BufferHead));	    
 		}
-		
-		_numDiskWrite++;
-		_diskWriteTimer.start();
-		writePage(bh.fileId, bh.pageId, bh.fixStat, (char*)((long)bufPtr + BUFFERHEAD), pLockHandle);
-		_diskWriteTimer.stop();
-		
-		_avgWriteDelay = ( _diskWriteTimer.getSum() /  (_numDiskWrite+(long)1) ) / 1000;
-		
-		
-		bh.isDirty = 0;
-		memcpy(bufPtr, &bh, sizeof(BufferHead));	    
+		bufPtr = (void*)((long long)bufPtr + (long long)(BUFFERHEAD + _pageSize));
+		pageCount++;
 	    }
-	    bufPtr = (void*)((long)bufPtr + (long)(BUFFERHEAD + _pageSize));
-	    pageCount++;
 	}
+	// set back tableset runstate to saved value
+	setTableSetRunState(tabSetId, currentState);
+	// sync control information to xml database file
+	doc2Xml();
+	
     }
     catch ( Exception e )
     {
 	pLockHandle->unlockBufferPool();
 	throw e;
     }
-    
+
     pLockHandle->unlockBufferPool();    
     return lsn;
 }
@@ -710,60 +755,70 @@ void CegoBufferPool::writeAndRemoveTabSet(int tabSetId, CegoLockHandler *pLockHa
     {
 	throw Exception(EXLOC, "No valid bufferpool" );
     }
-    
-    void *bufPtr = (void*)( (long)_pBufPool + BUFFERPOOLHEAD );
-    
-    int pageCount = 0;
-    
-    while ( pageCount < _numPages )
+
+    for ( int i=0; i<_numSegment; i++)
     {
-	BufferHead bh;
-	memcpy(&bh, bufPtr, sizeof(BufferHead));
+	void *bufPtr = (void*)( (long long)_pBufPool[i] + BUFFERPOOLHEAD );
 	
-	if ( bh.tabSetId == tabSetId )
+	int pageCount = 0;
+	
+	while ( pageCount < _numPages )
 	{
+	    BufferHead bh;
+	    memcpy(&bh, bufPtr, sizeof(BufferHead));
 	    
-	    
-	    if ( bh.isDirty != 0 )
+	    if ( bh.tabSetId == tabSetId )
 	    {
-		_numDiskWrite++;
 		
-		_diskWriteTimer.start();
-		writePage(bh.fileId, bh.pageId, bh.fixStat, (char*)((long)bufPtr + BUFFERHEAD), pLockHandle);
-		_diskWriteTimer.stop();
 		
-		_avgWriteDelay = ( _diskWriteTimer.getSum() /  (_numDiskWrite+(long)1) ) / 1000;
+		if ( bh.isOccupied != NOT_OCCUPIED && bh.isDirty != 0 )
+		{
+		    _numDiskWrite++;
+		    
+		    _diskWriteTimer.start();
+		    writePage(bh.fileId, bh.pageId, bh.fixStat, (char*)((long long)bufPtr + BUFFERHEAD), pLockHandle);
+		    _diskWriteTimer.stop();
+		    
+		    _avgWriteDelay = ( _diskWriteTimer.getSum() /  (_numDiskWrite+(unsigned long long)1) ) / 1000;
+		}
+		
+		bh.isOccupied = NOT_OCCUPIED;
+		bh.numFixes=0;
+		
+		memcpy(bufPtr, &bh, sizeof(BufferHead));
+		
 	    }
-	    
-	    bh.isOccupied = NOT_OCCUPIED;
-	    bh.numFixes=0;
-	    
-	    memcpy(bufPtr, &bh, sizeof(BufferHead));
-	    
+	    bufPtr = (void*)((long long)bufPtr + (long long)(BUFFERHEAD + _pageSize));
+	    pageCount++;
 	}
-	bufPtr = (void*)((long)bufPtr + (long)(BUFFERHEAD + _pageSize));
-	pageCount++;
-    }
-    
+    }    
     releaseFiles(tabSetId);
 }
 
+int CegoBufferPool::uptime() const
+{
+    Datetime ts;
+    return ts.asInt() - _poolStart;
+}
+
 void CegoBufferPool::poolInfo(int& pageSize, 
-			      long& numTotal, 
-			      long& numUsed, 
-			      long& numFree, 
-			      long& numDirty, 
-			      long& numFixes, 
-			      long& numPersistent, 
-			      long& numNoSync, 
-			      long& numDiskRead, 
-			      long& numDiskWrite,
-			      long& hitRate,
-			      long& readDelay,
-			      long& writeDelay,
-			      long& curFixCount,
-			      long& maxFixCount,
-			      int& statStart)
+			      unsigned long long& numTotal, 
+			      unsigned long long& numUsed, 
+			      unsigned long long& numFree, 
+			      unsigned long long& numDirty, 
+			      unsigned long long& numFixes, 
+			      unsigned long long& numPersistent, 
+			      unsigned long long& numNoSync, 
+			      unsigned long long& numDiskRead, 
+			      unsigned long long& numDiskWrite,
+			      double& hitRate,
+			      double& spreadRate,
+			      unsigned long long& readDelay,
+			      unsigned long long& writeDelay,
+			      unsigned long long& curFixCount,
+			      unsigned long long& maxFixCount,
+			      int& statStart,
+                              int& uptime) const
 {
 
     if (_pBufPool == 0)
@@ -772,100 +827,153 @@ void CegoBufferPool::poolInfo(int& pageSize,
     }
     
     pageSize = _pageSize;
-    numTotal = _numPages;
+    numTotal = _numPages * _numSegment;
     numUsed=0;
     numFree=0;
     numFixes=0;
     numDirty=0;
     numPersistent=0;
     numNoSync=0;
+
+    unsigned long long numUsage = 0;
     
-    void* bufPtr = (void*)( (long)_pBufPool + BUFFERPOOLHEAD);
-    
-    long pageCount = 0;
-    
-    while ( pageCount < _numPages )
-    {    
-	BufferHead bh;
-	memcpy(&bh, bufPtr, sizeof(BufferHead));
+    for ( int i=0; i<_numSegment; i++)
+    {
+	void *bufPtr = (void*)( (long long)_pBufPool[i] + BUFFERPOOLHEAD );
 	
-	numFixes += (long)bh.numFixes;
-	numDirty += (long)bh.isDirty;
+	unsigned long long pageCount = 0;
+
+	unsigned long long numSegPageUsed = 0;
 	
-	if (bh.isOccupied == NOT_OCCUPIED)
-	    numFree++;
-	else if (bh.isOccupied == WRITE_ON_SYNC)
-	{
-	    numUsed++;
+	while ( pageCount < _numPages )
+	{    
+	    BufferHead bh;
+	    memcpy(&bh, bufPtr, sizeof(BufferHead));
+	    
+	    numFixes += (long)bh.numFixes;
+	    numDirty += (long)bh.isDirty;
+
+	    // we don't count persistent hot spot pages
+	    if ( bh.isOccupied != PERSISTENT_OCCUPIED )
+		numUsage += bh.numUsage;
+	    
+	    if (bh.isOccupied == NOT_OCCUPIED)
+	    {
+		numFree++;
+	    }
+	    else if (bh.isOccupied == WRITE_ON_SYNC)
+	    {
+		numSegPageUsed++;
+		numUsed++;
+	    }
+	    else if (bh.isOccupied == WRITE_ON_DIRTY)
+	    {
+		numSegPageUsed++;
+		numUsed++;
+		numNoSync++;
+	    }
+	    else if (bh.isOccupied == PERSISTENT_OCCUPIED)
+	    {
+		numSegPageUsed++;
+		numUsed++;
+		numPersistent++;
+	    }   
+	    
+	    bufPtr = (void*)((long long)bufPtr + (long long)(BUFFERHEAD + _pageSize));     
+	    pageCount++;
 	}
-	else if (bh.isOccupied == WRITE_ON_DIRTY)
-	{
-	    numUsed++;
-	    numNoSync++;
-	}
-	else if (bh.isOccupied == PERSISTENT_OCCUPIED)
-	{
-	    numUsed++;
-	    numPersistent++;
-	}   
-	
-	bufPtr = (void*)((long)bufPtr + (long)(BUFFERHEAD + _pageSize));     
-	pageCount++;
-    }
-    
+	// cout << "Segment " << i << " : " << numSegPageUsed << " pages used" << endl; 
+    }	
+
     numDiskWrite = _numDiskWrite;
     numDiskRead = _numDiskRead;
+	
+    // hitRate = BUPMNG_STATSPERIOD - _numDiskRead / ( _fixCount / BUPMNG_STATSPERIOD + 1) ;
+    hitRate = 100.0 * ( (double)_fixCount - _numDiskRead + 1 ) / ( (double)_fixCount + 1 ); 
+
+    double usageMedian = (double)numUsage / (double)( _numSegment * _numPages );
+    double usageDist = 0.0;
     
-    hitRate = 100000 - _numDiskRead / ( _fixCount / 100000 + 1) ;
+    for ( int i=0; i<_numSegment; i++)
+    {
+	void *bufPtr = (void*)( (long long)_pBufPool[i] + BUFFERPOOLHEAD );
+	
+	unsigned long long pageCount = 0;
+    
+	while ( pageCount < _numPages )
+	{    
+	    BufferHead bh;
+	    memcpy(&bh, bufPtr, sizeof(BufferHead));
+
+	    if ( bh.isOccupied != PERSISTENT_OCCUPIED )
+	    {
+		usageMedian > bh.numUsage ? usageDist += usageMedian - (double)bh.numUsage : usageDist += (double)bh.numUsage - usageMedian;
+	    }
+
+	    bufPtr = (void*)((long long)bufPtr + (long long)(BUFFERHEAD + _pageSize));
+	    pageCount++;
+	}
+    }
+
+    double avgDist = usageDist / (double)( _numSegment * _numPages );
+    // cout << "DistMed = " << usageMedian << endl;
+    // cout << "AvgDist = " << avgDist << endl;
+    // cout << "NumUsage  = " << numUsage << endl;
+    spreadRate =  avgDist / ( usageMedian + 1 ); 
+
+    // cout << "Fix Dist Rate = " << fixDistRate << endl;
     
     readDelay = _avgReadDelay;
     writeDelay = _avgWriteDelay;
     statStart = _statStart;
     curFixCount = _fixCount;
     maxFixCount = BUPMNG_STATSPERIOD;
+    
+    Datetime ts;
+    uptime = ts.asInt() - _poolStart;    
 }
-
-
+    
 void CegoBufferPool::getPoolEntryList(ListT<CegoBufferPoolEntry>& entryList)
 {
-
     if (_pBufPool == 0)
     {
 	throw Exception(EXLOC, "No valid bufferpool" );
     }
-    
-    void* bufPtr = (void*)( (long)_pBufPool + BUFFERPOOLHEAD);
-    
-    long pageCount = 0;
-    
+
     entryList.Empty();
     
-    while ( pageCount < _numPages )
-    {    
-	BufferHead bh;
-	memcpy(&bh, bufPtr, sizeof(BufferHead));
-	
-	bool isDirty = false;
-	if ( bh.isDirty )
-	    isDirty=true;
-	
-	Chain occState;
-	if (bh.isOccupied == NOT_OCCUPIED)
-	    occState=Chain("NOTOCCUPIED");
-	else if (bh.isOccupied == WRITE_ON_SYNC)
-	    occState=Chain("WRITEONSYNC");
-	else if (bh.isOccupied == WRITE_ON_DIRTY)
-	    occState=Chain("WRITEONDIRTY");
-	else if (bh.isOccupied == PERSISTENT_OCCUPIED)
-	    occState=Chain("PERSISTENT");
-	
-	CegoBufferPoolEntry bpe(pageCount, occState, isDirty, bh.numFixes, bh.tabSetId, bh.fileId, bh.pageId, bh.fixStat);
-	entryList.Insert(bpe);
-	
-	bufPtr = (void*)((long)bufPtr + (long)(BUFFERHEAD + _pageSize));     
-	pageCount++;
+    for ( int seg=0; seg<_numSegment; seg++)
+    {
+	void *bufPtr = (void*)( (long long)_pBufPool[seg] + BUFFERPOOLHEAD );
+	    
+	unsigned long long pageCount = 0;
+    	
+	while ( pageCount < _numPages )
+	{    
+	    BufferHead bh;
+	    memcpy(&bh, bufPtr, sizeof(BufferHead));
+	    
+	    bool isDirty = false;
+	    if ( bh.isDirty )
+		isDirty=true;
+	    
+	    Chain occState;
+	    if (bh.isOccupied == NOT_OCCUPIED)
+		occState=Chain("NOTOCCUPIED");
+	    else if (bh.isOccupied == WRITE_ON_SYNC)
+		occState=Chain("WRITEONSYNC");
+	    else if (bh.isOccupied == WRITE_ON_DIRTY)
+		occState=Chain("WRITEONDIRTY");
+	    else if (bh.isOccupied == PERSISTENT_OCCUPIED)
+		occState=Chain("PERSISTENT");
+	    
+	    CegoBufferPoolEntry bpe(seg, pageCount, occState, isDirty, bh.numFixes, bh.tabSetId, bh.fileId, bh.pageId, bh.fixStat, bh.numUsage);
+	    entryList.Insert(bpe);
+	    
+	    bufPtr = (void*)((long long)bufPtr + (long long)(BUFFERHEAD + _pageSize));     
+	    pageCount++;
+	}	
     }
-    
 }
 
 void CegoBufferPool::resetStats()
@@ -883,9 +991,7 @@ void CegoBufferPool::resetStats()
 
 
 void CegoBufferPool::logBM(int tabSetId, int fileId, unsigned* fbm, int fbmSize)
-{
-    
-    
+{    
     CegoLogRecord lr;
     
     lr.setAction(CegoLogRecord::LOGREC_BUFBM);
@@ -917,55 +1023,49 @@ void CegoBufferPool::logBM(int tabSetId, int fileId, unsigned* fbm, int fbmSize)
 
 void CegoBufferPool::printPool()
 {
-
     if (_pBufPool == 0)
     {
 	throw Exception(EXLOC, "No valid bufferpool" );
     }
     
     cout << "--- BufferPool ---" << endl;
-    cout << "BasePtr: " << (long)_pBufPool << endl;
+    cout << "BasePtr: " << (long long)_pBufPool << endl;
     cout << "PageSize: " << _pageSize << endl;
     cout << "NumPages: " << _numPages << endl;
     
-    long numUsed=0;
-    long numFree=0;
-    long numFixes=0;
-    long numDirty=0;
+    unsigned long long numUsed=0;
+    unsigned long long numFree=0;
+    unsigned long long numFixes=0;
+    unsigned long long numDirty=0;
 
-    void* bufPtr = (void*)( (long)_pBufPool + BUFFERPOOLHEAD);
-
-    long pageCount = 0;
-    
-    while ( pageCount < _numPages )
-    {    
-	BufferHead bh;
-	memcpy(&bh, bufPtr, sizeof(BufferHead));
-
-	numFixes += (long)bh.numFixes;
-	numDirty += (long)bh.isDirty;
-
-	if (bh.isOccupied == 0)
-	    numFree++;
-	else
-	    numUsed++;
+    for ( int i=0; i<_numSegment; i++)
+    {
+	void *bufPtr = (void*)( (long long)_pBufPool[i] + BUFFERPOOLHEAD );
 	
-	bufPtr = (void*)((long)bufPtr + (long)(BUFFERHEAD + _pageSize));     
-	pageCount++;
-    }
+	unsigned long long pageCount = 0;
     
+	while ( pageCount < _numPages )
+	{    
+	    BufferHead bh;
+	    memcpy(&bh, bufPtr, sizeof(BufferHead));
+	    
+	    numFixes += bh.numFixes;
+	    numDirty += (unsigned long long)bh.isDirty;
+	    
+	    if (bh.isOccupied == 0)
+		numFree++;
+	    else
+		numUsed++;
+	    
+	    bufPtr = (void*)((long long)bufPtr + (long long)(BUFFERHEAD + _pageSize));     
+	    pageCount++;
+	}
+    }
+	
     cout << "NumUsed: " << numUsed << endl;
     cout << "NumFixes: " << numFixes << endl;
     cout << "NumDirty: " << numDirty << endl;
     cout << "NumFree: " << numFree << endl;
 
     cout << "NumFree: " << numFree << endl;
-
 }
-
-
-
-
-
-
-
