@@ -6,30 +6,15 @@
 //
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2012 Bjoern Lemke                        
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke                        
 //
 // IMPLEMENTATION MODULE
 //
 // Class: main
-// 
-// Description: 
 //
-// Status: QG-2.6
+// Description: The cego SQL console program
+//
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -40,6 +25,7 @@
 #include <readline/readline.h>
 #include <readline/history.h>
 
+// LFC INCLUDES
 #include <lfcbase/Chain.h>
 #include <lfcbase/Exception.h>
 #include <lfcbase/GetLongOpt.h>
@@ -49,59 +35,59 @@
 #include <lfcbase/Net.h>
 #include <lfcbase/SigHandler.h>
 #include <lfcbase/Tokenizer.h>
-
 #include <lfcxml/Element.h>
 #include <lfcxml/Document.h>
 #include <lfcxml/XMLSuite.h>
 
+// CEGO INCLUDES
 #include "CegoOutput.h"
-#include "CegoViewObject.h"
-#include "CegoProcObject.h"
-#include "CegoKeyObject.h"
-#include "CegoDbHandler.h"
 #include "CegoModule.h"
 #include "CegoNet.h"
 #include "CegoXMLdef.h"
 
-#include "CegoDefs.h"
+// #include "CegoDefs.h"
 
+#define DEFAULTPORT 2200
+#define DEFAULTSERVER "localhost"
+#define DEFAULTPROTOCOL "serial"
+#define DEFAULTMAXLINE 10000
+#define DEFAULTMAXRESULT 100000
 
 #define USAGE "Usage: cgclt --user=<user>/<password>\n\
-          [ --server=<host>]\n\
+          [ --server=<host> ]\n\
           [ --port=<port> ]\n\
           [ --protocol={serial|xml} ]\n\
 	  [ --tableset=<tableset> ]\n\
           [ --batchfile=<batchfile> ]\n\
           [ --dumpfile=<dumpfile> ]\n\
+          [ --qescmode ]\n\
           [ --structure ]\n\
           [ --cmd=<command> ]\n\
           [ --logfile=<logfile> ]\n\
           [ --raw ]\n\
+          [ --flow ]\n\
+          [ --maxline=<maxline> ]\n\
+          [ --maxresult=<maxresult> ]\n\
           [ --ignore ]\n\
-          [ --profile=<pid> ]\n\
+          [ --profile=<profile id> ]\n\
           [ --debug ] [ --version  ] [ --help ]"
 
-
-#define DEFAULTPORT 2200
-#define DEFAULTSERVER "localhost"
-#define DEFAULTPROTOCOL "xml"
-
+#define DEFAULTRAWSEP ","
 #define HISTFILE ".cgclt_history"
 #define PROMPT "CGCLT > "
 #define PROMPT2 "> "
-
 #define CGPROFILE ".cgprofile"
-
 #define LANGUAGE_ENV "LANG"
-#define LOCALE_CATEGORY LC_TIME
-
-#define MAXCMDLEN 5000
+#define LOCALE_CATEGORY LC_ALL
 
 enum RunMode { INTERACTIVE, BATCH, COMMAND, DUMP };
 
 extern char __lfcVersionString[];
 extern char __lfcxmlVersionString[];
 extern char __quoteEscapeFlag;
+extern Chain __dateTimeFormat;
+extern char __currencySymbol;
+extern char __decimalPoint;
 
 bool flowCtl = false;
 int flowCount = 10;
@@ -116,7 +102,7 @@ public:
 		 const Chain& tableSet,
 		 const Chain& user, 
 		 const Chain& password,		 
-		 long tid, 
+		 unsigned long long tid, 
 		 CegoModule *pModule);
     ~AbortHandler();
     void sigCatch(int sig);
@@ -125,7 +111,7 @@ public:
     
 private:
 
-    long _tid;
+    unsigned long long _tid;
     bool _doAbort;
     CegoModule *_pModule;
     int _portNo;
@@ -174,17 +160,19 @@ private:
 };
 
 int execute();
-bool dispatchCmd(CegoDbHandler* pNetHandle, AbortHandler* pAH, Chain cmd, bool outMode, int& exitCode, Chain& msg);
+bool dispatchCmd(bool outputBuffered, CegoNet* pCegoNet, AbortHandler* pAH, Chain cmd, bool rawMode, int& exitCode, Chain& msg);
 void loadProfile(const Chain& rcFileName, ListT<ProfileEntry>& pfeList);
 void dumpTableSet(CegoNet *pCegoNet, const Chain& dumpFileName, bool isStructure);
 void dumpCounter(CegoNet *pCegoNet, File *pDF);
 void dumpTable(CegoNet *pCegoNet, File *pDF, bool isStructure);
 void dumpIndex(CegoNet *pCegoNet, File *pDF);
+void dumpBtree(CegoNet *pCegoNet, File *pDF);
 void dumpKey(CegoNet *pCegoNet, File *pDF);
 void dumpCheck(CegoNet *pCegoNet, File *pDF);
 void dumpView(CegoNet *pCegoNet, File *pDF);
 void dumpProcedure(CegoNet *pCegoNet, File *pDF);
 
+
 Chain user;
 Chain password;
 Chain serverName;
@@ -194,7 +182,11 @@ Chain authString;
 Chain promptString;
 Chain protocol;
 
+unsigned long long maxLine;
+unsigned long long maxResult;
 bool rawMode = false;
+bool isBuffered = false;
+Chain rawSep;
 bool ignoreError = false;
 bool isStructure = false;
 Chain batchFileName;
@@ -205,12 +197,11 @@ RunMode runMode = INTERACTIVE;
 bool debug=false;
 CegoModule *pModule = 0;
 unsigned long modId;
-Chain homeDir;
-Chain histPath;
 CegoDbHandler::ProtocolType protType;
+Chain histPath;
 
 int main(int argc, char **argv)
-{	
+{
     
     GetLongOpt longOpt(argc, argv);
     
@@ -224,8 +215,12 @@ int main(int argc, char **argv)
     longOpt.addOpt("dumpfile");
     longOpt.addOpt("structure");
     longOpt.addOpt("qescmode");
+    longOpt.addOpt("maxline", Chain(DEFAULTMAXLINE));
+    longOpt.addOpt("maxresult", Chain(DEFAULTMAXRESULT));
+    
     longOpt.addOpt("debug");
-    longOpt.addOpt("raw");
+    longOpt.addOpt("raw", Chain(DEFAULTRAWSEP));
+    longOpt.addOpt("flow");
     longOpt.addOpt("ignore");
     longOpt.addOpt("server", DEFAULTSERVER);
     longOpt.addOpt("protocol", DEFAULTPROTOCOL);
@@ -245,13 +240,12 @@ int main(int argc, char **argv)
 	exit(1);	
     }
     
-
-#ifdef HAVE_MINGW32
-    homeDir = getenv("HOMEPATH");
+#ifdef HAVE_MINGW
+    Chain homeDir = getenv("HOMEPATH");
     histPath = homeDir + "\\" + Chain(HISTFILE);
 #else
     Chain homeDir(getenv("HOME"));
-    Chain histPath = homeDir + "/" + Chain(HISTFILE);
+    histPath = homeDir + "/" + Chain(HISTFILE);
 #endif
     
     if ( longOpt.isSet("debug") )
@@ -261,11 +255,16 @@ int main(int argc, char **argv)
 	__quoteEscapeFlag=1;
     else
 	__quoteEscapeFlag=0;
-    
 
     if ( longOpt.isSet("raw") )
+    {
 	rawMode=true;
-
+	rawSep = longOpt.getOptValue("raw");
+    }
+    
+    if ( longOpt.isSet("flow") )
+	isBuffered=true;
+    
     if ( longOpt.isSet("ignore") )
 	ignoreError=true;
 
@@ -281,14 +280,13 @@ int main(int argc, char **argv)
 
     if ( longOpt.isSet( Chain("version") ) )
     {
-	cout << CEGO_PRODUCT << " (" << sizeof(long) * 8 << " bit), Version " << CEGO_VERSION 
-	     << " [ lfc : " << __lfcVersionString  
-	     << ", lfcxml : " <<  __lfcxmlVersionString << " ]" << endl;
+	cout << CEGO_PRODUCT << " SQL Client (" << sizeof(void*) * 8 << " bit), Version " << CEGO_VERSION 
+	     << " [ lfc: " << __lfcVersionString  
+	     << ", lfcxml: " <<  __lfcxmlVersionString << " ]" << endl;
 	cout << CEGO_COPYRIGHT << endl;
 	exit(0);
     }
 
-    
     ListT<ProfileEntry> pfeList;
     
     try
@@ -321,7 +319,6 @@ int main(int argc, char **argv)
 	command = longOpt.getOptValue("cmd");
     }
 
-
     if ( longOpt.isSet("profile") )
     {
 	Chain profile;
@@ -399,11 +396,18 @@ int main(int argc, char **argv)
 	}
     }
 
+    const struct lconv * const currentlocale = localeconv();
+
+    __currencySymbol = *currentlocale->currency_symbol;
+    __decimalPoint = *currentlocale->decimal_point;        
+    
+    maxLine = longOpt.getOptValue("maxline").asUnsignedLongLong();
+    maxResult = longOpt.getOptValue("maxresult").asUnsignedLongLong();
+
     if ( logFile.length() > 0 )
-	pModule = new CegoModule(logFile);
+	pModule = new CegoModule(logFile, Chain(""));
     else
 	pModule = new CegoModule;
-
     
     modId = pModule->getModId("CegoClient");
     
@@ -440,48 +444,40 @@ int main(int argc, char **argv)
 int execute()
 {
     int exitCode = 0;
-	
+
+    Chain logMode("notice");
+    if ( debug )
+	logMode=Chain("debug");
+        
     switch ( runMode )
     {
 	
     case COMMAND:
     {
-	
-	Net n (  NETMNG_MSG_BUFLEN, NETMNG_SIZEBUFLEN );
-	NetHandler* pN;
+		
+	CegoNet *pCegoNet = new CegoNet( protType, logFile, Chain(""), logMode );	
 	
 	try
 	{
-	    pN = n.connect(serverName, portNo);	    
+	    pCegoNet->connect(serverName, portNo, tableSet, user, password);
 	}
 	catch ( Exception e )
 	{
 	    cerr << "Cannot connect to database" << endl;
+	    delete pCegoNet;
 	    return 1;
 	}
 	
-	CegoDbHandler* pSH = new CegoDbHandler(pN, protType, pModule);
-	
-	CegoDbHandler::ResultType res = pSH->requestSession(tableSet, user, password);
-	if ( res != CegoDbHandler::DB_OK )
-	{
-	    Chain msg = pSH->getMsg();
-	    delete pSH;
-	    cerr << msg << endl;
-	    return 1;
-	}
-			   		    
 	Timer t(6,3);
 	t.start();
 
 	Chain msg;
-	dispatchCmd(pSH, 0, command, rawMode, exitCode, msg);
+	dispatchCmd(false, pCegoNet, 0, command, rawMode, exitCode, msg);
 	t.stop();
 
-	pSH->closeSession();
+	pCegoNet->disconnect();
 	
-	delete pSH;
-	delete pN;
+	delete pCegoNet;
 
 	if ( exitCode != 0 )
 	{
@@ -499,66 +495,39 @@ int execute()
     case BATCH:
     {
 	int lineNo=0;
-	    
-	Net n (  NETMNG_MSG_BUFLEN, NETMNG_SIZEBUFLEN );
-	NetHandler* pN;
+
+	CegoNet *pCegoNet = new CegoNet( protType, logFile, Chain(""), logMode );	
 	
 	try
 	{
-	    pN = n.connect(serverName, portNo);	    
-	}
-	catch ( Exception e )
-	{
-	    cerr << "Cannot connect to database" << endl;
-	    return 1;
-	}
-	
-	CegoDbHandler* pSH = new CegoDbHandler(pN, protType, pModule);
-	
-	CegoDbHandler::ResultType res = pSH->requestSession(tableSet, user, password);
-	if ( res != CegoDbHandler::DB_OK )
-	{
-	    Chain msg = pSH->getMsg();
-	    delete pSH;
-	    cerr << msg << endl;
-	    return 1;
-	}
 
-	File cmdFile(batchFileName);
-	if ( cmdFile.exists() == false )
-	{
-	    cerr << "File " << batchFileName << " does not exist" << endl;
-	    return 1;
-	}
-	
-	try 
-	{
+	    File cmdFile(batchFileName);
+	    if ( cmdFile.exists() == false )
+	    {
+		Chain msg = Chain("File ") + batchFileName + Chain(" does not exist");
+		throw Exception(EXLOC, msg);
+	    }
+
+	    pCegoNet->connect(serverName, portNo, tableSet, user, password);
+		    
 	    cmdFile.open(File::READ);	    
-	}
-	catch ( Exception e)
-	{
-	    cout << "Cannot open batchfile " + batchFileName << endl;
-	    return 1;
-	}
-	
-	try 
-	{
-	    
+
 	    Chain cmd;
 	    Chain line;
 	    bool disableDelimiter=false;
 	    bool goOn = true;
-
-	    while (goOn && cmdFile.readLine(line, MAXCMDLEN))
+	    
+	    while (goOn && cmdFile.readLine(line, maxLine))
 	    {
 		lineNo++;
-		line = line.cutTrailing(" \t");
 		
 		bool isCommentLine = false;
 		
-		if ( line.length() >= 2 )
-		{		    
-		    if (line.cutTrailing(" \t").subChain(1, 2) == Chain("--"))
+		line = line.truncRight(" \t");
+	    
+		if ( line.truncLeft(" \t").length() >= 2 )
+		{
+		    if (line.truncLeft(" \t").subChain(1, 2) == Chain("--"))
 		    {
 			isCommentLine = true;
 		    }
@@ -577,24 +546,42 @@ int execute()
 		    else
 		    {
 			// cmd = line;
-			cmd = cmd + Chain(" ") + line;
-			cmd = cmd.cutTrailing(" \t");
+			cmd = cmd + Chain(" ") + line;			
+			// cmd = cmd + line;
 		    }
-			 
-		    if ( cmd.length() > 0 )
-		    {
 
+		    // cmd = cmd.cutTrailing(" \t");
+
+		    if ( cmd.length() > 0 )
+		    {		       
 			if (cmd.subChain(cmd.length()-1, cmd.length()) == Chain(";") 
 			    && disableDelimiter==false)
 			{
-			    		       
+
 			    // send command to server here
-			    
+
+			    Chain bval;
+			    if ( cmd.replaceAll(Chain("\\n"), Chain("\n"), bval) > 0 )
+				cmd = bval;
+			    			    
+			    // escape quotes
+			    Chain qval;
+			    if ( __quoteEscapeFlag )
+			    {
+				if ( cmd.replaceAll(Chain("''"), Chain("'"), qval) > 0 )
+				    cmd = qval;
+			    }
+			    else
+			    {
+				if ( cmd.replaceAll(Chain("\\'"), Chain("'"), qval) > 0 )
+				    cmd = qval;
+			    }			  			    
+			   			    
 			    Timer t(6,3);
 			    t.start();
 			    			   
 			    Chain msg;
-			    goOn = dispatchCmd(pSH, 0, cmd, rawMode, exitCode, msg);
+			    goOn = dispatchCmd(false, pCegoNet, 0, cmd, rawMode, exitCode, msg);
 			    
 			    t.stop();
 
@@ -604,8 +591,11 @@ int execute()
 			    }
 			    else
 			    {
-				cout << msg << endl;
-				cout << "ok ( " << t << " s )" << endl;
+				if ( rawMode == false )
+				{
+				    cout << msg << endl;				
+				    cout << "ok ( " << t << " s )" << endl;
+				}
 			    }
 
 			    if ( exitCode != 0 && ignoreError )
@@ -614,9 +604,14 @@ int execute()
 			    cmd = Chain("");
 			    
 			}
+			else
+			{
+			    // collect further data
+			}
 		    }
 		}
 	    }
+	    cmd = cmd.cutTrailing(" \t");
 	    if ( cmd.length() > 1 )
 	    {
 		cout << "Incomplete command ..<" << cmd << ">" << endl;
@@ -624,31 +619,28 @@ int execute()
 	    
 	    cmdFile.close();
 
-	    pSH->closeSession();
+	    pCegoNet->disconnect();
 
-	    delete pSH;
-	    delete pN;
+	    delete pCegoNet;
 
 	}
-	catch ( Exception e)
+	catch ( Exception e )
 	{
 	    Chain msg;
 	    e.pop(msg);
+	    
 	    cerr << msg << endl;
+	    delete pCegoNet;
 	    return 1;
 	}
-	
+		    	
 	break;
 
     }
     case DUMP:
     {
-	
-	Chain logMode("notice");
-	if ( debug )
-	    logMode=Chain("debug");
-
-	CegoNet *pCegoNet = new CegoNet( protType, logFile, logMode );	
+     
+	CegoNet *pCegoNet = new CegoNet( protType, logFile, Chain(""), logMode );	
 	
 	try
 	{
@@ -662,6 +654,8 @@ int execute()
 	}
 
 	dumpTableSet(pCegoNet, dumpFileName, isStructure);
+
+	pCegoNet->disconnect();
 	
 	delete pCegoNet;
 	
@@ -672,39 +666,14 @@ int execute()
     case INTERACTIVE:
     {
 
-	Net n (  NETMNG_MSG_BUFLEN, NETMNG_SIZEBUFLEN );
-	NetHandler* pN;
-	
+	CegoNet *pCegoNet = new CegoNet( protType, logFile, Chain(""), logMode );	
+
 	try
 	{
-	    pN = n.connect(serverName, portNo);
-	}
-	catch ( Exception e )
-	{
-	    cerr << "Cannot connect to database" << endl;
-	    return 1;	    
-	}
-
-	CegoDbHandler* pSH = new CegoDbHandler(pN, protType, pModule);
-	
-#ifdef DEBUG
-	pModule->log(modId, Logger::DEBUG, Chain("Requesting session ..."));
-#endif
-
-	CegoDbHandler::ResultType res = pSH->requestSession(tableSet, user, password);
-	if ( res != CegoDbHandler::DB_OK )
-	{
-	    Chain msg = pSH->getMsg();
-	    delete pSH;
-	    cerr << msg << endl;
-	    return 1;
-	}
-
-	
-	try 
-	{	    	    
-
-	    long tid = pSH->getTid();
+	    
+	    pCegoNet->connect(serverName, portNo, tableSet, user, password);
+	    
+	    unsigned long long tid = pCegoNet->getTid();
 	    
 	    AbortHandler *pAH = new AbortHandler(serverName, portNo, tableSet, user, password, tid, pModule);
 	    
@@ -764,7 +733,7 @@ int execute()
 		    t.start();
 
 		    Chain msg;
-		    goOn = dispatchCmd(pSH, pAH, cs, rawMode, exitCode, msg);
+		    goOn = dispatchCmd(isBuffered, pCegoNet, pAH, cs, rawMode, exitCode, msg);
 
 		    t.stop();
 
@@ -776,8 +745,11 @@ int execute()
 		    }
 		    else
 		    {
-			cout << msg << endl;
-			cout << "ok ( " << t << " s )" << endl;
+			if ( rawMode == false )
+			{			   
+			    cout << msg << endl;
+			    cout << "ok ( " << t << " s )" << endl;
+			}
 		    }
 		    write_history((char*)histPath);
 		}
@@ -788,15 +760,13 @@ int execute()
 		}
 	    }
 	    
-	    pSH->closeSession();
-
-	    delete pSH;
-	    delete pN;
-	    delete pAH;
+	    pCegoNet->disconnect();
+	    delete pCegoNet;
 	    
 	}
 	catch ( Exception e)
 	{
+	    delete pCegoNet;
 	    Chain msg;
 	    e.pop(msg);
 	    cerr << msg << endl;
@@ -810,8 +780,7 @@ int execute()
     return exitCode;
 }
 
-
-bool dispatchCmd(CegoDbHandler *pSH, AbortHandler *pAH, Chain cmd, bool outMode, int& exitCode, Chain& msg)
+bool dispatchCmd(bool outputBuffered, CegoNet *pCegoNet, AbortHandler *pAH, Chain cmd, bool rawMode, int& exitCode, Chain& msg)
 {
     
     exitCode = 0;
@@ -821,163 +790,95 @@ bool dispatchCmd(CegoDbHandler *pSH, AbortHandler *pAH, Chain cmd, bool outMode,
 	msg = Chain("Goodbye");
 	return false;
     }
-
-    Tokenizer tok(cmd, Chain(" ;"));
-    Chain flowTok;
-    if ( tok.nextToken(flowTok) )
-    {
-	
-	if ( flowTok == Chain("flowctl") )
-	{
-	    Chain flowOpt;
-	    if ( tok.nextToken(flowOpt) )
-	    {
-		if ( flowOpt == Chain("off") )
-		{
-		    flowCtl=false;
-		    cout << "Flow control disabled" << endl;
-		}
-		else
-		{
-		    flowCtl=true;
-		    if ( flowOpt.asInteger() > 0 )
-			flowCount=flowOpt.asInteger();
-		    cout << "Flow control enabled" << endl;    
-		}
-	    }	    
-	    return true;
-	}
-    }
     
     CegoDbHandler::ResultType res;
-
+    
     if ( pAH )
 	pAH->setQueryAbort(true);
-    
-    res = pSH->reqQueryOp(cmd);
-    
+
+    try
+    {
+	unsigned long long affCount = pCegoNet->doQuery(cmd);
+    }
+    catch ( Exception e )
+    {
+	if ( pAH )
+	    pAH->setQueryAbort(false);
+	
+	msg = pCegoNet->getMsg();
+	exitCode = 1;
+	return false;
+    }
+
     if ( pAH )
 	pAH->setQueryAbort(false);
-
-
-    if ( res == CegoDbHandler::DB_OK )
-    {
-	msg = pSH->getMsg();
-	long affCount = pSH->getAffected();
-	
-	// if ( msg == Chain("Goodbye") )
-	//     return true;
-
-	// cout << msg << endl; // " ( " << affCount << " affected )" << endl;
-	
-    }
-    else if ( res == CegoDbHandler::DB_FIN )
-    {
-	msg = Chain("No rows");
-    }
-    else if ( res == CegoDbHandler::DB_INFO )
-    {
-
-	CegoObject::ObjectType objType = pSH->getObjType();
-
-	switch ( objType )
-	{
-	case CegoObject::SYSTEM:
-	case CegoObject::TABLE:
-	case CegoObject::PINDEX:
-	case CegoObject::UINDEX:
-	case CegoObject::INDEX:
-	{   
-	    CegoTableObject to;
-	    to.putElement( pSH->getObjElement() );
-	    cout << to.getFormatted();
-	    break;
-	}
-	case CegoObject::VIEW:
-	{
-	    CegoViewObject vo;
-	    vo.putElement( pSH->getObjElement() );
-	    cout << vo.getFormatted();
-	    break;
-	}
-	case CegoObject::PROCEDURE:
-	{
-	    CegoProcObject po;
-	    po.putElement( pSH->getObjElement() );
-	    cout << po.getFormatted();
-	    break;
-	}
-	case CegoObject::FKEY:
-	{
-	    CegoKeyObject ko;
-	    ko.putElement( pSH->getObjElement() );
-	    cout << ko.getFormatted();
-	    break;
-	}
-	default:
-	    break;
-	}
-    }
-    else if ( res == CegoDbHandler::DB_DATA )
-    {
+    
+    if ( pCegoNet->isFetchable() )
+    {		
 	ListT<CegoField> schema;
-	Chain format;
+	pCegoNet->getSchema(schema);
 
-	schema = pSH->getSchema();
-	format = pSH->getFormat();
+	Chain format;
+	pCegoNet->getFormat(format);
 	
 	CegoOutput output(schema, format);
-	
-	output.setRawMode(outMode);
 
-	output.headOut();
-	
-	// CegoDbHandler::QuerySync querySync;
+	output.setRawMode(rawMode);
+	if ( rawMode )
+	    output.setRawSeparator(rawSep);
 
-	// querySync=CegoDbHandler::QUERY_MOREDATA;
+	ListT< ListT<CegoFieldValue> > outData;
 
+	if ( outputBuffered == false )
+	    output.headOut();
+		
 	ListT<CegoFieldValue> fvl;
-	long count=0;
 	
-	while ( ( res = pSH->receiveTableData(schema, fvl) ) == CegoDbHandler::DB_DATA )
-	{	    
-	    count++;
-	    
-	    output.rowOut(fvl);
-	    fvl.Empty();		    
+	unsigned long long count=0;
 
-	    if ( flowCtl )
+	while ( pCegoNet->fetchData(schema, fvl)  )
+	{
+
+	    count++;
+
+	    if ( count > maxResult )
 	    {
-		if ( count % flowCount == 0 )
+		pCegoNet->abortQuery();
+		exitCode = 1;
+		msg=Chain("Result is too large");
+		return false;
+	    }
+	    
+	    if ( outputBuffered )
+	    {       	
+		outData.Insert(fvl);
+		if ( count % 100 )
 		{
-		    cout << "Next, Reset, Abort ? ( n, r, a ) ";
-		    char resp;
-		    cin >> resp;
-		    if ( resp == 'a' )
-		    {
-			pSH->abortQuery();
-			break;
-		    }
-		    if ( resp == 'r' )
-		    {
-			pSH->resetQuery();
-		    }
-		    else
-		    {
-			// nothing
-		    }		    
+		    cout << "Collecting data ... " << count << " rows\r";
+		    cout.flush();
 		}
 	    }
-		
+	    else
+	    {
+		output.rowOut(fvl);
+	    }
+
+	    fvl.Empty();	    
+	}
+
+	if ( outputBuffered )
+	{
+	    output.manageOutput(cmd, outData);
+	}
+	else
+	{	    
+	    output.tailOut();
 	}
-	output.tailOut();
 	msg=Chain(count) + Chain(" tuples");
     }
-    if ( res == CegoDbHandler::DB_ERROR )
-    {
-	msg = pSH->getMsg();
-	exitCode = 1;
-	return false;	
+    else
+    {	      
+	msg = pCegoNet->getMsg();
     }
     
     return true;
@@ -994,6 +895,7 @@ void dumpTableSet(CegoNet *pCegoNet, const Chain& dumpFileName, bool isStructure
 	dumpCounter(pCegoNet, pDF);	
 	dumpTable(pCegoNet, pDF, isStructure);
 	dumpIndex(pCegoNet, pDF);
+	dumpBtree(pCegoNet, pDF);
 	dumpKey(pCegoNet, pDF);
 	dumpCheck(pCegoNet, pDF);
 	dumpView(pCegoNet, pDF);
@@ -1014,7 +916,7 @@ void dumpTableSet(CegoNet *pCegoNet, const Chain& dumpFileName, bool isStructure
 void dumpCounter(CegoNet *pCegoNet, File *pDF)
 {
 	
-    long res;
+    unsigned long long res;
     res = pCegoNet->doQuery(Chain("list counter;"));
         
     ListT<CegoField> schema;
@@ -1045,7 +947,7 @@ void dumpCounter(CegoNet *pCegoNet, File *pDF)
 void dumpTable(CegoNet *pCegoNet, File *pDF, bool isStructure)
 {
 	
-    long res;
+    unsigned long long res;
     res = pCegoNet->doQuery(Chain("list table;"));
         
     ListT<CegoField> schema;
@@ -1092,6 +994,8 @@ void dumpTable(CegoNet *pCegoNet, File *pDF, bool isStructure)
 	    Chain attrSize = pFV->valAsChain();
 	    pFV = fvl.Next();
 	    Chain attrDef = pFV->valAsChain();
+	    Chain attrDefValue = pFV->toChain();
+	    
 	    pFV = fvl.Next();
 	    Chain attrNull = pFV->valAsChain();
 
@@ -1102,7 +1006,12 @@ void dumpTable(CegoNet *pCegoNet, File *pDF, bool isStructure)
 		 || attrType == Chain(XML_FIXED_VALUE))
 		cmd +=  Chain("(") + attrSize + Chain(")");
 	    if ( attrDef != Chain(XML_NULL_VALUE))
-		cmd += Chain(" default ") + attrDef;
+	    {
+		if ( attrType == Chain(XML_STRING_VALUE) )
+		    cmd += Chain(" default ") + attrDefValue;
+		else
+		    cmd += Chain(" default ") + attrDef;
+	    }
 	    if ( attrNull == Chain("no"))
 		cmd += Chain(" not null");
 	    fvl.Empty();	
@@ -1146,11 +1055,10 @@ void dumpTable(CegoNet *pCegoNet, File *pDF, bool isStructure)
     }
 }
 
-
 void dumpIndex(CegoNet *pCegoNet, File *pDF)
 {
 	
-    long res;
+    unsigned long long res;
     res = pCegoNet->doQuery(Chain("list index;"));
         
     ListT<CegoField> schema;
@@ -1226,11 +1134,89 @@ void dumpIndex(CegoNet *pCegoNet, File *pDF)
     }
 }
 
+void dumpBtree(CegoNet *pCegoNet, File *pDF)
+{
+	
+    unsigned long long res;
+    res = pCegoNet->doQuery(Chain("list btree;"));
+        
+    ListT<CegoField> schema;
+    pCegoNet->getSchema(schema);
+    
+    ListT<Chain> indexList;
+    ListT<CegoFieldValue> fvl;
+    while ( pCegoNet->fetchData(schema, fvl)  )
+    {
+	CegoFieldValue *pFV = fvl.First();
+	Chain indexName = pFV->valAsChain();
+	indexList.Insert(indexName);
+	fvl.Empty();	
+    }
+    
+    Chain *pIndex = indexList.First();
+    while ( pIndex ) 
+    {
+	res = pCegoNet->doQuery(Chain("desc btree ") + *pIndex + Chain(";"));
+
+
+	ListT<CegoField> schema;
+	pCegoNet->getSchema(schema);
+    
+	Chain tableName;
+	Chain indexType;
+	ListT<Chain> attrList;
+	
+	ListT<CegoFieldValue> fvl;	
+	while ( pCegoNet->fetchData(schema, fvl)  )
+	{	    
+	    CegoFieldValue *pFV = fvl.First();
+	    Chain attrName = pFV->valAsChain();
+	    attrList.Insert(attrName);
+	    pFV = fvl.Next();
+	    tableName = pFV->valAsChain();
+	    pFV = fvl.Next();
+	    indexType = pFV->valAsChain();
+
+	    fvl.Empty();
+	}
+
+	Chain cmd;
+
+	cmd=Chain("drop if exists btree ") + *pIndex + Chain(";\n");
+	pDF->writeChain(cmd);
+
+	if ( indexType == Chain(XML_PBTREE_VALUE) )
+	    cmd=Chain("create primary btree ");
+	else if ( indexType == Chain(XML_UBTREE_VALUE) )
+	    cmd=Chain("create unique btree ") + *pIndex; 
+	else
+	    cmd=Chain("create btree ") + *pIndex;
+
+	cmd += Chain(" on ") + tableName + Chain("(");
+	
+	Chain *pAttr = attrList.First();
+	bool isFirst=true;
+	while ( pAttr )
+	{
+	    if ( isFirst == false )
+		cmd += Chain(",");
+	    isFirst=false;
+
+	    cmd += *pAttr;
+	    pAttr = attrList.Next();
+	}
+	
+	cmd += Chain(");\n");
+	pDF->writeChain(cmd);
+
+	pIndex = indexList.Next();
+    }
+}
 
 void dumpKey(CegoNet *pCegoNet, File *pDF)
 {
 	
-    long res;
+    unsigned long long res;
     res = pCegoNet->doQuery(Chain("list key;"));
         
     ListT<CegoField> schema;
@@ -1262,7 +1248,6 @@ void dumpKey(CegoNet *pCegoNet, File *pDF)
 	ListT<CegoFieldValue> fvl;
 	while ( pCegoNet->fetchData(schema, fvl)  )
 	{
-
 	    CegoFieldValue *pFV = fvl.First();
 
 	    Chain tableName = pFV->valAsChain();
@@ -1319,9 +1304,8 @@ void dumpKey(CegoNet *pCegoNet, File *pDF)
 }
 
 void dumpCheck(CegoNet *pCegoNet, File *pDF)
-{
-	
-    long res;
+{	
+    unsigned long long res;
     res = pCegoNet->doQuery(Chain("list check;"));
         
     ListT<CegoField> schema;
@@ -1369,9 +1353,8 @@ void dumpCheck(CegoNet *pCegoNet, File *pDF)
 }
 
 void dumpView(CegoNet *pCegoNet, File *pDF)
-{
-	
-    long res;
+{	
+    unsigned long long res;
     res = pCegoNet->doQuery(Chain("list view;"));
         
     ListT<CegoField> schema;
@@ -1392,7 +1375,6 @@ void dumpView(CegoNet *pCegoNet, File *pDF)
     {
 	res = pCegoNet->doQuery(Chain("show view ") + *pView + Chain(";"));
 
-
 	ListT<CegoField> schema;
 	pCegoNet->getSchema(schema);
     
@@ -1420,9 +1402,8 @@ void dumpView(CegoNet *pCegoNet, File *pDF)
 }
 
 void dumpProcedure(CegoNet *pCegoNet, File *pDF)
-{
-	
-    long res;
+{	
+    unsigned long long res;
     res = pCegoNet->doQuery(Chain("list procedure;"));
         
     ListT<CegoField> schema;
@@ -1473,8 +1454,7 @@ void loadProfile(const Chain& rcFileName, ListT<ProfileEntry>& pfeList)
 {
     File rcFile(rcFileName);
     if ( rcFile.exists() )
-    {
-	
+    {	
 	rcFile.open(File::READ);
 
 	Chain rcLine;
@@ -1537,12 +1517,12 @@ AbortHandler::AbortHandler(const Chain& serverName,
 			   const Chain& tableSet,
 			   const Chain& user, 
 			   const Chain& password,
-			   long tid, 
+			   unsigned long long tid, 
 			   CegoModule *pModule) :  SigHandler() 
 {
     init();
     install(SIGINT);
-#ifndef HAVE_MINGW32
+#ifndef HAVE_MINGW
     install(SIGPIPE);
 #endif
     _tid = tid;
@@ -1562,45 +1542,35 @@ void AbortHandler::sigCatch(int sig)
 {
     
     try {
+	
 	install(SIGINT);
-#ifndef HAVE_MINGW32
+#ifndef HAVE_MINGW
 	install(SIGPIPE);
 #endif
-
 	if ( _doAbort )
 	{
-
 	    cout << endl << "Aborting query .." << endl;
-	    
 
-	    Net n (  NETMNG_MSG_BUFLEN, NETMNG_SIZEBUFLEN );
-	    NetHandler* pN;
+	    Chain logMode("notice");
+	    if ( debug )
+		logMode=Chain("debug");
 	    
-	    pN = n.connect(_serverName, _portNo);
+	    CegoNet *pCegoNet = new CegoNet( protType, logFile, Chain(""), logMode );	
 	    
-	    CegoDbHandler* pSH = new CegoDbHandler(pN, protType, _pModule);
-	    	    
-	    CegoDbHandler::ResultType res = pSH->requestSession(_tableSet, _user, _password);
-	    if ( res != CegoDbHandler::DB_OK )
+	    try
 	    {
-		Chain msg = pSH->getMsg();
-		throw Exception(EXLOC, msg);
+		pCegoNet->connect(_serverName, _portNo, _tableSet, _user, _password);
 	    }
-    
-	    Chain msg;
-	    	    
-	    res = pSH->reqQueryAbort(_tid);
-	    	    
-	    if ( res == CegoDbHandler::DB_OK )
+	    catch ( Exception e )
 	    {
-		Chain msg = pSH->getMsg();
+		delete pCegoNet;
+		throw Exception(EXLOC, "Cannot connect to database");
 	    }
-	    
-	    pSH->closeSession();
-
-	    delete pSH;
-	    delete pN;
+	    	        
+	    pCegoNet->reqAbortQuery(_tid);	    	    	    
+	    pCegoNet->disconnect();
 
+	    delete pCegoNet;
 	}
 	else
 	{
@@ -1621,7 +1591,6 @@ void AbortHandler::setQueryAbort(bool doAbort)
     _doAbort = doAbort;
 }
 
-
 ProfileEntry::ProfileEntry()
 {
 }
@@ -1652,7 +1621,6 @@ ProfileEntry::~ProfileEntry()
 {
 } 
 
-
 const Chain& ProfileEntry::getId() const
 {
     return _pid;
@@ -1706,3 +1674,4 @@ bool ProfileEntry::operator == ( const ProfileEntry& pe)
 	return true;
     return false;
 }
+
