@@ -8,38 +8,27 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2010 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
-// Class: CegoGlobalSpace 
+// Class: CegoDatabaseManager 
 // 
 // Description: General database manager
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
+// LFC INCLUDES
 #include <lfcbase/ListT.h>
 #include <lfcbase/Chain.h>
 
+// CEGO INCLUDES
 #include "CegoBufferPool.h"
 #include "CegoDistDbHandler.h"
+#include "CegoQueryCache.h"
+#include "CegoTableCache.h"
 #include "CegoDefs.h"
 
 class CegoDatabaseManager : public CegoBufferPool {
@@ -48,11 +37,23 @@ class CegoDatabaseManager : public CegoBufferPool {
 
     enum RecoveryMode { REQOFF, OFF, ON};
 
-    enum ObjectUseMode { SHARED, EXCLUSIVE, EXCLUSIVE_WRITE };
+    enum ObjectUseMode { SHARED, EXCLUSIVE_WRITE };
 
-    CegoDatabaseManager(const Chain& xmlDef, const Chain& lckFileName, const Chain& logFile, CegoDbHandler::ProtocolType protType = CegoDbHandler::SERIAL);
+    CegoDatabaseManager(const Chain& xmlDef, const Chain& lckFileName, const Chain& logFile, const Chain& progName, CegoDbHandler::ProtocolType protType = CegoDbHandler::SERIAL);
     ~CegoDatabaseManager();
 
+    void allocateQueryCache(const Chain& tableSet);
+    void allocateTableCache(const Chain& tableSet);
+    void releaseQueryCache(const Chain& tableSet);
+    void releaseTableCache(const Chain& tableSet);
+
+    CegoQueryCache* getQueryCache(const Chain& tableSet) const;
+    CegoQueryCache* getQueryCache(int tabSetId) const;
+    CegoTableCache* getTableCache(const Chain& tableSet) const;
+    CegoTableCache* getTableCache(int tabSetId) const;
+
+    void cleanCache(int tabSetId, CegoObject::ObjectType objType, const Chain& objName);
+    
     void beat();
 
     void startRecovery(const Chain& tableSet);
@@ -74,11 +75,13 @@ class CegoDatabaseManager : public CegoBufferPool {
     void setCopyStatus(int id, const Chain& msg);
 
     void addObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type);
-    void useObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type, ObjectUseMode mode, long tid = 0 );
-    void unuseObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type, ObjectUseMode mode);
+    void useObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type, ObjectUseMode mode, unsigned long long tid = 0 );
+    void unuseObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type);
     void removeObject(int tabSetId, const Chain& objName, CegoObject::ObjectType type);
 
     bool objectExists(int tabSetId, const Chain& objName, CegoObject::ObjectType type);
+    
+    void printObjectList();
 
     void setThreadInfo(int numDbThread, int numAdmThread, int numLogThread);
     void getThreadInfo(int& numDbThread, int& numAdmThread, int& numLogThread,
@@ -93,10 +96,7 @@ class CegoDatabaseManager : public CegoBufferPool {
     void increaseActiveLogThread();
     void decreaseActiveLogThread();
 
-    Chain getSysFileName(const Chain& tableSet);    
-    Chain getTmpFileName(const Chain& tableSet);
-
-    void getDBMLockStat(Chain& lockName, long& lockCount, long &numRdLock, long &numWrLock, long &sumRdDelay, long &sumWrDelay);
+    void getDBMLockStat(Chain& lockName, int& lockCount, unsigned long long &numRdLock, unsigned long long &numWrLock, unsigned long long &sumRdDelay, unsigned long long &sumWrDelay);
 
     CegoDistDbHandler* allocateSession(const Chain& hostName, const Chain& tableSet, 
 				       const Chain& userName, const Chain& password);
@@ -110,6 +110,7 @@ class CegoDatabaseManager : public CegoBufferPool {
     void configureLogger(Logger::LogLevel level);
     bool isLoggerConfigured();
 
+    bool verifyJDBC(const Chain& user);
     bool verifyAccess(const int tabSetId, const Chain& objName, CegoObject::ObjectType type, CegoXMLSpace::AccessMode mode, const Chain& user);
 
     void initLogFiles(const Chain& tableSet, bool overwrite);
@@ -139,6 +140,9 @@ private:
 
 	const int getTabSetId() const;
 
+	const Chain& getName() const;
+	const CegoObject::ObjectType getType() const;
+
 	void incUsed();
 	void decUsed();
 	int numUsed() const;
@@ -146,8 +150,8 @@ private:
 	void setMode(ObjectUseMode mode); 
 	ObjectUseMode getMode() const;
 
-	void setTid(long tid);	
-	long getTid() const;
+	void setTid(unsigned long long tid);	
+	unsigned long long getTid() const;
 
 	ObjectRecord& operator = ( const ObjectRecord& t);
         bool operator == ( const ObjectRecord& t);
@@ -165,7 +169,7 @@ private:
 	int _tabSetId;
 	int _numUsed;
 	ObjectUseMode _mode;
-	long _tid;
+	unsigned long long _tid;
     };
 
     class CopyRecord {
@@ -267,12 +271,16 @@ private:
 
     int _nextCopyId;
 
-    unsigned long _nextBeat;
+    unsigned long long _nextBeat;
 
     CegoDbHandler::ProtocolType _protType;
 
     Chain _lckFileName;
     bool _logConfigured;
+
+    CegoQueryCache* _pQueryCache[TABMNG_MAXTABSET];
+    CegoTableCache* _pTableCache[TABMNG_MAXTABSET];
+    
     unsigned long _modId;
 };
 
