@@ -6,41 +6,24 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2010 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoDistDbHandler
-// 
-// Description: 
 //
-// Status: QG-2.6
+// Description: Database client interface to the access the database backend via network 
+//
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
-// base includes
+// LFC INCLUDES
 #include <lfcbase/Datetime.h>
 #include <lfcbase/NetHandler.h>
-
-// xml includes
 #include <lfcxml/Element.h>
 
-// cego includes
+// CEGO INCLUDES
 #include "CegoDistDbHandler.h"
 #include "CegoTypeConverter.h"
 #include "CegoXMLdef.h"
@@ -54,11 +37,9 @@ CegoDistDbHandler::~CegoDistDbHandler()
 {
 }
 
-
 void CegoDistDbHandler::getDeleteArg(Chain& tableSet, Chain& tableName, CegoPredDesc*& pPred,
 				     CegoDistManager *pGTM)
 {
-
     if ( _protType == CegoDbHandler::XML )
     {
 	Element *pRoot = _xml.getDocument()->getRootElement();
@@ -83,7 +64,6 @@ void CegoDistDbHandler::getDeleteArg(Chain& tableSet, Chain& tableName, CegoPred
     }
 }
 
-
 void CegoDistDbHandler::getUpdateArg(Chain& tableSet, 
 				     Chain& tableName,  
 				     ListT<CegoField>& fl,
@@ -91,7 +71,6 @@ void CegoDistDbHandler::getUpdateArg(Chain& tableSet,
 				     CegoPredDesc*& pPred,
 				     CegoDistManager *pGTM)
 {
-
     if ( _protType == CegoDbHandler::XML )
     {
 	Element *pRoot = _xml.getDocument()->getRootElement();
@@ -125,10 +104,8 @@ void CegoDistDbHandler::getUpdateArg(Chain& tableSet,
     }	
 }
 
-
 void CegoDistDbHandler::getCreateTableArg(Chain& tableSet, Chain& tableName, ListT<CegoField>& fl, ListT<CegoField>& idxList)
-{
-    
+{    
     if ( _protType == CegoDbHandler::XML )
     {
 	
@@ -158,8 +135,7 @@ void CegoDistDbHandler::getCreateTableArg(Chain& tableSet, Chain& tableName, Lis
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
 void CegoDistDbHandler::getCreateViewArg(Chain& tableSet, Chain& viewName, ListT<CegoField>& fl, Chain& viewText)
@@ -194,12 +170,10 @@ void CegoDistDbHandler::getCreateViewArg(Chain& tableSet, Chain& viewName, ListT
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
     }	
-
 }
 
 void CegoDistDbHandler::getCreateProcedureArg(Chain& tableSet, Chain& procName, Chain& procText)
 {
-
     if ( _protType == CegoDbHandler::XML )
     {
 	
@@ -214,18 +188,14 @@ void CegoDistDbHandler::getCreateProcedureArg(Chain& tableSet, Chain& procName,
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
-
 void CegoDistDbHandler::getCreateCheckArg(Chain& tableSet, Chain& checkName, Chain& tableName, CegoPredDesc*& pPred,
 				     CegoDistManager *pGTM)
 {
-
     if ( _protType == CegoDbHandler::XML )
     {
-	
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
@@ -279,10 +249,8 @@ void CegoDistDbHandler::getAlterTableArg(Chain& tableSet, Chain& tableName, List
 
 void CegoDistDbHandler::getDropTableArg(Chain& tableSet, Chain& tableName, CegoObject::ObjectType& type)
 {
-
     if ( _protType == CegoDbHandler::XML )
     {
-	
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
@@ -295,16 +263,13 @@ void CegoDistDbHandler::getDropTableArg(Chain& tableSet, Chain& tableName, CegoO
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
 void CegoDistDbHandler::getCreateIndexArg(Chain& tableSet, Chain& indexName, Chain& tableName, ListT<CegoField>& idxList, CegoObject::ObjectType& type)
 {
-
     if ( _protType == CegoDbHandler::XML )
-    {
-    
+    {   
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
@@ -326,27 +291,23 @@ void CegoDistDbHandler::getCreateIndexArg(Chain& tableSet, Chain& indexName, Cha
 	    }
 	    
 	    if ( pRoot->getAttributeValue(XML_INDEXTYPE_ATTR) == Chain(XML_INDEX_VALUE) )
-		type = CegoObject::INDEX;
+		type = CegoObject::AVLTREE;
 	    else if ( pRoot->getAttributeValue(XML_INDEXTYPE_ATTR) == Chain(XML_PINDEX_VALUE) )
-		type = CegoObject::PINDEX;
+		type = CegoObject::PAVLTREE;
 	    else if ( pRoot->getAttributeValue(XML_INDEXTYPE_ATTR) == Chain(XML_UINDEX_VALUE) )
-		type = CegoObject::UINDEX;
-	    
+		type = CegoObject::UAVLTREE;	    
 	}
     }
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
-
 void CegoDistDbHandler::getObjectInfoArg(int& tabSetId, Chain& tableName, CegoObject::ObjectType& type)
 {
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
@@ -360,15 +321,13 @@ void CegoDistDbHandler::getObjectInfoArg(int& tabSetId, Chain& tableName, CegoOb
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
 void CegoDistDbHandler::getGetTableArg(int& tabSetId, Chain& tableName, CegoObject::ObjectType& type)
 {
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
@@ -383,16 +342,13 @@ void CegoDistDbHandler::getGetTableArg(int& tabSetId, Chain& tableName, CegoObje
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
 void CegoDistDbHandler::getRenameArg(Chain& tableSet, Chain& objName, CegoObject::ObjectType& type, Chain& newObjName)
 {
-
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
@@ -409,17 +365,13 @@ void CegoDistDbHandler::getRenameArg(Chain& tableSet, Chain& objName, CegoObject
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
-    
+    }
 }
 
 void CegoDistDbHandler::getReorgArg(Chain& tableSet, Chain& objName, CegoObject::ObjectType& type)
 {
-
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
@@ -433,39 +385,32 @@ void CegoDistDbHandler::getReorgArg(Chain& tableSet, Chain& objName, CegoObject:
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
 void CegoDistDbHandler::getGetObjectListArg(int& tabSetId, CegoObject::ObjectType& type)
 {
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
 	    tabSetId = pRoot->getAttributeValue(XML_TSID_ATTR).asInteger();
 	    
 	    CegoTypeConverter tc;
-	    type = tc.getObjectTypeId(pRoot->getAttributeValue(XML_TABLETYPE_ATTR));
-	    
-	    
+	    type = tc.getObjectTypeId(pRoot->getAttributeValue(XML_TABLETYPE_ATTR));	    
 	}
     }
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
 void CegoDistDbHandler::getGetObjectByTableListArg(Chain& tableSet, Chain& tableName)
 {
-    
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
@@ -476,15 +421,13 @@ void CegoDistDbHandler::getGetObjectByTableListArg(Chain& tableSet, Chain& table
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
     
 void CegoDistDbHandler::getSyncArg(Chain& tableSet, Chain& escCmd, int& timeout)
 {
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
@@ -496,15 +439,13 @@ void CegoDistDbHandler::getSyncArg(Chain& tableSet, Chain& escCmd, int& timeout)
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
 void CegoDistDbHandler::getPageCountArg(Chain& tableSet, Chain& objName, CegoObject::ObjectType& type)
 {
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
@@ -518,15 +459,13 @@ void CegoDistDbHandler::getPageCountArg(Chain& tableSet, Chain& objName, CegoObj
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqInsertOp(const Chain& tableSet, const Chain& tableName, const ListT<CegoField>& fl)
 {
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	_xml.getDocument()->clear();
 	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
@@ -581,8 +520,7 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqInsertOp(const Chain& tableS
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqUpdateOp(const Chain& tableSet, 
@@ -592,9 +530,7 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqUpdateOp(const Chain& tableS
 							     CegoPredDesc* pPred)
 {   
     if ( _protType == CegoDbHandler::XML )
-    {
-	
-	
+    {	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	
 	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
@@ -613,26 +549,20 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqUpdateOp(const Chain& tableS
 	    pExpr = exprList.Next();
 	}
 	
-	
 	pRoot->addContent(pPred->toElement());
 	
-	
 	return sendXMLReq(XML_UPDATE_REQUEST, pRoot);
     }
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
     }	
-    
-
 }
 
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqDeleteOp(const Chain& tableSet, const Chain& tableName, CegoPredDesc* pPred)
 {    
-    
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	
 	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
@@ -645,18 +575,14 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqDeleteOp(const Chain& tableS
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-    
-
+    }
 }
 
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqCreateTableOp(const Chain& tableSet, const Chain& tableName, CegoObject::ObjectType type,
 								  const ListT<CegoField>& fl, const ListT<CegoField>& idxList)
 {
-    
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	
 	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
@@ -675,23 +601,18 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqCreateTableOp(const Chain& t
 	    pRoot->addContent(pColElement);
 	    pF = fl.Next();
 	}
-	
 	return sendXMLReq(XML_CREATETABLE_REQUEST, pRoot);    
-
     }
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqAlterTableOp(const Chain& tableSet, const Chain& tableName, const ListT<CegoAlterDesc>& alterList)
 {
-
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	
 	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
@@ -704,8 +625,7 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqAlterTableOp(const Chain& ta
 	    pAD = alterList.Next();
 	}
 	
-	return sendXMLReq(XML_ALTERTABLE_REQUEST, pRoot);    
-	
+	return sendXMLReq(XML_ALTERTABLE_REQUEST, pRoot);    	
     }
     else
     {
@@ -718,24 +638,21 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqCreateIndexOp(const Chain& t
 								  const Chain& tableName, 
 								  CegoObject::ObjectType type, 
 								  const ListT<CegoField>& idxList)
-{    
-    
+{
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	
 	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
 	pRoot->setAttribute(XML_INDEXNAME_ATTR, indexName);
 	pRoot->setAttribute(XML_TABLENAME_ATTR, tableName);
-	if ( type == CegoObject::INDEX )
+	if ( type == CegoObject::AVLTREE )
 	    pRoot->setAttribute(XML_INDEXTYPE_ATTR, XML_INDEX_VALUE);
-	else if ( type == CegoObject::PINDEX )
+	else if ( type == CegoObject::PAVLTREE )
 	    pRoot->setAttribute(XML_INDEXTYPE_ATTR, XML_PINDEX_VALUE);
-	else if ( type == CegoObject::UINDEX )
+	else if ( type == CegoObject::UAVLTREE )
 	    pRoot->setAttribute(XML_INDEXTYPE_ATTR, XML_UINDEX_VALUE);
-	
-	
+		
 	CegoField *pF = idxList.First();
 	while ( pF ) 
 	{
@@ -751,8 +668,7 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqCreateIndexOp(const Chain& t
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqCreateFKeyOp(int tabSetId, const Chain& fkey, 
@@ -761,18 +677,15 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqCreateFKeyOp(int tabSetId, c
 								 const Chain& refTable, 
 								 const ListT<CegoField>& refList)
 {
-    
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	
 	pRoot->setAttribute(XML_TSID_ATTR, Chain(tabSetId));
 	pRoot->setAttribute(XML_FKEY_ATTR, fkey);
 	pRoot->setAttribute(XML_TABLENAME_ATTR, tableName);
 	pRoot->setAttribute(XML_REFTABLENAME_ATTR, refTable);
-	
-	
+		
 	CegoField *pK = keyList.First();
 	while ( pK ) 
 	{
@@ -803,10 +716,8 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqCreateFKeyOp(int tabSetId, c
 
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqTableDataOp(int tabSetId, const Chain& tableName, CegoObject::ObjectType type)
 {
-    
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	pRoot->setAttribute(XML_TSID_ATTR, Chain(tabSetId));
 	pRoot->setAttribute(XML_TABLENAME_ATTR, tableName);
@@ -815,21 +726,17 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqTableDataOp(int tabSetId, co
 	pRoot->setAttribute(XML_TABLETYPE_ATTR,  tc.getObjectTypeString(type));
 	
 	return sendXMLReq(XML_GETTABLE_REQUEST, pRoot);
-
     }
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqGetObjectListOp(int tabSetId, CegoObject::ObjectType type)
 {
-
     if ( _protType == CegoDbHandler::XML )
     {
-
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	pRoot->setAttribute(XML_TSID_ATTR, Chain(tabSetId));
 	
@@ -837,42 +744,33 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqGetObjectListOp(int tabSetId
 	pRoot->setAttribute(XML_TABLETYPE_ATTR, tc.getObjectTypeString(type));
 	
 	return sendXMLReq(XML_GETOBJLIST_REQUEST, pRoot);
-
     }
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqGetObjectByTableListOp(const Chain& tableSet, const Chain& tableName)
 {
-    
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
 	pRoot->setAttribute(XML_TABLENAME_ATTR, tableName);
 	
 	return sendXMLReq(XML_GETOBJLISTBYTABLE_REQUEST, pRoot);
-    
     }
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
-
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqRenameOp(const Chain& tableSet, const Chain& objName, CegoObject::ObjectType type, const Chain& newObjName)
 {
-
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
 	pRoot->setAttribute(XML_OBJNAME_ATTR, objName);
@@ -882,21 +780,17 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqRenameOp(const Chain& tableS
 	pRoot->setAttribute(XML_NEWOBJNAME_ATTR, newObjName);
        	
 	return sendXMLReq(XML_OBJRENAME_REQUEST, pRoot);
-
     }
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqSyncOp(const Chain& tableSet, const Chain& escCmd, int timeout)
 {
-
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
 	pRoot->setAttribute(XML_ESCCMD_ATTR, escCmd);
@@ -907,16 +801,13 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqSyncOp(const Chain& tableSet
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-    
+    }
 }
 
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqGetPageCount(const Chain& tableSet, const Chain& objName, CegoObject::ObjectType type)
 {
-
     if ( _protType == CegoDbHandler::XML )
     {
-
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
 	pRoot->setAttribute(XML_OBJNAME_ATTR, objName);
@@ -930,12 +821,10 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqGetPageCount(const Chain& ta
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
     }	
-
 }
 
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqReorgObjectOp(const Chain& tableSet, const Chain& objName, CegoObject::ObjectType type)
 {
-    
     if ( _protType == CegoDbHandler::XML )
     {
 	
@@ -951,14 +840,11 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqReorgObjectOp(const Chain& t
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
-
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqCreateCheckOp(const Chain& tableSet, const Chain& checkName, const Chain& tableName, CegoPredDesc* pPred)
 {    
-
     if ( _protType == CegoDbHandler::XML )
     {
 	
@@ -969,23 +855,18 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqCreateCheckOp(const Chain& t
 	pRoot->setAttribute(XML_TABLENAME_ATTR, tableName);    
 	pRoot->addContent(pPred->toElement());
 	
-	return sendXMLReq(XML_CREATECHECK_REQUEST, pRoot);    
-	
+	return sendXMLReq(XML_CREATECHECK_REQUEST, pRoot);	
     }
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
-
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqObjectInfoOp(int tabSetId, const Chain& objName, CegoObject::ObjectType type)
 {
-
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 
 	pRoot->setAttribute(XML_TSID_ATTR, Chain(tabSetId));
@@ -995,20 +876,17 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqObjectInfoOp(int tabSetId, c
 	pRoot->setAttribute(XML_OBJTYPE_ATTR, tc.getObjectTypeString(type));
 	
 	return sendXMLReq(XML_OBJECTINFO_REQUEST, pRoot);
-     
     }
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqDropObjectOp(const Chain& tableSet, const Chain& objName, CegoObject::ObjectType type)
 {
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	
 	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
@@ -1018,21 +896,17 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqDropObjectOp(const Chain& ta
 	pRoot->setAttribute(XML_OBJTYPE_ATTR, tc.getObjectTypeString(type));
 	
 	return sendXMLReq(XML_DROP_OBJECT_REQUEST, pRoot);
-    
     }
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-    
+    }
 }
 
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqCreateViewOp(const Chain& tableSet, const Chain& viewName, const ListT<CegoField>& fl, const Chain& viewText)
 {
-    
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
 	pRoot->setAttribute(XML_VIEWNAME_ATTR, viewName);
@@ -1057,15 +931,13 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqCreateViewOp(const Chain& ta
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqCreateProcOp(const Chain& tableSet, const Chain& procName, const Chain& procText)
 {
     if ( _protType == CegoDbHandler::XML )
     {
-
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
 	pRoot->setAttribute(XML_PROCNAME_ATTR, procName);
@@ -1075,16 +947,13 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqCreateProcOp(const Chain& ta
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-    
+    }
 }
 
-
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqStartTransactionOp(const Chain& tableSet)
 {
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
 	return sendXMLReq(XML_STARTTRANSACTION_REQUEST, pRoot);       
@@ -1092,31 +961,27 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqStartTransactionOp(const Cha
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-    
+    }
 }
 
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqCommitTransactionOp(const Chain& tableSet)
 {
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
-	return sendXMLReq(XML_COMMITTRANSACTION_REQUEST, pRoot);        
- 
+	return sendXMLReq(XML_COMMITTRANSACTION_REQUEST, pRoot);
     }
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
     }	
 }
-    
+
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqRollbackTransactionOp(const Chain& tableSet)
 {
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
 	return sendXMLReq(XML_ROLLBACKTRANSACTION_REQUEST, pRoot);        
@@ -1124,34 +989,27 @@ CegoDistDbHandler::ResultType CegoDistDbHandler::reqRollbackTransactionOp(const
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
 CegoDistDbHandler::ResultType CegoDistDbHandler::reqGetTidOp(const Chain& tableSet)
 {
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);
 	return sendXMLReq(XML_GETTID_REQUEST, pRoot);        
-
     }
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
-    }	
-
+    }
 }
 
-
 void CegoDistDbHandler::getArgValue(const Chain& arg, Chain& value)
 {
-
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
@@ -1162,15 +1020,12 @@ void CegoDistDbHandler::getArgValue(const Chain& arg, Chain& value)
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
     }	
-
 }
 
 void CegoDistDbHandler::getInsertArg(Chain& tableSet, Chain& tableName, ListT<CegoField>& fl)
 {
-
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
@@ -1210,14 +1065,12 @@ void CegoDistDbHandler::getInsertArg(Chain& tableSet, Chain& tableName, ListT<Ce
 
 void CegoDistDbHandler::sendObjList(const ListT<Chain>& objList)
 {
-    
 #ifdef DEBUG
     _pModule->log(_modId, Logger::DEBUG, Chain("Sending object info"));
 #endif
 
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	_xml.getDocument()->clear();
 	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
@@ -1244,12 +1097,10 @@ void CegoDistDbHandler::sendObjList(const ListT<Chain>& objList)
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
 #endif
 	
-	
 	_pN->setMsg((char*)response, response.length());
 	_pN->writeMsg();
 	
-	_xml.getDocument()->clear();
-	
+	_xml.getDocument()->clear();	
     }
     else
     {
@@ -1257,15 +1108,12 @@ void CegoDistDbHandler::sendObjList(const ListT<Chain>& objList)
     }
 }
 
-
 void CegoDistDbHandler::sendObjByTableList(const ListT<CegoTableObject>& idxList,
 					   const ListT<CegoKeyObject>& keyList,
 					   const ListT<CegoCheckObject>& checkList)
-{
-    
+{    
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	_xml.getDocument()->clear();
 	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
@@ -1308,18 +1156,15 @@ void CegoDistDbHandler::sendObjByTableList(const ListT<CegoTableObject>& idxList
 	_pN->writeMsg();
 	
 	_xml.getDocument()->clear();
-    
     }
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
     }
-
 }
 
 void CegoDistDbHandler::sendTID(int tid)
 {
-    
     if ( _protType == CegoDbHandler::XML )
     {
 	
@@ -1340,27 +1185,22 @@ void CegoDistDbHandler::sendTID(int tid)
 	_pModule->log(_modId, Logger::DEBUG, response);
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
 #endif
-	
-	
+		
 	_pN->setMsg((char*)response, response.length());
 	_pN->writeMsg();
 	
 	_xml.getDocument()->clear();
-	
     }
     else
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
     }
-
 }
 
 void CegoDistDbHandler::sendPageCount(int pageCount)
 {
-
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	_xml.getDocument()->clear();
 	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
@@ -1388,21 +1228,16 @@ void CegoDistDbHandler::sendPageCount(int pageCount)
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
     }
-
 }
 
-
-
 void CegoDistDbHandler::getObjectList(ListT<Chain>& objList)
-{
-    
+{   
     if ( _protType == CegoDbHandler::XML )
     {
 	
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
-	{
-	    
+	{	    
 	    ListT<Element*> objElementList = pRoot->getChildren(XML_OBJ_ELEMENT);
 	    
 	    Element **pObjEntry = objElementList.First();
@@ -1417,21 +1252,17 @@ void CegoDistDbHandler::getObjectList(ListT<Chain>& objList)
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
     }
-
 }
 
 void CegoDistDbHandler::getObjectByTableList(ListT<CegoTableObject>& idxList, 
 					     ListT<CegoKeyObject>& keyList,
 					     ListT<CegoCheckObject>& checkList)
 {
-
     if ( _protType == CegoDbHandler::XML )
-    {
-	
+    {	
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
-	    
 	    ListT<Element*> objElementList = pRoot->getChildren(XML_OBJ_ELEMENT);
 	    
 	    Element **pObjEntry = objElementList.First();
@@ -1443,9 +1274,9 @@ void CegoDistDbHandler::getObjectByTableList(ListT<CegoTableObject>& idxList,
 		
 		switch ( type )
 		{
-		case CegoObject::INDEX:
-		case CegoObject::PINDEX:
-		case CegoObject::UINDEX:
+		case CegoObject::AVLTREE:
+		case CegoObject::PAVLTREE:
+		case CegoObject::UAVLTREE:
 		{
 		    CegoTableObject io;
 		    io.putElement(*pObjEntry);
@@ -1480,14 +1311,12 @@ void CegoDistDbHandler::getObjectByTableList(ListT<CegoTableObject>& idxList,
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
     }
-
 }
 
 void CegoDistDbHandler::getPageCount(int& pageCount)
 {
     if ( _protType == CegoDbHandler::XML )
     {
-
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
@@ -1498,5 +1327,4 @@ void CegoDistDbHandler::getPageCount(int& pageCount)
     {
 	throw Exception(EXLOC, Chain("Serial protocol still not supported"));
     }
-
 }
