@@ -8,34 +8,23 @@
 //
 // Design and Implementation by Bjoern Lemke               
 //
-// (C)opyright 2000-2010 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
 // Class: CegoExpr
 // 
-// Description: 
+// Description: Query expression container class 
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
-// INCLUDES
+// LFC INCLUDES
+#include <lfcbase/ListT.h>
+#include <lfcbase/SetT.h>
+
+// CEGO INCLUDES
 #include "CegoTerm.h"
 #include "CegoProcStmt.h"
 
@@ -43,12 +32,12 @@ class CegoDistManager;
 
 class CegoExpr {
 
- public:
+public:
 
     enum ExpType { ADD, SUB, TERM, CONCAT  };
-
+    
     CegoExpr();
-    CegoExpr(char* buf, CegoDistManager *pGTM);
+    CegoExpr(char* buf, CegoDistManager *pGTM, int tabSetId);
     CegoExpr(Element* pExpElement, CegoDistManager *pGTM);
     CegoExpr(CegoExpr *pExpr, CegoTerm *pTerm, ExpType expType);
     CegoExpr(CegoTerm *pTerm);
@@ -58,13 +47,14 @@ class CegoExpr {
     void setAlias(const Chain& alias);
     const Chain& getAlias() const;
 
-    bool getTableDesc(CegoTableDesc& td) const;
-
     bool checkVar(Chain& var) const;
     CegoAttrDesc* checkAttr() const;
 
+    void getFunctionList(SetT<CegoObject>& funcList);
+    
     void setBlock(CegoProcBlock *pBlock);
-    void setFieldListArray(ListT<CegoField> *fla, int size);
+    void setFieldListArray(ListT<CegoField>* pFL);
+    void setFieldListArray(ListT<CegoField> **pFLA);
 
     ListT<CegoAttrDesc*> getAttrRefList() const;
     int evalReferences(CegoContentObject *pCO, const ListT<CegoField>& fl);
@@ -81,32 +71,39 @@ class CegoExpr {
     CegoTerm *getTerm() const;
     CegoExpr *getExpr() const;
    
-    ListT<CegoAggregation*> getAggregationList();
+    ListT<CegoAggregation*>& getAggregationList();
 
     CegoExpr* clone(bool isAttrRef = false);
 
     CegoExpr& operator = ( const CegoExpr& e);
 
+    Chain getId() const;
     Chain toChain(const Chain& indent = Chain("")) const;
+    Chain dbFormat(CegoDatabaseFormater *pForm) const;
+
     Element* toElement() const;
     void fromElement(Element *pExpElement, CegoDistManager *pGTM);
 
     friend ostream& operator << (ostream& s, const CegoExpr& e);
 
     void encode(char *buf);
-    void decode(char *buf, CegoDistManager *pGTM);
+    void decode(char *buf, CegoDistManager *pGTM, int tabSetId);
     int getEncodingLength() const;
 
     void getPlanList(ListT<Element*>& planList);
 
- private:
-
+private:
+    
+    bool _isAgg;
+    ListT<CegoAggregation*> _aggList;
+    
     ExpType _expType;
 
     Chain _alias;
     CegoExpr *_pExpr;
     CegoTerm *_pTerm;
 
-};
+    ListT<CegoField>* _fla[2];
 
+};
 #endif
