@@ -6,22 +6,7 @@
 //
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2012 Bjoern Lemke                        
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke 
 //
 // IMPLEMENTATION MODULE
 //
@@ -29,7 +14,7 @@
 // 
 // Description: Container class to describe a factor element
 //
-// Status: QS-2.6
+// Status: CLEAN
 // 
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -43,10 +28,12 @@
 #include "CegoXMLdef.h"
 #include "CegoTypeConverter.h"
 
+#include "CegoDatabaseFormater.h"
+
 #include <string.h>
 #include <stdlib.h>
 
-CegoFactor::CegoFactor(char* buf, CegoDistManager *pGTM)
+CegoFactor::CegoFactor(char* buf, CegoDistManager *pGTM, int tabSetId)
 {
     _pAttrDesc = 0;
     _pExpr = 0;
@@ -55,12 +42,12 @@ CegoFactor::CegoFactor(char* buf, CegoDistManager *pGTM)
     _pAggr = 0;
     _pFunction = 0;
     _pSelect = 0;
-    _fla = 0;
+    _pFLA = 0;
     _pCaseCond = 0;
 
     _flaCached = false;
 
-    decode(buf, pGTM);
+    decode(buf, pGTM, tabSetId);
 }
 
 CegoFactor::CegoFactor(Element* pFactorElement, CegoDistManager *pGTM)
@@ -72,7 +59,7 @@ CegoFactor::CegoFactor(Element* pFactorElement, CegoDistManager *pGTM)
     _pAggr = 0;
     _pFunction = 0;
     _pSelect = 0;
-    _fla = 0;
+    _pFLA = 0;
     _pCaseCond = 0;
 
     _flaCached = false;
@@ -96,7 +83,7 @@ CegoFactor::CegoFactor(const Chain& varName)
     _pAggr = 0;
     _pFunction = 0;
     _pSelect = 0;
-    _fla = 0;
+    _pFLA = 0;
     _pCaseCond = 0;
 
     _flaCached = false;
@@ -113,7 +100,7 @@ CegoFactor::CegoFactor(const CegoFieldValue& fv)
     _pAggr = 0;
     _pFunction = 0;
     _pSelect = 0;
-    _fla = 0;
+    _pFLA = 0;
     _pCaseCond = 0;
 
     _flaCached = false;
@@ -129,7 +116,7 @@ CegoFactor::CegoFactor(CegoExpr *pExpr)
     _pAggr = 0;
     _pFunction = 0;
     _pSelect = 0;
-    _fla = 0;
+    _pFLA = 0;
     _pCaseCond = 0;
 
     _flaCached = false;
@@ -145,7 +132,7 @@ CegoFactor::CegoFactor(CegoProcFetch *pFetch)
     _pAggr = 0;
     _pFunction = 0;
     _pSelect = 0;
-    _fla = 0;
+    _pFLA = 0;
     _pCaseCond = 0;
 
     _flaCached = false;
@@ -162,7 +149,7 @@ CegoFactor::CegoFactor(CegoAttrDesc* pAttrDesc, bool isAttrRef)
     _pAggr = 0;
     _pFunction = 0;
     _pSelect = 0;
-    _fla = 0;
+    _pFLA = 0;
     _pCaseCond = 0;
 
     _flaCached = false;
@@ -178,7 +165,7 @@ CegoFactor::CegoFactor(CegoFunction* pFunction)
     _pFetch = 0;
     _pAggr = 0;
     _pSelect = 0;
-    _fla = 0;
+    _pFLA = 0;
     _pCaseCond = 0;
     _flaCached = false;
 }
@@ -193,7 +180,7 @@ CegoFactor::CegoFactor(CegoAggregation* pAggr)
     _pExpr = 0;
     _pFunction = 0;
     _pSelect = 0;
-    _fla = 0;
+    _pFLA = 0;
     _pCaseCond = 0;
     _flaCached = false;
 }
@@ -206,7 +193,7 @@ CegoFactor::CegoFactor(CegoSelect* pSelect)
     _pExpr = 0;
     _pFetch = 0;
     _pAggr = 0;
-    _fla = 0;
+    _pFLA = 0;
     _pFunction = 0;
     _pSelect = pSelect;
     _pCaseCond = 0;
@@ -221,14 +208,13 @@ CegoFactor::CegoFactor(CegoCaseCond* pCaseCond)
     _pExpr = 0;
     _pFetch = 0;
     _pAggr = 0;
-    _fla = 0;
+    _pFLA = 0;
     _pFunction = 0;
     _pSelect = 0;
     _pCaseCond = pCaseCond;
     _flaCached = false;
 }
 
-
 CegoFactor::~CegoFactor()
 {
     if ( _pAttrDesc && _isAttrRef == false)
@@ -313,51 +299,48 @@ void CegoFactor::clearAttrCache()
 	_pCaseCond->clearAttrCache();
 }
 
-bool CegoFactor::getTableDesc(CegoTableDesc& td) const
+void CegoFactor::getFunctionList(SetT<CegoObject>& funcList)
 {
-    if ( _type == ATTR )
+    if ( _type == FUNCTION )
     {
-	td.setTableAlias(_pAttrDesc->getTableName());
-	return true;
+	funcList.Insert(CegoObject(CegoObject::PROCEDURE, _pFunction->getName(), _pFunction->getTabSetId()));
     }
-    return false;
 }
 
-void CegoFactor::setFieldListArray(ListT<CegoField> *fla, int size)
+void CegoFactor::setFieldListArray(ListT<CegoField>** pFLA)
 {
     // cout << "Setting field list of size " << size  << endl;
     switch ( _type ) 
     {
     case ATTR:
     {
-	_fla = fla;
-	_flaSize = size;
+	_pFLA = pFLA;
 	break;
     }
     case EXPR:
     {
-	_pExpr->setFieldListArray(fla, size);
+	_pExpr->setFieldListArray(pFLA);
 	break;
     }
     case FUNCTION:
     {
-	_pFunction->setFieldListArray(fla, size);
+	_pFunction->setFieldListArray(pFLA);
 	break;
     }
     case AGGREGATION:
     {
-	_pAggr->setFieldListArray(fla, size);
+	_pAggr->setFieldListArray(pFLA);
 	break;
     }
     case QUERY:
     {
-	_pSelect->setParentJoinBuf(fla, size);
+	_pSelect->setParentJoinBuf(pFLA);
 	_pSelect->prepare(); // not necessary
 	break;
     }
     case CASECOND:
     {
-	_pCaseCond->setFieldListArray(fla, size);
+	_pCaseCond->setFieldListArray(pFLA);
 	break;
     }
     default:
@@ -380,7 +363,7 @@ CegoFactor& CegoFactor::operator = ( const CegoFactor& pf)
     _pBlock = pf._pBlock;
     _pCaseCond = pf._pCaseCond;
     _flaCached = pf._flaCached;
-
+    _pFLA = pf._pFLA;
     return (*this);
 }
 
@@ -438,7 +421,7 @@ ListT<CegoField> CegoFactor::getFieldList() const
 	
     case CegoFactor::ATTR:
     {
-	if ( _fla == 0)
+	if ( _pFLA == 0)
 	{
 	    Chain msg = Chain("Cannot get value for attribute " + _pAttrDesc->toChain());
 	    throw Exception(EXLOC, msg);
@@ -446,12 +429,11 @@ ListT<CegoField> CegoFactor::getFieldList() const
 	
 	int i=0;
 	CegoField* pF = 0;
-	while ( i < _flaSize && pF == 0 )
+	while ( _pFLA[i] && pF == 0 )
 	{		    
-	    if ( ( pF = _fla[i].Find(CegoField(_pAttrDesc->getTableName(), _pAttrDesc->getAttrName()))) != 0 )
+	    if ( ( pF = _pFLA[i]->Find(CegoField(_pAttrDesc->getTableName(), _pAttrDesc->getAttrName()))) != 0 )
 	    {
-		fl.Insert(*pF);
-		
+		fl.Insert(*pF);	       
 	    }
 	    i++;
 	}
@@ -541,7 +523,7 @@ int CegoFactor::evalReferences(CegoContentObject *pCO, const ListT<CegoField>& f
 
     switch ( _type )
     {
-	
+
     case CegoFactor::ATTR:
     {
 	refCount += _pAttrDesc->evalReferences(pCO, fl);
@@ -566,8 +548,8 @@ int CegoFactor::evalReferences(CegoContentObject *pCO, const ListT<CegoField>& f
 	break;
     }
     case CegoFactor::QUERY:
-    {
-	_pSelect->prepare();
+    {	
+	_pSelect->prepare();	
 	refCount += _pSelect->evalExtTableReferences(pCO, fl);
 	break;
     }
@@ -613,7 +595,7 @@ CegoField CegoFactor::evalField() const
     }
     case CegoFactor::ATTR:
     {
-	if ( _fla == 0)
+	if ( _pFLA == 0)
 	{
        	
 	    f = CegoField(_pAttrDesc->getTableName(), 
@@ -627,9 +609,19 @@ CegoField CegoFactor::evalField() const
 	    CegoField *pF = 0;
 	    
 	    int i=0;
-	    while ( i < _flaSize && pF == 0)
+	    while ( _pFLA[i] && pF == 0)
 	    {
-		if ( ( pF = _fla[i].Find(CegoField(_pAttrDesc->getTableName(), _pAttrDesc->getAttrName()))) != 0 )
+
+		/*
+		CegoField *pXF = _fla[i].First();
+		while ( pXF )
+		{
+		    cout << "FLA : Table = " << pXF->getTableName() << "." << pXF->getAttrName() << endl;
+		    pXF = _fla[i].Next();
+		}
+		*/
+		
+		if ( ( pF = _pFLA[i]->Find(CegoField(_pAttrDesc->getTableName(), _pAttrDesc->getAttrName()))) != 0 )
 		{
 		    f = CegoField(_pAttrDesc->getTableName(), 
 				  _pAttrDesc->getTableName(),
@@ -651,7 +643,7 @@ CegoField CegoFactor::evalField() const
     }
     case CegoFactor::FUNCTION:
     {
-	f = CegoField("FUNC", "FUNC", _pFunction->toChain(), VARCHAR_TYPE, 20 );
+	f = CegoField("FUNC", "FUNC", _pFunction->toChain(), _pFunction->getReturnType(), _pFunction->getReturnTypeLen());
 	break;
     }
     case CegoFactor::AGGREGATION:
@@ -660,7 +652,14 @@ CegoField CegoFactor::evalField() const
 	if ( _pAggr->getExpr() )
 	{
 	    CegoField af = _pAggr->getExpr()->evalField();
-	    f = CegoField("AGGR", "AGGR", _pAggr->toChain(), af.getType(), af.getLength() );
+	    if ( _pAggr->getType() == CegoAggregation::COUNT )
+	    {
+		f = CegoField("AGGR", "AGGR", _pAggr->toChain(), INT_TYPE,  4);
+	    }
+	    else
+	    {
+		f = CegoField("AGGR", "AGGR", _pAggr->toChain(), af.getType(), af.getLength() );
+	    }
 	}
 	else
 	{
@@ -723,24 +722,25 @@ CegoFieldValue CegoFactor::evalFieldValue()
     }
     case CegoFactor::ATTR:
     {
-	if ( _flaCached )
-	{
-	    // cout << "Using cached field for " << _pAttrDesc->toChain() << " with pos " << _flaCacheArrayPos << " / " << _flaCacheFieldPos << endl;
-	    return _fla[_flaCacheArrayPos][_flaCacheFieldPos].getValue();
-	}
-	if ( _fla == 0)
+	if ( _pFLA == 0 )
 	{
 	    Chain msg = Chain("Cannot get value for attribute " + _pAttrDesc->toChain());
 	    throw Exception(EXLOC, msg);
 	}
+
+	if ( _flaCached )
+	{
+	    // cout << "Using cached field for " << _pAttrDesc->toChain() << " with pos " << _flaCacheArrayPos << " / " << _flaCacheFieldPos << endl;	    
+	    return (*_pFLA[_flaCacheArrayPos])[_flaCacheFieldPos].getValue();
+	}
 	
 	CegoField *pF = 0;
 	
 	_flaCacheArrayPos=0;
-	while ( _flaCacheArrayPos < _flaSize && pF == 0)
+	while ( _pFLA[_flaCacheArrayPos] && pF == 0)
 	{
 
-	    CegoField *pF = _fla[_flaCacheArrayPos].First();
+	    CegoField *pF = _pFLA[_flaCacheArrayPos]->First();
 
 	    _flaCacheFieldPos = 0;
 	    while ( pF )
@@ -750,40 +750,14 @@ CegoFieldValue CegoFactor::evalFieldValue()
 		    _flaCached = true;
 		    return pF->getValue();
 		}  
-		pF = _fla[_flaCacheArrayPos].Next();
+		pF = _pFLA[_flaCacheArrayPos]->Next();
 		_flaCacheFieldPos++;
 	    }
-
-	    /*
-	    if ( ( pF = _fla[i].Find(CegoField(_pAttrDesc->getTableName(), _pAttrDesc->getAttrName()))) != 0 )
-	    {
-		_pCacheField = pF;
-
-		return pF->getValue();
-	    }
-	    */
 	    _flaCacheArrayPos++;
 	}
 	
 	if ( pF == 0 )
 	{
-	    
-	    
-	    /*
-	    int i=0;
-	    while ( i < _flaSize)
-	    {
-		CegoField *pF = _fla[i].First();
-		while ( pF )
-		{
-		    cout << "Fla Field = " << pF->getTableName() << "." << pF->getAttrName() << endl;
-		    pF = _fla[i].Next();
-		}
-		i++;
-	    }
-	    */
-	    	    
-	    
 	    Chain msg("Unknown attribute field <" + _pAttrDesc->getTableName() + "." + _pAttrDesc->getAttrName() + ">");
 	    throw Exception(EXLOC, msg);
 	}       
@@ -799,31 +773,28 @@ CegoFieldValue CegoFactor::evalFieldValue()
     }
     case CegoFactor::QUERY:
     {
-
 	try 
 	{
-
-	    // _pSelect->cleanUp();
 	    _pSelect->prepare();
-	    // _pSelect->reset(false);
+	    _pSelect->reset(false);
 
 	    ListT<CegoField> fl;
 
 	    if ( ! _pSelect->nextTuple(fl) )
-	    {	       
+	    {
 		_pSelect->reset(true);
 		return CegoFieldValue();
 	    }
 	    
 	    CegoField *pSF = fl.First();
-	    
 	    _pSelect->reset(true);
+	    CegoFieldValue val = pSF->getValue();
 	    return pSF->getValue();
 	}
 	catch ( Exception e ) 
 	{
 	    _pSelect->cleanUp();
-	    throw Exception(EXLOC, Chain("Cannot eval field value"), e);
+	    throw e;
 	}
     }
     case CegoFactor::CASECOND:
@@ -836,6 +807,51 @@ CegoFieldValue CegoFactor::evalFieldValue()
     }
 }
 
+ListT<CegoAggregation*> CegoFactor::getAggregationList()
+{
+
+    ListT<CegoAggregation*> aggList;
+
+    switch ( _type )
+    {
+    case CegoFactor::ATTR:
+    case CegoFactor::CONSTVAL:
+    case CegoFactor::VAR:
+    {
+	// nothing to add
+	break;
+    }
+    case CegoFactor::EXPR:
+    {
+	aggList = _pExpr->getAggregationList();
+	break;
+    }
+    case CegoFactor::FUNCTION:
+    {
+	aggList = _pFunction->getAggregationList();
+	break;
+    }
+    case CegoFactor::QUERY:
+    {
+	throw Exception(EXLOC, "Query not supported in aggregation");
+    }
+    case CegoFactor::CASECOND:
+    {
+	throw Exception(EXLOC, "Case condition not supported in aggregation");
+    }
+    case CegoFactor::FETCH:
+    {
+	throw Exception(EXLOC, "Fetch not supported in aggregation");
+    }
+    case CegoFactor::AGGREGATION:
+    {
+	aggList.Insert ( _pAggr );
+	break;
+    }
+    }
+    return aggList;
+}
+
 CegoExpr* CegoFactor::getExpr() const
 {
     return _pExpr;
@@ -903,6 +919,65 @@ CegoFactor* CegoFactor::clone(bool isAttrRef)
     return 0;
 }
 
+Chain CegoFactor::getId() const
+{
+    Chain s;
+    switch (_type)
+    {
+    case CegoFactor::CONSTVAL:
+    {
+	s = _fv.valAsChain(true);
+	break;
+    }
+    case CegoFactor::ATTR:
+    {
+	s = _pAttrDesc->getId();
+	break;
+    }
+    case CegoFactor::VAR:
+    {
+	CegoFieldValue fv;
+	if ( _pBlock )
+	{
+	    fv = _pBlock->getValue(_varName);
+	}
+	s = fv.toChain();
+	break;
+    }
+    case CegoFactor::EXPR:
+    {
+	s = Chain("(") + _pExpr->getId() + Chain(")");
+	break;
+    }
+    case CegoFactor::FETCH:
+    {
+	s = _pFetch->toChain();
+	break;
+    }
+    case CegoFactor::AGGREGATION:
+    {
+	s = _pAggr->getId();
+	break;
+    }
+    case CegoFactor::FUNCTION:
+    {
+	s = _pFunction->getId();
+	break;
+    }
+    case CegoFactor::QUERY:
+    {
+	s = Chain("(") + _pSelect->getQueryId() + Chain(")");
+	break;
+    }
+    case CegoFactor::CASECOND:
+    {
+	s = _pCaseCond->getId();
+	break;
+    }
+    }
+    return s;
+}
+
 Chain CegoFactor::toChain() const
 {
     Chain s;
@@ -939,6 +1014,10 @@ Chain CegoFactor::toChain() const
     return s;
 }
 
+Chain CegoFactor::dbFormat(CegoDatabaseFormater *pForm) const
+{
+    return pForm->formatFactor(_type, _fv, _pAttrDesc, _varName, _pExpr, _pFetch, _pAggr, _pFunction, _pSelect, _pCaseCond);
+}
 
 Element* CegoFactor::toElement() const
 {
@@ -1104,7 +1183,7 @@ void CegoFactor::encode(char *buf)
 	if ( _pBlock )
 	{
 	    fv = _pBlock->getValue(_varName);
-	}
+	}      
 	fv.encode(pE);
 	pE = pE + fv.getEncodingLength();
 	break;
@@ -1124,7 +1203,11 @@ void CegoFactor::encode(char *buf)
     }
     case CegoFactor::CASECOND:
     {
-	throw Exception(EXLOC, "No encoding supported for case condition");
+	memcpy( pE, &_type, sizeof(CegoFactor::FacType));
+	pE = pE + sizeof(CegoFactor::FacType);
+	_pCaseCond->encode(pE);
+	pE = pE + _pCaseCond->getEncodingLength();
+	break;
     }
     case CegoFactor::AGGREGATION:
     {
@@ -1153,7 +1236,7 @@ void CegoFactor::encode(char *buf)
     }
 }
 
-void CegoFactor::decode(char *buf, CegoDistManager* pGTM)
+void CegoFactor::decode(char *buf, CegoDistManager* pGTM, int tabSetId)
 {
 
     char* pE = (char*)buf;
@@ -1183,7 +1266,7 @@ void CegoFactor::decode(char *buf, CegoDistManager* pGTM)
     }
     case CegoFactor::EXPR:
     {
-	_pExpr = new CegoExpr(pE, pGTM);
+	_pExpr = new CegoExpr(pE, pGTM, tabSetId);
 	pE = pE + _pExpr->getEncodingLength();
 	break;
     }
@@ -1194,24 +1277,25 @@ void CegoFactor::decode(char *buf, CegoDistManager* pGTM)
     }
     case CegoFactor::CASECOND:
     {
-	throw Exception(EXLOC, "No decoding supported for case condition");
+	_pCaseCond = new CegoCaseCond(pE, pGTM, tabSetId);
+	pE = pE + _pCaseCond->getEncodingLength();
 	break;
     }
     case CegoFactor::AGGREGATION:
     {
-	_pAggr = new CegoAggregation(pE, pGTM);
+	_pAggr = new CegoAggregation(pE, pGTM, tabSetId);
 	pE = pE + _pAggr->getEncodingLength();
 	break;
     }
     case CegoFactor::FUNCTION:
     {
-	_pFunction = new CegoFunction(pE, pGTM);
+	_pFunction = new CegoFunction(pE, pGTM, tabSetId);
 	pE = pE + _pFunction->getEncodingLength();
 	break;
     }
     case CegoFactor::QUERY:
     {
-	_pSelect = new CegoSelect(pE, pGTM);
+	_pSelect = new CegoSelect(pE, pGTM, tabSetId);
 	pE = pE + _pSelect->getEncodingLength();
 	break;
     }
@@ -1260,7 +1344,7 @@ int CegoFactor::getEncodingLength() const
     }
     case CegoFactor::CASECOND:
     {
-	// no encoding supported for casecond
+	len += _pCaseCond->getEncodingLength();
 	break;
     }
     case CegoFactor::AGGREGATION:
@@ -1295,4 +1379,3 @@ ostream& operator << (ostream& s, const CegoFactor& pf)
     s << pf.toChain();
     return s;
 }
-
