@@ -6,34 +6,17 @@
 //     
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2010 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoFileHandler
 // 
-// Description: 
-// 
-// The filehandler class provides access to all datafiles. Datafiles consists
-// of a defined number of pages managed by the this class.
-// At the header of a datafile, a bitmap is stored for used page information
+// Description: The filehandler class provides access to all datafiles. Datafiles consists
+//              of a defined number of pages managed by the this class.
+//              At the header of a datafile, a bitmap is stored for used page information
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -44,14 +27,16 @@
 
 #define BIT_PER_BYTE 8
 
-CegoFileHandler::CegoFileHandler(const Chain& logFile, int pageSize) : CegoModule(logFile)
+CegoFileHandler::CegoFileHandler(const Chain& logFile, const Chain& progName, int pageSize) : CegoModule(logFile, progName)
 {
+    _currentPageOffset=0;
     for (int i = 0; i<FILMNG_MAXDATAFILE; i++)
     {
 	_isReg[i]=false;
 	_tabSetId[i]=0;
 	_fhList[i]=0;
 	_buMask[i]=0;
+	_regPageOffset[i]=0;
     }
     _modId = getModId("CegoFileHandler");
 }
@@ -110,7 +95,7 @@ void CegoFileHandler::markPage(int fileId, int pageId)
     
     unsigned bm = _buMask[fileId][bmid];
     
-    unsigned f = ~(~0 << 1);
+    unsigned f = ~(~(unsigned)0 << 1);
     f = f << bmoffset;
     
     _buMask[fileId][bmid] = bm | f;
@@ -131,7 +116,7 @@ bool CegoFileHandler::isMarked(int fileId, int pageId)
     
     unsigned bm = _buMask[fileId][bmid];    
         
-    unsigned checkbit = bm >> bmoffset & ~(~0 << 1);    
+    unsigned checkbit = bm >> bmoffset & ~(~(unsigned)0 << 1);    
     
     if (  checkbit )
     {
@@ -141,6 +126,11 @@ bool CegoFileHandler::isMarked(int fileId, int pageId)
 
 }
 
+unsigned long long CegoFileHandler::getRegPageOffset(int fileId)
+{
+    return _regPageOffset[fileId];
+}
+
 int CegoFileHandler::getFBMSize(int fileId)
 {
     return (_numPages[fileId] / (BIT_PER_BYTE * sizeof(int))) + 1;
@@ -156,7 +146,7 @@ void CegoFileHandler::readFBM(int fileId, unsigned *fbm, CegoLockHandler *pLockH
     }
     catch ( Exception e )
     {
-	throw Exception(EXLOC, "Cannot read fbm", e);
+	throw e;
     }
 
     pLockHandle->lockDataFile(fileId, CegoLockHandler::WRITE);
@@ -182,7 +172,7 @@ void CegoFileHandler::readFBM(int fileId, unsigned *fbm, CegoLockHandler *pLockH
     catch ( Exception e )
     {
 	pLockHandle->unlockDataFile(fileId);
-	throw Exception(EXLOC, "Cannot read fbm", e);
+	throw e;
     }
 }
 
@@ -197,7 +187,7 @@ void CegoFileHandler::writeFBM(int fileId, unsigned *fbm, CegoLockHandler *pLock
     }
     catch ( Exception e )
     {
-	throw Exception(EXLOC, "Cannot write fbm", e);
+	throw e;
     }
 
     pLockHandle->lockDataFile(fileId, CegoLockHandler::WRITE);
@@ -225,7 +215,7 @@ void CegoFileHandler::writeFBM(int fileId, unsigned *fbm, CegoLockHandler *pLock
     catch ( Exception e )
     {
 	pLockHandle->unlockDataFile(fileId);
-	throw Exception(EXLOC, "Cannot write fbm", e);
+	throw e;
     }
 }
 
@@ -248,7 +238,7 @@ void CegoFileHandler::cleanDataFile(int fileId, CegoLockHandler *pLockHandle)
     }
     catch ( Exception e )
     {
-	throw Exception(EXLOC, "Cannot clean datafile", e);
+	throw e;
     }
     
     int offset = sizeof(FileType) +  3 * sizeof(int)
@@ -298,7 +288,7 @@ void CegoFileHandler::initDataFile(int tabSetId, const Chain& path, int fileId,
 	
 	pF->writeByte((char*)&tabSetId, sizeof(int));
 	pF->writeByte((char*)&ft, sizeof(FileType));
-	// cout << "Init datafile with numPages " << numPages << endl;
+	// cout << "Init datafile " << path << " with numPages " << numPages << endl;
 	pF->writeByte((char*)&numPages, sizeof(int));
 		
 	int i;
@@ -319,7 +309,7 @@ void CegoFileHandler::initDataFile(int tabSetId, const Chain& path, int fileId,
 	    pF->writeByte((char*)&fixStat, sizeof(int));
 	    pF->writeByte(initBuf, _pageSize);
 	}
-	delete initBuf;
+	delete[] initBuf;
 	
 	pF->close();
 	delete pF;
@@ -327,7 +317,7 @@ void CegoFileHandler::initDataFile(int tabSetId, const Chain& path, int fileId,
     catch ( Exception e )
     {
 	delete pF;
-	throw Exception(EXLOC, "Cannot init datafile", e);
+	throw e;
     }
 }
 
@@ -336,7 +326,6 @@ void CegoFileHandler::initDataFile(int tabSetId, const Chain& path, int fileId,
 
 void CegoFileHandler::regDataFile(int tabSetId, const Chain& path, int fileId, CegoLockHandler *pLockHandle)
 {
-
     pLockHandle->lockDataFile(fileId, CegoLockHandler::READ);
 
     if ( _isReg[fileId] )
@@ -364,18 +353,21 @@ void CegoFileHandler::regDataFile(int tabSetId, const Chain& path, int fileId, C
 
 	File checkFile(path);	    
 	checkFile.open(File::READ);
-
 	
 	checkFile.readByte((char*)&checkTabSetId, sizeof(int));	
 	checkFile.readByte((char*)&ft, sizeof(FileType));
 	checkFile.readByte((char*)&numPage, sizeof(int));
+
+	checkFile.close();
 	
     }
     catch ( Exception e )
     {
+	Chain ferr;
+	e.pop(ferr);
 	pLockHandle->unlockDataFile(fileId);
-	Chain msg = Chain("Cannot access datafile <") + path + Chain(">");
-	throw Exception(EXLOC, msg, e);
+	Chain msg = Chain("Cannot access datafile <") + path + Chain("> :") + ferr;
+	throw Exception(EXLOC, msg);
     }
     
     if ( tabSetId != checkTabSetId )
@@ -383,13 +375,16 @@ void CegoFileHandler::regDataFile(int tabSetId, const Chain& path, int fileId, C
 	pLockHandle->unlockDataFile(fileId);
 	throw Exception(EXLOC, "Table Set Id does not match");
     }
-	
+    
+    _regPageOffset[fileId] = _currentPageOffset;
     _isReg[fileId] = true;
     _path[fileId] = path;
     _tabSetId[fileId] = tabSetId;
     _fileType[fileId] = ft;
     _numPages[fileId] = numPage;
 
+    _currentPageOffset += numPage;
+
     pLockHandle->unlockDataFile(fileId);
 }
 
@@ -418,7 +413,12 @@ void CegoFileHandler::releaseFiles(int tabSetId)
 
 void CegoFileHandler::writePage(int fileId, int pageId, int fixStat, char* pageData, CegoLockHandler *pLockHandle)
 {
-    
+
+    // if ( isClaimed(fileId, pageId, pLockHandle) == false )
+    //	cout << "WRITE PAGE " << pageId << " IS NOT CLAIMED" << endl;
+
+    // cout << "Write page " << fileId << " " << pageId << endl;
+
     File* pF;
 
     try 
@@ -427,7 +427,7 @@ void CegoFileHandler::writePage(int fileId, int pageId, int fixStat, char* pageD
     }
     catch ( Exception e )
     {
-	throw Exception(EXLOC, "Cannot write page", e);
+	throw e;
     }
 
 
@@ -454,7 +454,7 @@ void CegoFileHandler::writePage(int fileId, int pageId, int fixStat, char* pageD
     catch ( Exception e )
     {
 	pLockHandle->unlockDataFile(fileId);
-	throw Exception(EXLOC, "Cannot write page", e);
+	throw e;
     }
 
 }
@@ -464,6 +464,10 @@ void CegoFileHandler::writePage(int fileId, int pageId, int fixStat, char* pageD
 
 void CegoFileHandler::readPage(int fileId, int pageId, int& tabSetId, int& fixStat, char* pageData, CegoLockHandler *pLockHandle)
 {
+
+    // if ( isClaimed(fileId, pageId, pLockHandle) == false )
+    //	cout << "READ PAGE " << pageId << "IS NOT CLAIMED" << endl;
+
     File *pF;
 
     try 
@@ -472,7 +476,7 @@ void CegoFileHandler::readPage(int fileId, int pageId, int& tabSetId, int& fixSt
     }
     catch ( Exception e )
     {
-	throw Exception(EXLOC, "Cannot read page", e);
+	throw e;
     }
 
     pLockHandle->lockDataFile(fileId, CegoLockHandler::WRITE);
@@ -485,13 +489,23 @@ void CegoFileHandler::readPage(int fileId, int pageId, int& tabSetId, int& fixSt
 	
 	int offset = sizeof(int) + sizeof(FileType) + 2 * sizeof(int) 
 	    +  (_numPages[fileId] / ( BIT_PER_BYTE * sizeof(int) )) * sizeof(int) ;
-	
+
 	pF->seek(offset + pageId * ( _pageSize + sizeof(int)) );
 	
 	pF->readByte((char*)&fixStat, sizeof(int));
 	
-	if ( pF->readByte(pageData, _pageSize) != _pageSize )
-	    throw Exception(EXLOC, "Incomplete file read");
+	unsigned long rb = pF->readByte(pageData, _pageSize);
+	while ( rb < _pageSize )
+	{
+	  
+	    unsigned long rrb = pF->readByte((char*)((long long)pageData + rb), _pageSize - rb);
+	    
+	    
+	    if ( rrb == 0 )
+	      throw Exception(EXLOC, Chain("Incomplete file read"));
+	    
+	    rb+=rrb;
+	}
 	
 	pLockHandle->unlockDataFile(fileId);
 
@@ -499,7 +513,7 @@ void CegoFileHandler::readPage(int fileId, int pageId, int& tabSetId, int& fixSt
     catch ( Exception e )
     {
 	pLockHandle->unlockDataFile(fileId);
-	throw Exception(EXLOC, "Cannot read page", e);
+	throw e;
     }
 }
 
@@ -518,7 +532,7 @@ void CegoFileHandler::claimPage(int fileId, int pageId, CegoLockHandler *pLockHa
     }
     catch ( Exception e )
     {
-	throw Exception(EXLOC, "Cannot claim page", e);
+	throw e;
     }
 
     pLockHandle->lockDataFile(fileId, CegoLockHandler::WRITE);
@@ -534,7 +548,7 @@ void CegoFileHandler::claimPage(int fileId, int pageId, CegoLockHandler *pLockHa
 	unsigned bm;
 	pF->readByte((char*)&bm, sizeof(unsigned));
 	
-	unsigned f = ~(~0 << 1);
+	unsigned f = ~(~(unsigned)0 << 1);
 	f = f << bmoffset;
 	
 	bm = bm | f;
@@ -544,15 +558,24 @@ void CegoFileHandler::claimPage(int fileId, int pageId, CegoLockHandler *pLockHa
 
 	if ( _buMask[fileId] )
 	    _fbmMask[fileId] = true;
-
 	
 	pLockHandle->unlockDataFile(fileId);
+
     }
     catch ( Exception e )
     {
 	pLockHandle->unlockDataFile(fileId);
 	throw e;
     }
+
+    // write page data to datafile to avoid, that cleanup procedure frees the page
+    
+    char* initBuf = new char[_pageSize];
+    CegoBufferPage ip(initBuf, _pageSize);
+    ip.initPage(CegoBufferPage::TABLE);
+    
+    writePage(fileId, pageId, 1, initBuf, pLockHandle);
+    
 }
 
 /* any new required page can be requested using the allocatePage method.
@@ -581,7 +604,7 @@ void CegoFileHandler::allocatePage(int tabSetId, FileType ft, int& fileId, int&
 
 		int k;
 
-		if ( doAppend == false  || _appendPos[fid] == 0)
+		if ( doAppend == false || _appendPos[fid] == 0)
 		{
 
 		    /*
@@ -599,6 +622,8 @@ void CegoFileHandler::allocatePage(int tabSetId, FileType ft, int& fileId, int&
 
 		    pF->seek(2*sizeof(int) + sizeof(FileType));
 		    k = 0;
+		    // we reset the append postion to zero, so next allocation with append = true will search from the beginning
+		    _appendPos[fid] = 0;
 		}
 		else
 		{		    
@@ -627,7 +652,7 @@ void CegoFileHandler::allocatePage(int tabSetId, FileType ft, int& fileId, int&
 			{
 			    if ( k+l < _numPages[fid])
 			    {
-				unsigned checkbit = bm >> l & ~(~0 << 1);
+				unsigned checkbit = bm >> l & ~(~(unsigned)0 << 1);
 				if (checkbit == 0)
 				{
 				    
@@ -636,7 +661,7 @@ void CegoFileHandler::allocatePage(int tabSetId, FileType ft, int& fileId, int&
 				    
 				    // cout << "FH: " << fid << "," <<  k << "," << l << " using pageId " << pageId << endl;
 				    
-				    unsigned f = ~(~0 << 1);
+				    unsigned f = ~(~(unsigned)0 << 1);
 				    f = f << l;
 				    // printBitMap(bm);
 				    // cout << "1 bm=" << bm << endl;
@@ -688,20 +713,28 @@ void CegoFileHandler::allocatePage(int tabSetId, FileType ft, int& fileId, int&
 	    catch ( Exception e )
 	    {
 		pLockHandle->unlockDataFile(fid);
-		throw Exception(EXLOC, "File allocation error", e);
+		throw e;
 	    }		
 	} 
 	fid++;		    
     }
-    Chain fileType;
-    if ( ft == DATAFILE )
-	fileType = Chain("Data");
-    else if ( ft == SYSTEMFILE )
-	fileType = Chain("System");
-    else if ( ft == TEMP )
-	fileType = Chain("Temp");
-    
-    throw Exception(EXLOC, fileType + Chain(" file pages exceeded "));
+
+    if ( doAppend )
+    {
+	// if no more pages available with append mode, we make another try without append
+	allocatePage(tabSetId, ft, fileId, pageId, pLockHandle, fbm, fbmSize, false);
+    }
+    else
+    {    
+	Chain fileType;
+	if ( ft == DATAFILE )
+	    fileType = Chain("Data");
+	else if ( ft == SYSTEMFILE )
+	    fileType = Chain("System");
+	else if ( ft == TEMP )
+	    fileType = Chain("Temp");	
+	throw Exception(EXLOC, fileType + Chain(" file pages exceeded "));
+    }
 }
 
 
@@ -711,9 +744,11 @@ void CegoFileHandler::allocatePage(int tabSetId, FileType ft, int& fileId, int&
 void CegoFileHandler::releasePage(int fileId, int pageId, CegoLockHandler *pLockHandle, unsigned* &fbm, int& fbmSize)
 {
 
+    // cout << "Releasing pageId " << pageId << endl;
+
     File* pF;
     pF = getHandle(fileId);
-    
+
     pLockHandle->lockDataFile(fileId, CegoLockHandler::WRITE);
 
     try
@@ -746,27 +781,35 @@ void CegoFileHandler::releasePage(int fileId, int pageId, CegoLockHandler *pLock
 	unsigned bm;
 	pF->readByte((char*)&bm, sizeof(unsigned));
 	
-	unsigned f = ~(~0 << 1);
+	unsigned f = ~(~(unsigned)0 << 1);
 	f = f << bmoffset;
-	// printBitMap(f);
-	// printBitMap(bm);			    
-	bm = bm & ~f;
+
+	// cout << "Bitmap before " << endl;
 	// printBitMap(bm);
-	
+
+	// printBitMap(f);	    
+	bm = bm & ~f;
+
+	// cout << "Bitmap after " << endl;
+	// printBitMap(bm);
+
+	// cout << "Write to pos " << sizeof(int) + sizeof(FileType) + sizeof(int) + ( bmid * sizeof(int)) << endl;
+
 	pF->seek(sizeof(int) + sizeof(FileType) + sizeof(int) + ( bmid * sizeof(int)));
 	pF->writeByte((char*)&bm, sizeof(unsigned));
 	
 	if ( _buMask[fileId] )
 	    _fbmMask[fileId] = true;
-	
+
 	pLockHandle->unlockDataFile(fileId);
 	
     }
     catch ( Exception e )
     {
 	pLockHandle->unlockDataFile(fileId);
-	throw Exception(EXLOC, "Cannot release page", e);
+	throw e;
     }
+
 }
 
 /* returns the number of total pages for the given fileId */
@@ -780,7 +823,7 @@ int CegoFileHandler::getNumPages(int fileId)
     }
     catch ( Exception e )
     {
-	throw Exception(EXLOC, "Cannot get num pages", e);
+	throw e;
     }
 
     return _numPages[fileId];
@@ -799,7 +842,7 @@ int CegoFileHandler::getNumUsedPages(int fileId, CegoLockHandler *pLockHandle)
     }
     catch ( Exception e )
     {
-	throw Exception(EXLOC, "Cannot get num used pages", e);
+	throw e;
     }
 
     pLockHandle->lockDataFile(fileId, CegoLockHandler::WRITE);
@@ -849,7 +892,7 @@ int CegoFileHandler::getNumUsedPages(int fileId, CegoLockHandler *pLockHandle)
     catch ( Exception e )
     {
 	pLockHandle->unlockDataFile(fileId);
-	throw Exception(EXLOC, "Cannot get num used paged", e);
+	throw e;
     }
 }
 
@@ -867,13 +910,18 @@ bool CegoFileHandler::isClaimed(int fileId, int pageId, CegoLockHandler *pLockHa
 
     try 
     {
-	pF = getHandle(fileId);
+	pF = getHandle(fileId);	
     }
     catch ( Exception e )
     {
-	throw Exception(EXLOC, "Cannot check for claimed status", e);
+	throw e;
     }
 
+    if ( pageId >= _numPages[fileId] )
+    {
+	throw Exception(EXLOC, Chain("Invalid page id"));
+    }
+    
     pLockHandle->lockDataFile(fileId, CegoLockHandler::WRITE);
 
     try 
@@ -889,7 +937,7 @@ bool CegoFileHandler::isClaimed(int fileId, int pageId, CegoLockHandler *pLockHa
 	
 	pLockHandle->unlockDataFile(fileId);
 	
-	unsigned checkbit = bm >> bmoffset & ~(~0 << 1);    
+	unsigned checkbit = bm >> bmoffset & ~(~(unsigned)0 << 1);    
 	
 	if (  checkbit )
 	{
@@ -900,7 +948,7 @@ bool CegoFileHandler::isClaimed(int fileId, int pageId, CegoLockHandler *pLockHa
     catch ( Exception e )
     {
 	pLockHandle->unlockDataFile(fileId);
-	throw Exception(EXLOC, "Cannot check for claimed status", e);
+	throw e;
     }
 }
 
@@ -959,8 +1007,10 @@ File* CegoFileHandler::getHandle(int fileId)
 	}
 	catch (Exception e)
 	{
-	    Chain msg = "Cannot get file handle " + Chain(fileId);
-	    throw Exception(EXLOC, msg, e);
+	    Chain ferr;
+	    e.pop(ferr);
+	    Chain msg = "Cannot get file handle " + Chain(fileId) + Chain(": ") + ferr;
+	    throw Exception(EXLOC, msg);
 	}
     }
     File *pF = _fhList[fileId];
