@@ -6,30 +6,18 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2010 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoLockHandler
 // 
-// Description: 
-//
-// Status: QG-2.6
+// Description: All semaphore lock operations on database files, pool buffers, pages and records level
+//              are managed using the LockHandler class.
+//              A number of available semaphores is distributed among all required lock operations.
+//              
+//             
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -37,25 +25,25 @@
 #define _REENTRANT    /* basic 3-lines for threads */
 #endif
 
-// base includes
+// LFC INCLUDES
 #include <lfcbase/Exception.h>
 #include <lfcbase/ThreadLock.h>
 #include <lfcbase/Tokenizer.h>
 
-// cego includes
+// CEGO INCLUDES
 #include "CegoLockHandler.h"
 #include "CegoDatabaseManager.h"
 
+// POSIX INCLUDES
 #include <string.h>
 #include <stdlib.h>
 		 
 static ThreadLock** _lockArray;
 
-bool __lockStatOn = true;
+bool __lockStatOn = false;
 
 CegoLockHandler::CegoLockHandler(CegoDatabaseManager* pDBMng)
 {
-    
     _numRecSema = pDBMng->getNumRecordSema();
     _numRBRecSema = pDBMng->getNumRBRecordSema();
     _numSysRecSema = pDBMng->getNumSysRecordSema();
@@ -88,7 +76,6 @@ CegoLockHandler::CegoLockHandler(CegoDatabaseManager* pDBMng)
     
     _lockCount = new int[_numSema];
 
-   
     _recordLock = (RecordLock*)malloc( LCKMNG_NUMLOCKS * sizeof( struct RecordLock));
     _rbRecordLock = (RecordLock*)malloc( LCKMNG_NUMLOCKS * sizeof( struct RecordLock));
     _sysRecordLock = (RecordLock*)malloc( LCKMNG_NUMLOCKS * sizeof( struct RecordLock));
@@ -98,7 +85,6 @@ CegoLockHandler::CegoLockHandler(CegoDatabaseManager* pDBMng)
     _idxPageLock = (PageLock*)malloc( LCKMNG_NUMLOCKS * sizeof( struct PageLock));
     _rbPageLock = (PageLock*)malloc( LCKMNG_NUMLOCKS * sizeof( struct PageLock));
     
-
     int i;
     for (int i = 0; i<_numSema; i++)
     {
@@ -133,11 +119,9 @@ CegoLockHandler::CegoLockHandler(CegoDatabaseManager* pDBMng)
 	_rbPageLock[i].lockId=0;
 	_rbPageLock[i].fileId=0;
 	_rbPageLock[i].pageId=0;
-	_rbPageLock[i].semId=0;
-		
+	_rbPageLock[i].semId=0;		
     }
     _nextLockId = 1;
-
 }
 
 CegoLockHandler::~CegoLockHandler()
@@ -280,7 +264,6 @@ void CegoLockHandler::initLocks()
 	_lockArray[semId]->setId( Chain(DATAFILELOCKGROUP) + Chain("-") + Chain(semId-low));
     }
   
-
     low = _numRecSema 
 	+ _numRBRecSema
 	+ _numSysRecSema
@@ -318,34 +301,14 @@ void CegoLockHandler::deleteLocks()
     }
 }
 
-CegoBufferPage CegoLockHandler::getPage(unsigned long lockId)
+unsigned long long CegoLockHandler::lockRecord(const CegoDataPointer& dp, enum LockMode lockMode)
 {
 
-    int i=0;
-    while (i < LCKMNG_NUMLOCKS && _recordLock[i].lockId != lockId )
-    {
-	i++;
-    }
-    
-    if (_recordLock[i].lockId == lockId)
-    {
-	return _recordLock[i].bp;
-    }
-    else
-    {
-	Chain msg = Chain("Invalid page lockid ") + Chain(lockId);
-	throw Exception(EXLOC, msg);
-    }
-}
-
-unsigned long CegoLockHandler::lockRecord(const CegoBufferPage &bp, const CegoDataPointer& dp, enum LockMode lockMode)
-{
-
-    int semId = ( dp.getFileId() * dp.getPageId() + dp.getOffset()) % _numRecSema;
-    
     int timeout = _recLockTimeout;
     int numTries = _numLockTries;
 
+    int semId = ( dp.getFileId() * dp.getPageId() + dp.getOffset()) % _numRecSema;
+
     int i=0;
     while ( _recordLock[i].lockId != 0 && i < LCKMNG_NUMLOCKS)
     {
@@ -353,9 +316,9 @@ unsigned long CegoLockHandler::lockRecord(const CegoBufferPage &bp, const CegoDa
     }
     if (i == LCKMNG_NUMLOCKS)
     {
-	throw Exception(EXLOC, "Number of record locks exceeded");
+	throw Exception(EXLOC, "Number of record locks exceeded");    
     }
-    _recordLock[i].lockId=_nextLockId;    
+    _recordLock[i].lockId=_nextLockId;
     _nextLockId++;
 
     _recordLock[i].semId=semId;
@@ -376,11 +339,21 @@ unsigned long CegoLockHandler::lockRecord(const CegoBufferPage &bp, const CegoDa
 		numTries--;									
 		if ( numTries == 0 )
 		{
-		    Chain msg = Chain("Lock record failed for fileId ") 
-			+ Chain(dp.getFileId()) 
-			+ Chain(" pageId ") + Chain(dp.getPageId()) + Chain(" offset=") + Chain(dp.getOffset());
+		    Chain lockError;
+		    e.pop(lockError);
 
-		    throw Exception(EXLOC, msg, e); 
+		    Chain lockMode;
+		    if ( lockMode == READ )
+			lockMode = Chain("Read lock");
+		    else
+			lockMode = Chain("Write lock");
+
+
+		    Chain msg = lockMode + Chain(" for record failed, fileId ") 
+			+ Chain(dp.getFileId())
+			+ Chain(" pageid ") + Chain(dp.getPageId()) + Chain(", offset ") + Chain(dp.getOffset()) + Chain(", reason ") + lockError; 
+
+		    throw Exception(EXLOC, msg, e);
 		}
 		else
 		{
@@ -395,15 +368,12 @@ unsigned long CegoLockHandler::lockRecord(const CegoBufferPage &bp, const CegoDa
     {
 	_lockCount[semId]++;
     }
-    
-    _recordLock[i].dp=dp;
-    _recordLock[i].bp=bp;
-
+        
     return _recordLock[i].lockId;
     
 }
 
-void CegoLockHandler::unlockRecord(unsigned long lockId)
+void CegoLockHandler::unlockRecord(unsigned long long lockId)
 {
 
     int i=0;
@@ -435,14 +405,14 @@ void CegoLockHandler::unlockRecord(unsigned long lockId)
     }
 }
 
-unsigned long CegoLockHandler::lockRBRecord(const CegoBufferPage &bp, const CegoDataPointer& dp, enum LockMode lockMode)
+unsigned long long CegoLockHandler::lockRBRecord(const CegoDataPointer& dp, enum LockMode lockMode)
 {
 
+    int timeout = _recLockTimeout;
+    int numTries = _numLockTries;
+
     int semId = _numRecSema
 	+ ( dp.getFileId() * dp.getPageId() + dp.getOffset()) % _numRBRecSema;
-    
-    int timeout = _recLockTimeout;
-    int numTries = _numLockTries;
 
     int i=0;
     while ( _rbRecordLock[i].lockId != 0 && i < LCKMNG_NUMLOCKS)
@@ -456,7 +426,6 @@ unsigned long CegoLockHandler::lockRBRecord(const CegoBufferPage &bp, const Cego
     _rbRecordLock[i].lockId=_nextLockId;
     _nextLockId++;
 
-
     _rbRecordLock[i].semId=semId;
     if ( _lockCount[semId] == 0 )
     {
@@ -475,15 +444,25 @@ unsigned long CegoLockHandler::lockRBRecord(const CegoBufferPage &bp, const Cego
 		numTries--;									
 		if ( numTries == 0 )
 		{
-		    Chain msg = Chain("Lock rb record failed for fileId ") 
-			+ Chain(dp.getFileId()) 
-			+ Chain(" pageId ") + Chain(dp.getPageId()) + Chain(" offset=") + Chain(dp.getOffset());
+		    Chain lockError;
+		    e.pop(lockError);
+
+		    Chain lockMode;
+		    if ( lockMode == READ )
+			lockMode = Chain("Read lock");
+		    else
+			lockMode = Chain("Write lock");
+
+
+		    Chain msg = lockMode + Chain(" for rb record failed, fileId ") 
+			+ Chain(dp.getFileId())
+			+ Chain(" pageid ") + Chain(dp.getPageId()) + Chain(", offset ") + Chain(dp.getOffset()) + Chain(", reason ") + lockError; 
 
 		    throw Exception(EXLOC, msg, e);
 		}
 		else
 		{
-		    _pDBMng->log(_modId, Logger::NOTICE, Chain("rb lock record failed, still ") + Chain(numTries) + Chain(" tries"));
+		    _pDBMng->log(_modId, Logger::NOTICE, Chain("RBlock record failed, still ") + Chain(numTries) + Chain(" tries"));
 		}
 	    }
 	}
@@ -494,16 +473,12 @@ unsigned long CegoLockHandler::lockRBRecord(const CegoBufferPage &bp, const Cego
     {
 	_lockCount[semId]++;
     }
-    
-    _rbRecordLock[i].dp=dp;
-    _rbRecordLock[i].bp=bp;
-
-    
+        
     return _rbRecordLock[i].lockId;
     
 }
 
-void CegoLockHandler::unlockRBRecord(unsigned long lockId)
+void CegoLockHandler::unlockRBRecord(unsigned long long lockId)
 {
 
     int i=0;
@@ -530,12 +505,12 @@ void CegoLockHandler::unlockRBRecord(unsigned long lockId)
     }
     else
     {
-	Chain msg = Chain("Invalid rb record lockid ") + Chain(lockId);
+	Chain msg = Chain("Invalid RB record lockid ") + Chain(lockId);
 	throw Exception(EXLOC, msg);
     }
 }
 
-unsigned long CegoLockHandler::lockSysRecord(const CegoBufferPage &bp, const CegoDataPointer& dp, enum LockMode lockMode)
+unsigned long long CegoLockHandler::lockSysRecord(const CegoDataPointer& dp, enum LockMode lockMode)
 {
 
     int timeout = _recLockTimeout;
@@ -575,14 +550,25 @@ unsigned long CegoLockHandler::lockSysRecord(const CegoBufferPage &bp, const Ceg
 		numTries--;									
 		if ( numTries == 0 )
 		{
-		    Chain msg = Chain("Lock sys record failed for fileId ") 
-			+ Chain(dp.getFileId()) 
-			+ Chain(" pageId ") + Chain(dp.getPageId()) + Chain(" offset=") + Chain(dp.getOffset());
+		    Chain lockError;
+		    e.pop(lockError);
+
+		    Chain lockMode;
+		    if ( lockMode == READ )
+			lockMode = Chain("Read lock");
+		    else
+			lockMode = Chain("Write lock");
+
+
+		    Chain msg = lockMode + Chain(" for sys record failed, fileId ") 
+			+ Chain(dp.getFileId())
+			+ Chain(" pageid ") + Chain(dp.getPageId()) + Chain(", offset ") + Chain(dp.getOffset()) + Chain(", reason ") + lockError; 
+
 		    throw Exception(EXLOC, msg, e);
 		}
 		else
 		{
-		    _pDBMng->log(_modId, Logger::NOTICE, Chain("SYS lock record failed, still ") + Chain(numTries) + Chain(" tries"));
+		    _pDBMng->log(_modId, Logger::NOTICE, Chain("Syslock record failed, still ") + Chain(numTries) + Chain(" tries"));
 		}
 	    }
 	}
@@ -593,15 +579,12 @@ unsigned long CegoLockHandler::lockSysRecord(const CegoBufferPage &bp, const Ceg
     {
 	_lockCount[semId]++;
     }
-    
-    _sysRecordLock[i].dp=dp;
-    _sysRecordLock[i].bp=bp;
-    
+        
     return _sysRecordLock[i].lockId;
     
 }
 
-void CegoLockHandler::unlockSysRecord(unsigned long lockId)
+void CegoLockHandler::unlockSysRecord(unsigned long long lockId)
 {
 
     int i=0;
@@ -633,7 +616,7 @@ void CegoLockHandler::unlockSysRecord(unsigned long lockId)
     }
 }
 
-unsigned long CegoLockHandler::lockSysPage(int fileId, int pageId, enum LockMode lockMode)
+unsigned long long CegoLockHandler::lockSysPage(int fileId, int pageId, enum LockMode lockMode)
 {
 
     int semId = _numRecSema 
@@ -671,15 +654,25 @@ unsigned long CegoLockHandler::lockSysPage(int fileId, int pageId, enum LockMode
 	    }
 	    catch ( Exception e )
 	    {		
+
 		numTries--;									
 		if ( numTries == 0 )
 		{
-		    Chain msg = Chain("Lock sys page failed for fileId ") + Chain(fileId) + Chain(" pageId ") + Chain(pageId);
+		    Chain lockError;
+		    e.pop(lockError);
+
+		    Chain lockMode;
+		    if ( lockMode == READ )
+			lockMode = Chain("Read lock");
+		    else
+			lockMode = Chain("Write lock");
+
+		    Chain msg = lockMode + Chain(" for sys page failed, fileid ") + Chain(fileId) + Chain(", pageid ") + Chain(pageId) + Chain(", reason ") + lockError;
 		    throw Exception(EXLOC, msg, e);
 		}
 		else
 		{
-		    _pDBMng->log(_modId, Logger::NOTICE, Chain("Lock sys page failed, still ") + Chain(numTries) + Chain(" tries"));
+		    _pDBMng->log(_modId, Logger::NOTICE, Chain("Syslock page failed, still ") + Chain(numTries) + Chain(" tries"));
 		}
 	    }
 	}
@@ -695,11 +688,10 @@ unsigned long CegoLockHandler::lockSysPage(int fileId, int pageId, enum LockMode
     _sysPageLock[i].fileId = fileId;
     _sysPageLock[i].pageId = pageId;
     
-    return _sysPageLock[i].lockId;
-    
+    return _sysPageLock[i].lockId;    
 }
 
-void CegoLockHandler::unlockSysPage(unsigned long lockId)
+void CegoLockHandler::unlockSysPage(unsigned long long lockId)
 {
 
     int i=0;
@@ -730,10 +722,9 @@ void CegoLockHandler::unlockSysPage(unsigned long lockId)
     }
 }
 
-unsigned long CegoLockHandler::lockData(CegoObject::ObjectType type, int fileId, int pageId, enum LockMode lockMode)
-{
-
-    if ( type == CegoObject::INDEX || type == CegoObject::UINDEX || type == CegoObject::PINDEX )
+unsigned long long CegoLockHandler::lockData(CegoObject::ObjectType type, int fileId, int pageId, enum LockMode lockMode)
+{   
+    if ( type == CegoObject::AVLTREE || type == CegoObject::UAVLTREE || type == CegoObject::PAVLTREE || type == CegoObject::BTREE || type == CegoObject::UBTREE || type == CegoObject::PBTREE )
     {
 	return lockIndexPage(fileId, pageId, lockMode);
     }
@@ -744,10 +735,9 @@ unsigned long CegoLockHandler::lockData(CegoObject::ObjectType type, int fileId,
     return lockDataPage(fileId, pageId, lockMode);
 }
 
-
-void CegoLockHandler::unlockData(CegoObject::ObjectType type, unsigned long lockId)
-{
-    if ( type == CegoObject::INDEX || type == CegoObject::UINDEX || type == CegoObject::PINDEX )
+void CegoLockHandler::unlockData(CegoObject::ObjectType type, unsigned long long lockId)
+{    
+    if ( type == CegoObject::AVLTREE || type == CegoObject::UAVLTREE || type == CegoObject::PAVLTREE || type == CegoObject::BTREE || type == CegoObject::UBTREE || type == CegoObject::PBTREE )
     {
 	return unlockIndexPage(lockId);
     }
@@ -758,7 +748,7 @@ void CegoLockHandler::unlockData(CegoObject::ObjectType type, unsigned long lock
     return unlockDataPage(lockId);
 }
 
-unsigned long CegoLockHandler::lockDataPage(int fileId, int pageId, enum LockMode lockMode)
+unsigned long long CegoLockHandler::lockDataPage(int fileId, int pageId, enum LockMode lockMode)
 {
 
     int semId = _numRecSema 
@@ -799,7 +789,17 @@ unsigned long CegoLockHandler::lockDataPage(int fileId, int pageId, enum LockMod
 		numTries--;						
 		if ( numTries == 0 )
 		{
-		    Chain msg = Chain("Lock data page failed for fileId ") + Chain(fileId) + Chain(" pageId ") + Chain(pageId);
+		    Chain lockError;
+		    e.pop(lockError);
+
+		    Chain lockMode;
+		    if ( lockMode == READ )
+			lockMode = Chain("Read lock");
+		    else
+			lockMode = Chain("Write lock");
+
+		    Chain msg = lockMode + Chain(" for data page failed, fileid ") + Chain(fileId) + Chain(", pageid ") + Chain(pageId) + Chain(", reason ") + lockError;
+
 		    throw Exception(EXLOC, msg, e);
 		}
 		else
@@ -820,14 +820,11 @@ unsigned long CegoLockHandler::lockDataPage(int fileId, int pageId, enum LockMod
     _dataPageLock[i].fileId = fileId;
     _dataPageLock[i].pageId = pageId;
 
-
-    return _dataPageLock[i].lockId;
-    
+    return _dataPageLock[i].lockId;    
 }
 
-void CegoLockHandler::unlockDataPage(unsigned long lockId)
+void CegoLockHandler::unlockDataPage(unsigned long long lockId)
 {
-
     int i=0;
     while (i < LCKMNG_NUMLOCKS && _dataPageLock[i].lockId != lockId )
     {
@@ -856,7 +853,7 @@ void CegoLockHandler::unlockDataPage(unsigned long lockId)
     }
 }
 
-unsigned long CegoLockHandler::lockIndexPage(int fileId, int pageId, enum LockMode lockMode)
+unsigned long long CegoLockHandler::lockIndexPage(int fileId, int pageId, enum LockMode lockMode)
 {
 
     int semId = _numRecSema
@@ -898,7 +895,17 @@ unsigned long CegoLockHandler::lockIndexPage(int fileId, int pageId, enum LockMo
 		numTries--;									
 		if ( numTries == 0 )
 		{
-		    Chain msg = Chain("Lock index page failed for fileId ") + Chain(fileId) + Chain(" pageId ") + Chain(pageId);
+		    Chain lockError;
+		    e.pop(lockError);
+
+		    Chain lockMode;
+		    if ( lockMode == READ )
+			lockMode = Chain("Read lock");
+		    else
+			lockMode = Chain("Write lock");
+		    
+		    Chain msg = lockMode + Chain(" for index page failed, fileid ") + Chain(fileId) + Chain(", pageid ") + Chain(pageId) + Chain(", reason ") + lockError;
+
 		    throw Exception(EXLOC, msg, e);
 		}
 		else
@@ -919,14 +926,11 @@ unsigned long CegoLockHandler::lockIndexPage(int fileId, int pageId, enum LockMo
     _idxPageLock[i].fileId = fileId;
     _idxPageLock[i].pageId = pageId;
 
-
-    return _idxPageLock[i].lockId;
-    
+    return _idxPageLock[i].lockId;   
 }
 
-void CegoLockHandler::unlockIndexPage(unsigned long lockId)
+void CegoLockHandler::unlockIndexPage(unsigned long long lockId)
 {
-
     int i=0;
     while (i < LCKMNG_NUMLOCKS && _idxPageLock[i].lockId != lockId )
     {
@@ -955,7 +959,7 @@ void CegoLockHandler::unlockIndexPage(unsigned long lockId)
     }
 }
 
-unsigned long CegoLockHandler::lockRBPage(int fileId, int pageId, enum LockMode lockMode)
+unsigned long long CegoLockHandler::lockRBPage(int fileId, int pageId, enum LockMode lockMode)
 {
 
     int semId = _numRecSema 
@@ -999,7 +1003,18 @@ unsigned long CegoLockHandler::lockRBPage(int fileId, int pageId, enum LockMode
 		numTries--;
 		if ( numTries == 0 )
 		{
-		    Chain msg = Chain("Lock rb page failed for fileId ") + Chain(fileId) + Chain(" pageId ") + Chain(pageId);
+
+		    Chain lockError;
+		    e.pop(lockError);
+
+		    Chain lockMode;
+		    if ( lockMode == READ )
+			lockMode = Chain("Read lock");
+		    else
+			lockMode = Chain("Write lock");
+		    
+		    Chain msg = lockMode + Chain(" for rb page failed, fileid ") + Chain(fileId) + Chain(", pageid ") + Chain(pageId) + Chain(", reason ") + lockError;
+
 		    throw Exception(EXLOC, msg, e);
 		}
 		else
@@ -1025,7 +1040,7 @@ unsigned long CegoLockHandler::lockRBPage(int fileId, int pageId, enum LockMode
     
 }
 
-void CegoLockHandler::unlockRBPage(unsigned long lockId)
+void CegoLockHandler::unlockRBPage(unsigned long long lockId)
 {
 
     int i=0;
@@ -1091,7 +1106,18 @@ void CegoLockHandler::lockDataFile(int fileId, enum LockMode lockMode)
 		numTries--;
 		if ( numTries == 0 )
 		{
-		    Chain msg = Chain("Lock data file failed for fileId ") + Chain(fileId);
+
+		    Chain lockError;
+		    e.pop(lockError);
+
+		    Chain lockMode;
+		    if ( lockMode == READ )
+			lockMode = Chain("Read lock");
+		    else
+			lockMode = Chain("Write lock");
+		    
+		    Chain msg = lockMode + Chain(" for data file failed, fileid ") + Chain(fileId) + Chain(", reason ") + lockError;
+
 		    throw Exception(EXLOC, msg, e);
 		}
 		else
@@ -1106,8 +1132,7 @@ void CegoLockHandler::lockDataFile(int fileId, enum LockMode lockMode)
     else
     {
 	_lockCount[semId]++;
-    }
-    
+    }    
 }
 
 void CegoLockHandler::unlockDataFile(int fileId)
@@ -1165,7 +1190,16 @@ void CegoLockHandler::lockBufferPool(int hashId, enum LockMode lockMode)
 		numTries--;					
 		if ( numTries == 0 )
 		{
-		    throw Exception(EXLOC, "Lock buffer pool failed", e);
+		    Chain lockError;
+		    e.pop(lockError);
+
+		    Chain lockMode;
+		    if ( lockMode == READ )
+			lockMode = Chain("Read lock");
+		    else
+			lockMode = Chain("Write lock");
+
+		    throw Exception(EXLOC, lockMode + Chain(" for buffer pool failed, reason ") + lockError);
 		}
 		else
 		{
@@ -1179,8 +1213,7 @@ void CegoLockHandler::lockBufferPool(int hashId, enum LockMode lockMode)
     else
     {
 	_lockCount[semId]++;
-    }
-    
+    }    
 }
 
 void CegoLockHandler::unlockBufferPool(int hashId)
@@ -1209,19 +1242,17 @@ void CegoLockHandler::unlockBufferPool(int hashId)
 
 void CegoLockHandler::lockBufferPool()
 {
-
     for ( int i=0; i<_numBufferPoolSema; i++ )
 	lockBufferPool(i, WRITE);
 }
 
 void CegoLockHandler::unlockBufferPool()
 {
-
     for ( int i=0; i<_numBufferPoolSema; i++ )
 	unlockBufferPool(i);
 }
 
-void CegoLockHandler::getLockStat(int semId, Chain& lockName, long& lockCount,  long &numRdLock, long &numWrLock, long &sumRdDelay, long &sumWrDelay)
+void CegoLockHandler::getLockStat(int semId, Chain& lockName, int& lockCount, unsigned long long &numRdLock, unsigned long long &numWrLock, unsigned long long &sumRdDelay, unsigned long long &sumWrDelay)
 {
     
     lockName = _lockArray[semId]->getId();
@@ -1238,7 +1269,7 @@ void CegoLockHandler::getLockStat(int semId, Chain& lockName, long& lockCount,
 
 }
 
-void CegoLockHandler::getAggLockStat(const Chain& lockGroup, int& numLock, long& lockCount,  long &numRdLock, long &numWrLock, long &sumRdDelay, long &sumWrDelay)
+void CegoLockHandler::getAggLockStat(const Chain& lockGroup, int& numLock, int& lockCount, unsigned long long &numRdLock, unsigned long long &numWrLock, unsigned long long &sumRdDelay, unsigned long long &sumWrDelay)
 {
 
     lockCount = 0;
@@ -1269,5 +1300,4 @@ void CegoLockHandler::getAggLockStat(const Chain& lockGroup, int& numLock, long&
 		sumWrDelay += _lockArray[semId]->sumWriteDelay() / LCKMNG_DELRES;
 	}
     }
-
 }
