@@ -8,72 +8,42 @@
 //
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2010 Bjoern Lemke                        
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
 // Class: CegoProcCursorCreateStmt
 // 
-// Description: 
+// Description: Stored procedure statement for cursor creation
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
-// base includes
+// LFC INCLUDES
 #include <lfcbase/Chain.h>
-
-// cego includes
+// CEGO INCLUDES
 #include "CegoProcStmt.h"
 #include "CegoExpr.h"
 #include "CegoProcBlock.h"
 
-class CegoProcCursorCreateStmt : public CegoProcStmt  {
+class CegoProcCursorCreateStmt : public CegoProcStmt {
     
 public:
     
     CegoProcCursorCreateStmt(const Chain& name, CegoSelect *pSelect, CegoProcBlock *pBlock);
-
+    
     ~CegoProcCursorCreateStmt();
 
     CegoException execute();
-
+    
     Chain toChain(const Chain& indent) const;
-
+    Chain dbFormat(CegoDatabaseFormater *pForm) const;
+    
 private:
-
+    
     Chain _name;
     CegoSelect *_pSelect;
-
+    
 };
-
 #endif
-
-
-
-
-
-
-
-
-
-
-
-
-
-
