@@ -6,30 +6,15 @@
 //      
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2010 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoTableCursor
-// 
-// Description: 
 //
-// Status: QG-2.6
+// Description: Basic table cursor implementation
+//
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -56,10 +41,10 @@ CegoTableCursor::CegoTableCursor(CegoTableManager* pTM, int tabSetId, const Chai
     _tabSetId = tabSetId;
     _ignoreTouched = ignoreTouched;
     _idxSetup = false;
-    _pOE = 0;
-    _pBTO = 0;
-
-    _pLogger = _pTM->getDBMng();
+    _idxCached = false;
+    
+    _pDBMng = _pTM->getDBMng();
+	
     _modId = _pTM->getDBMng()->getModId("CegoTableCursor");
 }
 
@@ -86,152 +71,187 @@ CegoTableCursor::~CegoTableCursor()
 CegoAttrCond::IndexMatch CegoTableCursor::setup( const CegoAttrCond& attrCond )
 {
 
+
+    if ( attrCond.numComp() == 0 )
+    {
+	_idxSetup = false;
+	return CegoAttrCond::INAPP;
+    }
+
     if ( _idxSetup == false || attrCond.diff(_attrCond) == false )
     {
 #ifdef CGDEBUG
-	_pLogger->log(_modId, Logger::DEBUG, Chain("Setting up tablecursor with ") + attrCond.toChain());
+	_pDBMng->log(_modId, Logger::DEBUG, Chain("Setting up tablecursor for " ) + _tableName + Chain (" with ") + attrCond.toChain());
 #endif
 
 	_idxSetup = true;
 
-	_pTM->getObjectListByTable(_tabSetId, _tableName, _idxList, _btreeList, _keyList, _checkList);
+	if ( _idxCached == false )
+	{
+
+	    ListT<CegoKeyObject> keyList;
+	    ListT<CegoCheckObject> checkList;
+	    int numInvalid;
     
-	_pOE = _idxList.First();
+	    _pTM->getObjectListByTable(_tabSetId, _tableName, _idxList, _btreeList, keyList, checkList, numInvalid);
+	    _idxCached = true;
+	}
+    
+	CegoTableObject *pIOE = _idxList.First();
 	
 	_idxMatch = CegoAttrCond::INAPP;
 	
-	int strength = 0;
-
 	// first check for AVL index objects
 	
-	while ( _pOE && _idxMatch != CegoAttrCond::FULL )
+	int strength = 0;
+	int relevance = 0;
+	    
+	while ( pIOE && _idxMatch != CegoAttrCond::FULL )
 	{	    
-	    if (_pOE->getType() == CegoObject::PINDEX
-		|| _pOE->getType() == CegoObject::INDEX
-		|| _pOE->getType() == CegoObject::UINDEX )
+	    if ( pIOE->isValid())
 	    {
 
 #ifdef CGDEBUG
-		_pLogger->log(_modId, Logger::DEBUG, Chain("Checking index ") + _pOE->getName());
+		_pDBMng->log(_modId, Logger::DEBUG, Chain("Checking index ") + pIOE->getName());
 #endif
 
-		CegoAttrCond::IndexMatch indexMatch = attrCond.checkIndex(_pOE->getSchema());
+		CegoAttrCond::IndexMatch indexMatch = attrCond.checkIndex(pIOE->getSchema());
 
 		if ( indexMatch == CegoAttrCond::FULL || indexMatch == CegoAttrCond::PART )
 		{
 
-		    CegoAttrCond checkCond = attrCond.getIndexCond(_pOE->getSchema());
+		    CegoAttrCond checkCond = attrCond.getIndexCond(pIOE->getSchema());
 
 		    if ( checkCond.getStrength() > strength )
 		    {
 
 			strength = checkCond.getStrength();
 #ifdef CGDEBUG
-			_pLogger->log(_modId, Logger::DEBUG, Chain("Strength ") + Chain(strength) + Chain(" tops for ") + checkCond.toChain());
+			_pDBMng->log(_modId, Logger::DEBUG, Chain("Strength ") + Chain(strength) + Chain(" tops for ") + checkCond.toChain());
 #endif
 
 			_attrCond = checkCond; 
-			_attrCond.setIdxSchema(_pOE->getSchema());
+			_attrCond.setIdxSchema(pIOE->getSchema());
 			
-			_idxName = _pOE->getName();
+			_idxName = pIOE->getName();
 #ifdef CGDEBUG
-			_pLogger->log(_modId, Logger::DEBUG, Chain("Using index ") + _idxName );
+			_pDBMng->log(_modId, Logger::DEBUG, Chain("Using index ") + _idxName );
 #endif
 			
-			_idxSchema = _pOE->getSchema();
-			_type = _pOE->getType();
+			_idxSchema = pIOE->getSchema();
+			_type = pIOE->getType();
 			_idxMatch = indexMatch;
 		    }
 		    else
 		    {
 #ifdef CGDEBUG
-			_pLogger->log(_modId, Logger::DEBUG, Chain("Strength ") + Chain(strength) + Chain(" to weak for ") + checkCond.toChain());
+			_pDBMng->log(_modId, Logger::DEBUG, Chain("Strength ") + Chain(strength) + Chain(" to weak for ") + checkCond.toChain());
 #endif
 		    }
 		}
 	    }
-	    _pOE = _idxList.Next();
+	    pIOE = _idxList.Next();
 	}
 	
 	if ( _idxMatch == CegoAttrCond::INAPP )
 	{
 	    // now check for btree index objects
-
-	    _pBTO = _btreeList.First();
-	
-	    strength = 0;
-	    	    
-	    while ( _pBTO && _idxMatch != CegoAttrCond::FULL )
+	    
+	    CegoBTreeObject *pBTO = _btreeList.First();
+		    	    
+	    while ( pBTO )
 	    {	    
-		if (_pBTO->getType() == CegoObject::PBTREE
-		    || _pBTO->getType() == CegoObject::BTREE
-		    || _pBTO->getType() == CegoObject::UBTREE )
-		{
-		    
+		
 #ifdef CGDEBUG
-		    _pLogger->log(_modId, Logger::DEBUG, Chain("Checking btree ") + _pBTO->getName());
+		_pDBMng->log(_modId, Logger::DEBUG, Chain("Checking btree ") + pBTO->getName());
 #endif
-
-		    CegoAttrCond::IndexMatch indexMatch = attrCond.checkIndex(_pBTO->getSchema());
+		if ( pBTO->isValid() )
+		{
+		    CegoAttrCond::IndexMatch indexMatch = attrCond.checkIndex(pBTO->getSchema());
 		    
 		    if ( indexMatch == CegoAttrCond::FULL || indexMatch == CegoAttrCond::PART )
 		    {
 			
-			CegoAttrCond checkCond = attrCond.getIndexCond(_pBTO->getSchema());
+			CegoAttrCond checkCond = attrCond.getIndexCond(pBTO->getSchema());
 			
 			if ( checkCond.getStrength() > strength )
 			{
 			    
 			    strength = checkCond.getStrength();
+			    relevance = pBTO->getRelevance();
 #ifdef CGDEBUG
-			    _pLogger->log(_modId, Logger::DEBUG, Chain("Strength ") + Chain(strength) + Chain(" tops for ") + checkCond.toChain());
+			    _pDBMng->log(_modId, Logger::DEBUG, Chain("Strength ") + Chain(strength) + Chain(" tops for ") + checkCond.toChain());
 #endif
-
+			    
 			    _attrCond = checkCond; 
-			    _attrCond.setIdxSchema(_pBTO->getSchema());
-			
-			    _idxName = _pBTO->getName();
+			    _attrCond.setIdxSchema(pBTO->getSchema());
+			    
+			    _idxName = pBTO->getName();
 #ifdef CGDEBUG
-			    _pLogger->log(_modId, Logger::DEBUG, Chain("Using btree ") + _idxName );
+			    _pDBMng->log(_modId, Logger::DEBUG, Chain("Using btree ") + _idxName );
 #endif
-			
-			    _idxSchema = _pBTO->getSchema();
-			    _type = _pBTO->getType();
+			    
+			    _idxSchema = pBTO->getSchema();
+			    _type = pBTO->getType();
 			    _idxMatch = indexMatch;
 			}
+			else if ( checkCond.getStrength() == strength )
+			{
+#ifdef CGDEBUG
+			    _pDBMng->log(_modId, Logger::DEBUG, Chain("Strength ") + Chain(strength) + Chain(" equals, checking for btree relevance ") + checkCond.toChain());
+#endif
+			    
+			    if ( relevance < pBTO->getRelevance() )
+			    {
+#ifdef CGDEBUG
+				_pDBMng->log(_modId, Logger::DEBUG, Chain("Relevance ") + Chain(pBTO->getRelevance()) + Chain(" tops ") + Chain(relevance));
+#endif
+				
+				_attrCond = checkCond; 
+				_attrCond.setIdxSchema(pBTO->getSchema());
+				
+				_idxName = pBTO->getName();
+#ifdef CGDEBUG
+				_pDBMng->log(_modId, Logger::DEBUG, Chain("Using btree ") + _idxName );
+#endif
+				
+				_idxSchema = pBTO->getSchema();
+				_type = pBTO->getType();
+				_idxMatch = indexMatch;
+				
+			    }
+			}
 			else
 			{
 #ifdef CGDEBUG
-			    _pLogger->log(_modId, Logger::DEBUG, Chain("Strength ") + Chain(strength) + Chain(" to weak for ") + checkCond.toChain());
+			    _pDBMng->log(_modId, Logger::DEBUG, Chain("Strength ") + Chain(strength) + Chain(" to weak for ") + checkCond.toChain());
 #endif
 			}
 		    }
 		}
-		_pBTO = _btreeList.Next();
+		pBTO = _btreeList.Next();
 	    }
 	}
     }
     else
     {
 #ifdef CGDEBUG
-	_pLogger->log(_modId, Logger::DEBUG, Chain("Tablecursor already set up "));
+	_pDBMng->log(_modId, Logger::DEBUG, Chain("Tablecursor already set up "));
 #endif
     }
-    // Setting up attribute condition to actual field values
-
 
 #ifdef CGDEBUG    
 
     if ( _idxMatch == CegoAttrCond::FULL )
     {
-	_pLogger->log(_modId, Logger::DEBUG, Chain("Index classified as full qualified"));
+	_pDBMng->log(_modId, Logger::DEBUG, Chain("Index classified as full qualified"));
     }
     else if ( _idxMatch == CegoAttrCond::PART )
     {
-	_pLogger->log(_modId, Logger::DEBUG, Chain("Index classified as part qualified"));
+	_pDBMng->log(_modId, Logger::DEBUG, Chain("Index classified as part qualified"));
     }
     else
-	_pLogger->log(_modId, Logger::DEBUG, Chain("Index classified as not qualified"));
+	_pDBMng->log(_modId, Logger::DEBUG, Chain("Index classified as not qualified"));
 #endif
 
     return _idxMatch;
@@ -251,9 +271,9 @@ bool CegoTableCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 
     if ( _idxMatch == CegoAttrCond::FULL || _idxMatch == CegoAttrCond::PART  )
     {
-	if ( _type == CegoObject::INDEX 
-	     || _type == CegoObject::UINDEX 
-	     || _type == CegoObject::PINDEX )
+	if ( _type == CegoObject::AVLTREE 
+	     || _type == CegoObject::UAVLTREE 
+	     || _type == CegoObject::PAVLTREE )
 	{
 	    if (_pIC)
 	    {
@@ -277,6 +297,7 @@ bool CegoTableCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 	    {
 		_pBTC = new CegoBTreeCursor(_pTM, _tabSetId, _idxName, _type, &_attrCond, _ignoreTouched, _readUncommitted);
 	    }
+	    
 	    return _pBTC->getFirst(fl, dp);
 	} 
 	else
@@ -304,16 +325,14 @@ bool CegoTableCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 
 	    bool useEntry=true;
 	    
-	    int tid;
-	    int tastep;
+	    unsigned long long tid;
+	    unsigned long long tastep;
 	    CegoTupleState ts;
-	    memcpy(&tid, pc, sizeof(int));
 
-	    char* pctastep = pc + sizeof(int);
-	    memcpy(&tastep, pctastep, sizeof(int));
+	    int toff = _qh.decodeTupleHeader(tid, tastep, ts, pc);
 
-	    char* pcts = pc + sizeof(int) + sizeof(int);
-	    memcpy(&ts, pcts, sizeof(CegoTupleState));
+	    char* tp = pc + toff;
+	    int tlen = len - toff;
 	    
 	    if ( tid != 0  )
 	    {
@@ -324,8 +343,7 @@ bool CegoTableCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 			 && tastep < _pTM->getTAStep(_tabSetId) ) 
 
 		    {
-			CegoTupleState ts;
-			_qh.decodeFVL(fl, pc, len, tid, tastep, ts);
+			_qh.decodeFVL(fl, tp, tlen);
 			return true;
 		    }
 		    else
@@ -341,8 +359,7 @@ bool CegoTableCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 			      && (( ts == INSERTED && tid == _pTM->getTID(_tabSetId)) 
 				  || ( ts == DELETED && tid != _pTM->getTID(_tabSetId)))))
 		    {
-			CegoTupleState ts;
-			_qh.decodeFVL(fl, pc, len, tid, tastep, ts);
+			_qh.decodeFVL(fl, tp, tlen);
 			return true;
 		    }
 		    else
@@ -352,8 +369,7 @@ bool CegoTableCursor::getFirst(ListT<CegoField>& fl, CegoDataPointer& dp)
 		}
 	    }
 
-
-	    _qh.decodeFVL(fl, pc, len, tid, tastep, ts);
+	    _qh.decodeFVL(fl, tp, tlen);
 
 	    return true;
 	}
@@ -367,9 +383,9 @@ bool CegoTableCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
     if ( _idxMatch == CegoAttrCond::FULL || _idxMatch == CegoAttrCond::PART  )
     {
 
-	if ( _type == CegoObject::INDEX 
-	     || _type == CegoObject::UINDEX 
-	     || _type == CegoObject::PINDEX )
+	if ( _type == CegoObject::AVLTREE 
+	     || _type == CegoObject::UAVLTREE 
+	     || _type == CegoObject::PAVLTREE )
 	{
 
 	    return _pIC->getNext(fl, dp);
@@ -387,7 +403,6 @@ bool CegoTableCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
     }
     else
     {
-
 	if ( _pC == 0 )
 	{
 	    throw Exception(EXLOC, "Invalid ObjectCursor");
@@ -401,22 +416,17 @@ bool CegoTableCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
 
 	    if ( pc && len > 0 )
 	    {
-
+	       	      
 		bool useEntry=true;
 		
-		int tid;
-		int tastep;
-
+		unsigned long long tid;
+		unsigned long long tastep;
 		CegoTupleState ts;
 
-		memcpy(&tid, pc, sizeof(int));
-
-		char* pctastep = pc + sizeof(int);
-		memcpy(&tastep, pctastep, sizeof(int));
-
-		char* pcts = pc + sizeof(int) + sizeof(int);
-		memcpy(&ts, pcts, sizeof(CegoTupleState));
+		int toff = _qh.decodeTupleHeader(tid, tastep, ts, pc);
 
+		char* tp = pc + toff;
+		int tlen = len - toff;
 
 		if (tid != 0 )
 		{
@@ -428,8 +438,7 @@ bool CegoTableCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
 			     && tid == _pTM->getTID(_tabSetId) 
 			     && tastep < _pTM->getTAStep(_tabSetId) )
 			{
-			    CegoTupleState ts;
-			    _qh.decodeFVL(fl, pc, len, tid, tastep, ts);			    
+			    _qh.decodeFVL(fl, tp, tlen);
 			    return true;   
 			}
 			else 
@@ -447,8 +456,7 @@ bool CegoTableCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
 				      || ( ts == DELETED && tid != _pTM->getTID(_tabSetId)))))
 			    
 			{
-			    CegoTupleState ts;
-			    _qh.decodeFVL(fl, pc, len, tid, tastep, ts);			    
+			    _qh.decodeFVL(fl, tp, tlen);			    
 			    return true;   
 			}
 			else 
@@ -460,8 +468,7 @@ bool CegoTableCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
 		
 		if ( useEntry )
 		{	       
-		    CegoTupleState ts;
-		    _qh.decodeFVL(fl, pc, len, tid, tastep, ts);
+		    _qh.decodeFVL(fl, tp, tlen);
 		    return true;
 		}
 	    }
@@ -473,6 +480,11 @@ bool CegoTableCursor::getNext(ListT<CegoField>& fl, CegoDataPointer& dp)
     }    
 }
 
+const CegoObject::ObjectType CegoTableCursor::getIndexType() const
+{
+    return _type;
+}
+
 const Chain& CegoTableCursor::getIndexName() const
 {
     return _idxName;
@@ -492,10 +504,10 @@ void CegoTableCursor::abort()
     {
 	_pC->abort();
     }
+
+    /* still not clear, if we have to reset this too
+    _idxMatch = CegoAttrCond::INAPP;
+    _idxSetup = false;
+    _idxCached = false;
+    */ 
 }
-
-
-
-
-
-
