@@ -6,30 +6,15 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2013 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoTransactionManager
 // 
-// Description: 
+// Description: Transaction data structures
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -54,6 +39,7 @@
 #include "CegoTupleState.h"
 #include "CegoTableManager.h"
 #include "CegoAVLIndexManager.h"
+#include "CegoBTreeManager.h"
 
 #define RBSEP "#"
 #define SYS_RB "rbcatlog"
@@ -83,12 +69,12 @@ CegoTransactionManager::TAEntry::TAEntry()
 {
 }
 
-CegoTransactionManager::TAEntry::TAEntry(const int tid)
+CegoTransactionManager::TAEntry::TAEntry(const unsigned long long tid)
 {
     _tid = tid;
 }
 
-CegoTransactionManager::TAEntry::TAEntry(const int tid, const CegoBufferPage& bp, const CegoTableObject& oe)
+CegoTransactionManager::TAEntry::TAEntry(const unsigned long long tid, const CegoBufferPage& bp, const CegoTableObject& oe)
 {
     _tid = tid;
     _bp = bp;
@@ -99,12 +85,11 @@ CegoTransactionManager::TAEntry::~TAEntry()
 {
 }
 
-const int CegoTransactionManager::TAEntry::getTid() const
+const unsigned long long CegoTransactionManager::TAEntry::getTid() const
 {
     return _tid;
 }
 
-
 CegoBufferPage& CegoTransactionManager::TAEntry::getBufferPage()
 {
     return _bp;
@@ -188,7 +173,7 @@ void CegoTransactionManager::initLock(int tabSetId)
     // nothing to do anymore
 }
 
-void CegoTransactionManager::newRBEntry(int tabSetId, int tid, int fileId, int pageId, int offset, const Chain& tableName)
+void CegoTransactionManager::newRBEntry(int tabSetId, unsigned long long tid, int fileId, int pageId, int offset, const Chain& tableName)
 {
 
     TAEntry *pTAE = _taList.Find( TAEntry(tid));
@@ -207,6 +192,7 @@ void CegoTransactionManager::newRBEntry(int tabSetId, int tid, int fileId, int p
 	{
 	    _pTM->createDataTable(tabSetId, taTable, CegoObject::RBSEG, _rbcatSchema);
 	}
+
 	_pTM->getObjectWithFix(tabSetId, taTable, CegoObject::RBSEG, oe, bp);
 	
 	TAEntry tae(tid, bp, oe);
@@ -215,8 +201,9 @@ void CegoTransactionManager::newRBEntry(int tabSetId, int tid, int fileId, int p
 	pTAE = _taList.Find( TAEntry(tid));
     }
     
-    CegoFieldValue fv1(INT_TYPE, &tid, sizeof(int));
-    CegoField f1(CegoField(SYS_RB, SYS_RB, SYS_RB_TID, INT_TYPE, sizeof(int), fv1, false, 1));
+
+    // CegoFieldValue fv1(ULONG_TYPE, &tid, sizeof(unsigned long long));
+    // CegoField f1(CegoField(SYS_RB, SYS_RB, SYS_RB_TID, ULONG_TYPE, sizeof(int), fv1, false, 1));
     
     CegoFieldValue fv2(INT_TYPE, &fileId, sizeof(int));
     CegoField f2(CegoField(SYS_RB, SYS_RB, SYS_RB_FILEID, INT_TYPE, sizeof(int), fv2, true, 2));	
@@ -231,14 +218,14 @@ void CegoTransactionManager::newRBEntry(int tabSetId, int tid, int fileId, int p
     CegoField f5(CegoField(SYS_RB, SYS_RB, SYS_RB_TABLE, VARCHAR_TYPE, MAX_OBJNAME_LEN, fv5, false, 5));	
     
     ListT<CegoField> fl;
-    fl.Insert(f1);
+    // fl.Insert(f1);
     fl.Insert(f2);
     fl.Insert(f3);
     fl.Insert(f4);
     fl.Insert(f5);
         
-    int rbtid = 0;
-    int rbtastep = 0;
+    unsigned long long rbtid = 0;
+    unsigned long long rbtastep = 0;
     CegoTupleState ts = COMMITTED;
     
     char *pBufBase = 0;
@@ -266,7 +253,7 @@ void CegoTransactionManager::newRBEntry(int tabSetId, int tid, int fileId, int p
    a possible deadlock
 */
    
-void CegoTransactionManager::getTransactionAffectedTables(int tabSetId, int tid, SetT<Chain>& tableList)
+void CegoTransactionManager::getTransactionAffectedTables(int tabSetId, unsigned long long tid, SetT<Chain>& tableList)
 {
 
     CegoObjectCursor *pOC = 0;
@@ -320,9 +307,9 @@ void CegoTransactionManager::getTransactionAffectedTables(int tabSetId, int tid,
     }
 }
 
-long CegoTransactionManager::commitTransaction(int tabSetId, int tid)
+unsigned long long CegoTransactionManager::commitTransaction(int tabSetId, unsigned long long tid)
 {
-    long opCount = 0;
+    unsigned long long opCount = 0;
 
     TAEntry *pTAE = _taList.Find( TAEntry(tid) );
     
@@ -347,9 +334,9 @@ long CegoTransactionManager::commitTransaction(int tabSetId, int tid)
     Chain commitTa = Chain(SYS_RBCOMMIT) + Chain(RBSEP) + Chain(tid);
     _pTM->renameObject(tabSetId, taTable, CegoObject::RBSEG, commitTa);
     pTAE->getTableObject().setName(commitTa);
-    
+ 
     opCount = doCommit(tabSetId, commitTa);
-    
+
     // Step 2 : Cleaning up rollback segment
     
     _pDBMng->bufferUnfix(pTAE->getBufferPage(), true, _pTM->getLockHandler());
@@ -359,10 +346,10 @@ long CegoTransactionManager::commitTransaction(int tabSetId, int tid)
     return opCount;
 }
 
-long CegoTransactionManager::doCommit(int tabSetId, const Chain& rbo)
+unsigned long long CegoTransactionManager::doCommit(int tabSetId, const Chain& rbo)
 {
 
-    long opCount = 0;
+    unsigned long long opCount = 0;
 
     CegoObjectCursor *pOC = _pTM->getObjectCursor(tabSetId, rbo, rbo, CegoObject::RBSEG);
     
@@ -382,10 +369,12 @@ long CegoTransactionManager::doCommit(int tabSetId, const Chain& rbo)
 	ListT<CegoBTreeObject> cachedBTreeList;
 	ListT<CegoKeyObject> cachedKeyList;
 	ListT<CegoCheckObject> cachedCheckList;
-	
+	int numInvalid;
+
 	while (moreTuple)
 	{
 	    
+
 	    int fileId, pageId, offset;
 	    
 	    CegoField *pF = schema.Find(CegoField(SYS_RB, SYS_RB_FILEID));
@@ -402,10 +391,10 @@ long CegoTransactionManager::doCommit(int tabSetId, const Chain& rbo)
 	    
 	    CegoDataPointer dp(fileId, pageId, offset);
 	    
-	    int tid;
-	    int tastep;
+	    unsigned long long tid;
+	    unsigned long long tastep;
 	    CegoTupleState ts;
-	    
+
 	    _pTM->getTupleInfo(tabSetId, dp, tid, tastep, ts);			
 	    _pTM->setTupleInfo(tabSetId, dp, 0, 0, COMMITTED);
 
@@ -430,32 +419,35 @@ long CegoTransactionManager::doCommit(int tabSetId, const Chain& rbo)
 		    cachedBTreeList.Empty();
 		    cachedKeyList.Empty();
 		    cachedCheckList.Empty();
-		    _pTM->getObjectListByTable(tabSetId, cachedTable, cachedIdxList, cachedBTreeList, cachedKeyList, cachedCheckList);
+		    _pTM->getObjectListByTable(tabSetId, cachedTable, cachedIdxList, cachedBTreeList, cachedKeyList, cachedCheckList, numInvalid);
 		    
 		}
 		
 		char* p;
 		int len;
-		unsigned long lockId;
+
+		CegoBufferPage bp;
 		
 		try
 		{
-		    lockId = _pTM->claimDataPtr(tabSetId, CegoLockHandler::READ, CegoBufferPool::SYNC, dp, p, len);
-		    
-		    int tid;
-		    int tastep;
-		    CegoTupleState ts;
-		    _qh.decodeFVL(cachedFvl, p, len, tid, tastep, ts);
-		    
-		    _pTM->deleteDataTable(tabSetId, cachedTable, CegoObject::TABLE, dp, cachedFvl, cachedIdxList, cachedBTreeList, cachedKeyList, false);
+		    bp = _pTM->claimDataPtrUnlocked(tabSetId, CegoBufferPool::SYNC, dp, p, len);
+
+		    int toff = _qh.skipTupleHeader();
+
+		    char* tp = p + toff;
+		    int tlen = len - toff;
+		   
+		    _qh.decodeFVL(cachedFvl, tp, tlen);
+
+		    _pTM->deleteDataTableEntry(tabSetId, cachedTable, CegoObject::TABLE, dp, cachedFvl, cachedIdxList, cachedBTreeList, cachedKeyList, false);
 		    
 		}
 		catch ( Exception e)
 		{   
-		    _pTM->releaseDataPtr(lockId);
-		    throw Exception(EXLOC, Chain("Cannot commit transaction"), e );
+		    _pTM->releaseDataPtrUnlocked(bp);
+		    throw e;
 		}
-		_pTM->releaseDataPtr(lockId);		    
+		_pTM->releaseDataPtrUnlocked(bp);		    
 		
 	    }
 
@@ -472,10 +464,13 @@ long CegoTransactionManager::doCommit(int tabSetId, const Chain& rbo)
     {
 	pOC->abort();
 	delete pOC;
-	throw Exception(EXLOC, Chain("Cannot commit transaction"), e);
+
+	Chain msg;
+	e.pop(msg);
+	
+	throw Exception(EXLOC, Chain("Cannot commit transaction : ") + msg);
     }
     
-    
     pOC->abort();
     delete pOC;
 
@@ -534,10 +529,10 @@ void CegoTransactionManager::finishOpenTransaction(int tabSetId)
     }    
 }
 
-long CegoTransactionManager::rollbackTransaction(int tabSetId, int tid)
+unsigned long long CegoTransactionManager::rollbackTransaction(int tabSetId, unsigned long long tid)
 {
     
-    long opCount = 0;
+    unsigned long long opCount = 0;
 
     TAEntry *pTAE = _taList.Find( TAEntry(tid) );
     
@@ -545,7 +540,7 @@ long CegoTransactionManager::rollbackTransaction(int tabSetId, int tid)
     {	    
 	
 	// no transaction entry found. It seems, no 
-	// modifying operations have been take place, so we can remove the segemtn and return immediately
+	// modifying operations have been take place, so we can remove the segment and return immediately
 	
 	Chain taTable = Chain(SYS_RB) + Chain(RBSEP) + Chain(tid);
 	if ( _pTM->objectExists(tabSetId, taTable, CegoObject::RBSEG) )
@@ -573,11 +568,11 @@ long CegoTransactionManager::rollbackTransaction(int tabSetId, int tid)
     
 }
 
-long CegoTransactionManager::doRollback(int tabSetId, const Chain& rbo)
+unsigned long long CegoTransactionManager::doRollback(int tabSetId, const Chain& rbo)
 {
 
-    long opCount = 0;
-
+    unsigned long long opCount = 0;
+    unsigned long long recLock = 0;
     CegoObjectCursor *pOC = 0;
 
     try
@@ -598,7 +593,8 @@ long CegoTransactionManager::doRollback(int tabSetId, const Chain& rbo)
 	ListT<CegoBTreeObject> cachedBTreeList;
 	ListT<CegoKeyObject> cachedKeyList;
 	ListT<CegoCheckObject> cachedCheckList;
-	
+	int numInvalid;
+
 	while (moreTuple)
 	{
 	    
@@ -624,12 +620,17 @@ long CegoTransactionManager::doRollback(int tabSetId, const Chain& rbo)
 	    
 	    CegoDataPointer dp(fileId, pageId, offset);
 	    
-	    int tid;
-	    int tastep;
+	    unsigned long long tid;
+	    unsigned long long tastep;
 	    CegoTupleState ts;
-	    
+
+	    recLock = _pTM->getLockHandler()->lockRecord(dp, CegoLockHandler::WRITE);
+
 	    _pTM->getTupleInfo(tabSetId, dp, tid, tastep, ts);			
 	    _pTM->setTupleInfo(tabSetId, dp, 0, 0, COMMITTED);
+
+	    _pTM->getLockHandler()->unlockRecord(recLock);
+	    
 	    opCount++;
 	    
 	    if ( ts == INSERTED || ts == OBSOLETE ) 
@@ -645,36 +646,37 @@ long CegoTransactionManager::doRollback(int tabSetId, const Chain& rbo)
 		    cachedBTreeList.Empty();
 		    cachedKeyList.Empty();
 		    cachedCheckList.Empty();
-		    _pTM->getObjectListByTable(tabSetId, cachedTable, cachedIdxList, cachedBTreeList, cachedKeyList, cachedCheckList);
+		    _pTM->getObjectListByTable(tabSetId, cachedTable, cachedIdxList, cachedBTreeList, cachedKeyList, cachedCheckList, numInvalid);
 		    
 		}
 		
 		char* p;
 		int len;
-		unsigned long lockId;
+
+		CegoBufferPage bp;
 		
 		try
 		{
-		    lockId = _pTM->claimDataPtr(tabSetId, CegoLockHandler::READ, CegoBufferPool::SYNC, dp, p, len);
+		    bp = _pTM->claimDataPtrUnlocked(tabSetId, CegoBufferPool::SYNC, dp, p, len);
 		    
-		    int tid;
-		    CegoTupleState ts;
-		    _qh.decodeFVL(cachedFvl, p, len, tid, tastep, ts);
+
+		    int toff = _qh.skipTupleHeader();
+
+		    char* tp = p + toff;
+		    int tlen = len - toff;
 		    
-		    _pTM->deleteDataTable(tabSetId, tableName, CegoObject::TABLE, dp, cachedFvl, cachedIdxList, cachedBTreeList, cachedKeyList, false, true);
+		    _qh.decodeFVL(cachedFvl, tp, tlen);
+		    
+		    _pTM->deleteDataTableEntry(tabSetId, tableName, CegoObject::TABLE, dp, cachedFvl, cachedIdxList, cachedBTreeList, cachedKeyList, false, true);
 		    
 		}
 		catch ( Exception e)
 		{   
-		    _pTM->releaseDataPtr(lockId);
-		    
-		    Chain msg;
-		    e.pop(msg);
-		    
-		    // _pDBMng->log(_modId, Logger::NOTICE, Chain("Incomplete rollback for table ") + tableName + Chain(" : ") + msg);
-		    throw Exception(EXLOC, Chain("Cannot delete entry for table"), e);
+		    _pTM->releaseDataPtrUnlocked(bp);
+
+		    throw e;
 		}
-		_pTM->releaseDataPtr(lockId);
+		_pTM->releaseDataPtrUnlocked(bp);
 		
 	    }
 
@@ -689,6 +691,9 @@ long CegoTransactionManager::doRollback(int tabSetId, const Chain& rbo)
     }
     catch ( Exception e )
     {
+	if ( recLock )
+	    _pTM->getLockHandler()->unlockRecord(recLock);
+	
 	if ( pOC )
 	{
 	    pOC->abort();
@@ -739,8 +744,7 @@ void CegoTransactionManager::getTransactionInfo(int tabSetId, const Chain& rbo,
     }
 }
 
-
-void CegoTransactionManager::recordUpdate(int tabSetId, int tid, const CegoDataPointer& dp )
+void CegoTransactionManager::recordUpdate(int tabSetId, unsigned long long tid, const CegoDataPointer& dp )
 {
 
     TAEntry *pUDE = _udList.Find( TAEntry(tid));
@@ -767,8 +771,8 @@ void CegoTransactionManager::recordUpdate(int tabSetId, int tid, const CegoDataP
     int pageId = dp.getPageId();
     int offset = dp.getOffset();
 
-    CegoFieldValue fv1(INT_TYPE, &tid, sizeof(int));
-    CegoField f1(CegoField(SYS_UPDTAB, SYS_UPDTAB, SYS_UPDTAB_TID, INT_TYPE, sizeof(int), fv1, true, 1));	
+    // CegoFieldValue fv1(ULONG_TYPE, &tid, sizeof(int));
+    // CegoField f1(CegoField(SYS_UPDTAB, SYS_UPDTAB, SYS_UPDTAB_TID, ULONG_TYPE, sizeof(int), fv1, true, 1));	
 
     CegoFieldValue fv2(INT_TYPE, &fileId, sizeof(int));
     CegoField f2(CegoField(SYS_UPDTAB, SYS_UPDTAB, SYS_UPDTAB_FILEID, INT_TYPE, sizeof(int), fv2, true, 2));	
@@ -780,13 +784,13 @@ void CegoTransactionManager::recordUpdate(int tabSetId, int tid, const CegoDataP
     CegoField f4(CegoField(SYS_UPDTAB, SYS_UPDTAB, SYS_UPDTAB_OFFSET, INT_TYPE, sizeof(int), fv4, true, 4));
 
     ListT<CegoField> fl;
-    fl.Insert(f1);
+    // fl.Insert(f1);
     fl.Insert(f2);
     fl.Insert(f3);
     fl.Insert(f4);
 
-    int rbtid = 0;
-    int rbtastep = 0;
+    unsigned long long rbtid = 0;
+    unsigned long long rbtastep = 0;
     CegoTupleState ts = COMMITTED;
 
     char *pBufBase = 0;
@@ -815,14 +819,14 @@ void CegoTransactionManager::recordUpdate(int tabSetId, int tid, const CegoDataP
    In this case, the ( virgin ) index still has to expand for the updated tuples
 */
 
-void CegoTransactionManager::commitUpdate(int tabSetId, const Chain& idxName, int tid, bool doAppend)
+void CegoTransactionManager::commitUpdate(int tabSetId, const Chain& idxName, const CegoObject::ObjectType idxType, unsigned long long tid, bool doAppend)
 {
 
     CegoObjectCursor *pOC = 0;
-    unsigned long lockId = 0;
     CegoBufferPage bp;
+    CegoBufferPage cbp;
     bool isFixed = false;
-    
+
     try
     {
 
@@ -831,98 +835,188 @@ void CegoTransactionManager::commitUpdate(int tabSetId, const Chain& idxName, in
 	if ( pUDE == 0 )
 	{	    
 	    // no update entry found. It seems, no 
-	    // modifying operations have been take place, so we can retrn immediately
+	    // modifying operations have been take place, so we can return immediately
 	    return;
 	    // throw Exception(EXLOC, Chain("Cannot find transaction ") + Chain(tid));
 	}
 
-
 	Chain udTable = pUDE->getTableObject().getName();
-	
-	CegoTableObject ioe;
-	_pTM->getObjectWithFix(tabSetId, idxName, CegoObject::INDEX, ioe, bp);			
-	isFixed = true;
-	
-	pOC = _pTM->getObjectCursor(tabSetId, udTable, udTable, CegoObject::RBSEG);
-	
 
-	ListT<CegoField> schema = _updSchema;
-		
-	CegoDataPointer updp;
-	bool moreTuple = _pTM->getFirstTuple(pOC, schema, updp);
-    
-	while ( moreTuple)
+	if ( idxType  == CegoObject::AVLTREE 
+	     || idxType == CegoObject::UAVLTREE 
+	     || idxType == CegoObject::PAVLTREE )
 	{
-	    
-	    int fileId, pageId, offset;
-	    
-	    CegoField *pF = schema.Find(CegoField(SYS_UPDTAB, SYS_UPDTAB_FILEID));
-	    if (pF)
-		fileId = *(int*)pF->getValue().getValue();
-	    
-	    pF = schema.Find(CegoField(SYS_UPDTAB, SYS_UPDTAB_PAGEID));
-	    if (pF)
-		pageId = *(int*)pF->getValue().getValue();
-	    
-	    pF = schema.Find(CegoField(SYS_UPDTAB, SYS_UPDTAB_OFFSET));
-	    if (pF)
-		offset = *(int*)pF->getValue().getValue();
-	    
-	    CegoDataPointer dp(fileId, pageId, offset);
-	    
-	    ListT<CegoField> fvl = ioe.getSchema();
-	    
-	    char* p;
-	    int len;
-	    
-	    CegoDataPointer sysEntry(bp.getFileId(), bp.getPageId(), bp.getEntryPos());
-	
-	    
-	    lockId = _pTM->claimDataPtr(tabSetId, CegoLockHandler::READ, CegoBufferPool::SYNC, dp, p, len);
-	    
-	    int tid;
-	    int tastep;
-	    CegoTupleState ts;
-	    _qh.decodeFVL(fvl, p, len, tid, tastep, ts);
-	    
-	    char ip[TABMNG_MAXINDEXVALUE];
-	    int ilen = TABMNG_MAXINDEXVALUE;
-	    int idxLen;
-	    CegoDataPointer ritp;
-	    	   
-	    _pTM->extractIndexValue(fvl, ioe.getSchema(), ip, ilen, idxLen);
-	
-	    CegoAVLIndexManager idxMng(_pTM);
-	    idxMng.insertNativeIndexTable(ioe, sysEntry, dp, ip, idxLen, tid, doAppend, ritp);
 
-	    _pTM->releaseDataPtr(lockId);
-	    lockId = 0;
 	    
+	    CegoTableObject ioe;
+	    _pTM->getObjectWithFix(tabSetId, idxName, idxType, ioe, bp);			
+	    isFixed = true;
+	    
+	    pOC = _pTM->getObjectCursor(tabSetId, udTable, udTable, CegoObject::RBSEG);
+	    
+	    
+	    ListT<CegoField> schema = _updSchema;
+	    
+	    CegoDataPointer updp;
+	    bool moreTuple = _pTM->getFirstTuple(pOC, schema, updp);
+	    
+	    while ( moreTuple)
+	    {
+		
+		int fileId, pageId, offset;
+		
+		CegoField *pF = schema.Find(CegoField(SYS_UPDTAB, SYS_UPDTAB_FILEID));
+		if (pF)
+		    fileId = *(int*)pF->getValue().getValue();
+		
+		pF = schema.Find(CegoField(SYS_UPDTAB, SYS_UPDTAB_PAGEID));
+		if (pF)
+		    pageId = *(int*)pF->getValue().getValue();
+		
+		pF = schema.Find(CegoField(SYS_UPDTAB, SYS_UPDTAB_OFFSET));
+		if (pF)
+		    offset = *(int*)pF->getValue().getValue();
+		
+		CegoDataPointer dp(fileId, pageId, offset);
+		
+		ListT<CegoField> fvl = ioe.getSchema();
+		
+		char* p;
+		int len;
+		
+		CegoDataPointer sysEntry(bp.getFileId(), bp.getPageId(), bp.getEntryPos());
+				
+		cbp = _pTM->claimDataPtrUnlocked(tabSetId, CegoBufferPool::SYNC, dp, p, len);
+	      		
 
-	    moreTuple = _pTM->getNextTuple(pOC, schema, updp);
+		int toff = _qh.skipTupleHeader();
+
+		char* tp = p + toff;
+		int tlen = len - toff;
+
+		_qh.decodeFVL(fvl, tp, tlen);
+		
+		char ip[TABMNG_MAXINDEXVALUE];
+		int ilen = TABMNG_MAXINDEXVALUE;
+		int idxLen;
+		CegoDataPointer ritp;
+		
+		_pTM->extractIndexValue(fvl, ioe.getSchema(), ip, ilen, idxLen);
+		
+		CegoAVLIndexManager idxMng(_pTM);
+		idxMng.insertNativeIndexTable(ioe, sysEntry, dp, ip, idxLen, tid, doAppend, ritp);
+
+		_pTM->releaseDataPtrUnlocked(cbp);
+		cbp = CegoBufferPage();
+		
+		
+		moreTuple = _pTM->getNextTuple(pOC, schema, updp);
+
+	    }
+
+	    delete pOC;
+	    pOC = 0;
+	    
+	    // now we can release the index root page
+	    _pTM->getDBMng()->bufferUnfix(bp, true, _pTM->getLockHandler());
+	    isFixed = false;
+	    
+	    _pTM->removeObject(tabSetId, udTable, CegoObject::RBSEG);
+	    
+	    // release the update commit object
+	    _pTM->getDBMng()->bufferUnfix(pUDE->getBufferPage(), true, _pTM->getLockHandler());	
+	    _udList.Remove(TAEntry(tid));
 	    
 	}
+	else if ( idxType == CegoObject::BTREE 
+		  || idxType == CegoObject::UBTREE 
+		  || idxType == CegoObject::PBTREE )
+	{
+	    
+
+	    CegoBTreeObject boe;
+	    _pTM->getObjectWithFix(tabSetId, idxName, idxType, boe, bp);			
+	    isFixed = true;
+	    
+	    pOC = _pTM->getObjectCursor(tabSetId, udTable, udTable, CegoObject::RBSEG);
+	    
+	    
+	    ListT<CegoField> schema = _updSchema;
+	    
+	    CegoDataPointer updp;
+	    bool moreTuple = _pTM->getFirstTuple(pOC, schema, updp);
+	    
+	    while ( moreTuple)
+	    {
+		
+		int fileId, pageId, offset;
+		
+		CegoField *pF = schema.Find(CegoField(SYS_UPDTAB, SYS_UPDTAB_FILEID));
+		if (pF)
+		    fileId = *(int*)pF->getValue().getValue();
+		
+		pF = schema.Find(CegoField(SYS_UPDTAB, SYS_UPDTAB_PAGEID));
+		if (pF)
+		    pageId = *(int*)pF->getValue().getValue();
+		
+		pF = schema.Find(CegoField(SYS_UPDTAB, SYS_UPDTAB_OFFSET));
+		if (pF)
+		    offset = *(int*)pF->getValue().getValue();
+		
+		CegoDataPointer dp(fileId, pageId, offset);
+		
+		ListT<CegoField> fvl = boe.getSchema();
+		
+		char* p;
+		int len;
+		
+		CegoDataPointer sysEntry(bp.getFileId(), bp.getPageId(), bp.getEntryPos());
+		
+		cbp = _pTM->claimDataPtrUnlocked(tabSetId, CegoBufferPool::SYNC, dp, p, len);
+
+		int toff = _qh.skipTupleHeader();
+
+		char* tp = p + toff;
+		int tlen = len - toff;
+
+		_qh.decodeFVL(fvl, tp, tlen);
+
+		CegoBTreeValue iv;
+		iv.valueFromSchema(fvl, boe.getSchema());
+		
+		CegoBTreeManager btreeMng(_pTM, &boe);
+		btreeMng.insertBTree(dp,
+				     iv,
+				     tid);
+	    
+		_pTM->releaseDataPtrUnlocked(cbp, true);
+		cbp = CegoBufferPage();
+		
+		
+		moreTuple = _pTM->getNextTuple(pOC, schema, updp);
+
+	    }
+
+	    delete pOC;
+	    pOC = 0;
+	    
+	    // now we can release the index root page
+	    _pTM->getDBMng()->bufferUnfix(bp, true, _pTM->getLockHandler());
+	    isFixed = false;
+	    
+	    _pTM->removeObject(tabSetId, udTable, CegoObject::RBSEG);
+	    
+	    // release the update commit object
+	    _pTM->getDBMng()->bufferUnfix(pUDE->getBufferPage(), true, _pTM->getLockHandler());	
+	    _udList.Remove(TAEntry(tid));
+	        
+	}
 	
-	delete pOC;
-	pOC = 0;
-	
-	// now we can release the index root page
-	_pTM->getDBMng()->bufferUnfix(bp, true, _pTM->getLockHandler());
-	isFixed = false;
-
-	_pTM->removeObject(tabSetId, udTable, CegoObject::RBSEG);
-
-	// release the update commit object
-	_pTM->getDBMng()->bufferUnfix(pUDE->getBufferPage(), true, _pTM->getLockHandler());	
-	_udList.Remove(TAEntry(tid));
-
     }
     catch ( Exception e )
     {
-	if ( lockId )
-	{
-	    _pTM->releaseDataPtr(lockId);
-	}
+	_pTM->releaseDataPtrUnlocked(cbp);
+	
 	if ( isFixed )
 	{
 	    _pTM->getDBMng()->bufferUnfix(bp, true, _pTM->getLockHandler());
@@ -941,4 +1035,3 @@ void CegoTransactionManager::commitUpdate(int tabSetId, const Chain& idxName, in
 	delete pOC;
     }
 }
-
