@@ -6,30 +6,15 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2010 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoXMLSpace
 // 
-// Description: 
+// Description: XML based database control file management
 //
-// Status: QG-2.6
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -39,11 +24,11 @@
 
 // base includes
 #include <lfcbase/File.h>
-#include <lfcbase/Host.h>
 #include <lfcbase/Datetime.h>
 #include <lfcbase/ThreadLock.h>
 #include <lfcbase/Tokenizer.h>
 #include <lfcbase/Matcher.h>
+#include <lfcbase/Directory.h>
 
 // xml includes
 #include <lfcxml/XMLSuite.h>
@@ -398,6 +383,17 @@ int CegoXMLSpace::getMaxFixTries()
     return maxFixTries;    
 }
 
+/* no more used
+int CegoXMLSpace::getBufferPoolHashKey()
+{
+    P();
+    int hashKey = _pDoc->getRootElement()->getAttributeValue(XML_BUFFERPOOLHASHKEY_ATTR).asInteger();
+    V();
+    if ( hashKey == 0 )
+	hashKey = BUPMNG_HASHKEY;
+    return hashKey;    
+}
+*/ 
 int CegoXMLSpace::getPageLockTimeout()
 {
     P();
@@ -448,16 +444,38 @@ int CegoXMLSpace::getNumLockTries()
     return numLockTries;
 }
 
-bool CegoXMLSpace::isCaseSensitiveMode()
+char CegoXMLSpace::getCaseSensitiveFlag()
 {
     P();
     Chain mode = _pDoc->getRootElement()->getAttributeValue(XML_CSMODE_ATTR);
     V();
+    if ( mode == Chain(XML_NONE_VALUE) )
+	return 2;
+    if ( mode == Chain(XML_STR_VALUE) )
+	return 1;
+    return 0;
+}
+
+bool CegoXMLSpace::allowDuplicateNull()
+{
+    P();
+    Chain mode = _pDoc->getRootElement()->getAttributeValue(XML_DUPLICATENULL_ATTR);
+    V();
     if ( mode == Chain(XML_ON_VALUE) )
 	return true;
     return false;    
 }
 
+void CegoXMLSpace::setQuoteEscapeMode(bool m)
+{
+    P();
+    if ( m )
+	_pDoc->getRootElement()->setAttribute(XML_QESCMODE_ATTR, XML_ON_VALUE);
+    else 
+	_pDoc->getRootElement()->setAttribute(XML_QESCMODE_ATTR, XML_OFF_VALUE);
+    V();
+}
+
 bool CegoXMLSpace::isQuoteEscapeMode()
 {
     P();
@@ -468,14 +486,41 @@ bool CegoXMLSpace::isQuoteEscapeMode()
     return false;    
 }
 
-Chain CegoXMLSpace::getDateFormatString()
+Chain CegoXMLSpace::getDateTimeFormat()
 {
+    Chain dateTimeFormat;
+    
     P();
-    Chain dtFormat = _pDoc->getRootElement()->getAttributeValue(XML_DTFORMAT_ATTR);
+    dateTimeFormat = _pDoc->getRootElement()->getAttributeValue(XML_DATETIMEFORMAT_ATTR);
     V();
-    return dtFormat;
+    
+    return dateTimeFormat;
 }
 
+ListT<Chain> CegoXMLSpace::getDateFormatList()
+{
+    ListT<Chain> formatList;
+    
+    P();
+
+    Element *pRoot = _pDoc->getRootElement();    
+    if ( pRoot )
+    {
+	
+	ListT<Element*> formatElementList = pRoot->getChildren(XML_DATEFORMAT_ELEMENT);
+	
+	Element **pFormat = formatElementList.First();
+	while ( pFormat )
+	{
+	    formatList.Insert((*pFormat)->getAttributeValue(XML_VALUE_ATTR));
+	    pFormat = formatElementList.Next();
+	}	
+    }
+    V();
+
+    return formatList;
+}    
+
 void CegoXMLSpace::setArchRestoreProg(const Chain& progName)
 {
     P();
@@ -632,7 +677,6 @@ Chain CegoXMLSpace::getMediator(const Chain& tabSetName)
     return mediator;
 }
 
-
 void CegoXMLSpace::setPrimary(const Chain& tabSetName, const Chain& hostName)
 {
     P();
@@ -724,7 +768,7 @@ void CegoXMLSpace::getTSforMedAndPrim(const Chain& mediator, const Chain& primar
     V();
 }
 
-int CegoXMLSpace::getTabSetId(const Chain& tabSetName)
+int CegoXMLSpace::getTabSetId(const Chain& tabSetName) const
 {
     int tabSetId;
 
@@ -744,11 +788,10 @@ int CegoXMLSpace::getTabSetId(const Chain& tabSetName)
     return tabSetId;
 }
 
-
 Chain CegoXMLSpace::getTabSetName(int tabSetId)
 {
     Chain tabSetName;
-
+    
     Element *pTSE = getCachedTableSetElement(tabSetId);
     if ( pTSE == 0 )
     {
@@ -781,7 +824,6 @@ int CegoXMLSpace::getTmpFid(const Chain& tabSetName)
     return tmpFid;
 }
 
-
 void CegoXMLSpace::setTSSysSize(const Chain& tableSet, int sysSize)
 {
     P();
@@ -800,7 +842,6 @@ void CegoXMLSpace::setTSSysSize(const Chain& tableSet, int sysSize)
     V();
 }
 
-
 int CegoXMLSpace::getSysSize(const Chain& tabSetName)
 {
     int sysSize;
@@ -821,7 +862,6 @@ int CegoXMLSpace::getSysSize(const Chain& tabSetName)
     return sysSize;
 }
 
-
 void CegoXMLSpace::setTSTmpSize(const Chain& tableSet, int tmpSize)
 {
     P();
@@ -840,7 +880,6 @@ void CegoXMLSpace::setTSTmpSize(const Chain& tableSet, int tmpSize)
     V();
 }
 
-
 int CegoXMLSpace::getTmpSize(const Chain& tabSetName)
 {
     int tmpSize;
@@ -889,7 +928,7 @@ void CegoXMLSpace::setTSLogNum(const Chain& tableSet, int logNum)
 		{
 		    Element *pLE = new Element(XML_LOGFILE_ELEMENT);
 		    
-		    Chain logName = tsRoot + Chain("/") + tableSet + Chain("redo") +  + Chain(i) + Chain(".log");
+		    Chain logName = tsRoot + Chain("/") + tableSet + Chain("redo") +  + Chain(i) + Chain(TABMNG_LOGSUFFIX);
 		    pLE->setAttribute(XML_NAME_ATTR, logName);
 		    pLE->setAttribute(XML_SIZE_ATTR, Chain(logSize));
 		    pLE->setAttribute(XML_STATUS_ATTR, XML_FREE_VALUE);
@@ -946,6 +985,65 @@ void CegoXMLSpace::setTSLogSize(const Chain& tableSet, int logSize)
     throw Exception(EXLOC, "Root element not found");    
 }
 
+void CegoXMLSpace::resetTSRoot(const Chain& tableSet, const Chain& tsRoot)
+{
+
+    P();
+    Element *pRoot = _pDoc->getRootElement();
+    if ( pRoot ) 
+    {
+	ListT<Element*> tabSetList = pRoot->getChildren(XML_TABLESET_ELEMENT);
+	Element **pTSE = tabSetList.First();
+	while ( pTSE ) 
+	{
+	    if ( (*pTSE)->getAttributeValue(XML_NAME_ATTR) == tableSet )
+	    {
+		ListT<Element*> logFileList = (*pTSE)->getChildren(XML_LOGFILE_ELEMENT);
+		Element **pLF = logFileList.First();
+		int i=0;
+		while ( pLF )
+		{
+		    Chain logName = tsRoot + Chain("/") + tableSet + Chain("redo") +  + Chain(i) + Chain(TABMNG_LOGSUFFIX);
+		    (*pLF)->setAttribute(XML_NAME_ATTR, logName);
+		    pLF = logFileList.Next();
+		    i++;
+		}
+
+		(*pTSE)->setAttribute(XML_TSTICKET_ATTR, tsRoot + Chain("/") + tableSet + Chain("_ticket.xml"));
+		
+		(*pTSE)->setAttribute(XML_SYSNAME_ATTR, tsRoot + Chain("/") + tableSet + Chain(TABMNG_SYSSUFFIX));
+		(*pTSE)->setAttribute(XML_TMPNAME_ATTR, tsRoot + Chain("/") + tableSet + Chain(TABMNG_TEMPSUFFIX));
+
+		Chain oldTsRoot =  (*pTSE)->getAttributeValue(XML_TSROOT_ATTR);  
+
+		ListT<Element*> fileList = (*pTSE)->getChildren(XML_DATAFILE_ELEMENT);
+		Element **pFile = fileList.First();
+		while ( pFile )
+		{
+		    Chain oldName = (*pFile)->getAttributeValue(XML_NAME_ATTR);
+		    Chain newName;
+		    oldName.replace(oldTsRoot, tsRoot, newName);
+		    (*pFile)->setAttribute(XML_NAME_ATTR, newName);
+		    
+		    pFile = fileList.Next();
+		}
+
+		(*pTSE)->setAttribute(XML_TSROOT_ATTR, tsRoot);
+						
+		V();
+		return;
+	    }
+	    pTSE = tabSetList.Next();
+	}
+	
+	V();
+	Chain msg = Chain("Unknown tableset ") + tableSet; 
+	throw Exception(EXLOC, msg);
+    }
+    V();
+    throw Exception(EXLOC, "Root element not found");    
+}
+
 void CegoXMLSpace::setTSSortAreaSize(const Chain& tableSet, int sortAreaSize)
 {
     P();
@@ -964,16 +1062,345 @@ void CegoXMLSpace::setTSSortAreaSize(const Chain& tableSet, int sortAreaSize)
     V();
 }
 
+void CegoXMLSpace::setTSAppSize(const Chain& tableSet, int appSize)
+{
+    P();
+    Element *pTS = getTableSetElement(tableSet);
+
+    if ( pTS )
+    {
+
+	ListT<Element*> fileList = pTS->getChildren(XML_DATAFILE_ELEMENT);
+	Element **pFile = fileList.First();
+	if ( pFile )
+	{
+	    if ( appSize > 0 )
+	    {
+		(*pFile)->setAttribute(XML_SIZE_ATTR, Chain(appSize));
+	    }
+	    else
+	    {
+		pTS->removeChild( *pFile );
+	    }
+	}
+    }
+    else
+    {
+	V();
+	Chain msg = "Unknown tableset <" + tableSet + ">"; 
+	throw Exception(EXLOC, msg);
+    }
+    V();
+}
+
+void CegoXMLSpace::setAutoCorrect(const Chain& tabSetName, bool isAutoCorrect)
+{
+
+    P();
+    Element *pTSE = getTableSetElement(tabSetName);
+    if ( pTSE )
+    {	
+	if ( isAutoCorrect )
+	    pTSE->setAttribute(XML_AUTOCORRECT_ATTR, XML_ON_VALUE);  
+	else
+	    pTSE->setAttribute(XML_AUTOCORRECT_ATTR, XML_OFF_VALUE);  
+    }
+    else
+    {
+	V();
+	Chain msg = Chain("Unknown tableset name ") + tabSetName; 
+	throw Exception(EXLOC, msg);
+    }
+    V();
+}
+
+bool CegoXMLSpace::getAutoCorrect(const Chain& tabSetName)
+{
+    bool isAutoCorrect=false;
+
+    P();
+    Element *pTSE = getTableSetElement(tabSetName);
+    if ( pTSE )
+    {	
+	if ( pTSE->getAttributeValue(XML_AUTOCORRECT_ATTR) == Chain(XML_ON_VALUE))
+	    isAutoCorrect = true;
+    }
+    else
+    {
+	V();
+	Chain msg = Chain("Unknown tableset name ") + tabSetName; 
+	throw Exception(EXLOC, msg);
+    }
+    V();
+    return isAutoCorrect;
+}
+
+void CegoXMLSpace::setTableCacheMode(const Chain& tabSetName, bool isEnabled)
+{
+
+    P();
+    Element *pTSE = getTableSetElement(tabSetName);
+    if ( pTSE )
+    {	
+	if ( isEnabled )
+	    pTSE->setAttribute(XML_TABLECACHE_ATTR, XML_ON_VALUE);  
+	else
+	    pTSE->setAttribute(XML_TABLECACHE_ATTR, XML_OFF_VALUE);  
+    }
+    else
+    {
+	V();
+	Chain msg = Chain("Unknown tableset name ") + tabSetName; 
+	throw Exception(EXLOC, msg);
+    }
+    V();
+}
+
+bool CegoXMLSpace::getTableCacheMode(const Chain& tabSetName)
+{
+    bool isEnabled=false;
+
+    P();
+    Element *pTSE = getTableSetElement(tabSetName);
+    if ( pTSE )
+    {	
+	if ( pTSE->getAttributeValue(XML_TABLECACHE_ATTR) == Chain(XML_ON_VALUE))
+	    isEnabled = true;
+    }
+    else
+    {
+	V();
+	Chain msg = Chain("Unknown tableset name ") + tabSetName; 
+	throw Exception(EXLOC, msg);
+    }
+    V();
+    return isEnabled;
+}
+
+void CegoXMLSpace::setMaxTableCacheEntry(const Chain& tableSet, int numEntry)
+{
+    P();
+    Element *pTS = getTableSetElement(tableSet);
+
+    if ( pTS )
+    {
+	pTS->setAttribute(XML_MAXTABLECACHEENTRY_ATTR, Chain(numEntry));
+    }
+    else
+    {
+	V();
+	Chain msg = "Unknown tableset <" + tableSet + ">"; 
+	throw Exception(EXLOC, msg);
+    }
+    V();
+}
+
+int CegoXMLSpace::getMaxTableCacheEntry(const Chain& tabSetName)
+{
+    int numEntry = 0;
+
+    P();
+    Element *pTSE = getTableSetElement(tabSetName);
+    if ( pTSE )
+    {
+	numEntry =  pTSE->getAttributeValue(XML_MAXTABLECACHEENTRY_ATTR).asInteger();  
+    }
+    else
+    {
+	V();
+	Chain msg = Chain("Unknown tableset name ") + tabSetName; 
+	throw Exception(EXLOC, msg);
+    }
+    V();
+    return numEntry;
+}
+
+void CegoXMLSpace::setMaxTableCacheSize(const Chain& tableSet, int cacheSize)
+{
+    P();
+    Element *pTS = getTableSetElement(tableSet);
+
+    if ( pTS )
+    {
+	pTS->setAttribute(XML_MAXTABLECACHESIZE_ATTR, Chain(cacheSize));
+    }
+    else
+    {
+	V();
+	Chain msg = "Unknown tableset <" + tableSet + ">"; 
+	throw Exception(EXLOC, msg);
+    }
+    V();
+}
+
+int CegoXMLSpace::getMaxTableCacheSize(const Chain& tabSetName)
+{
+    int cacheSize = 0;
+
+    P();
+    Element *pTSE = getTableSetElement(tabSetName);
+    if ( pTSE )
+    {
+	cacheSize =  pTSE->getAttributeValue(XML_MAXTABLECACHESIZE_ATTR).asInteger();  
+    }
+    else
+    {
+	V();
+	Chain msg = Chain("Unknown tableset name ") + tabSetName; 
+	throw Exception(EXLOC, msg);
+    }
+    V();
+    return cacheSize;
+}
+
+void CegoXMLSpace::setMaxQueryCacheEntry(const Chain& tableSet, int numEntry)
+{
+    P();
+    Element *pTS = getTableSetElement(tableSet);
+
+    if ( pTS )
+    {
+	pTS->setAttribute(XML_MAXQUERYCACHEENTRY_ATTR, Chain(numEntry));
+    }
+    else
+    {
+	V();
+	Chain msg = "Unknown tableset <" + tableSet + ">"; 
+	throw Exception(EXLOC, msg);
+    }
+    V();
+}
+
+int CegoXMLSpace::getMaxQueryCacheEntry(const Chain& tabSetName)
+{
+    int numEntry = 0;
+
+    P();
+    Element *pTSE = getTableSetElement(tabSetName);
+    if ( pTSE )
+    {
+	numEntry =  pTSE->getAttributeValue(XML_MAXQUERYCACHEENTRY_ATTR).asInteger();  
+    }
+    else
+    {
+	V();
+	Chain msg = Chain("Unknown tableset name ") + tabSetName; 
+	throw Exception(EXLOC, msg);
+    }
+    V();
+    return numEntry;
+}
+
+void CegoXMLSpace::setMaxQueryCacheSize(const Chain& tableSet, int cacheSize)
+{
+    P();
+    Element *pTS = getTableSetElement(tableSet);
+
+    if ( pTS )
+    {
+	pTS->setAttribute(XML_MAXQUERYCACHESIZE_ATTR, Chain(cacheSize));
+    }
+    else
+    {
+	V();
+	Chain msg = "Unknown tableset <" + tableSet + ">"; 
+	throw Exception(EXLOC, msg);
+    }
+    V();
+}
+
+int CegoXMLSpace::getMaxQueryCacheSize(const Chain& tabSetName)
+{
+    int cacheSize = 0;
+
+    P();
+    Element *pTSE = getTableSetElement(tabSetName);
+    if ( pTSE )
+    {
+	cacheSize =  pTSE->getAttributeValue(XML_MAXQUERYCACHESIZE_ATTR).asInteger();  
+    }
+    else
+    {
+	V();
+	Chain msg = Chain("Unknown tableset name ") + tabSetName; 
+	throw Exception(EXLOC, msg);
+    }
+    V();
+    return cacheSize;
+}
+
+void CegoXMLSpace::setQueryCacheMode(const Chain& tabSetName, bool isEnabled)
+{
+
+    P();
+    Element *pTSE = getTableSetElement(tabSetName);
+    if ( pTSE )
+    {	
+	if ( isEnabled )
+	    pTSE->setAttribute(XML_QUERYCACHE_ATTR, XML_ON_VALUE);  
+	else
+	    pTSE->setAttribute(XML_QUERYCACHE_ATTR, XML_OFF_VALUE);  
+    }
+    else
+    {
+	V();
+	Chain msg = Chain("Unknown tableset name ") + tabSetName; 
+	throw Exception(EXLOC, msg);
+    }
+    V();
+}
+
+bool CegoXMLSpace::getQueryCacheMode(const Chain& tabSetName)
+{
+    bool isEnabled=false;
+
+    P();
+    Element *pTSE = getTableSetElement(tabSetName);
+    if ( pTSE )
+    {	
+	if ( pTSE->getAttributeValue(XML_QUERYCACHE_ATTR) == Chain(XML_ON_VALUE))
+	    isEnabled = true;
+    }
+    else
+    {
+	V();
+	Chain msg = Chain("Unknown tableset name ") + tabSetName; 
+	throw Exception(EXLOC, msg);
+    }
+    V();
+    return isEnabled;
+}
+
 void CegoXMLSpace::addTableSetDef(const Chain& tabSetName, 
 				  const Chain& tsRoot, 
-				  const Chain& tsTicket, 
 				  const Chain& primary,
 				  const Chain& secondary, 
 				  const Chain& mediator,
-				  int sysFileId, int tempFileId, int sysFileSize, int tmpFileSize,
-				  int logFileSize, int logFileNum, long sortAreaSize)
+				  int sysFileId, int tempFileId, int sysFileSize, int tmpFileSize, int appFileSize,
+				  int logFileSize, int logFileNum, unsigned long long sortAreaSize)
 {
 
+    // first check some constraints here
+
+    if ( logFileNum > LOGMNG_MAXNUMLOGFILE )
+    {
+	Chain msg = Chain("Too many logfiles, up to ") + Chain(LOGMNG_MAXNUMLOGFILE) + Chain(" logfiles allowed"); 
+	throw Exception(EXLOC, msg);
+    }
+
+    Chain tsPath;
+    if ( tsRoot.length() < 2 )
+    {
+	Directory d;
+	d.setCurrent();
+	tsPath = d.getName();
+    }
+    else
+    {
+	tsPath = tsRoot;
+    }
+    
+
     P();
     Element *pRoot = _pDoc->getRootElement();
     if ( pRoot ) 
@@ -994,14 +1421,16 @@ void CegoXMLSpace::addTableSetDef(const Chain& tabSetName,
 
     Element *pTSE = new Element(XML_TABLESET_ELEMENT);
     pTSE->setAttribute(XML_NAME_ATTR, tabSetName);
-    pTSE->setAttribute(XML_TSROOT_ATTR, tsRoot);
+    
+    pTSE->setAttribute(XML_TSROOT_ATTR, tsPath);
+
     pTSE->setAttribute(XML_PRIMARY_ATTR, primary);
     pTSE->setAttribute(XML_SECONDARY_ATTR, secondary);
     pTSE->setAttribute(XML_MEDIATOR_ATTR, mediator);
     pTSE->setAttribute(XML_RUNSTATE_ATTR, XML_DEFINED_VALUE);
     pTSE->setAttribute(XML_SYNCSTATE_ATTR, XML_SYNCHED_VALUE);
 
-    pTSE->setAttribute(XML_TSTICKET_ATTR, tsTicket);
+    pTSE->setAttribute(XML_TSTICKET_ATTR, tsPath + Chain("/") + tabSetName + Chain("_ticket") + Chain(TABMNG_XMLSUFFIX));
     
     pTSE->setAttribute(XML_TSID_ATTR, Chain( sysFileId ) );
     pTSE->setAttribute(XML_TMPFID_ATTR, Chain( tempFileId ) );
@@ -1009,13 +1438,19 @@ void CegoXMLSpace::addTableSetDef(const Chain& tabSetName,
     pTSE->setAttribute(XML_SYSSIZE_ATTR, Chain( sysFileSize ) );
     pTSE->setAttribute(XML_TMPSIZE_ATTR, Chain( tmpFileSize ) );
 
+    pTSE->setAttribute(XML_SYSNAME_ATTR, tsPath + Chain("/") + tabSetName + Chain(TABMNG_SYSSUFFIX));
+    pTSE->setAttribute(XML_TMPNAME_ATTR, tsPath + Chain("/") + tabSetName + Chain(TABMNG_TEMPSUFFIX));
+    
     pTSE->setAttribute(XML_SORTAREASIZE_ATTR, Chain( sortAreaSize ) );
 
+    /* default tableset values */ 
+    pTSE->setAttribute(XML_AUTOCORRECT_ATTR, XML_ON_VALUE);
+
     for ( int i = 0 ; i < logFileNum ; i++ )
     {
 	Element *pLE = new Element(XML_LOGFILE_ELEMENT);
 
-	Chain logName = tsRoot + Chain("/") + tabSetName + Chain("redo") +  + Chain(i) + Chain(".log");
+	Chain logName = tsPath + Chain("/") + tabSetName + Chain("_redo") +  Chain(i) + Chain(TABMNG_LOGSUFFIX);
 	pLE->setAttribute(XML_NAME_ATTR, logName);
 	pLE->setAttribute(XML_SIZE_ATTR, Chain(logFileSize));
 	pLE->setAttribute(XML_STATUS_ATTR, XML_FREE_VALUE);
@@ -1024,8 +1459,15 @@ void CegoXMLSpace::addTableSetDef(const Chain& tabSetName,
     }
 
     _pDoc->getRootElement()->addContent(pTSE);
-    
+
     V();
+
+    if ( appFileSize > 0 )
+    {
+	int appFid = nextFID();
+	Chain appFileName = tsPath + Chain("/") + tabSetName + Chain("_data") + Chain(TABMNG_APPSUFFIX);
+	addDataFile(tabSetName, Chain(XML_APPFILE_VALUE), appFid, appFileName, appFileSize);
+    }
 }
 
 void CegoXMLSpace::rmTableSetDef(const Chain& tabSetName)
@@ -1109,7 +1551,6 @@ bool CegoXMLSpace::isArchiveMode(const Chain& tableSet)
     throw Exception(EXLOC, msg);
 }    
 
-
 void CegoXMLSpace::enableArchLog(const Chain& tableSet)
 {
     P();
@@ -1431,6 +1872,31 @@ void CegoXMLSpace::assignUserRole(const Chain& user, const Chain& role)
     P();
     Element *pRoot = _pDoc->getRootElement();
 
+    if ( role != Chain(ROLE_ADMIN) && role != Chain(ROLE_JDBC) )
+    {
+	bool roleFound = false;
+		    
+	ListT<Element*> roleList = pRoot->getChildren(XML_ROLE_ELEMENT);
+	Element **pRE = roleList.First();
+	while ( pRE && roleFound == false)
+	{
+	    if ( (*pRE)->getAttributeValue(XML_NAME_ATTR) == role )
+	    {	    
+		roleFound=true;	    
+	    }
+	    else
+	    {
+		pRE = roleList.Next();
+	    }
+	}
+	if ( roleFound == false )
+	{
+	    V();
+	    Chain msg = Chain("Unknown role ") + role; 
+	    throw Exception(EXLOC, msg);
+	}
+    }
+
     ListT<Element*> userList = pRoot->getChildren(XML_USER_ELEMENT);
     Element **pUE = userList.First();
     while ( pUE )
@@ -1472,7 +1938,6 @@ void CegoXMLSpace::assignUserRole(const Chain& user, const Chain& role)
     throw Exception(EXLOC, msg);
 }
 
-
 void CegoXMLSpace::removeUserRole(const Chain& user, const Chain& role)
 {
     P();
@@ -1545,6 +2010,19 @@ Chain CegoXMLSpace::getUserRole(const Chain& user)
 
 void CegoXMLSpace::createRole(const Chain& role)
 {
+
+    if ( role == Chain(ROLE_ADMIN) )
+    {
+	Chain msg = Chain("Role admin cannot be created"); 
+	throw Exception(EXLOC, msg);
+    }
+
+    if ( role == Chain(ROLE_JDBC) )
+    {
+	Chain msg = Chain("Role jdbc cannot be created"); 
+	throw Exception(EXLOC, msg);
+    }
+
     P();
     
     Element *pRoot = _pDoc->getRootElement();
@@ -1578,6 +2056,42 @@ void CegoXMLSpace::dropRole(const Chain& role)
     Element *pRoot = _pDoc->getRootElement();
     if ( pRoot ) 
     {
+
+	// drop role from all users
+
+	ListT<Element*> userList = pRoot->getChildren(XML_USER_ELEMENT);
+
+	Element **pUser = userList.First();
+	while ( pUser )
+	{
+
+	    Chain roleString = (*pUser)->getAttributeValue(XML_ROLE_ATTR);	    
+	    Tokenizer t(roleString, ",");
+
+	    Chain assignedRole;
+	    SetT<Chain> roleSet; 
+	    while ( t.nextToken(assignedRole) )
+	    {
+		roleSet.Insert(assignedRole);
+	    }
+
+	    roleSet.Remove(role);
+
+	    Chain newRoleString;
+	    Chain *pRole = roleSet.First();
+	    while ( pRole )
+	    {
+		newRoleString += *pRole;
+		pRole = roleSet.Next();
+		if ( pRole ) 
+		    newRoleString += ",";
+	    }
+	    
+	    (*pUser)->setAttribute(XML_ROLE_ATTR, newRoleString);
+
+	    pUser = userList.Next();
+	}
+
 	ListT<Element*> roleList = pRoot->getChildren(XML_ROLE_ELEMENT);
 	Element **pRE = roleList.First();
 	while ( pRE )
@@ -1637,10 +2151,21 @@ Element* CegoXMLSpace::getPermInfo(const Chain& role)
 
 }
 
-
 void CegoXMLSpace::setPerm(const Chain& role, const Chain& permid, const Chain& tableset, const Chain& filter, const Chain& perm)
 {
 
+    if ( role == Chain(ROLE_ADMIN) )
+    {
+	Chain msg = Chain("Permission canot be added to admin role"); 
+	throw Exception(EXLOC, msg);
+    }
+
+    if ( role == Chain(ROLE_JDBC) )
+    {
+	Chain msg = Chain("Permission canot be added to jdbc role"); 
+	throw Exception(EXLOC, msg);
+    }
+
     Chain uperm = perm.toUpper();
 
     if ( uperm != Chain("READ") 
@@ -1884,7 +2409,7 @@ bool CegoXMLSpace::checkUser(const Chain& user, const Chain& password, Chain& ms
 		
 		if ( (*pUE)->getAttributeValue(XML_TRACE_ATTR) == Chain(XML_ON_VALUE) )
 		{
-		    int numReq = (*pUE)->getAttributeValue(XML_NUMREQUEST_ATTR).asInteger();
+		    unsigned long long numReq = (*pUE)->getAttributeValue(XML_NUMREQUEST_ATTR).asUnsignedLongLong();
 		    numReq++;
 		    (*pUE)->setAttribute(XML_NUMREQUEST_ATTR, Chain(numReq));
 		    isTrace=true;
@@ -1943,7 +2468,7 @@ void CegoXMLSpace::incUserQuery(const Chain& user)
 	{
 	    if ( (*pUE)->getAttributeValue(XML_NAME_ATTR) == user )
 	    {
-		int numQuery = (*pUE)->getAttributeValue(XML_NUMQUERY_ATTR).asInteger();
+		unsigned long long numQuery = (*pUE)->getAttributeValue(XML_NUMQUERY_ATTR).asUnsignedLongLong();
 		numQuery++;
 		(*pUE)->setAttribute(XML_NUMQUERY_ATTR, Chain(numQuery));
 		V();
@@ -1991,7 +2516,6 @@ Element* CegoXMLSpace::getUserList()
     return 0;
 }    
 
-
 Element* CegoXMLSpace::getRoleList()
 {
     P();
@@ -2020,7 +2544,6 @@ Element* CegoXMLSpace::getRoleList()
     return 0;
 }    
 
-
 void CegoXMLSpace::getDataPort(int& dataPort)
 {
     P();
@@ -2081,7 +2604,6 @@ void CegoXMLSpace::setTSRoot(const Chain& tableSet, const Chain& tsRoot)
     V();
 }
 
-
 Chain CegoXMLSpace::getTSRoot(const Chain& tableSet)
 {
     Chain tsRoot;
@@ -2123,7 +2645,8 @@ void CegoXMLSpace::initDoc()
 {
     P();
     ListT<Element*> nodeList;
-    
+    Chain dbHost;
+
     nodeList = _pDoc->getRootElement()->getChildren(XML_NODE_ELEMENT);
     Element **pNode = nodeList.First();
     while ( pNode ) 
@@ -2132,11 +2655,12 @@ void CegoXMLSpace::initDoc()
 	nodeList = _pDoc->getRootElement()->getChildren(XML_NODE_ELEMENT);
 	pNode = nodeList.First();
     }
-    
+
+    dbHost = _pDoc->getRootElement()->getAttributeValue(XML_HOSTNAME_ATTR);    
+
     V();
-    
-    Host h;
-    addHost(h.getName(), XML_ONLINE_VALUE);
+        
+    addHost(dbHost, XML_ONLINE_VALUE);
    
     return;
 }
@@ -2146,8 +2670,7 @@ void CegoXMLSpace::setXmlDef(const Chain& xmlDef)
     _xmlDef = xmlDef;
 }
 
-
-void CegoXMLSpace::initXml(const Chain& dbName, int pageSize, const Chain& hostName, int dbPort, int admPort, int logPort, const Chain& pidFile, const Chain& logLevel)
+void CegoXMLSpace::initXml(const Chain& dbName, int pageSize, const Chain& hostName, int dbPort, int admPort, int logPort, const Chain& pidFile, const Chain& logLevel, const Chain& csmode, bool qescmode)
 {
 
     P();
@@ -2168,6 +2691,14 @@ void CegoXMLSpace::initXml(const Chain& dbName, int pageSize, const Chain& hostN
 	pElement->setAttribute(XML_ADMINPORT_ATTR, Chain(admPort));
 	pElement->setAttribute(XML_LOGPORT_ATTR, Chain(logPort));
 	pElement->setAttribute(XML_PIDFILE_ATTR, pidFile);
+	pElement->setAttribute(XML_CSMODE_ATTR, csmode);
+
+	pElement->setAttribute(XML_MAXTSID_ATTR, Chain(0));
+	
+	if ( qescmode )
+	    pElement->setAttribute(XML_QESCMODE_ATTR, XML_ON_VALUE);
+	else 
+	    pElement->setAttribute(XML_QESCMODE_ATTR, XML_OFF_VALUE);
 
 	Element *pLog = new Element(XML_MODULE_ELEMENT);
 	pLog->setAttribute(XML_NAME_ATTR, Chain("ALL"));
@@ -2196,8 +2727,6 @@ void CegoXMLSpace::initXml(const Chain& dbName, int pageSize, const Chain& hostN
     V();
 }
 
-
-
 void CegoXMLSpace::xml2Doc()
 {
 
@@ -2278,7 +2807,6 @@ void CegoXMLSpace::doc2Xml()
     V();
 }
 
-
 void CegoXMLSpace::setTSTicket(const Chain& tableSet, const Chain& tsTicket)
 {
     P();
@@ -2306,11 +2834,11 @@ Chain CegoXMLSpace::getTSTicket(const Chain& tableSet)
     return tsTicket;
 }
 
-int CegoXMLSpace::getTID(const Chain& tableSet)
+unsigned long long CegoXMLSpace::getTID(const Chain& tableSet)
 {
     P();
     Element *pE = getTableSetElement(tableSet);
-    int tid = pE->getAttributeValue(XML_TID_ATTR).asInteger();
+    unsigned long long tid = pE->getAttributeValue(XML_TID_ATTR).asUnsignedLongLong();
     V();
     return tid;
 }
@@ -2325,7 +2853,7 @@ bool CegoXMLSpace::tableSetExists(const Chain& tableSet)
     return false;
 }
 
-long CegoXMLSpace::getSortAreaSize(int tabSetId)
+unsigned long long CegoXMLSpace::getSortAreaSize(int tabSetId)
 {
     Element *pE = getCachedTableSetElement(tabSetId);
 
@@ -2336,12 +2864,12 @@ long CegoXMLSpace::getSortAreaSize(int tabSetId)
     }
 
     P();
-    long sortAreaSize = pE->getAttributeValue(XML_SORTAREASIZE_ATTR).asLong();
+    unsigned long long sortAreaSize = pE->getAttributeValue(XML_SORTAREASIZE_ATTR).asUnsignedLongLong();
     V();
     return sortAreaSize;
 }
 
-int CegoXMLSpace::nextTID(int tabSetId)
+unsigned long long CegoXMLSpace::nextTID(int tabSetId)
 {
     Element *pE = getCachedTableSetElement(tabSetId);
 
@@ -2352,7 +2880,7 @@ int CegoXMLSpace::nextTID(int tabSetId)
     }
    
     P();
-    int tid = pE->getAttributeValue(XML_TID_ATTR).asInteger();
+    unsigned long long tid = pE->getAttributeValue(XML_TID_ATTR).asUnsignedLongLong();
     tid++;
     pE->setAttribute(XML_TID_ATTR, Chain(tid));
     V();
@@ -2362,27 +2890,58 @@ int CegoXMLSpace::nextTID(int tabSetId)
 int CegoXMLSpace::nextFID()
 {
     P();
-    Element *pE = _pDoc->getRootElement();;
 
-    int fid = pE->getAttributeValue(XML_MAXFID_ATTR).asInteger();
-    if ( fid <= TABMNG_MAXTABSET )
-	fid = TABMNG_MAXTABSET + 1;
-    else
-	fid++;
-    pE->setAttribute(XML_MAXFID_ATTR, Chain(fid));
+    Element *pRoot = _pDoc->getRootElement();
+
+    SetT<int> fidSet;
+    
+    ListT<Element*> tabSetList = pRoot->getChildren(XML_TABLESET_ELEMENT);
+    Element **pTSE = tabSetList.First();
+    while ( pTSE ) 
+    {
+
+	int tmpFid = (*pTSE)->getAttributeValue(XML_TMPFID_ATTR).asInteger();  
+	fidSet.Insert(tmpFid);
+	
+	ListT<Element*> fileList = (*pTSE)->getChildren(XML_DATAFILE_ELEMENT);
+	Element **pFile = fileList.First();
+	while ( pFile )
+	{
+	    int appFid = (*pFile)->getAttributeValue(XML_FILEID_ATTR).asInteger();
+	    fidSet.Insert(appFid);
+	    
+	    pFile = fileList.Next();
+	}
+
+	pTSE = tabSetList.Next();
+    }
+
     V();
-    return fid;
+    
+    int nextFid = TABMNG_MAXTABSET + 1; // values <= MAXTABSET are reserved for system files
+
+    while ( nextFid < FILMNG_MAXDATAFILE )
+    {
+	if ( fidSet.Find(nextFid) )
+	    nextFid++;
+	else
+	    return nextFid;
+    }
+
+    throw Exception(EXLOC, Chain("File Id exceeded"));
+    
 }
 
 int CegoXMLSpace::nextTSID()
 {
     P();
-    Element *pE = _pDoc->getRootElement();;
+    Element *pE = _pDoc->getRootElement();
 
     int tsid = pE->getAttributeValue(XML_MAXTSID_ATTR).asInteger();
     tsid++;
     if ( tsid > TABMNG_MAXTABSET )
     {
+	
 	V();
 	throw Exception ( EXLOC, "TableSet Ids exceeded");
     }
@@ -2392,7 +2951,7 @@ int CegoXMLSpace::nextTSID()
     return tsid;
 }
 
-void CegoXMLSpace::setCommittedLSN(int tabSetId, long lsn)
+void CegoXMLSpace::setCommittedLSN(int tabSetId, unsigned long long lsn)
 {   
     Element *pTS = getCachedTableSetElement(tabSetId);
 
@@ -2406,7 +2965,7 @@ void CegoXMLSpace::setCommittedLSN(int tabSetId, long lsn)
     V();
 }
 
-void CegoXMLSpace::setCommittedLSN(const Chain& tableSet, long lsn)
+void CegoXMLSpace::setCommittedLSN(const Chain& tableSet, unsigned long long lsn)
 {  
     P();
     Element *pTS = getTableSetElement(tableSet);
@@ -2422,15 +2981,15 @@ void CegoXMLSpace::setCommittedLSN(const Chain& tableSet, long lsn)
     V();
 }
 
-long CegoXMLSpace::getCommittedLSN(const Chain& tableSet)
+unsigned long long CegoXMLSpace::getCommittedLSN(const Chain& tableSet)
 {
-    long lsn;
+    unsigned long long lsn;
 
     P();
     Element *pTS = getTableSetElement(tableSet);
     if ( pTS )
     {
-	lsn = pTS->getAttributeValue(XML_LSN_ATTR).asLong() ;
+	lsn = pTS->getAttributeValue(XML_LSN_ATTR).asUnsignedLongLong() ;
     }
     else
     {
@@ -2442,7 +3001,6 @@ long CegoXMLSpace::getCommittedLSN(const Chain& tableSet)
     return lsn;
 }
 
-
 void CegoXMLSpace::getAllActiveTableSet(ListT<Chain>& tsList)
 {
     P();
@@ -2464,7 +3022,6 @@ void CegoXMLSpace::getAllActiveTableSet(ListT<Chain>& tsList)
     V();
 }
 
-
 void CegoXMLSpace::getActiveTableSet(const Chain& primary, ListT<Chain>& tsList, bool includeRecovery)
 {
     P();
@@ -2474,7 +3031,7 @@ void CegoXMLSpace::getActiveTableSet(const Chain& primary, ListT<Chain>& tsList,
 	ListT<Element*> tabSetList = pRoot->getChildren(XML_TABLESET_ELEMENT);
 	Element **pTSE = tabSetList.First();
 	while ( pTSE ) 
-	{
+	{	    
 	    if ( (*pTSE)->getAttributeValue(XML_PRIMARY_ATTR) == primary
 		 && ( (*pTSE)->getAttributeValue(XML_RUNSTATE_ATTR) == Chain(XML_ONLINE_VALUE)  
 		      || (*pTSE)->getAttributeValue(XML_RUNSTATE_ATTR) == Chain(XML_BACKUP_VALUE)
@@ -2488,6 +3045,35 @@ void CegoXMLSpace::getActiveTableSet(const Chain& primary, ListT<Chain>& tsList,
     V();
 }
 
+Chain CegoXMLSpace::getTableSetRunState(int tabSetId)
+{
+   
+    Element *pTS = getCachedTableSetElement(tabSetId);
+
+    if ( pTS == 0)
+    {
+	Chain msg = "Unknown tableset id <" + Chain(tabSetId) + ">"; 
+	throw Exception(EXLOC, msg);
+    }
+    P();
+    Chain runState = pTS->getAttributeValue(XML_RUNSTATE_ATTR);
+    V();
+    return runState;
+}
+
+void CegoXMLSpace::setTableSetRunState(int tabSetId, const Chain& status)
+{
+    Element *pTS = getCachedTableSetElement(tabSetId);
+
+    if ( pTS == 0)
+    {
+	Chain msg = "Unknown tableset id <" + Chain(tabSetId) + ">"; 
+	throw Exception(EXLOC, msg);
+    }    
+    P();
+    pTS->setAttribute(XML_RUNSTATE_ATTR, status);    
+    V();
+}
 
 Chain CegoXMLSpace::getTableSetRunState(const Chain& tableSet)
 {
@@ -2509,6 +3095,7 @@ Chain CegoXMLSpace::getTableSetRunState(const Chain& tableSet)
     return status;
 }
 
+
 void CegoXMLSpace::setTableSetRunState(const Chain& tableSet, const Chain& status)
 {
     P();
@@ -2603,7 +3190,6 @@ void CegoXMLSpace::getDataFileInfo(const Chain& tableSet, const Chain& type, Lis
     V();
 }
 
-
 void CegoXMLSpace::getLogFileInfo(const Chain& tableSet, ListT<Chain>& lfList, ListT<int>& sizeList, ListT<Chain>& statusList)
 {
     P();
@@ -2652,7 +3238,7 @@ void CegoXMLSpace::setLogFileStatus(const Chain& tableSet, const Chain& logFile,
     V();
 }
 
-void CegoXMLSpace::getXMLLockStat(Chain& lockName, long& lockCount, long &numRdLock, long &numWrLock, long &sumRdDelay, long &sumWrDelay)
+void CegoXMLSpace::getXMLLockStat(Chain& lockName, int& lockCount, unsigned long long &numRdLock, unsigned long long &numWrLock, unsigned long long &sumRdDelay, unsigned long long &sumWrDelay)
 {
     lockName = xmlLock.getId();
     lockCount = xmlLock.numLockTry();
@@ -2669,7 +3255,7 @@ void CegoXMLSpace::getXMLLockStat(Chain& lockName, long& lockCount, long &numRdL
 
 }
 
-void CegoXMLSpace::addCounter(int tabSetId, const Chain& counterName, long initValue, bool forceSet)
+void CegoXMLSpace::addCounter(int tabSetId, const Chain& counterName, unsigned long long initValue, bool forceSet)
 {
    
     Element *pTS = getCachedTableSetElement(tabSetId);
@@ -2710,7 +3296,7 @@ void CegoXMLSpace::addCounter(int tabSetId, const Chain& counterName, long initV
     V();
 }
 
-long CegoXMLSpace::getCounterValue(int tabSetId, const Chain& counterName, long incValue)
+unsigned long long CegoXMLSpace::getCounterValue(int tabSetId, const Chain& counterName, unsigned long long incValue)
 {   
     Element *pTS = getCachedTableSetElement(tabSetId);
 
@@ -2729,7 +3315,7 @@ long CegoXMLSpace::getCounterValue(int tabSetId, const Chain& counterName, long
 	    
 	    Chain counterVal = (*pCE)->getAttributeValue(XML_VALUE_ATTR);
 	    
-	    long v = counterVal.asLong();
+	    unsigned long long v = counterVal.asUnsignedLongLong();
 	    
 	    if ( incValue > 0 )
 	    {
@@ -2746,7 +3332,7 @@ long CegoXMLSpace::getCounterValue(int tabSetId, const Chain& counterName, long
     throw Exception(EXLOC, msg);
 }
 
-void CegoXMLSpace::setCounterValue(int tabSetId, const Chain& counterName, long value)
+void CegoXMLSpace::setCounterValue(int tabSetId, const Chain& counterName, unsigned long long value)
 {
     Element *pTS = getCachedTableSetElement(tabSetId);
 
@@ -2825,7 +3411,6 @@ void CegoXMLSpace::removeCounter(int tabSetId, const Chain& counterName)
     throw Exception(EXLOC, msg);
 }
 
-
 void CegoXMLSpace::setCheckpointInterval(const Chain& tableSet, int timeout)
 {
     P();
@@ -2864,7 +3449,6 @@ int CegoXMLSpace::getCheckpointInterval(const Chain& tableSet)
     return checkpoint;
 }
 
-
 void CegoXMLSpace::setTSInitFile(const Chain& tableSet, const Chain& initFile)
 {
     P();
@@ -2903,15 +3487,14 @@ Chain CegoXMLSpace::getTSInitFile(const Chain& tableSet)
     return initFile;
 }
 
-
 // private methods
 
-void CegoXMLSpace::P()
+void CegoXMLSpace::P() const
 {    
     xmlLock.writeLock(XS_LOCKTIMEOUT);
 }
 
-void CegoXMLSpace::V()
+void CegoXMLSpace::V() const
 {
     xmlLock.unlock();
 }
@@ -2944,33 +3527,6 @@ void CegoXMLSpace::getRoleSet(const Chain& user, SetT<Chain>& roleSet)
     V();
 }
 
-/*
-Chain CegoXMLSpace::getRoleAbbrev(const Chain& roleString)
-{
-    Chain abbrev;
-    Tokenizer t(roleString, ",");
-    Chain assignedRole;
-    while ( t.nextToken(assignedRole) )
-    {
-	if ( assignedRole == Chain(XML_ROLE_ALL_VALUE) )
-	    abbrev += "A";
-	else if ( assignedRole == Chain(XML_ROLE_READOBJ_VALUE) )
-	    abbrev += "R";
-	else if ( assignedRole == Chain(XML_ROLE_WRITEOBJ_VALUE) )
-	    abbrev += "W";
-	else if ( assignedRole == Chain(XML_ROLE_CREATEOBJ_VALUE) )
-	    abbrev += "C";
-	else if ( assignedRole == Chain(XML_ROLE_DROPOBJ_VALUE) )
-	    abbrev += "D";
-	else if ( assignedRole == Chain(XML_ROLE_EXECPROC_VALUE) )
-	    abbrev += "E";
-    }
-
-    return abbrev;
-
-}
-*/
-
 bool CegoXMLSpace::matchRole(const Chain& role, const Chain tableSet, const Chain& objPattern, AccessMode perm)
 {
     
@@ -3066,7 +3622,7 @@ bool CegoXMLSpace::fitsPerm(const Chain& perm, AccessMode req)
     return false;
 }
 
-Element* CegoXMLSpace::getTableSetElement(const Chain& tableSet)
+Element* CegoXMLSpace::getTableSetElement(const Chain& tableSet) const
 {
    
     Element *pRoot = _pDoc->getRootElement();
@@ -3078,7 +3634,7 @@ Element* CegoXMLSpace::getTableSetElement(const Chain& tableSet)
 	{
 	    
 	    bool isEqual;
-	    if ( __caseSensitiveFlag == 1 )
+	    if ( __caseSensitiveFlag == 0 )
 		isEqual = (*pTSE)->getAttributeValue(XML_NAME_ATTR) == tableSet;
 	    else
 		isEqual = (*pTSE)->getAttributeValue(XML_NAME_ATTR).toUpper() == tableSet.toUpper();
@@ -3186,5 +3742,46 @@ Logger::LogLevel CegoXMLSpace::getLogLevel(const Chain& module)
     return Logger::NONE;
 }
 
+Chain CegoXMLSpace::getSysFileName(const Chain& tableSet)
+{
 
+    Chain sysFileName;
 
+    P();
+
+    Element *pTSE = getTableSetElement(tableSet);
+    if ( pTSE )
+    {
+	sysFileName = pTSE->getAttributeValue(XML_SYSNAME_ATTR);  
+    }
+    else
+    {
+	V();
+	Chain msg = Chain("Unknown tableset name ") + tableSet; 
+	throw Exception(EXLOC, msg);
+    }
+    V();
+    return sysFileName;
+    
+}
+
+Chain CegoXMLSpace::getTmpFileName(const Chain& tableSet)
+{
+    Chain tmpFileName;
+
+    P();
+
+    Element *pTSE = getTableSetElement(tableSet);
+    if ( pTSE )
+    {
+	tmpFileName = pTSE->getAttributeValue(XML_TMPNAME_ATTR);  
+    }
+    else
+    {
+	V();
+	Chain msg = Chain("Unknown tableset name ") + tableSet; 
+	throw Exception(EXLOC, msg);
+    }
+    V();
+    return tmpFileName;
+}
