@@ -8,30 +8,15 @@
 //                
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2006 Bjoern Lemke
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2, or (at your option)
-// any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// (C)opyright 2000-2016 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
-// Class: 
+// Class: NA
 // 
-// Description:
+// Description: XML tag and attribute definitions
 //
-// Status: IMPLEMENTED
+// Status: CLEAN
 //
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -60,6 +45,17 @@
 #define XML_LOGTHREADINFO_REQUEST "LOGTHREADINFO"
 #define XML_DBSESSIONINFO_REQUEST "DBSESSIONINFO"
 #define XML_COPYINFO_REQUEST "COPYINFO"
+
+#define XML_LISTQUERYCACHE_REQUEST "LISTQUERYCACHE"
+#define XML_CLEANQUERYCACHE_REQUEST "CLEANQUERYCACHE"
+#define XML_SETQUERYCACHEENTRY_REQUEST "SETQUERYCACHEENTRY"
+#define XML_SETQUERYCACHESIZE_REQUEST "SETQUERYCACHESIZE"
+
+#define XML_LISTTABLECACHE_REQUEST "LISTTABLECACHE"
+#define XML_CLEANTABLECACHE_REQUEST "CLEANTABLECACHE"
+#define XML_SETTABLECACHEENTRY_REQUEST "SETTABLECACHEENTRY"
+#define XML_SETTABLECACHESIZE_REQUEST "SETTABLECACHESIZE"
+
 #define XML_DEFINE_TABLESET_REQUEST "DEFINE_TABLESET"
 #define XML_REMOVE_TABLESET_REQUEST "REMOVE_TABLESET"
 #define XML_RESET_TABLESET_REQUEST "RESET_TABLESET"
@@ -125,12 +121,17 @@
 #define XML_SET_TSROOTPATH_REQUEST "SET_TSROOTPATH"
 #define XML_SET_TSSYSSIZE_REQUEST "SET_TSSYSSIZE"
 #define XML_SET_TSTMPSIZE_REQUEST "SET_TSTMPSIZE"
+#define XML_SET_TSAPPSIZE_REQUEST "SET_TSAPPSIZE"
 
-#define XML_SET_TSTICKET_REQUEST "SET_TSTICKET"
 #define XML_SET_TSLOGNUM_REQUEST "SET_TSLOGNUM"
 #define XML_SET_TSLOGSIZE_REQUEST "SET_TSLOGSIZE"
 #define XML_SET_TSSORTAREASIZE_REQUEST "SET_TSSORTAREASIZE"
 
+#define XML_ENABLE_TABLECACHE_REQUEST "MED_ENABLE_TABLECACHE"
+#define XML_DISABLE_TABLECACHE_REQUEST "MED_DISABLE_TABLECACHE"
+#define XML_ENABLE_QUERYCACHE_REQUEST "MED_ENABLE_QUERYCACHE"
+#define XML_DISABLE_QUERYCACHE_REQUEST "MED_DISABLE_QUERYCACHE"
+
 // mediator requests
 #define XML_MED_NOTIFY_REQUEST "MED_NOTIFY"
 #define XML_MED_DEFINE_TABLESET_REQUEST "MED_DEFINE_TABLESET"
@@ -161,6 +162,8 @@
 #define XML_MED_VERIFY_TABLESET_REQUEST "MED_VERIFY_TABLESET"
 #define XML_MED_ENABLE_ARCHLOG_REQUEST "MED_ENABLE_ARCHLOG"
 #define XML_MED_DISABLE_ARCHLOG_REQUEST "MED_DISABLE_ARCHLOG"
+#define XML_MED_ENABLE_AUTOCORRECT_REQUEST "MED_ENABLE_AUTOCORRECT"
+#define XML_MED_DISABLE_AUTOCORRECT_REQUEST "MED_DISABLE_AUTOCORRECT"
 
 #define XML_SEC_MEDSWITCH_REQUEST "SEC_MEDSWITCH"
 #define XML_SEC_MEDRELOCATE_REQUEST "SEC_MEDRELOCATE"
@@ -189,6 +192,10 @@
 #define XML_PUTBLOB_REQUEST "PUTBLOB"
 #define XML_GETBLOB_REQUEST "GETBLOB"
 #define XML_DELBLOB_REQUEST "DELBLOB"
+#define XML_PUTCLOB_REQUEST "PUTCLOB"
+#define XML_GETCLOB_REQUEST "GETCLOB"
+#define XML_DELCLOB_REQUEST "DELCLOB"
+
 #define XML_STARTTRANSACTION_REQUEST "STARTTRANSACTION"
 #define XML_COMMITTRANSACTION_REQUEST "COMMITTRANSACTION"
 #define XML_ROLLBACKTRANSACTION_REQUEST "ROLLBACKTRANSACTION"
@@ -199,6 +206,7 @@
 #define XML_CEGO_DB_CONFIG_DOC "CEGO_DB_CONFIG"
 #define XML_ERROR_DOC "ERROR"
 #define XML_OK_DOC "OK"
+#define XML_SACK_DOC "SACK"
 #define XML_DATA_DOC "DATA"
 #define XML_INFO_DOC "INFO"
 #define XML_EXPORT_DOC "CEGO_EXPORT"
@@ -214,6 +222,8 @@
 #define XML_BLOB_ELEMENT "BLOB"
 #define XML_BUSINFO_ELEMENT "BUSINFO"
 #define XML_BUENTRY_ELEMENT "BUENTRY"
+#define XML_CACHE_ELEMENT "CACHE"
+#define XML_CACHEINFO_ELEMENT "CACHEINFO"
 #define XML_CASE_ELEMENT "CASE"
 #define XML_CASECLAUSE_ELEMENT "CASECLAUSE"
 #define XML_CHECK_ELEMENT "CHECK"
@@ -228,6 +238,7 @@
 #define XML_DATA_ELEMENT "DATA"
 #define XML_DATABASE_ELEMENT "DATABASE"
 #define XML_DATAFILE_ELEMENT "DATAFILE"
+#define XML_DATEFORMAT_ELEMENT "DATEFORMAT"
 #define XML_DBINFO_ELEMENT "DBINFO"
 #define XML_DBSESSION_ELEMENT "DBSESSION"
 #define XML_DBSESSIONINFO_ELEMENT "DBSESSIONINFO"
@@ -298,11 +309,14 @@
 #define XML_ADMINPORT_ATTR "ADMINPORT"
 #define XML_AFFECTED_ATTR "AFFCOUNT"
 #define XML_AGG_ATTR "AGG"
+#define XML_APPSIZE_ATTR "APPSIZE"
 #define XML_ARCHMODE_ATTR "ARCHMODE"
 #define XML_ARCHID_ATTR "ARCHID"
 #define XML_ARCHPATH_ATTR "ARCHPATH"
 #define XML_ATTRNAME_ATTR "ATTRNAME"
+#define XML_AUTOCORRECT_ATTR "AUTOCORRECT"
 #define XML_BPHITRATE_ATTR "BPHITRATE"
+#define XML_SPREADRATE_ATTR "SPREADRATE"
 #define XML_BUINFO_ATTR "BUINFO"
 #define XML_BUMSG_ATTR "BUMSG"
 #define XML_CSMODE_ATTR "CSMODE"
@@ -323,9 +337,11 @@
 #define XML_DATAPORT_ATTR "DATAPORT"
 #define XML_DATALEN_ATTR "DATALEN"
 #define XML_DATAID_ATTR "DATAID"
+#define XML_DATETIMEFORMAT_ATTR "DATETIMEFORMAT"
 #define XML_DBPRODNAME_ATTR "DBPRODNAME"
 #define XML_DBPRODVERSION_ATTR "DBPRODVERSION"
 #define XML_DOLOGGING_ATTR "DOLOGGING"
+#define XML_DUPLICATENULL_ATTR "DUPLICATENULL"
 #define XML_EXPR_ATTR "EXPR"
 #define XML_ESCCMD_ATTR "ESCCMD"
 #define XML_FACTOR_ATTR "FACTOR"
@@ -342,6 +358,7 @@
 #define XML_GROUPNAME_ATTR "GROUPNAME"
 #define XML_HOSTNAME_ATTR "HOSTNAME"
 #define XML_HOSTROLE_ATTR "HOSTROLE"
+#define XML_ID_ATTR "ID"
 #define XML_INDEXNAME_ATTR "INDEXNAME"
 #define XML_INDEXTYPE_ATTR "INDEXTYPE"
 #define XML_ISDIRTY_ATTR "ISDIRTY"
@@ -371,14 +388,20 @@
 #define XML_NUMLOCKTRIES_ATTR "NUMLOCKTRIES"
 #define XML_SORTAREASIZE_ATTR "SORTAREASIZE"
 #define XML_MAXFIXCOUNT_ATTR "MAXFIXCOUNT"
+#define XML_MAXTABLECACHEENTRY_ATTR "MAXTABLECACHEENTRY"
+#define XML_MAXTABLECACHESIZE_ATTR "MAXTABLECACHESIZE"
+#define XML_TABLECACHEUSEDSIZE_ATTR "TABLECACHEUSEDSIZE"
+#define XML_MAXQUERYCACHEENTRY_ATTR "MAXQUERYCACHEENTRY"
+#define XML_MAXQUERYCACHESIZE_ATTR "MAXQUERYCACHESIZE"
+#define XML_QUERYCACHEUSEDSIZE_ATTR "QUERYCACHEUSEDSIZE"
 #define XML_MVAL_ATTR "MVAL"
-#define XML_MAXFID_ATTR "MAXFID"
 #define XML_MAXTSID_ATTR "MAXTSID"
 #define XML_MEDIATOR_ATTR "MEDIATOR"
 #define XML_MODE_ATTR "MODE"
 #define XML_MSG_ATTR "MSG"
 #define XML_NAME_ATTR "NAME"
 #define XML_NEWOBJNAME_ATTR "NEWOBJNAME"
+#define XML_NUMENTRY_ATTR "NUMQUERY"
 #define XML_NUMDBTHREAD_ATTR "NUMDBTHREAD"
 #define XML_NUMADMTHREAD_ATTR "NUMADMTHREAD"
 #define XML_NUMLOCK_ATTR "NUMLOCK"
@@ -409,6 +432,7 @@
 #define XML_WRITEDELAY_ATTR "WRITEDELAY"
 #define XML_STATSTART_ATTR "STATSTART"
 #define XML_NUMROWS_ATTR "NUMROWS"
+#define XML_NUMHITS_ATTR "NUMHITS"
 #define XML_NUMRECSEMA_ATTR "NUMRECSEMA"
 #define XML_NUMRBRECSEMA_ATTR "NUMRBRECSEMA"
 #define XML_NUMSYSRECSEMA_ATTR "NUMSYSRECSEMA"
@@ -418,7 +442,9 @@
 #define XML_NUMRBPAGESEMA_ATTR "NUMRBPAGESEMA"
 #define XML_NUMDATAFILESEMA_ATTR "NUMDATAFILESEMA"
 #define XML_NUMBUFFERPOOLSEMA_ATTR "NUMBUFFERPOOLSEMA"
+#define XML_NUMUSAGE_ATTR "NUMUSAGE"
 #define XML_MAXFIXTRIES_ATTR "MAXFIXTRIES"
+// #define XML_BUFFERPOOLHASHKEY_ATTR "BUFFERPOOLHASHKEY" -- no more used
 #define XML_OBJNAME_ATTR "OBJNAME"
 #define XML_OBJTYPE_ATTR "OBJTYPE"
 #define XML_OCCSTATE_ATTR "OCCSTATE"
@@ -435,8 +461,10 @@
 #define XML_PROCNAME_ATTR "PROCNAME"
 #define XML_PROCTEXT_ATTR "PROCTEXT"
 #define XML_QESCMODE_ATTR "QESCMODE"
+#define XML_QUERYCACHE_ATTR "QUERYCACHE"
 #define XML_REFTABLENAME_ATTR "REFTABLENAME"
 #define XML_ROLE_ATTR "ROLE"
+#define XML_SEGMENT_ATTR "SEGMENT"
 #define XML_SECONDARY_ATTR "SECONDARY"
 #define XML_STATUS_ATTR "STATUS"
 #define XML_RUNSTATE_ATTR "RUNSTATE"
@@ -445,6 +473,7 @@
 #define XML_SIZE_ATTR "SIZE"
 #define XML_SYSFILE_ATTR "SYSFILE"
 #define XML_SYSFID_ATTR "SYSFID"
+#define XML_SYSNAME_ATTR "SYSNAME"
 #define XML_SYSTOTAL_ATTR "SYSTOTAL"
 #define XML_SYSUSED_ATTR "SYSUSED"
 #define XML_SVAL_ATTR "SVAL"
@@ -453,6 +482,7 @@
 #define XML_TABLETYPE_ATTR "TABLETYPE"
 #define XML_TARGETHOST_ATTR "TARGETHOST"
 #define XML_TABLESET_ATTR "TABLESET"
+#define XML_TABLECACHE_ATTR "TABLECACHE"
 #define XML_TERM_ATTR "TERM"
 #define XML_TID_ATTR "TID"
 #define XML_TIMESTAMP_ATTR "TIMESTAMP"
@@ -466,19 +496,20 @@
 #define XML_THID_ATTR "THID"
 #define XML_TMPFID_ATTR "TMPFID"
 #define XML_TMPSIZE_ATTR "TMPSIZE"
+#define XML_TMPNAME_ATTR "TMPNAME"
 #define XML_TEMPFILE_ATTR "TEMPFILE"
 #define XML_TEMPTOTAL_ATTR "TEMPTOTAL"
 #define XML_TEMPUSED_ATTR "TEMPUSED"
 #define XML_TYPE_ATTR "TYPE"
 #define XML_USAGE_ATTR "USAGE"
 #define XML_PASSWD_ATTR "PASSWD"
+#define XML_UPTIME_ATTR "UPTIME"
 #define XML_USER_ATTR "USER"
 #define XML_VALUE_ATTR "VALUE"
 #define XML_VIEWSTMT_ATTR "VIEWSTMT"
 #define XML_VIEWNAME_ATTR "VIEWNAME"
 #define XML_VIEWTEXT_ATTR "VIEWTEXT"
 #define XML_PAGECOUNT_ATTR "PAGECOUNT"
-#define XML_DTFORMAT_ATTR "DTFORMAT"
 #define XML_SYSUSAGE_ATTR "SYSUSAGE"
 #define XML_TEMPUSAGE_ATTR "TEMPUSAGE"
 #define XML_APPUSAGE_ATTR "APPUSAGE"
@@ -498,6 +529,7 @@
 #define XML_INT_VALUE "int"
 #define XML_LONG_VALUE "long"
 #define XML_BLOB_VALUE "blob"
+#define XML_CLOB_VALUE "clob"
 #define XML_NULL_VALUE "null"
 #define XML_INDEX_VALUE "index"
 #define XML_PINDEX_VALUE "primary"
@@ -545,6 +577,7 @@
 #define XML_OFFLINE_VALUE "OFFLINE"
 #define XML_BACKUP_VALUE "BACKUP"
 #define XML_RECOVERY_VALUE "RECOVERY"
+#define XML_CHECKPOINT_VALUE "CHECKPOINT"
 
 // tableset sync states
 #define XML_NOT_SYNCHED_VALUE "NOT_SYNCHED"
@@ -616,7 +649,8 @@
 #define XML_SELECT_VALUE "SELECT"
 
 #define XML_ADD_VALUE "ADD"
-#define XML_MODIFY_VALUE "MODIFY"
+#define XML_MODIFY_COLUMN_VALUE "MODIFY_COLUMN"
+#define XML_MODIFY_DEFAULT_VALUE "MODIFY_DEFAULT"
 #define XML_DROP_VALUE "DROP"
 #define XML_RENAME_VALUE "RENAME"
 
@@ -647,6 +681,9 @@
 #define XML_BITANDFUNC_VALUE "BITANDFUNC"
 #define XML_BITORFUNC_VALUE "BITORFUNC"
 #define XML_BITXORFUNC_VALUE "BITXORFUNC"
+#define XML_BLOBSIZEFUNC_VALUE "BLOBSIZEFUNC"
+#define XML_CLOBSIZEFUNC_VALUE "CLOBSIZEFUNC"
+#define XML_CLOB2STRFUNC_VALUE "CLOB2STRFUNC"
 #define XML_NEXTCOUNTFUNC_VALUE "NEXTCOUNT"
 #define XML_SETCOUNTFUNC_VALUE "SETCOUNT"
 
@@ -673,18 +710,20 @@
 #define XML_PLAIN_VALUE "PLAIN"
 
 #define XML_BLOBPREFIX "B@"
+#define XML_CLOBPREFIX "C@"
 
 #define XML_NOTICE_VALUE "NOTICE"
 #define XML_LOGERR_VALUE "LOGERR"
 #define XML_DEBUG_VALUE "DEBUG"
 #define XML_NONE_VALUE "NONE"
 
+#define XML_STR_VALUE "STR"
+#define XML_ID_VALUE "ID"
+
 #define XML_ADMIN_ROLE "admin"
 #define XML_CREATE_ROLE "create"
 #define XML_MODIFY_ROLE "modify"
 #define XML_READ_ROLE "read"
 #define XML_DROP_ROLE "drop"
 
-
-
 #endif
